@@ -1,6 +1,6 @@
-/* $Id: e2p_config.c 2344 2011-04-11 22:46:24Z tpgww $
+/* $Id: e2p_config.c 3047 2014-02-08 21:56:04Z tpgww $
 
-Copyright (C) 2006-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2006-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -31,16 +31,12 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #ifdef E2_MOUSECUSTOM
 # include "e2_mousebinding.h"
 #endif
+#include "e2_icons.h"
+#include "e2p_upgrade.h" //get OLDEST_UPGRADE
 
+//signature component, must match 'core' of this file name and likewise for corresponding icon file name
 #define ANAME "config"
-#ifndef UPGRADE_PNAME
-#define UPGRADE_PNAME "e2p_upgrade.so"
-#endif
-//NOTE this needs to be the minimum actionable version in the upgrade plugin
-//or else that will just install defaults
-#ifndef OLDEST_UPGRADE
-#define OLDEST_UPGRADE "0.4.1.3"
-#endif
+
 #define OLDEST_IMPORT "0.4.0"
 
 typedef struct _E2P_ConfigData
@@ -64,7 +60,7 @@ typedef enum
 #endif
 	PLUGINS_P, MENU_P, CUSTMENU_P,
 	PANEBAR1_P, PANEBAR2_P, TASKBAR_P, CMDBAR_P,
-	MAX_FLAGS	//no. of entries in the array
+	MAX_CONFFLAGS	//no. of entries in the array
 } flag_t;
 
 typedef enum
@@ -103,6 +99,8 @@ typedef enum
 //for determining index of some specific tree
 #define FIRSTTREE_P MARKS_P
 
+static PluginIface iface;
+
 //these treeset "internal" names are in same order as the flag_t enumerator
 static gchar *set_private_names [] =
 {
@@ -124,7 +122,7 @@ static gchar *set_private_names [] =
 };
 
 //cache for toggle values, static for the session
-static gboolean flags[MAX_FLAGS];
+static gboolean flags[MAX_CONFFLAGS];
 static GPtrArray *treeset_names = NULL;	//names to use when checking importable tree options
 static gboolean rebuild_needed;
 static E2P_ConfigData *srt;	//copy of runtime ptr for func(s) that don't get rt directly
@@ -170,7 +168,7 @@ The relevant array value is set
 */
 static void _e2pc_set_flag (flag_t f, gboolean value)
 {
-	if (f < MAX_FLAGS)
+	if (f < MAX_CONFFLAGS)
 		flags[ (gint) f] = value;
 }
 /**
@@ -183,7 +181,7 @@ static void _e2pc_set_flag (flag_t f, gboolean value)
 */
 static gboolean _e2pc_get_flag (flag_t f)
 {
-	if (f < MAX_FLAGS)
+	if (f < MAX_CONFFLAGS)
 		return (flags[(gint) f]);
 	else
 		return (FALSE);
@@ -211,7 +209,11 @@ static void _e2pc_toggle_cb (GtkWidget *widget, gpointer flagnum)
 	  }
 	} */
 	if (flg == CUSTOM_P)
+	{
+		NEEDCLOSEBGL
 		gtk_expander_set_expanded (GTK_EXPANDER (srt->expander), newflag);
+		NEEDOPENBGL
+	}
 }
 /**
 @brief create and show a button in a specified container
@@ -418,7 +420,7 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 	//convert import flags to bitflags that we can play with
 	bitflag_t import_flags = 0;
 	gint i;
-	for (i = 0; i < MAX_FLAGS; i++)
+	for (i = 0; i < MAX_CONFFLAGS; i++)
 	{
 		if (flags[i])
 			import_flags |= 1 << i;
@@ -432,7 +434,7 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 	if (import_flags & (E2PC_ALL | E2PC_ALLTREE))
 		import_flags |= E2PC_ALLTREEMASK;
 	//setup tree-option private (untranslated) names, for comparison when parsing
-	treeset_names = g_ptr_array_sized_new (MAX_FLAGS);	//plenty of space, even with trailing NULL
+	treeset_names = g_ptr_array_sized_new (MAX_CONFFLAGS);	//plenty of space, even with trailing NULL
 	//FIXME make this dynamic
 	if (import_flags & E2PC_MARKS)
 		g_ptr_array_add (treeset_names, (gpointer) set_private_names[MARKS_P - FIRSTTREE_P]);
@@ -464,6 +466,7 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 		g_ptr_array_add (treeset_names, (gpointer) set_private_names[CMDBAR_P - FIRSTTREE_P]);
 
 	gboolean upgrade = FALSE;	//TRUE when importing an old config file that probably needs upgrade
+	NEEDCLOSEBGL
 	const gchar *filepath = gtk_entry_get_text (GTK_ENTRY (rt->open_entry));
 	gchar *realpath = (*filepath != '\0') ? (gchar *) filepath :
 		g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
@@ -561,7 +564,7 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 				gchar *path2 = g_build_filename (e2_cl_options.config_dir, name2, NULL);
 				gchar *local2 = F_FILENAME_TO_LOCALE (path2);
 				gchar *savepath = e2_utils_get_tempname (local2);
-				gdk_threads_leave ();	//downstream errors invoke local mutex locking
+				OPENBGL	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
 				VPATH ddata = { local1, NULL };	//local config files only
 				VPATH tdata = { savepath, NULL };	//local config files only
@@ -569,7 +572,7 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 #else
 				e2_task_backend_rename (local1, savepath);
 #endif
-				gdk_threads_enter ();
+				CLOSEBGL
 				g_free (path1);
 				F_FREE (local1, path1);
 				g_free (name2);
@@ -579,8 +582,8 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 				//save updated config file
 				e2_option_file_write (NULL);
 				//do any upgrades needed
-				Plugin *p = e2_plugins_open1 (PLUGINS_DIR G_DIR_SEPARATOR_S UPGRADE_PNAME);	//localised path
-				if (p != NULL)
+				E2P_InitData data;
+				if (e2_plugins_open_module (PLUGINS_DIR G_DIR_SEPARATOR_S UPGRADE_PNAME, &data))	//localised path
 				{
 					//fake the config version for the plugin
 					//NOTE this needs to be the minimum actionable version in the
@@ -592,12 +595,17 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 						sv = OLDEST_UPGRADE;
 					g_strlcpy (app.cfgfile_version, sv, sizeof (app.cfgfile_version));
 					//app.cfgfile_version is reverted in plugin
-					if (!p->plugin_init (p))	//do any upgrades
+					Plugin *p = (*data.init) (E2P_SETUP);	//do any upgrades
+					if (p->refcount == 1) //success
+					{
+						p->module = data.module;
+						e2_plugins_unload1 (p, FALSE); //cleanup then dump
+					}
+					else
 					{
 						printd (ERROR, "Can't initialize upgrade plugin: "UPGRADE_PNAME);
 						g_strlcpy (app.cfgfile_version, VERSION RELEASE, sizeof (app.cfgfile_version));
 					}
-					e2_plugins_unload1 (p, FALSE);
 				}
 				else
 					printd (ERROR, "Can't find upgrade plugin "UPGRADE_PNAME", so can't upgrade the imported config data");
@@ -624,24 +632,16 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 
 			if (import_flags & E2PC_PLUGINS)
 			{
-				//this is like e2_plugins_unload_all() but must not unload this plugin
 				if (app.plugins != NULL)
 				{
-					GList *member;
-					for (member = app.plugins; member != NULL; member = member->next)
-					{
-						if (member->data != NULL)
-						{
-
-							if (strcmp(((Plugin *)member->data)->signature, ANAME VERSION))
-							//FIXME handle plugins that are not allowed to be unloaded
-							{
-								e2_plugins_unload1 ((Plugin *)member->data, TRUE);
-								member->data = NULL;
-							}
-						}
-					}
-					app.plugins = g_list_remove_all (app.plugins, NULL);
+					//take data for this plugin out of harm's way
+					PluginAction *pa = iface.actsarray;
+					g_ptr_array_remove_fast (app.plugacts, pa);
+					g_hash_table_steal (app.plugins, ANAME VERSION); //the allocated key leaks
+					//unload all plugins except this one
+					e2_plugins_unload_all (TRUE);
+					g_hash_table_replace (app.plugins, g_strdup(ANAME VERSION), &iface);
+					g_ptr_array_add (app.plugacts, pa);
 				}
 			}
 			//clear relevant current information
@@ -653,13 +653,13 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 //			if (reload)
 //			{
 //				printd (INFO, "reloading config due to external change");
-//				e2_option_file_read ();
+//				e2_option_file_read (NULL);
 //			}
 
 //			e2_option_tree_install_defaults ();
 
 			if (import_flags & E2PC_PLUGINS)
-				e2_plugins_load_all (); // load plugins (if any) CHECKME what if this plugin is reloaded ?
+				e2_plugins_load_configured (); // load plugins (if any)
 
 			//re-initialise things that are not done in normal 'recreate' functions
 			e2_pane_create_option_data (&app.pane1);
@@ -693,6 +693,9 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 		gchar *msg = g_strdup_printf (_("Error reading file %s"), realpath);
 		e2_output_print_error (msg, TRUE);
 	}
+
+	NEEDOPENBGL
+
 	if (*filepath == '\0')
 		g_free (realpath);
 	F_FREE (localpath, realpath);
@@ -708,11 +711,9 @@ static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 */
 static void _e2pc_select_config_cb (GtkButton *button, E2P_ConfigData *rt)
 {
+	NEEDCLOSEBGL
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
-		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_OPEN,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-		NULL);
+		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL);
 
 	e2_dialog_setup_chooser (dialog,
 		_("select configuration data file"),
@@ -720,7 +721,10 @@ static void _e2pc_select_config_cb (GtkButton *button, E2P_ConfigData *rt)
 		GTK_FILE_CHOOSER_ACTION_OPEN,
 		TRUE,	//show hidden
 		FALSE,	//single-selection
-		GTK_RESPONSE_OK);	//default response
+		GTK_RESPONSE_OK,	//default response
+		STOCK_NAME_CANCEL, GTK_RESPONSE_CANCEL,
+		STOCK_NAME_OPEN, GTK_RESPONSE_OK,
+		NULL);
 
 	gint response;
 	while ((response = e2_dialog_run_simple (dialog, app.main_window)) == E2_RESPONSE_USER1)
@@ -735,6 +739,7 @@ static void _e2pc_select_config_cb (GtkButton *button, E2P_ConfigData *rt)
 		F_FREE (openpath, local);
 	}
 	gtk_widget_destroy (dialog);
+	NEEDOPENBGL
 }
 /**
 @brief save config file with specified name
@@ -746,6 +751,7 @@ static void _e2pc_select_config_cb (GtkButton *button, E2P_ConfigData *rt)
 */
 static void _e2pc_save_cb (GtkButton *button, E2P_ConfigData *rt)
 {
+	NEEDCLOSEBGL
 	const gchar *savepath = gtk_entry_get_text (GTK_ENTRY (rt->save_entry));
 	if (*savepath != '\0')
 	{
@@ -760,7 +766,7 @@ static void _e2pc_save_cb (GtkButton *button, E2P_ConfigData *rt)
 			if (e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
 #endif
 			{
-				gdk_threads_leave ();
+				OPENBGL
 				DialogButtons choice = e2_dialog_ow_check (NULL,
 #ifdef E2_VFS
 					&ddata,
@@ -768,9 +774,10 @@ static void _e2pc_save_cb (GtkButton *button, E2P_ConfigData *rt)
 					dlocal,
 #endif
 					NONE);
-				gdk_threads_enter ();
+				CLOSEBGL
 				if (choice != OK)
 				{
+					NEEDOPENBGL
 					g_free (dlocal);
 					return;
 				}
@@ -779,6 +786,7 @@ static void _e2pc_save_cb (GtkButton *button, E2P_ConfigData *rt)
 		}
 		e2_option_file_write (savepath);
 	}
+	NEEDOPENBGL
 }
 /**
 @brief save config file with new name
@@ -790,11 +798,9 @@ static void _e2pc_save_cb (GtkButton *button, E2P_ConfigData *rt)
 */
 static void _e2pc_saveas_cb (GtkButton *button, E2P_ConfigData *rt)
 {
+	NEEDCLOSEBGL
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
-		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SAVE,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_SAVE, GTK_RESPONSE_OK,
-		NULL);
+		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SAVE, NULL, NULL);
 
 	e2_dialog_setup_chooser (dialog,
 		_("save configuration data file"),
@@ -802,7 +808,10 @@ static void _e2pc_saveas_cb (GtkButton *button, E2P_ConfigData *rt)
 		GTK_FILE_CHOOSER_ACTION_SAVE,
 		FALSE,	//hide hidden
 		FALSE,	//single selection
-		GTK_RESPONSE_OK);	//default response
+		GTK_RESPONSE_OK,	//default response
+		STOCK_NAME_CANCEL, GTK_RESPONSE_CANCEL,
+		STOCK_NAME_SAVE, GTK_RESPONSE_OK,
+		NULL);
 
 	gint response;
 	while ((response = e2_dialog_run_simple (dialog, app.main_window)) == E2_RESPONSE_USER1)
@@ -823,7 +832,7 @@ static void _e2pc_saveas_cb (GtkButton *button, E2P_ConfigData *rt)
 		&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
 #endif
 		{
-			gdk_threads_leave ();
+			OPENBGL
 			DialogButtons choice = e2_dialog_ow_check (NULL,
 #ifdef E2_VFS
 				&ddata,
@@ -831,10 +840,11 @@ static void _e2pc_saveas_cb (GtkButton *button, E2P_ConfigData *rt)
 				local,
 #endif
 				NONE);
-			gdk_threads_enter ();
+			CLOSEBGL
 			if (choice != OK)
 			{
 				gtk_widget_destroy (dialog);
+				NEEDOPENBGL
 				g_free (local);
 				return;
 			}
@@ -846,6 +856,7 @@ static void _e2pc_saveas_cb (GtkButton *button, E2P_ConfigData *rt)
 		F_FREE (savepath, local);
 	}
 	gtk_widget_destroy (dialog);
+	NEEDOPENBGL
 }
 /**
 @brief select directory containing custom icons to be used
@@ -857,11 +868,9 @@ static void _e2pc_saveas_cb (GtkButton *button, E2P_ConfigData *rt)
 */
 static void _e2pc_select_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
+	NEEDCLOSEBGL
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
-		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-		NULL);
+		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, NULL, NULL);
 
 	e2_dialog_setup_chooser (dialog,
 		_("select icons directory"),
@@ -869,7 +878,10 @@ static void _e2pc_select_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 		TRUE,	//show hidden
 		FALSE,	//single-selection
-		GTK_RESPONSE_OK);	//default response
+		GTK_RESPONSE_OK,	//default response
+		STOCK_NAME_CANCEL, GTK_RESPONSE_CANCEL,
+		STOCK_NAME_OPEN, GTK_RESPONSE_OK,
+		NULL);
 
 	gint response;
 	while ((response = e2_dialog_run_simple (dialog, app.main_window)) == E2_RESPONSE_USER1)
@@ -887,6 +899,7 @@ static void _e2pc_select_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 		g_free (local);
 	}
 	gtk_widget_destroy (dialog);
+	NEEDOPENBGL
 }
 /**
 @brief apply icon directory
@@ -898,6 +911,7 @@ static void _e2pc_select_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 */
 static void _e2pc_apply_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
+	NEEDCLOSEBGL
 	gchar *utfpath = g_strdup (gtk_entry_get_text (GTK_ENTRY (rt->icondir_entry)));
 	if (g_str_has_suffix (utfpath, G_DIR_SEPARATOR_S))
 		*(utfpath + strlen (utfpath) - sizeof(gchar)) = '\0';
@@ -912,6 +926,7 @@ static void _e2pc_apply_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 		e2_option_str_set_direct (set, utfpath);
 		e2_toolbar_recreate_all ();
 	}
+	NEEDOPENBGL
 	g_free (utfpath);
 	F_FREE (localpath, utfpath);
 }
@@ -925,11 +940,9 @@ static void _e2pc_apply_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 */
 static void _e2pc_select_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
+	NEEDCLOSEBGL
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
-		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_SAVE, GTK_RESPONSE_OK,
-		NULL);
+		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER, NULL, NULL);
 
 	e2_dialog_setup_chooser (dialog,
 		_("select icons directory"),
@@ -937,7 +950,10 @@ static void _e2pc_select_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 		GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER,
 		FALSE,	//hide-hidden
 		FALSE,	//single-selection
-		GTK_RESPONSE_OK);	//default response
+		GTK_RESPONSE_OK,	//default response
+		STOCK_NAME_CANCEL, GTK_RESPONSE_CANCEL,
+		STOCK_NAME_SAVE, GTK_RESPONSE_OK,
+		NULL);
 
 	gint response;
 	while ((response = e2_dialog_run_simple (dialog, app.main_window)) == E2_RESPONSE_USER1)
@@ -958,7 +974,7 @@ static void _e2pc_select_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 			&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
 #endif
 		{
-			gdk_threads_leave ();
+			OPENBGL
 			DialogButtons choice = e2_dialog_ow_check (NULL,
 #ifdef E2_VFS
 				&ddata,
@@ -966,11 +982,12 @@ static void _e2pc_select_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 				local,
 #endif
 				NONE);
-			gdk_threads_enter ();
+			CLOSEBGL
 			if (choice != OK)
 			{
 				gtk_widget_destroy (dialog);
 				g_free (local);
+				NEEDOPENBGL
 				return;
 			}
 		}
@@ -981,6 +998,7 @@ static void _e2pc_select_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 		F_FREE (openpath, local);
 	}
 	gtk_widget_destroy (dialog);
+	NEEDOPENBGL
 }
 /**
 @brief copy icons to specified directory
@@ -996,8 +1014,9 @@ static void _e2pc_apply_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 	gchar *slocal, *dest, *dlocal;
 
 	//can't have trailing / for copy func
-	slocal = e2_utils_get_icons_path (FALSE);
+	slocal = e2_icons_get_custom_path (FALSE);
 
+	NEEDCLOSEBGL
 	path = gtk_entry_get_text (GTK_ENTRY (rt->iconsavedir_entry));
 	if (!g_path_is_absolute (path))
 //E2_VFSTMPOK
@@ -1022,7 +1041,7 @@ static void _e2pc_apply_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 #ifdef E2_VFS
 		VPATH sdata = { slocal, NULL };
 #endif
-		gdk_threads_leave ();
+		OPENBGL
 		result = e2_dialog_ow_check (
 #ifdef E2_VFS
 			&sdata, &ddata,
@@ -1030,7 +1049,7 @@ static void _e2pc_apply_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 			slocal, dlocal,
 #endif
 			NONE);
-		gdk_threads_enter ();
+		CLOSEBGL
 	}
 	else
 		result = OK;
@@ -1040,15 +1059,16 @@ static void _e2pc_apply_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 #ifdef E2_VFS
 		VPATH sdata = { slocal, NULL };
 #endif
-		gdk_threads_leave ();	//downstream errors invoke local mutex locking
+		OPENBGL	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
 		e2_task_backend_copy (&sdata, &ddata, E2_FTM_NORMAL);
 #else
 		e2_task_backend_copy (slocal, dlocal, E2_FTM_NORMAL);
 #endif
-		gdk_threads_enter ();
+		CLOSEBGL
 	}
 
+	NEEDOPENBGL
 	g_free (slocal);
 	g_free (dest);
 	F_FREE (dlocal, dest);
@@ -1068,7 +1088,11 @@ static void _e2pc_apply_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 */
 static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
+#ifdef USE_GTK3_0
+	GtkWidget *vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
+#endif
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
@@ -1130,15 +1154,19 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	g_free (temppath);
 	F_FREE (savepath, temppath);
 
+#ifdef USE_GTK3_2
+	GtkWidget *hbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
+#else
 	GtkWidget *hbox = gtk_hbutton_box_new ();
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
-	GtkWidget *btn = e2_button_get (_("Se_lect"), GTK_STOCK_SAVE_AS,
+	GtkWidget *btn = e2_button_get (_("Se_lect"), STOCK_NAME_SAVE_AS,
 		_("Select the file in which to store the config data"), _e2pc_saveas_cb, rt);
 	gtk_container_add (GTK_CONTAINER (hbox), btn);
-	btn = e2_button_get (_("_Save"), GTK_STOCK_SAVE,
+	btn = e2_button_get (_("_Save"), STOCK_NAME_SAVE,
 		_("Save current config data in the specified file"), _e2pc_save_cb, rt);
 	gtk_container_add (GTK_CONTAINER (hbox), btn);
 
@@ -1158,7 +1186,11 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 */
 static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
+#ifdef USE_GTK3_0
+	GtkWidget *vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
+#endif
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
@@ -1187,16 +1219,20 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 #endif
 	gtk_widget_set_size_request (rt->open_entry, 400, -1);
 
+#ifdef USE_GTK3_2
+	GtkWidget *hbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
+#else
 	GtkWidget *hbox = gtk_hbutton_box_new ();
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
-	GtkWidget *btn = e2_button_get (_("Se_lect"), GTK_STOCK_OPEN,
+	GtkWidget *btn = e2_button_get (_("Se_lect"), STOCK_NAME_OPEN,
 		_("Select the config file from which to get the data"),
 		_e2pc_select_config_cb, rt);
 	gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
-	btn = e2_button_get (_("_Apply"), GTK_STOCK_APPLY,
+	btn = e2_button_get (_("_Apply"), STOCK_NAME_APPLY,
 		_("Import config data in accord with choices below"), _e2pc_import_cb, rt);
 	gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
 
@@ -1217,7 +1253,11 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		FALSE, _("_specific group option(s)"));
 	rt->expander = gtk_expander_new_with_mnemonic (_("_groups"));
 	gtk_box_pack_start (GTK_BOX (vbox), rt->expander, FALSE, FALSE, 0);
+#ifdef USE_GTK3_0
+	GtkWidget *expvbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	GtkWidget *expvbox = gtk_vbox_new (FALSE, 0);
+#endif
 	gtk_container_add (GTK_CONTAINER (rt->expander), expvbox);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
@@ -1291,7 +1331,11 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 */
 static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
+#ifdef USE_GTK3_0
+	GtkWidget *vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
+#endif
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
@@ -1299,7 +1343,7 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		0.5, 0.5, FALSE, E2_PADDING);
 
 	gchar *openpath;
-	gchar *localpath = e2_utils_get_icons_path (FALSE);
+	gchar *localpath = e2_icons_get_custom_path (FALSE);
 	//default icons in config dir if that's usable
 #ifdef E2_VFS
 	VPATH ddata = { localpath, NULL };
@@ -1319,16 +1363,20 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	g_free (openpath);
 	gtk_widget_set_size_request (rt->icondir_entry, 400, -1);
 
+#ifdef USE_GTK3_2
+	GtkWidget *hbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
+#else
 	GtkWidget *hbox = gtk_hbutton_box_new ();
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
-	GtkWidget *btn = e2_button_get (_("Se_lect"), GTK_STOCK_OPEN,
+	GtkWidget *btn = e2_button_get (_("Se_lect"), STOCK_NAME_OPEN,
 		_("Select the directory where the icons are"),
 		_e2pc_select_icondir_cb, rt);
 	gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
-	btn = e2_button_get (_("_Apply"), GTK_STOCK_APPLY,
+	btn = e2_button_get (_("_Apply"), STOCK_NAME_APPLY,
 		_("Apply the chosen icon directory"), _e2pc_apply_icondir_cb, rt);
 	gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
 
@@ -1340,8 +1388,9 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	e2_widget_add_label (vbox, _("Copy current icons to"),
 		0.5, 0.5, FALSE, E2_PADDING);
 
-	openpath = g_strconcat (e2_cl_options.config_dir,G_DIR_SEPARATOR_S BINNAME"-",
-		_("icons"), NULL);
+	gchar *basename = e2_utils_strcat (BINNAME"-", _("icons"));
+	openpath = g_build_filename (e2_cl_options.config_dir, basename, NULL);
+	g_free (basename);
 	rt->iconsavedir_entry = e2_widget_add_entry (vbox, openpath, TRUE, TRUE);
 #ifdef E2_ASSISTED
 	e2_widget_set_label_relations (label, rt->iconsavedir_entry);
@@ -1349,16 +1398,20 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	g_free (openpath);
 	gtk_widget_set_size_request (rt->iconsavedir_entry, 400, -1);
 
+#ifdef USE_GTK3_2
+	hbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
+#else
 	hbox = gtk_hbutton_box_new ();
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
-	btn = e2_button_get (_("Se_lect"), GTK_STOCK_OPEN,
+	btn = e2_button_get (_("Se_lect"), STOCK_NAME_OPEN,
 		_("Select the directory where the icons will be saved"),
 		_e2pc_select_iconsavedir_cb, rt);
 	gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
-	btn = e2_button_get (_("C_opy"), GTK_STOCK_COPY,
+	btn = e2_button_get (_("C_opy"), STOCK_NAME_COPY,
 		_("Copy the icons to the chosen directory"), _e2pc_apply_iconsavedir_cb, rt);
 	gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
 
@@ -1410,34 +1463,19 @@ static gboolean _e2p_config_dialog_create (gpointer from, E2_ActionRuntime *art)
 	return TRUE;
 }
 
-//aname must be confined to this module
-static gchar *aname;
 /**
 @brief plugin initialization function, called by main program
 
-@param p ptr to plugin data struct
+@param mode flags enumerating what sort of init to perform
 
-@return TRUE if the initialization succeeds, else FALSE
+@return Plugin*, with refcount 1 if @a mode included runtime setup and that succeeded
 */
-gboolean init_plugin (Plugin *p)
+Plugin *init_plugin (E2PInit mode)
 {
-  aname = _("manage");
-  p->signature = ANAME VERSION;
-  p->menu_name = _("_Configure..");
-  p->description = _("Export or import configuration data");
-  p->icon = "plugin_config"E2ICONTB;
-
-  if (p->action == NULL)
-  {
-	//don't free name string here
-	E2_Action plugact =
-	{g_strconcat (_A(3),".",aname,NULL),_e2p_config_dialog_create,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
-	p->action = e2_plugins_action_register (&plugact);
-	if (p->action != NULL)
-		return TRUE;
-	g_free (plugact.name);
-  }
-  return FALSE;
+	PLUGINIT_ONEACTION_SIMPLE (_A(3),_("manage"),_e2p_config_dialog_create,
+		_("_Configure.."),
+		_("Export or import configuration data"),
+		"plugin_"ANAME E2ICONTB)
 }
 /**
 @brief cleanup transient things for this plugin
@@ -1448,9 +1486,6 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-  gchar *action_name = g_strconcat (_A(3),".",aname,NULL);
-  gboolean ret = e2_plugins_action_unregister (action_name);
-  g_free (action_name);
-  return ret;
+	PLUGIN_CLEAR_ACTIONS (p)
+	return ret;
 }
-
