@@ -1,6 +1,6 @@
-/* $Id: e2p_glob.c 2344 2011-04-11 22:46:24Z tpgww $
+/* $Id: e2p_glob.c 2978 2013-11-30 02:56:32Z tpgww $
 
-Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -31,6 +31,9 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #include "e2_dialog.h"
 #include "e2_filelist.h"
 
+//signature component, must match 'core' of this file name and likewise for corresponding icon file name
+#define ANAME "glob"
+
 //various dialog widgets ...
 typedef struct _E2_FltDlgRuntime
 {
@@ -88,6 +91,8 @@ enum
 	MAXDATECHOICES
 };
 
+static PluginIface iface;
+
 static gboolean use_name, use_size, use_date;
 static gint date_index;
 static gchar *previous_pattern;
@@ -284,6 +289,7 @@ static void _e2p_glob_toggle_cb (GtkToggleButton *togglebutton,
 #else
 		togglebutton->active;
 #endif
+	NEEDCLOSEBGL
 	if (togglebutton == (GtkToggleButton*)data->usename_check)
 	{
 		use_name = state; //cache for session
@@ -317,6 +323,7 @@ static void _e2p_glob_toggle_cb (GtkToggleButton *togglebutton,
 	 || GTK_TOGGLE_BUTTON (data->usedate_check)->active;
 #endif
 	gtk_widget_set_sensitive (data->apply_button, state);
+	NEEDOPENBGL
 }
 /**
 @brief callback for all glob-dialog responses
@@ -344,6 +351,7 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 			E2_FltDlgData choices;
 			//assignment for complier-warning prevention only
 			memset (&choices, '\0', sizeof (E2_FltDlgData));
+			NEEDCLOSEBGL
 			//get all the relevant parameters from the dialog
 			choices.name_filter.use =
 #ifdef USE_GTK2_14
@@ -465,6 +473,7 @@ static void _e2p_glob_response_cb (GtkDialog *dialog, gint response,
 					e2_filelist_enable_refresh ();
 				}
 			}
+			NEEDOPENBGL
 		}
 			break;
 		default:
@@ -640,9 +649,11 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 	e2_dialog_add_defined_button (dialog, &E2_BUTTON_CANCEL);
 	data.apply_button = e2_dialog_add_defined_button (dialog, &local_btn);
 	//set initial sensitivities
+	NEEDOPENBGL
 	_e2p_glob_toggle_cb ((GtkToggleButton*)data.usename_check, &data);
 	_e2p_glob_toggle_cb ((GtkToggleButton*)data.usesize_check, &data);
 	_e2p_glob_toggle_cb ((GtkToggleButton*)data.usedate_check, &data);
+	NEEDCLOSEBGL
 
 	e2_dialog_setup (dialog, app.main_window);
 //	e2_dialog_resize (dialog, 1.3);	//really, we only want to make it wider ..
@@ -652,36 +663,19 @@ static gboolean _e2p_glob (gpointer from, E2_ActionRuntime *art)
 	return TRUE;
 }
 
-//aname must be confined to this module
-static gchar *aname;
 /**
 @brief plugin initialization function, called by main program
 
-@param p ptr to plugin data struct
+@param mode flags enumerating what sort of init to perform
 
-@return TRUE if the initialization succeeds, else FALSE
+@return Plugin*, with refcount 1 if @a mode included runtime setup and that succeeded
 */
-gboolean init_plugin (Plugin *p)
+Plugin *init_plugin (E2PInit mode)
 {
-#define ANAME "glob"
-	aname = _("glob");
-
-	p->signature = ANAME VERSION;
-	p->menu_name = _("_Glob..");
-	p->description = _("Select items matching a specified pattern");
-	p->icon = "plugin_"ANAME E2ICONTB;  //use icon file pathname if appropriate
-
-	if (p->action == NULL)
-	{
-		//don't free name string here
-		E2_Action plugact =
-		{g_strconcat (_A(7),".",aname,NULL),_e2p_glob,TRUE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
-		p->action = e2_plugins_action_register (&plugact);
-		if G_LIKELY((p->action != NULL))
-			return TRUE;
-		g_free (plugact.name);
-	}
-	return FALSE;
+	PLUGINIT_ONEACTION_SIMPLE (_A(7),_("glob"),_e2p_glob,
+		_("_Glob.."),
+		_("Select items matching a specified pattern"),
+		"plugin_"ANAME E2ICONTB)
 }
 /**
 @brief cleanup transient things for this plugin
@@ -692,9 +686,7 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-	gchar *action_name = g_strconcat (_A(7),".",aname,NULL);
-	gboolean ret = e2_plugins_action_unregister (action_name);
-	g_free (action_name);
+	PLUGIN_CLEAR_ACTIONS (p)
 	if (previous_pattern != NULL)
 		g_free (previous_pattern);
 	return ret;
