@@ -1,6 +1,6 @@
-/* $Id: e2p_selmatch.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2p_selmatch.c 2865 2013-10-27 09:18:13Z tpgww $
 
-Copyright (C) 2009-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2009-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -27,6 +27,11 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #include "e2_fileview.h"
 #include "e2_filelist.h"
 
+//signature component, must match 'core' of this file name and likewise for corresponding icon file name
+#define ANAME "selmatch"
+
+static PluginIface iface;
+
 static void _e2p_selsame_reselect (GtkTreePath *tp, GtkTreeSelection *sel)
 {
 	gtk_tree_selection_select_path (sel, tp);
@@ -219,129 +224,67 @@ static gboolean _e2p_select_same (gpointer from, E2_ActionRuntime *art)
 	return FALSE;
 }
 
-//aname must be confined to this module
-static gchar *aname, *aname2;
+/**
+@brief iterate over active pane file list to check for matches
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE if action completed successfully, else FALSE
+*/
+static gboolean _e2p_select_like (gpointer from, E2_ActionRuntime *art)
+{
+	art->action->data = GINT_TO_POINTER (1); //non-NULL to trigger partial-matches
+	return _e2p_select_same (from, art);
+}
+
 /**
 @brief plugin initialization function, called by main program
 
-@param p ptr to plugin data struct
+@param mode flags enumerating what sort of init to perform
 
-@return TRUE if the initialization succeeds, else FALSE
+@return Plugin*, with refcount 1 if @a mode included runtime setup and that succeeded
 */
-gboolean init_plugin (Plugin *p)
+Plugin *init_plugin (E2PInit mode)
 {
-#define ANAME "selmatch"
-	aname = _("selmatch");
-	aname2 = _("selmatchpart");
+	const gchar *aname2 = _("selmatchpart");
 
-	p->signature = ANAME VERSION;
-	p->menu_name = _("_Select same");
-	p->description = "";
-	p->icon = "plugin_"ANAME E2ICONTB;  //prepend path if appropriate
+	PLUGINIT_INTRO
+	PLUGINIT_NUMBERED_ALLOCATE(2)
 
-	//child UI data
-	gchar *sig1 = "0-"ANAME VERSION;
-	gchar *label1 = _("_Whole");
-//	gchar *icon1 = "";	//no icon "plugin_"ANAME E2ICONTB;  //use icon file pathname if appropriate
-	gchar *tip1 = _("Select items whose whole name matches a selected item in the other pane");
-	gchar *sig2 = "1-"ANAME VERSION;
-	gchar *label2 = _("_Partial");
-//	gchar *icon2 = "";
-	gchar *tip2 = _("Select items whose name partially matches a selected item in the other pane");
+	PLUGINIT_NUMBERED_ACTION(1,_A(7),_("selmatch"),_e2p_select_same,
+		_("_Select same"),
+		_("Select items whose whole name matches a selected item in the other pane"),
+		"plugin_"ANAME E2ICONTB)
+	PLUGINIT_NUMBERED_ACTION(2,_A(7),aname2,_e2p_select_like,
+		_("Select _like"),
+		_("Select items whose name partially matches a selected item in the other pane"),
+		NULL)
 
-	if (p->action == NULL)
+	//if the above included init, and it succeeded, more init needed
+	if (iface.refcount == 1)
 	{
-		gboolean retval;
-		Plugin *pc = e2_plugins_create_child (p);
-		if (pc != NULL)
-		{
-			//for reconciling with config treestore data, signatures must reflect
-			//0-based index of each child's order in the list of children
-			pc->signature = sig1; //begin with "n-", n=0,1, ...
-			//these will generallly be discarded in favour of config treestore data
-			//meaning that any non-constant string will leak
-			pc->menu_name = label1;	//or whatever
-//			pc->description = _("Copy selected item(s), with displayed progress details");
-			pc->description = tip1;	//or whatever
-//			pc->icon = "plugin_copy"E2ICONTB;  //use icon file pathname if appropriate
-			//normally don't free name string here
-			E2_Action plugact =
-			{g_strconcat (_A(7),".",aname,NULL),_e2p_select_same,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL}; // not _A(6) !
-			pc->action = e2_plugins_action_register (&plugact);
-			retval = (pc->action != NULL);
-			if (!retval)
-				g_free (plugact.name);
-		}
-		else
-			retval = FALSE;
+		E2_OptionSetupExtra ex;
+		gchar *group = g_strconcat(_C(34),".",_C(27),":",aname2,NULL); //_("plugins.options:selmatchpart"
+		memset (&ex, 0, sizeof (E2_OptionSetupExtra));
+		ex.exbool = TRUE;
+		E2_OptionSet *set = e2_plugins_option_register (E2_OPTION_TYPE_BOOL, "selmatch-start",
+			group, _("match to first separator"),
+			_("If enabled, name matching stops at the first instance of any specified separator, otherwise, at the last instance"),
+			NULL, &ex, E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_ADVANCED);
+		//because plugins are loaded after config data, config options need to
+		//get any data from unknown-options data
+		e2_option_transient_value_get (set);
 
-		//this will not be used, but MUST be set to allow checking whether to show
-		//the item in the context menu
-		if (retval)
-			p->action = pc->action;
-
-		pc = e2_plugins_create_child (p);
-		if (pc != NULL)
-		{
-			pc->signature = sig2;	//for reconciling with config treestore data
-			pc->menu_name = label2;
-			pc->description = tip2;
-//			pc->icon = "plugin_copy"E2ICONTB;  //use icon file pathname if appropriate
-			//don't free name string here
-			E2_Action plugact =
-			{g_strconcat (_A(7),".",aname2,NULL),_e2p_select_same,FALSE,E2_ACTION_TYPE_ITEM,0,GINT_TO_POINTER(1),NULL}; // not _A(6) !
-			pc->action = e2_plugins_action_register (&plugact);
-			retval = (pc->action != NULL);
-		}
-		else
-			retval = FALSE;
-
-		if (retval)
-		{
-			E2_OptionSet *set;
-			E2_OptionSetupExtra ex;
-			gchar *group = g_strconcat(_C(34),".",_C(27),":",aname2,NULL); //_("plugins.options:selmatchpart"
-			memset (&ex, 0, sizeof (E2_OptionSetupExtra));
-			ex.exbool = TRUE;
-			set = e2_plugins_option_register (E2_OPTION_TYPE_BOOL, "selmatch-start",
-				group, _("match to first separator"),
-				_("If enabled, name matching stops at the first instance of any specified separator, otherwise, at the last instance"),
-				NULL, &ex, E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_ADVANCED);
-			//because plugins are loaded after config data, config options need to
-			//get any data from unknown-options data
-			e2_option_transient_value_get (set);
-
-			ex.exstr = ".";
-			set = e2_plugins_option_register (E2_OPTION_TYPE_STR, "selmatch-separators",
-				group, _("separator character(s)"),
-				_("String comprising all chars considered to be a 'separator'"),
-				NULL, &ex, E2_OPTION_FLAG_ADVANCED);
-			e2_option_transient_value_get (set);
-		}
-
-		if (retval && (p->action == NULL))
-			p->action = pc->action;
-
-		return retval;
-	}
-	else	//setup children UI data for pushing into a config dialog
-	{
-		E2_Sextet *uidata;
-		uidata = e2_utils_sextet_new ();
-		p->child_list = g_list_append (p->child_list, uidata);
-		uidata->a = label1;
-		uidata->b = "";	//no icon
-		uidata->c = tip1;
-		uidata->d = sig1;
-		uidata = e2_utils_sextet_new ();
-		p->child_list = g_list_append (p->child_list, uidata);
-		uidata->a = label2;
-		uidata->b = "";
-		uidata->c = tip2;
-		uidata->d = sig2;
+		ex.exstr = ".";
+		set = e2_plugins_option_register (E2_OPTION_TYPE_STR, "selmatch-separators",
+			group, _("separator character(s)"),
+			_("String comprising all chars considered to be a 'separator'"),
+			NULL, &ex, E2_OPTION_FLAG_ADVANCED);
+		e2_option_transient_value_get (set);
 	}
 
-	return FALSE;
+	PLUGINIT_NUMBERED_END
+
 }
 /**
 @brief cleanup transient things for this plugin
@@ -352,16 +295,12 @@ gboolean init_plugin (Plugin *p)
 */
 gboolean clean_plugin (Plugin *p)
 {
-	gchar *action_name = g_strconcat (_A(7),".",aname,NULL);
-	gboolean ret1 = e2_plugins_action_unregister (action_name);
-	g_free (action_name);
-	action_name = g_strconcat (_A(7),".",aname2,NULL);
-	gboolean ret2 = e2_plugins_action_unregister (action_name);
-	g_free (action_name);
-	if (ret1 && ret2)
-	{
-		ret1 = ret1 && e2_plugins_option_unregister ("selmatch-start");
-		ret2 = ret2 && e2_plugins_option_unregister ("selmatch-separators");
-	}
-	return (ret1 && ret2);
+	PLUGIN_CLEAR_ACTIONS (p)
+
+	if (ret)
+		ret = e2_plugins_option_unregister ("selmatch-start");
+	if (ret)
+		ret = e2_plugins_option_unregister ("selmatch-separators");
+
+	return ret;
 }
