@@ -1,6 +1,6 @@
-/* $Id: e2p_unpack.c 2309 2011-01-17 00:12:22Z tpgww $
+/* $Id: e2p_unpack.c 3039 2014-02-02 21:37:42Z tpgww $
 
-Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2014 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -24,12 +24,24 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 
 #include "emelfm2.h"
 #include <string.h>
+#include <glob.h>
 #include "e2_plugins.h"
 #include "e2_dialog.h"
 #include "e2_task.h"
 #include "e2_filelist.h"
-//same enum as in pack plugin, though it need not be so
-enum { TAR_GZ, TAR_BZ2, TAR_LZMA, TAR_XZ, TAR, /*DEB, RPM, */ZIP, Z7Z, RAR, ARJ, ZOO, MAXTYPES };
+#include "e2_icons.h"
+
+//signature component, must match 'core' of this file name and likewise for corresponding icon file name
+#define ANAME "unpack"
+
+//NOT same enum as in pack plugin
+typedef enum
+{
+	UPTYPENONE = -1,
+	//NOTE some handler-string-arrays are assumed to be arranged in same order as this enum
+	TAR_GZ, TAR_BZ2, TAR_XZ, ZIP, TAR_7Z, Z7Z, TAR_LZMA, TAR, DEB, RPM, RAR, ARJ, ZOO, ISO,
+	UPTYPESCOUNT
+} E2_UnPackType;
 
 typedef struct _E2P_Unpackdata
 {
@@ -40,17 +52,35 @@ typedef struct _E2P_Unpackdata
 	glong thispid;	//id of process which is re-packing an archive
 //	guint chdir_id;	//id of timer which checks for whether to ask user what to do
 	guint pack_id;	//id of timer which checks for repack completion
+	E2_UnPackType type; //enum for type of package being processed
+	guint compress_type; //enum for type of tarball compression in .deb package
+	gpointer from;	//widget which initiated the unpack action
 	E2_CDType cd_completed;	//flag set when cd to temp dir is completed
-	gchar *command;	//the upack command to be run, UTF-8 string
+	gchar *command;	//the pack (and later, unpack) command to be run, UTF-8 string
 	gboolean departing;	//TRUE when the temp dir is waiting to be processed after
 						//the user selects from a what-to-do dialog
-						//also used to block reentrant use of the hook function
+						//also used to block re-entrant use of the hook function
 } E2P_Unpackdata;
 
 static gboolean _e2p_unpack_delete_dir (E2P_Unpackdata *data);
 static gboolean _e2p_unpack_change_dir_hook (gchar *path, E2P_Unpackdata *data);
 
-static gchar *unpack_tmp;	//absolute path of 'base working' directory (no trailer), utf-8 string
+/*
+//strings used in processing compressed items in .deb files
+//each row has a suffix (to be appended to ...tar) and associated uncompress-to-stdout command
+static gchar *unsqueezers[5][2] =
+{
+	{"",	"cat"},
+	{".gz",	"gzip -cd"},
+	{".bz2","bzip2 -cd"},
+	{".lzma","lzma -cd"},
+	{".xz",	"xz -cd"}
+};
+*/
+
+static PluginIface iface;
+
+static gchar *unpack_tmp = NULL;	//absolute path of 'base working' directory (no trailer), UTF-8 string
 
 /**
 @brief cleanup plugin data
@@ -73,125 +103,282 @@ static void _e2p_unpack_cleanup (E2P_Unpackdata *data)
 
 @param localpath pointer to archive data, with localised path
 
-@return the number, or -1 upon no match
+@return the number, or UPTYPENONE upon no match
 */
-static gint _e2p_unpack_match_type (VPATH *localpath)
+static E2_UnPackType _e2p_unpack_match_type (VPATH *localpath)
 {
-	gint typecode, count;
-/*	//FIXME use a better way to distinguish filetypes eg magic number
-	//e.g mimetype
-	file string needs to be localised VPATH *
-	file(1) cannot detect all these mimetypes
-*/
+	gint i, count;
+	E2_UnPackType thistype = UPTYPENONE;
 	gchar *thismime = e2_utils_get_mimetype (localpath);
+
 	if (thismime != NULL)
 	{
 		const gchar *mimes [] =
 		{
 			"x-gzip",
 			"x-bzip2",
-			//CHECKME not sure about lzma mimetype yet
-			"x-lzma-compressed-tar",
-			//this may be version-specific i.e. NOT for LZMA_Alone tool in LZMA SDK <= v.4.43
-			"x-lzma",
-			"x-xz"
+			"x-xz-compressed-tar",
 			"x-tar",
-//			"x-deb",
-//			"x-rpm",
 			"zip",
-			"x-7z", //"x-7z-compressed" ?
-			"x-rar", //"x-rar-compressed" ?
-			"x-arj", //"arj" ?
-			"x-zoo"
+			"x-7z-compressed-tar",	//this is probably N/A ATM
+			"x-7z-compressed",		//this may be reported for .tar.7z as well as .7z - further check needed
+			"x-lzma-compressed-tar",//CHECKME not sure about lzma mimetype yet
+			"x-deb",
+			"x-rpm",
+			"x-rar-compressed",
+			"arj",
+			"x-arj",
+			"zoo",
+			"x-zoo",
+			"x-cd-image",
+			"iso-image"
 		};
-		gint mimecodes [] =
+		E2_UnPackType mimecodes [] =
 		{
 			TAR_GZ,
 			TAR_BZ2,
-			TAR_LZMA,	//2 alternates for lzma
-			TAR_LZMA,
 			TAR_XZ,
 			TAR,
-	//		DEB,
-	//		RPM,
 			ZIP,
+			TAR_7Z,
 			Z7Z,
+			TAR_LZMA,
+			DEB,
+			RPM,
 			RAR,
 			ARJ,
+			ARJ,
 			ZOO,
+			ZOO,
+			ISO,
+			ISO
 		};
+
 		if (g_str_has_prefix (thismime, "application/"))
 		{
-			gchar *s = thismime + sizeof (gchar) * 12;
+			gchar *s = thismime + sizeof (gchar) * 12; //skip the prefix
 			count = sizeof (mimes) / sizeof (gchar *);
-			for (typecode = 0; typecode < count; typecode++)
+			for (i = 0; i < count; i++)
 			{
-				if (strcmp (s, mimes [typecode]) == 0)
+				if (strcmp (s, mimes [i]) == 0)
 				{
-					typecode = mimecodes [typecode];
+					if (i == 6)
+					{
+						if (g_str_has_suffix (VPCSTR(localpath), ".tar.7z")
+						 || g_str_has_suffix (VPCSTR(localpath), ".t7z"))
+						i = 5;
+					}
+					thistype = mimecodes [i];
 					break;
 				}
 			}
-			if (typecode == count)
-				typecode = -1;
 		}
-		else
-			typecode = -1;
-
 		g_free (thismime);
 	}
-	else //mime-check by xdg-utils(1) or file(1) not available, try some other approach
-		typecode = -1;
 
-	if (typecode == -1)
+	if (thistype == UPTYPENONE)
 	{
+		//mime-check by xdg-utils(1) or file(1) not available, try extension-match
 		const gchar *extensions [] =
 		{
-			".tar.gz", ".tgz", //compress(1) does ".taz", ".tar.z",
-			".tar.bz2", ".tbz2",
-			".tar.lzma", ".tlz",
+			".tar.gz", //compress(1) does ".taz", ".tar.z",
+			".tgz",
+			".tar.bz2",
+			".tbz2",
 			".tar.xz",
 			".tar",
-	//		".deb",
-	//		".rpm".
 			".zip",
+			".tar.7z",
+			".t7z",
 			".7z",
+			".tar.lzma",
+			".tlz",
+			".deb",
+			".rpm",
 			".rar",
 			".arj",
 			".zoo",
+			".iso"
 		};
 		//codes corresponding to extensions array, above
-		gint extcodes [] =
+		E2_UnPackType extcodes [] =
 		{
-			TAR_GZ, TAR_GZ,
-			TAR_BZ2, TAR_BZ2,
-			TAR_LZMA, TAR_LZMA,
+			TAR_GZ,
+			TAR_GZ,
+			TAR_BZ2,
+			TAR_BZ2,
 			TAR_XZ,
 			TAR,
-	//		DEB,
-	//		RPM,
 			ZIP,
+			TAR_7Z,
+			TAR_7Z,
 			Z7Z,
+			TAR_LZMA,
+			TAR_LZMA,
+			DEB,
+			RPM,
 			RAR,
 			ARJ,
 			ZOO,
+			ISO
 		};
 
 		//CHECKME matched string not at end of name - ok ?
 		count = sizeof (extensions) / sizeof (gchar *);
-		for (typecode = 0; typecode < count; typecode++)
+		for (i = 0; i < count; i++)
 		{
-			if (g_str_has_suffix (VPSTR (localpath), extensions[typecode]))
+			if (g_str_has_suffix (VPSTR (localpath), extensions[i]))
 			{
-				typecode = extcodes [typecode]; //must be < count
+				thistype = extcodes [i];
 				break;
 			}
 		}
-
-		if (typecode == count)
-			typecode = -1;
 	}
-	return typecode;
+	return thistype;
+}
+/**
+@brief check for, and if found, decompress, instances of a type of compressed file
+
+Tarballs need to be handled by 2 calls here.
+
+@param ext compressed file extension ".gz" ... ".xz", ".tar" or ""
+@param handler command to be run to process each individual item matching @a ext
+@param from the button, menu item etc which was activated to initiate this action
+@param localpath localised path string of temporary workdir containing
+ now-unpacked but still-compressed items (if any) to be processed
+
+@return 0 on success, command exit-code or glob error-code otherwise
+*/
+static gint _e2p_unpack_decompress_helper (const gchar *ext, const gchar *handler,
+	gpointer from, const gchar *localpath)
+{
+	gint retval;
+	gchar *pattern;
+	gchar base[16];
+	glob_t matches;
+
+	snprintf (base, sizeof(base), "*%s", ext);
+	pattern = g_build_filename (localpath, base, NULL);
+
+	retval = glob (pattern, GLOB_NOSORT, NULL, &matches);
+	if (retval == 0)
+	{
+		size_t i;
+
+		CLOSEBGL
+		for (i = 0; i < matches.gl_pathc; i++)
+		{
+			gchar *cmd = g_strdup_printf (handler, matches.gl_pathv[i]);
+			retval = e2_command_run_at (cmd, localpath, E2_COMMAND_RANGE_DEFAULT,
+				from
+#ifdef E2_COMMANDQ
+				, T/F ?
+#endif
+			);
+			g_free (cmd);
+			if (retval != 0)
+				break;
+		}
+		OPENBGL
+	}
+	g_free (pattern);
+	globfree (&matches);
+
+	return retval;
+}
+/**
+@brief decompress any compressed file(s) in @a localpath
+
+Assumes BGL open upon arrival. Downstream relies on that.
+Return value is per (now-unused) strings-array row-index, 0 for uncompressed tar
+... 4 for xz i.e. NOT a E2_UnPackType.
+
+@param from the button, menu item etc which was activated
+@param localpath temporary workdir containing unpacked items
+
+@return enumerator 0... representing 'latest-compression-type' of files processed,
+ or -1 upon error
+*/
+static gint _e2p_unpack_decompress (gpointer from, const gchar *localpath)
+{
+	gint retval = -1;
+
+#ifdef DEBUG_MESSAGES
+	gchar *output = NULL;
+	e2_fs_get_command_output ("pwd", (gpointer *)&output);
+	printd (DEBUG, "Decompressing unpacked archives in %s", output);
+	g_free (output);
+#endif
+	//cuz decompressions may be in series, the first ones block till done
+	//process all gz's
+	if (_e2p_unpack_decompress_helper (".gz", "|gzip -d %s", from, localpath) == 0)
+		retval = 1;
+	//process all bz2's
+	if (_e2p_unpack_decompress_helper (".bz2", "|bzip2 -d %s", from, localpath) == 0)
+		retval = 2;
+	//process all lzma's
+	if (_e2p_unpack_decompress_helper (".lzma", "|lzma -d %s", from, localpath) == 0)
+		retval = 3;
+	//process all xz's
+	if (_e2p_unpack_decompress_helper (".xz", "|xz -d %s", from, localpath) == 0)
+		retval = 4;
+	//prior unpack may be incomplete
+	if (retval != -1)
+	{
+		WAIT_FOR_EVENTS_UNLOCKED_SLOWLY
+	}
+	//process all tar's, perhaps created by prior call to helper
+	if (_e2p_unpack_decompress_helper (".tar",
+		"TARFILE=%s;tar -xpf $TARFILE && rm -f $TARFILE", from,	localpath) == 0
+		&& retval == -1)
+			retval = 0;
+
+	//clear assigned internal variable
+	gchar *cmd = g_strdup ("TARFILE=");
+	e2_command_run_at (cmd, localpath, E2_COMMAND_RANGE_DEFAULT, from
+#ifdef E2_COMMANDQ
+		, T/F ?
+#endif
+	);
+	g_free (cmd);
+
+#ifdef E2_FAM
+	e2_filelist_request_refresh (curr_view->dir, FALSE);
+//	e2_filelist_request_refresh (other_view->dir, TRUE);
+#else
+	e2_filelist_check_dirty (GINT_TO_POINTER (1));
+#endif
+
+	return retval;
+}
+/**
+@brief timer callback to complete unpacking of some sorts of item
+
+@param data pointer to unpack data struct
+
+@return FALSE (to cancel timer) after processing has been completed
+*/
+static gboolean _e2p_unpack_expand_contents (E2P_Unpackdata *data)
+{
+	if (!g_str_has_prefix (curr_view->dir, data->workdir))
+		return TRUE; //not at unpack dir yet, wait some more
+//	if (0) //TODO check for not ready yet
+//		return TRUE;
+
+	if (data->type == DEB)
+	{
+		data->compress_type = _e2p_unpack_decompress (data->from, data->workdir);
+	}
+	else if (data->type == RPM)
+	{
+		_e2p_unpack_decompress (data->from, data->workdir);
+	}
+
+	CLOSEBGL
+	e2_window_set_cursor (GDK_LEFT_PTR);
+	OPENBGL
+
+	return FALSE;
 }
 /**
 @brief timer callback to resume idle checking
@@ -208,8 +395,10 @@ static gboolean _e2p_unpack_pause (E2P_Unpackdata *data)
 }
 /**
 @brief idle callback to check whether temp-dir deletion can proceed safely
-We do this check at an idle time to reduce risk of conflict, but interpose a timer
-between idle callbacks to reduce their frequency
+
+We do this check at an idle time to reduce risk of conflict, but interpose a
+timer between idle callbacks to reduce their frequency
+
 @param data pointer to unpack data struct
 
 @return FALSE always
@@ -254,10 +443,38 @@ static gboolean _e2p_unpack_delete_dir (E2P_Unpackdata *data)
 #ifdef E2_VFS
 	VPATH ddata = { local, NULL };	//local unpacking only
 	if (e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
+	{
+		if (data->type == ISO)
+		{
+			//must unmount before dir can be deleted
+			gchar *cmd = e2_utils_strcat ("umount ", data->workdir);
+			e2_command_run_at (cmd, NULL,
+				E2_COMMAND_RANGE_DEFAULT, data->from
+#ifdef E2_COMMANDQ
+				, T/F ?
+#endif
+				);
+			g_free (cmd);
+		}
 		e2_task_backend_delete (&ddata);
+	}
 #else
 	if (e2_fs_access2 (local E2_ERR_NONE()) == 0)
+	{
+		if (data->type == ISO)
+		{
+			//must unmount before dir can be deleted
+			gchar *cmd = e2_utils_strcat ("umount ", data->workdir);
+			e2_command_run_at (cmd, NULL,
+				E2_COMMAND_RANGE_DEFAULT, data->from
+#ifdef E2_COMMANDQ
+				, T/F ?
+#endif
+				);
+			g_free (cmd);
+		}
 		e2_task_backend_delete (local);
+	}
 #endif
 
 	e2_filelist_enable_refresh ();
@@ -326,64 +543,74 @@ static void _e2p_unpack_repack (E2P_Unpackdata *data, gpointer from)
 		return;
 	}
 #endif
-	const gchar *pack_str [MAXTYPES] =
+
+	const gchar *cmd_str [UPTYPESCOUNT] =
 	{	//these command strings are in same order as enum
 		//they are all designed to be executed from the unpack temp dir, and
 		//on all its contents (recursive if appropriate)
 		//CHECKME "." always remains correct during repack execution ?
 		">tar cf - . | gzip - > %s",
 		">tar cf - . | bzip2 - > %s",	//default block size -9 is not significantly better, maybe use -3 instead ?
-		">tar cf - . | lzma - > %s",	//default compresssion -7, maybe use -2 instead ?
-		">tar cf - . | xz - > %s",
-		"tar cf %s .",
-//		deb 'pack' command ?
-//		rpm 'pack' command does not exist
+		">tar cf - . | xz - > %s",		//default compresssion -6, maybe use -3 instead ?
 		">zip -r - . > %s",
-		">7za a -t7z -so . > %s",	//CHECKME
+		">tar cf - . | 7za a -si %s 1>/dev/null", //default compresssion type 7z
+		">7za a %s . 1>/dev/null",
+		">tar cf - . | lzma - > %s",
+		"tar cf %s .",
+//		"echo \".deb package re-packing is not supported\"",	//deb 'pack' process N/A here
+		NULL,	//.deb pack command involves custom treatment - see below
+//		"echo \".rpm package re-packing is not supported\"",	//rpm 'pack' process does not exist, in general
+		NULL,	//.rpm pack command involves custom treatment - see below
 		//updates of archives probably don't capture renames,deletes
 		"rar u -as -ol -tl -r %s .", //OK
-
 		"arj a -u -r -s -a -2s %s .", //CHECKME
-		"zoo unP %s ."	//CHECKME replace, not update ? deletions ?
+		"zoo unP %s .",	//CHECKME replace, not update ? deletions ?
+		"mkisofs -o %s . && umount -lf %s" //NOTE extra %s, special handling needed
 	};
 /*
-compress ANSI files: 7za a -tzip archive.zip file1 file2 ... fileN
-compress ANSI dir:   7za a -tzip archive.zip dirnametocompress\
-compress UNICODE files: 7za a -t7z archive.7z file1 file2 ... fileN
-compress UNICODE dir:   7za a -t7z archive.7z dirnametocompress\
-To backup directories you must use tar :
-   - to backup a directory : tar cf - directory | 7za a -si directory.tar.7z
-   - to restore your backup : 7za x -so directory.tar.7z | tar xf -
-
-ar -x package-file.deb
-a deb top-level package contains a small file with a version number,
-and two tar archives - one contains the package description, MD5 checksums and
-install/uninstall scripts, the other the files to be installed.
+a deb top-level package contains "debian-binary", a small file with a version
+number, and two archives - one (control.tar.gz|bz2...) contains the package description,
+MD5 checksums and install/uninstall scripts (none of which have a path),
+the other (data.tar.gz|bz2....) holds the files to be installed, each with its OS path
+(i.e. unpacked relative to the unpack directory).
+so to repack:
+find . -maxdepth 1 \! -type d \! -name debian-binary -exec tar -c --owner=0 --group=0 '{}' + > control.tar.gz;
+find . -maxdepth 1 -type d \! -name . -exec tar -c --owner=0 --group=0 '{}' + > data.tar.gz;
+ar -rD %s debian-binary control.tar.gz data.tar.gz
 */
-	gchar *package = data->package;
-#ifdef E2_VFSTMP
-	VPATH ddata;
-	ddata.path = F_FILENAME_TO_LOCALE (package);
-	ddata.spacedata = curr_view->spacedata; //FIXME
-	gint index = _e2p_unpack_match_type (&ddata);
-#else
-	gchar *local = F_FILENAME_TO_LOCALE (package);
-	gint index = _e2p_unpack_match_type ((VPATH*)local);
-#endif
+	gint res;
 	gchar *fmt, *qp;
+	gchar *package = data->package;
+	gchar *local = F_FILENAME_TO_LOCALE (package);
+
 	g_free (data->command);
 
-	switch (index)
+	E2_UnPackType uptype = data->type;
+	switch (uptype)
 	{
+#if 0
+		case DEB:
+		/* Assume lzma compression is ok for tarballs in current deb packages.
+		   Or else, cache the unpack-type and customise this command accordingly */
+			cmd_str [index] =
+/*
+">find . -maxdepth 1 \\! -type d \\! -name debian-binary -exec tar -c --owner=0 --group=0 '{}' + | lzma - > control.tar.lzma;"
+">find . -maxdepth 1 -type d \\! -name . -exec tar -c --owner=0 --group=0 '{}' + | lzma - > data.tar.lzma;"
+"chown +0:+0 debian-binary control.tar.lzma data.tar.lzma;"
+"ar -r %s debian-binary control.tar.lzma data.tar.lzma"; //archive content-order matters
+*/
+			"echo \".deb package re-packing is not supported\"";
+#endif
 		case TAR_GZ:
 		case TAR_BZ2:
-		case TAR_LZMA:
 		case TAR_XZ:
-		case TAR:
 		case ZIP:
+		case TAR_7Z:
 		case Z7Z:
+		case TAR_LZMA:
+		case TAR:
 			//these types of archive are overwritten
-			fmt = g_strconcat (pack_str [index], " && mv -f %s %s && rm -rfd %s", NULL);
+			fmt = g_strconcat (cmd_str [uptype], " && mv -f %s %s && rm -rfd %s", NULL);
 			qp = e2_utils_quote_string (package);
 			gchar *tmp = e2_utils_get_tempname
 #ifdef E2_VFSTMP
@@ -402,14 +629,23 @@ install/uninstall scripts, the other the files to be installed.
 		case RAR:
 		case ARJ:
 		case ZOO:
+		case ISO:
 			//these types of archive are just updated
-			fmt = g_strconcat (pack_str [index], " && rm -rfd %s", NULL);
+			fmt = g_strconcat (cmd_str [uptype], " && rm -rfd %s", NULL);
 			qp = e2_utils_quote_string (package);
-			data->command = g_strdup_printf (fmt, qp, data->workdir, NULL);
+			if (uptype != ISO)
+				data->command = g_strdup_printf (fmt, qp, data->workdir, NULL);
+			else
+				data->command = g_strdup_printf (fmt, qp, data->workdir, data->workdir, NULL);
 //			printd (DEBUG, "repack command %s", data->command);
 			break;
+		case DEB:
+		case RPM:
+			fmt = qp = NULL;	//warning prevention
+			data->command = NULL;
+			break;
 		default:
-		//case -1:	//should never happen
+//		case UPNONE:	//should never happen
 			fmt = qp = NULL;	//warning prevention
 			_e2p_unpack_cleanup (data);
 			return;
@@ -417,19 +653,20 @@ install/uninstall scripts, the other the files to be installed.
 
 	g_free (fmt);
 	g_free (qp);
-
-#ifdef E2_VFSTMP
-	F_FREE (ddata.path, package);
-#else
 	F_FREE (local, package);
-#endif
 
-	gint res = e2_command_run_at (data->command, data->workdir,
+	if (data->command != NULL)
+	{
+		res = e2_command_run_at (data->command, data->workdir,
 		E2_COMMAND_RANGE_DEFAULT, from
 #ifdef E2_COMMANDQ
 		, T/F ?
 #endif
-	);
+		);
+	}
+	else
+		res = 0;
+
 	if (res == 0)
 	{
 /* use this if command doesn't include cleanups i.e. rm workdir
@@ -471,6 +708,7 @@ static void _e2p_unpack_response_decode_cb (GtkDialog *dialog, gint response,
 	e2_hook_unregister (&app.pane2.hook_change_dir,
 		(HookFunc)_e2p_unpack_change_dir_hook, data, TRUE);
 
+	NEEDCLOSEBGL
 	switch (response)
 	{
 		case E2_RESPONSE_USER1:	//repack the temp dir
@@ -485,6 +723,7 @@ static void _e2p_unpack_response_decode_cb (GtkDialog *dialog, gint response,
 			_e2p_unpack_clear (data);
 			break;
 	}
+	NEEDOPENBGL
 }
 /**
 @brief hook function for cd in either pane (app.paneX.hook_change_dir)
@@ -521,25 +760,30 @@ static gboolean _e2p_unpack_change_dir_hook (gchar *path, E2P_Unpackdata *data)
 	printd (DEBUG, "ready to cleanup unpack dir");
 
 	//ask user what to do with the unpacked items
-	gdk_threads_enter ();
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
-		_("What do you want to do with the unpacked item(s) ?"), NULL,
+	CLOSEBGL
+	GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION,
+		_("What do you want to do with the unpacked items ?"), NULL,
 		(ResponseFunc)_e2p_unpack_response_decode_cb, data);
-	gdk_threads_leave ();
+	OPENBGL
 
-	e2_dialog_add_simple_button (dialog, GTK_STOCK_CLEAR,
-		_("Re_pack"), E2_RESPONSE_USER1);
-	e2_dialog_add_simple_button (dialog, GTK_STOCK_APPLY,
+	//re-pack button for types that allow it
+	//we haven't yet decoded which re-pack commands are un-available, so need
+	//to hard-code the un-recoverable ones
+	if (!(data->type == DEB || data->type == RPM))
+		e2_dialog_add_simple_button (dialog, STOCK_NAME_CLEAR,
+			_("Re_pack"), E2_RESPONSE_USER1);
+
+	e2_dialog_add_simple_button (dialog, STOCK_NAME_APPLY,
 		_("_Retain"), E2_RESPONSE_USER2);
-	GtkWidget *button = e2_dialog_add_simple_button (dialog, GTK_STOCK_DELETE,
+	GtkWidget *button = e2_dialog_add_simple_button (dialog, STOCK_NAME_DELETE,
 		_("_Delete"), E2_RESPONSE_REMOVE);
 
-	gdk_threads_enter ();
+	CLOSEBGL
 	e2_dialog_setup (dialog, app.main_window);
-	e2_dialog_run (dialog, NULL, 0);
+	gtk_widget_show_all (dialog);
 	gtk_widget_grab_focus (button);
 	gtk_window_present (GTK_WINDOW (dialog));
-	gdk_threads_leave ();
+	OPENBGL
 
 	return TRUE; 	//no hook cleanup here
 					//done in response cb, as we need to check both panes and data
@@ -556,8 +800,8 @@ static gboolean _e2p_unpack (gpointer from, E2_ActionRuntime *art)
 {
 	//these unpack-command strings are in same order as enum
 	//all are executed from the temp dir (as some can only do that)
-	//all are ascii (no conversion to utf8 before execution)
-	static gchar *cmd_str [MAXTYPES] =
+	//all are ASCII (no conversion to UTF-8 before execution)
+	static gchar *cmd_str [UPTYPESCOUNT] =
 	{
 		//each unpack goes into a separate newly-created work dir, so there's
 		//no need to worry manage over-writing when unpacking with these commands
@@ -566,31 +810,30 @@ static gboolean _e2p_unpack (gpointer from, E2_ActionRuntime *art)
 		//ditto for --overwrite, and for -C, if that's wanted
 		">gzip -cd %s | tar -xpf -",
 		">bzip2 -cd %s | tar -xpf -",
-		">lzma -cd %s | tar -xpf -",
 		">xz -cd %s | tar -xpf -",
-		"tar -xpf %s",
 #else
 		//note: an --atime-preserve in these tar commands prevents the
 		//file-monitoring process from noticing anything in the temp dir
 		//--overwrite not needed
 		"tar -xpzf %s",
 		"tar -xpjf %s",
-		"tar --lzma -xpf %s",
 		"tar --xz -xpf %s",
-		"tar -xpf %s",
 #endif
-		//.deb unpack command by Martin Zelaia (has GNU-specific tar options)
-		//">>mkdir ./DEBIAN ./CONTENTS;ar -x %s | tar -xfz control.tar.gz -C ./DEBIAN | tar -xfz data.tar.gz -C ./CONTENTS; rm control.tar.gz data.tar.gz;cp ./DEBIAN/control ./INFO;rm ./debian-binary";
-		//">rpm2cpio %s | cpio -id"  this is ok, but no spec data from a non-src file, and there's no re-pack command available
 		"unzip -o %s",  //or "unzip -o -d %s %s"
-/*
-decompress ANSI:    7za x archive.zip -odirname -aoa
-decompress UNICODE: 7za x archive.7z -odirname -aoa
-*/
-		"7za x %s -aoa", //or ??
+		">7za x -so %s | tar -xpf -",
+		"7za x %s",
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(darwin)
+		">lzma -cd %s | tar -xpf -",
+#else
+		"tar --lzma -xpf %s",
+#endif
+		"tar -xpf %s",
+		"|ar -x %s",  //start the unpack, with block until done, to allow further processing
+		">|rpm2cpio %s | cpio -id", //ditto
 		"rar x -o+ %s", //rar will only extract to current dir
 		"arj x -y %s", //or "arj x -y "UNPACKPATH" "UNPACKPATH, //NOTE swapped order of archive & path
-		"zoo xO %s"	//zoo will only extract to current dir
+		"zoo xO %s",	//zoo will only extract to current dir
+		"mount -o loop %s ." //auto-detect FS-type, option -o loop needs superuser
 	};
 
 	//CHECKME 	e2_filelist_disable_refresh ();
@@ -605,14 +848,14 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 	VPATH ddata;
 	ddata.path = e2_utils_dircat (curr_view, info->filename, TRUE);
 	ddata.spacedata = curr_view->spacedata;
-	gint index = _e2p_unpack_match_type (&ddata);
+	E2_UnPackType uptype = _e2p_unpack_match_type (&ddata);
 	g_free (ddata.path);
 #else
 	gchar *local = e2_utils_dircat (curr_view, info->filename, TRUE);
-	gint index = _e2p_unpack_match_type ((VPATH*)local);
+	E2_UnPackType uptype = _e2p_unpack_match_type ((VPATH*)local);
 	g_free (local);
 #endif
-	if (index == -1)
+	if (uptype == UPTYPENONE)
 	{
 		e2_output_print_error (_("Selected item is not a supported archive"), FALSE);
 //		e2_filelist_enable_refresh ();
@@ -664,6 +907,7 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 	data->workdir = D_FILENAME_FROM_LOCALE (workdir);
 	g_free (workdir);
 
+	data->type = uptype;
 	converted = F_FILENAME_FROM_LOCALE (info->filename); //not much race chance, don't D_...
 #ifdef E2_VFSTMP
 	FIXME dir when not mounted local
@@ -674,7 +918,7 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 	gchar *qp = e2_utils_quote_string (data->package);
 #endif
 	//need no conversion of command encoding
-	data->command = g_strdup_printf (cmd_str [index], qp); //CHECKME append 1>/dev/null
+	data->command = g_strdup_printf (cmd_str [uptype], qp); //CHECKME append 1>/dev/null
 	g_free (qp);
 	e2_window_set_cursor (GDK_WATCH);
 	//unpack the archive into the temp directory
@@ -684,7 +928,7 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 		, T/F ?
 #endif
 		);
-	e2_window_set_cursor (GDK_LEFT_PTR);
+
 	if (result != 0)
 	{
 		workdir = F_FILENAME_TO_LOCALE (data->workdir);
@@ -697,10 +941,21 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 		F_FREE (workdir, data->workdir);
 		_e2p_unpack_cleanup (data);
 //		e2_filelist_enable_refresh ();
+		e2_window_set_cursor (GDK_LEFT_PTR);
 		return FALSE;
 	}
 	//now go see it
 	e2_pane_change_dir (NULL, data->workdir);
+
+	if (uptype == DEB || uptype == RPM)
+	{
+		//wait for async cd before finishing de-compression from there
+		data->from = from;
+		g_timeout_add (50, (GSourceFunc)_e2p_unpack_expand_contents, data);
+	}
+	else
+		e2_window_set_cursor (GDK_LEFT_PTR);
+
 	//setup to clean when temp dir not open anymore
 	e2_hook_register (&app.pane1.hook_change_dir,
 		(HookFunc)_e2p_unpack_change_dir_hook, data);
@@ -711,45 +966,34 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 	return TRUE;
 }
 
-//aname must be confined to this module
-static gchar *aname;
 /**
 @brief plugin initialization function, called by main program
 
-@param p ptr to plugin data struct
+@param mode flags enumerating what sort of init to perform
 
-@return TRUE if the initialization succeeds, else FALSE
+@return Plugin*, with refcount 1 if @a mode included runtime setup and that succeeded
 */
-gboolean init_plugin (Plugin *p)
+Plugin *init_plugin (E2PInit mode)
 {
-	//once-only, setup the working dir name
+	gchar *name;
+#ifdef E2_VFS
+	name = _("unpack_with_plugin");
+#else
+	name = _A(107);
+#endif
+
+	PLUGINIT_ONE_START(_A(6),name,_e2p_unpack,
+		_("_Unpack"),
+		_("Unpack archive file into a temporary directory"),
+		"plugin_"ANAME E2ICONTB)
+
+	//setup the working dir name
 	unpack_tmp = e2_utils_get_temp_path ("-unpack");
 	//strip the trailing ".tmp~0" as we will append a suffix like that for each unpack
 	gchar *s = strrchr (unpack_tmp, '.');
 	*s = '\0';
 
-#define ANAME "unpack"
-#ifdef E2_VFS
-	aname = _("unpack_with_plugin");
-#else
-	aname = _A(104);
-#endif
-	p->signature = ANAME VERSION;
-	p->menu_name = _("_Unpack");
-	p->description = _("Unpack archive (tar, tar.gz, tar.bz2, zip, 7z, rar, arj, zoo) into a temporary directory");
-	p->icon = "plugin_"ANAME E2ICONTB;  //use icon file pathname if appropriate
-
-	if (p->action == NULL)
-	{
-		//don't free name string here
-		E2_Action plugact =
-		{g_strconcat (_A(6),".",aname,NULL),_e2p_unpack,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
-		p->action = e2_plugins_action_register (&plugact);
-		if G_LIKELY((p->action != NULL))
-			return TRUE;
-		g_free (plugact.name);
-	}
-	return FALSE;
+	PLUGINIT_ONE_END
 }
 /**
 @brief cleanup transient things for this plugin
@@ -767,9 +1011,10 @@ gboolean clean_plugin (Plugin *p)
 	while (e2_hook_unregister (&app.pane2.hook_change_dir,
 		(HookFunc)_e2p_unpack_change_dir_hook, NULL, FALSE)) {}
 
-	gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
-	gboolean ret = e2_plugins_action_unregister (action_name);
-	g_free (action_name);
+	PLUGIN_CLEAR_ACTIONS (p)
+
 	g_free (unpack_tmp);
+	unpack_tmp = NULL; //CHECKME fix crash when repeated load/unload ?
+
 	return ret;
 }
