@@ -1,6 +1,6 @@
-/* $Id: e2_action.c 2439 2012-02-21 23:23:53Z tpgww $
+/* $Id: e2_action.c 2915 2013-11-13 01:08:10Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it under the
@@ -55,7 +55,10 @@ GtkTreeStore *actions_store;	//for action-names in config dialog combo renderers
 
 /**
 @brief process a toggle-button click
+
 This expects as data the key of the relevant toggle data struct in the toggles hash
+Expects BGL closed.
+
 @param from the button, menu item etc which was activated
 @param art action runtime data
 
@@ -89,7 +92,9 @@ static gboolean _e2_action_do_toggle (gpointer from, E2_ActionRuntime *art)
 			*gap = '\0';
 			gap = e2_utils_pass_whitespace (gap+1);
 		}
+		OPENBGL
 		retval = e2_action_run_simple_from (cc, gap, from);
+		CLOSEBGL
 	}
 	else
 	{
@@ -226,11 +231,9 @@ static gboolean _e2_action_visible_cb (GtkTreeModel *model, GtkTreeIter *iter, g
 	gboolean retval;
 	E2_ACTION_EXCLUDE ex1 = GPOINTER_TO_INT (data);
 	E2_ACTION_EXCLUDE ex2;
-	gchar *name;	//, *child;
-//	gboolean notfake, col5;
-	gtk_tree_model_get (model, iter, 0, &name, //1, &child,
-		2, &ex2, // 3, &notfake, 4, &col5,
-		-1);
+	gchar *name;
+
+	gtk_tree_model_get (model, iter, 0, &name, 2, &ex2, -1);
 	retval = !(ex1 & ex2 & 0xffff);
 	//special cases
 	if ((ex1 & E2_ACTION_INCLUDE_FILES) && name != NULL)
@@ -248,13 +251,20 @@ static GtkWidget *selitem = NULL;
 
 @return	TRUE if the event was processed
 */
-static gboolean _e2_action_hover_buttonpress_cb (GtkWidget *item, GdkEventButton *event,
-	E2HoverData *data)
+static gboolean _e2_action_hover_buttonpress_cb (GtkWidget *item,
+	GdkEventButton *event, E2HoverData *data)
 {
 	if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
 	{
+		NEEDCLOSEBGL
 		gtk_menu_shell_activate_item (GTK_MENU_SHELL (data->menu), item, TRUE);
-		data->menu = NULL;	//data->menu will have been destroyed as consequence of activation
+#ifdef USE_GTK3_0
+		gtk_widget_destroy (data->menu);
+#else
+		//data->menu will have been destroyed as consequence of activation
+#endif
+		NEEDOPENBGL
+		data->menu = NULL;
 		selitem = NULL;
 		return TRUE;
 	}
@@ -281,6 +291,7 @@ static void _e2_action_hover_setup (GtkWidget *menuitem, E2HoverData *data)
 static gboolean _e2_action_hover_pointermove_cb (GtkWidget *window,
 	GdkEventMotion *event, E2HoverData *data)
 {
+	NEEDCLOSEBGL
 	GtkWidget *menuitem = gtk_get_event_widget ((GdkEvent*) event);
 	if (selitem != menuitem)
 	{
@@ -296,6 +307,7 @@ static gboolean _e2_action_hover_pointermove_cb (GtkWidget *window,
 		gtk_menu_shell_select_item (GTK_MENU_SHELL (data->menu), menuitem);
 #endif
 	}
+	NEEDOPENBGL
 	return FALSE;
 }
 /**
@@ -316,12 +328,14 @@ static gboolean _e2_action_enter_menu_cb (GtkWidget *window, GdkEventCrossing *e
 		g_source_remove (data->timer_id);
 		data->timer_id = 0;
 	}
+	NEEDCLOSEBGL
 #ifdef USE_GTK2_18
 	if (G_LIKELY(!gtk_widget_has_grab (window)))
 #else
 	if (G_LIKELY(!GTK_WIDGET_HAS_GRAB (window)))
 #endif
 		gtk_grab_add (window); //apply keyboard and mouse focus
+	NEEDOPENBGL
 	g_signal_connect (G_OBJECT (window), "motion-notify-event",
 		G_CALLBACK (_e2_action_hover_pointermove_cb), data);
 
@@ -344,6 +358,7 @@ static gboolean _e2_action_leave_menu_cb (GtkWidget *window, GdkEventCrossing *e
 		data->timer_id = 0;
 	}
 	printd (DEBUG, "In _e2_action_leave_menu_cb()");
+	NEEDCLOSEBGL
 #ifdef USE_GTK2_18
 	if (G_LIKELY (gtk_widget_has_grab (window)))
 #else
@@ -351,6 +366,7 @@ static gboolean _e2_action_leave_menu_cb (GtkWidget *window, GdkEventCrossing *e
 #endif
 		gtk_grab_remove (window);
 	gtk_widget_destroy (window);
+	NEEDOPENBGL
 	data->menu = NULL;
 	selitem = NULL;
 	return FALSE;
@@ -392,11 +408,11 @@ static gboolean _e2_action_do_hover_timeout (E2HoverData *data)
 				gtk_widget_show (data->menu); //setup size for use when positioning ?
 				gint x, y;
 				gboolean push = TRUE;
+				CLOSEBGL
 				e2_toolbar_set_menu_position (GTK_MENU (data->menu), &x, &y, &push, data->hovered);
 				gtk_window_move (GTK_WINDOW (popwin), x, y); //this might be ignored by WM ?
-				gdk_threads_enter ();
 				gtk_widget_show_all (popwin);
-				gdk_threads_leave ();
+				OPENBGL
 			}
 		}
 		else
@@ -413,10 +429,10 @@ static gboolean _e2_action_clear_hover_cb (E2HoverData *data)
 {
 	if (GTK_IS_WIDGET (data->menu))
 	{
+		CLOSEBGL
 		GtkWidget *popwin = gtk_widget_get_toplevel (data->menu);
-		gdk_threads_enter ();
 		gtk_widget_destroy (popwin);
-		gdk_threads_leave ();
+		OPENBGL
 	}
 	data->menu = NULL;
 
@@ -437,11 +453,14 @@ static void _e2_action_abort_hover_cb (GtkButton *button, E2HoverData *data)
 		g_source_remove (data->timer_id);
 		data->timer_id = 0;
 	}
+
+	NEEDCLOSEBGL
 	if (GTK_IS_WIDGET (data->menu))
 	{
 		GtkWidget *popwin = gtk_widget_get_toplevel (data->menu);
 		gtk_widget_destroy (popwin);
 	}
+	NEEDOPENBGL
 	data->menu = NULL;
 }
 
@@ -469,6 +488,7 @@ gboolean e2_action_start_hover_cb (GtkWidget *button, GdkEventCrossing *event, E
 		g_source_remove (data->timer_id);
 		data->timer_id = 0;
 	}
+//	NEEDCLOSEBGL
 	if (data->menu == NULL)
 	{
 		//no popup if button is clicked
@@ -480,6 +500,7 @@ gboolean e2_action_start_hover_cb (GtkWidget *button, GdkEventCrossing *event, E
         g_object_get (G_OBJECT (defs), "gtk-menu-popup-delay", &delay, NULL);
 		data->timer_id = g_timeout_add (delay, (GSourceFunc)_e2_action_do_hover_timeout, data);
 	}
+//	NEEDOPENBGL
 	return FALSE;
 }
 /**
@@ -501,6 +522,7 @@ gboolean e2_action_end_hover_cb (GtkWidget *widget, GdkEventCrossing *event, E2H
 		g_source_remove (data->timer_id);
 		data->timer_id = 0;
 	}
+//	NEEDCLOSEBGL
 	if (GTK_IS_WIDGET (data->menu))
 	{
 		//allow short interval to move pointer from button into the menu or vice-versa, before killing it
@@ -514,6 +536,7 @@ gboolean e2_action_end_hover_cb (GtkWidget *widget, GdkEventCrossing *event, E2H
 	else
 		data->menu = NULL;
 
+//	NEEDOPENBGL
 	g_signal_handlers_disconnect_by_func ((gpointer)widget, _e2_action_abort_hover_cb, data);
 
 	return FALSE;
@@ -638,7 +661,7 @@ This is for plugin actions, essentially
 
 @return TRUE if the action was registered
 */
-gboolean e2_action_unregister (gchar *name)
+gboolean e2_action_unregister (const gchar *name)
 {
 	E2_Action *action = g_hash_table_lookup (actions_hash, name);
 	if (action == NULL)
@@ -667,7 +690,7 @@ cannot be found, NULL is returned.
 
 @return pointer to an action object or NULL
 */
-E2_Action *e2_action_get (gchar *name)
+E2_Action *e2_action_get (const gchar *name)
 {
 	return (g_hash_table_lookup (actions_hash, name));
 }
@@ -679,7 +702,7 @@ E2_Action *e2_action_get (gchar *name)
 @param use_arg pointer to store newly-allocated "real" argument string
 @return pointer to an action object, or NULL if @a taskname is NULL
 */
-E2_Action *e2_action_get_with_custom (gchar *taskname, gchar *arg, gchar **use_arg)
+E2_Action *e2_action_get_with_custom (const gchar *taskname, const gchar *arg, gchar **use_arg)
 {
 	if (taskname == NULL)
 		return NULL;
@@ -733,9 +756,10 @@ E2_Action* e2_action_check (gchar *command)
 @brief run a named 'task' (may be a command or an action)
 
 This function is to run a certain action. First, the action specified by
-@a name is looked up. Then a runtime object is created and the action is
-run. Afterwards the runtime object is freed again. If the action could not
-be found, an error message is printed to the output pane.
+@a name is looked up. Then a runtime object is created and the action is run.
+Afterwards the runtime object is freed again. If the action could not be found,
+an error message is printed to the output pane.
+Downstream expects BGL open/off.
 
 @param name action name or command, string
 @param arg pointer to data for the command, or possibly arguments or more of the command, or NULL
@@ -743,7 +767,7 @@ be found, an error message is printed to the output pane.
 
 @return  TRUE on success, FALSE on failure
 */
-gboolean e2_action_run_simple_from (gchar *name, gpointer arg, gpointer from)
+gboolean e2_action_run_simple_from (const gchar *name, gpointer arg, gpointer from)
 {
 	//CHECKME should these variables be volatilized ?
 	if ((name == NULL || *name == '\0')	//chained keybindings may have no command
@@ -769,9 +793,10 @@ so that any such locking can be managed locally, without mutux reentrance.
 */
 /*void e2_action_run_cb (gpointer from, E2_ActionRuntime *rt)
 {
-	gdk_threads_leave ();
+//	NEEDCLOSEBGLX
+	OPENBGL
 	e2_action_run (from, rt);
-	gdk_threads_enter ();
+	CLOSEBGL
 }*/
 /**
 @brief run a certain action
@@ -782,6 +807,7 @@ are managed.
 There are several wrappers to simplify access.
 Runtime data @a rt may contain additional user data.
 @a from may be used by the action handler to determine the context.
+Expects BGL open/off.
 
 @param from the item activated to trigger the action (eg toolbar GtkButton, maybe NULL)
 @param art runtime data for the action
@@ -799,15 +825,13 @@ gboolean e2_action_run (gpointer from, E2_ActionRuntime *art)
 	printd (DEBUG, "e2_action_run (from:, art:%s)", art->action->name);
 	if (from == NULL)
 		from = app.main_window;
+	CLOSEBGL
 	art->state = e2_action_get_current_state (from);
 	gboolean (*func) (gpointer, E2_ActionRuntime *) = art->action->func;
-#ifndef DEBUG_MESSAGES
-	return func (from, art);
-#else
 	gboolean res = func (from, art);
+	OPENBGL
 	printd (DEBUG,"e2_action_run () ends");
 	return res;
-#endif
 }
 /**
 @brief create an action runtime object
@@ -923,7 +947,9 @@ E2_ActionState e2_action_get_current_state (gpointer from)
 */
 static void _e2_action_free_toggle (E2_ToggleData *ex)
 {
-	e2_list_free_with_data (&ex->boxes);
+	GList *member;
+	for (member = ex->boxes; member != NULL; member = member->next)
+		DEALLOCATE (E2_ToggleBox, member->data);
 	if (ex->true_action != NULL)
 		g_free (ex->true_action);
 	if (ex->false_action != NULL)
@@ -953,32 +979,6 @@ static void _e2_action_clean1 (E2_Action *action)
 	DEALLOCATE (E2_Action, action);
 #endif
 }
-/* presently unused
-static void free_name (gchar *key, E2_Action *value, gpointer data)
-{
-	g_free (key); //=value->name
-	return;
-} */
-
-/**
-@brief internal action system cleanup
-
-After calling the function, the action system may not be used anymore.
-It is only called internally just before exit.
-
-@return
-*/
-/* this done only at session end, don't bother
-void e2_actions_clean ()
-{
-//CHECKME = should this be able to handle ->data &| ->data2 ?
-	g_object_unref (actions_store);
-	//clean up non-constant action name strings in the hash
-	g_hash_table_foreach (actions_hash, (GFunc) free_name, NULL);
-	g_hash_table_destroy (actions_hash);
-	//CHECKME = array cleanup ok for non-constant strings? could this replace the hash foreach?
-//	g_ptr_array_free (actions_array, TRUE);
-} */
 
 /**
 @brief internal action system initialization
@@ -1008,17 +1008,17 @@ void e2_actions_init (void)
 
 	toggles_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
 		(GDestroyNotify) _e2_action_free_toggle);
-	toggles_array [E2_TOGGLE_PANE1FULL] = g_strconcat (_A(11),".",_A(113),NULL);
-	toggles_array [E2_TOGGLE_PANE2FULL] = g_strconcat (_A(12),".",_A(113),NULL);
-	toggles_array [E2_TOGGLE_PANE1HIDDEN] = g_strconcat (_A(11),".",_A(84),NULL);
-	toggles_array [E2_TOGGLE_PANE2HIDDEN] = g_strconcat (_A(12),".",_A(84),NULL);
+	toggles_array [E2_TOGGLE_PANE1FULL] = g_strconcat (_A(11),".",_A(116),NULL);
+	toggles_array [E2_TOGGLE_PANE2FULL] = g_strconcat (_A(12),".",_A(116),NULL);
+	toggles_array [E2_TOGGLE_PANE1HIDDEN] = g_strconcat (_A(11),".",_A(87),NULL);
+	toggles_array [E2_TOGGLE_PANE2HIDDEN] = g_strconcat (_A(12),".",_A(87),NULL);
 	toggles_array [E2_TOGGLE_PANE1FILTERS] = g_strconcat (_A(11),".",_A(25),NULL);
 	toggles_array [E2_TOGGLE_PANE2FILTERS] = g_strconcat (_A(12),".",_A(25),NULL);
-	toggles_array [E2_TOGGLE_OUTPUTFULL] = g_strconcat (_A(10),".",_A(113),NULL);
-	toggles_array [E2_TOGGLE_OUTPUTSHADE] = g_strconcat (_A(10),".",_A(83),NULL);
+	toggles_array [E2_TOGGLE_OUTPUTFULL] = g_strconcat (_A(10),".",_A(116),NULL);
+	toggles_array [E2_TOGGLE_OUTPUTSHADE] = g_strconcat (_A(10),".",_A(86),NULL);
 #ifdef E2_VFS
-	toggles_array [E2_TOGGLE_PANE1SPACE] = g_strconcat (_A(11),".",_A(120),NULL);
-	toggles_array [E2_TOGGLE_PANE2SPACE] = g_strconcat (_A(12),".",_A(120),NULL);
+	toggles_array [E2_TOGGLE_PANE1SPACE] = g_strconcat (_A(11),".",_A(123),NULL);
+	toggles_array [E2_TOGGLE_PANE2SPACE] = g_strconcat (_A(12),".",_A(123),NULL);
 #endif
 	//register most actions
 	//note translated action labels needed here, i.e. before config loaded
@@ -1027,24 +1027,26 @@ void e2_actions_init (void)
 	{	//here we don't bother explicitly setting 3 trailing NULL parameters
 		{ g_strconcat(_A(6),".",_A(23),NULL), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_FILE_ACTIONS,
 			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOOLBAR | E2_ACTION_EXCLUDE_LAYOUT, },
-		{ g_strconcat(_A(16),".",_A(27),NULL), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_PLUGINS,
+		{ g_strconcat(_A(6),".",_A(26),NULL), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_FILE_HANDLERS,
+			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOOLBAR | E2_ACTION_EXCLUDE_LAYOUT, },
+		{ g_strconcat(_A(16),".",_A(28),NULL), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_PLUGINS,
 			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOOLBAR, },
 		{ g_strdup(_A(21)), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_SEPARATOR,
 			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_LAYOUT, },
 		{ g_strdup(_A(22)), (gboolean (*)(gpointer,E2_ActionRuntime*))NULL, FALSE, E2_ACTION_TYPE_SUBMENU,
 			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_LAYOUT, },
-		{ g_strconcat(_A(1),".",_A(72),NULL), e2_main_user_shutdown, E2_ACTION_TYPE_ITEM, FALSE, 0, },
+		{ g_strconcat(_A(1),".",_A(75),NULL), e2_main_user_shutdown, E2_ACTION_TYPE_ITEM, FALSE, 0, },
 
 		{ g_strconcat(_A(3),".",_C(34),NULL), e2_plugins_configure, FALSE, E2_ACTION_TYPE_ITEM, 0, },
 		{ g_strdup(_A(20)), _e2_action_custom_command, TRUE, E2_ACTION_TYPE_ITEM,
 			E2_ACTION_EXCLUDE_GENERAL, },
-		{ g_strconcat(_A(17),".",_A(115),NULL), _e2_action_do_toggle, TRUE, E2_ACTION_TYPE_TOGGLE,
+		{ g_strconcat(_A(17),".",_A(118),NULL), _e2_action_do_toggle, TRUE, E2_ACTION_TYPE_TOGGLE,
 			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOGGLE, },
-		{ g_strconcat(_A(17),".",_A(116),NULL), _e2_action_do_toggle, TRUE, E2_ACTION_TYPE_TOGGLE,
+		{ g_strconcat(_A(17),".",_A(119),NULL), _e2_action_do_toggle, TRUE, E2_ACTION_TYPE_TOGGLE,
 			E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_TOGGLE, },
 
-		{ g_strconcat(_A(3),".",_A(32),NULL), _e2_action_configure, FALSE, E2_ACTION_TYPE_ITEM, 0, },
-		{ g_strconcat(_A(3),".",_A(40),NULL), _e2_action_configure_default, FALSE, E2_ACTION_TYPE_ITEM, 0, },
+		{ g_strconcat(_A(3),".",_A(34),NULL), _e2_action_configure, FALSE, E2_ACTION_TYPE_ITEM, 0, },
+		{ g_strconcat(_A(3),".",_A(43),NULL), _e2_action_configure_default, FALSE, E2_ACTION_TYPE_ITEM, 0, },
 	};
 
 	guint i, count = sizeof (homeless_actions) / sizeof (E2_Action);
@@ -1122,125 +1124,127 @@ void e2_action_setup_labels (void)
 	action_labels[23] = _("<actions>");
 	action_labels[24] = _("<bookmarks>");
 	action_labels[25] = _("<filters>");
-	action_labels[26] = _("<line>");
-	action_labels[27] = _("<menu>");
-	action_labels[28] = _("about");
-	action_labels[29] = _("activate");
-	action_labels[30] = _("add");
-	action_labels[31] = _("adjust_ratio");
-	action_labels[32] = _("application");
-	//see 127 _("change");
-	action_labels[33] = _("clear");
-	action_labels[34] = _("clear_history");
-	action_labels[35] = _("complete");
-	action_labels[36] = _("copy");
-	action_labels[37] = _("copy_as");
-	action_labels[38] = _("copy_merge");
-	action_labels[39] = _("copy_with_time");
-	action_labels[40] = _("default");
-	action_labels[42] = _("delete");
-	action_labels[43] = _("edit");
-	action_labels[44] = _("edit_again");
-	action_labels[45] = _("filetype");
-	action_labels[46] = _("find");
-	action_labels[47] = _("focus");
-	action_labels[48] = _("fullscreen");
-	action_labels[49] = _("go_back");
-	action_labels[50] = _("go_forward");
-	action_labels[51] = _("go_up");
-	action_labels[52] = _("goto_bottom");
-	action_labels[53] = _("goto_top");
-	action_labels[54] = _("help");
-	action_labels[55] = _("info");
-	action_labels[56] = _("insert_selection");
-	action_labels[57] = _("invert_selection");
-	action_labels[58] = _("list");
-	action_labels[59] = _("mirror");
-	action_labels[60] = _("mkdir");
-	action_labels[61] = _("mountpoints");
-	action_labels[62] = _("move");
-	action_labels[63] = _("move_as");
-	action_labels[64] = _("open");
-	action_labels[65] = _("open_in_other");
-	action_labels[66] = _("open_with");
-	action_labels[67] = _("owners");
-	action_labels[68] = _("page_down");
-	action_labels[69] = _("page_up");
-	action_labels[70] = _("permissions");
-	action_labels[71] = _("print");
-	action_labels[72] = _("quit");
-	action_labels[73] = _("refresh");
-	action_labels[74] = _("refreshresume");
-	action_labels[75] = _("refreshsuspend");
-	action_labels[76] = _("rename");
-	action_labels[77] = _("scroll_down");
-	action_labels[78] = _("scroll_up");
-	action_labels[79] = _("search");  //= _C(37)
-	action_labels[80] = _("select_type");
-	action_labels[81] = _("send");
-	action_labels[82] = _("set");
-	action_labels[83] = _("show");
-	action_labels[84] = _("show_hidden");
-	action_labels[85] = _("show_menu");
-	action_labels[86] = _("sortaccesssed");
-	action_labels[87] = _("sortchanged");
-	action_labels[88] = _("sortext");
-	action_labels[89] = _("sortgroup");
-	action_labels[90] = _("sortmodified");
-	action_labels[91] = _("sortname");
-	action_labels[92] = _("sortpermission");
-	action_labels[93] = _("sortsize");
-	action_labels[94] = _("sortuser");
-	action_labels[95] = _("switch");
-	action_labels[96] = _("symlink");
-	action_labels[97] = _("symlink_as");
-	action_labels[98] = _("sync");
-	action_labels[99] = _("toggle_direction");
-	action_labels[100] = _("toggle_focus");
-	action_labels[101] = _("toggle_select_all");
-	action_labels[102] = _("toggle_selected");
+	action_labels[26] = _("<handlers>");
+	action_labels[27] = _("<line>");
+	action_labels[28] = _("<menu>");
+	action_labels[29] = _("about");
+	action_labels[30] = _("activate");
+	action_labels[31] = _("add");
+	action_labels[32] = _("add_history");
+	action_labels[33] = _("adjust_ratio");
+	action_labels[34] = _("application");
+	action_labels[35] = _("change");
+	action_labels[36] = _("clear");
+	action_labels[37] = _("clear_history");
+	action_labels[38] = _("complete");
+	action_labels[39] = _("copy");
+	action_labels[40] = _("copy_as");
+	action_labels[41] = _("copy_merge");
+	action_labels[42] = _("copy_with_time");
+	action_labels[43] = _("default");
+	action_labels[44] = _("del_history");
+	action_labels[45] = _("delete");
+	action_labels[46] = _("edit");
+	action_labels[47] = _("edit_again");
+	action_labels[48] = _("filetype");
+	action_labels[49] = _("find");
+	action_labels[50] = _("focus");
+	action_labels[51] = _("fullscreen");
+	action_labels[52] = _("go_back");
+	action_labels[53] = _("go_forward");
+	action_labels[54] = _("go_up");
+	action_labels[55] = _("goto_bottom");
+	action_labels[56] = _("goto_top");
+	action_labels[57] = _("help");
+	action_labels[58] = _("info");
+	action_labels[59] = _("insert_selection");
+	action_labels[60] = _("invert_selection");
+	action_labels[61] = _("list");
+	action_labels[62] = _("mirror");
+	action_labels[63] = _("mkdir");
+	action_labels[64] = _("mountpoints");
+	action_labels[65] = _("move");
+	action_labels[66] = _("move_as");
+	action_labels[67] = _("open");
+	action_labels[68] = _("open_in_other");
+	action_labels[69] = _("open_with");
+	action_labels[70] = _("owners");
+	action_labels[71] = _("page_down");
+	action_labels[72] = _("page_up");
+	action_labels[73] = _("permissions");
+	action_labels[74] = _("print");
+	action_labels[75] = _("quit");
+	action_labels[76] = _("refresh");
+	action_labels[77] = _("refreshresume");
+	action_labels[78] = _("refreshsuspend");
+	action_labels[79] = _("rename");
+	action_labels[80] = _("scroll_down");
+	action_labels[81] = _("scroll_up");
+	action_labels[82] = _("search");  //= _C(37)
+	action_labels[83] = _("select_type");
+	action_labels[84] = _("send");
+	action_labels[85] = _("set");
+	action_labels[86] = _("show");
+	action_labels[87] = _("show_hidden");
+	action_labels[88] = _("show_menu");
+	action_labels[89] = _("sortaccesssed");
+	action_labels[90] = _("sortchanged");
+	action_labels[91] = _("sortext");
+	action_labels[92] = _("sortgroup");
+	action_labels[93] = _("sortmodified");
+	action_labels[94] = _("sortname");
+	action_labels[95] = _("sortpermission");
+	action_labels[96] = _("sortsize");
+	action_labels[97] = _("sortuser");
+	action_labels[98] = _("switch");
+	action_labels[99] = _("symlink");
+	action_labels[100] = _("symlink_as");
+	action_labels[101] = _("sync");
+	action_labels[102] = _("toggle_direction");
+	action_labels[103] = _("toggle_focus");
+	action_labels[104] = _("toggle_select_all");
+	action_labels[105] = _("toggle_selected");
 #ifdef E2_TREEDIALOG
-	action_labels[103] = _("tree");
+	action_labels[106] = _("tree");
 #endif
-	action_labels[104] = _("unpack");
+	action_labels[107] = _("unpack");
 #ifdef WITH_UNTRASH
-	action_labels[105] = _("untrash");
+	action_labels[108] = _("untrash");
 #endif
-	action_labels[106] = _("view");  //= _C(42)
-	action_labels[107] = _("view_again");
-	action_labels[108] = _("view_at");
+	action_labels[109] = _("view");  //= _C(42)
+	action_labels[110] = _("view_again");
+	action_labels[111] = _("view_at");
 	//these are action _parameter_ strings
-	action_labels[109] = _("child");
-	action_labels[110] = _("ctrl");
-	action_labels[111] = _("dirs");
-	action_labels[112] = _("escape");
-	action_labels[113] = _("expand");
-	action_labels[114] = _("files");
-	action_labels[115] = _("off");
-	action_labels[116] = _("on");
-	action_labels[117] = _("quote");
-	action_labels[118] = _("shift");
-	action_labels[119] = _("top");
+	action_labels[112] = _("child");
+	action_labels[113] = _("ctrl");
+	action_labels[114] = _("dirs");
+	action_labels[115] = _("escape");
+	action_labels[116] = _("expand");
+	action_labels[117] = _("files");
+	action_labels[118] = _("off");
+	action_labels[119] = _("on");
+	action_labels[120] = _("quote");
+	action_labels[121] = _("shift");
+	action_labels[122] = _("top");
 	//late entries, out of order
 	//maybe more space here, see array definition in header file
 #ifdef E2_VFS
-	action_labels[120] = _("<spacemenu>");
-	action_labels[121] = _("dummy");
-	action_labels[122] = _("namespace");
-	action_labels[123] = _("unpack_in_other");
+	action_labels[123] = _("<spacemenu>");
+	action_labels[124] = _("dummy");
+	action_labels[125] = _("namespace");
+	action_labels[126] = _("unpack_in_other");
 #endif
 #ifdef WITH_KEYFAKE
-	action_labels[124] = _("key");
+	action_labels[127] = _("key");
 #endif
 #if defined(WITH_KEYFAKE) || defined(WITH_BUTTONFAKE)
-	action_labels[125] = _("fake");
+	action_labels[128] = _("fake");
 #endif
 #ifdef WITH_BUTTONFAKE
-	action_labels[126] = _("button");
+	action_labels[129] = _("button");
 #endif
-	action_labels[127] = _("change");
 }
-#define LAST_NAME_INDEX 127
+#define LAST_NAME_INDEX 129
 #if LAST_NAME_INDEX > ALLOCATED_NAMES - 1
 # warning must increase size of action names array
 #endif
