@@ -1,6 +1,6 @@
-/* $Id: e2_action.h 2377 2011-06-11 00:16:34Z tpgww $
+/* $Id: e2_action.h 2914 2013-11-13 01:06:23Z tpgww $
 
-Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -32,7 +32,8 @@ typedef enum
 {
 	E2_ACTION_TYPE_ITEM,	//all "non-special" (pseudo)actions default to this
 	E2_ACTION_TYPE_SUBMENU,
-	E2_ACTION_TYPE_FILE_ACTIONS,	//this is the context-menu widget
+	E2_ACTION_TYPE_FILE_ACTIONS,	//this is a context-menu widget
+	E2_ACTION_TYPE_FILE_HANDLERS,	//this is another context-menu widget
 	E2_ACTION_TYPE_CUSTOM_COMMAND,	//UNUSED
 	E2_ACTION_TYPE_PLUGINS,
 	E2_ACTION_TYPE_SEPARATOR,
@@ -145,44 +146,43 @@ GHashTable *toggles_hash;
 #endif
 #ifdef WITH_BUTTONFAKE
 //KEYFAKE number + 1
-# define ACTION_LABEL_COUNT 128
+# define ACTION_LABEL_COUNT 130
 #elif defined(WITH_KEYFAKE)
 //make sure this is big enough for all labels, 2 extra for this option
-# define ACTION_LABEL_COUNT 128
+# define ACTION_LABEL_COUNT 130
 //124
 #else
 //make sure this is big enough for all labels
-# define ACTION_LABEL_COUNT 128
+# define ACTION_LABEL_COUNT 130
 //122
 #endif
-#define ALLOCATED_NAMES 128
+#define ALLOCATED_NAMES 130
 typedef gchar *_action_labels[ACTION_LABEL_COUNT];
 _action_labels action_labels;
 #define _A(d) action_labels[d]
 
 GtkTreeModel *e2_action_filter_store (gpointer data);
-E2_Action *e2_action_register (const E2_Action *newaction);
-gboolean e2_action_unregister (gchar *name);
-E2_Action *e2_action_get (gchar *name);
-E2_Action *e2_action_get_with_custom (gchar *name, gchar *arg, gchar **use_arg);
+E2_Action *e2_action_register (const E2_Action *newaction) G_GNUC_MALLOC;
+gboolean e2_action_unregister (const gchar *name);
+E2_Action *e2_action_get (const gchar *name);
+E2_Action *e2_action_get_with_custom (const gchar *name, const gchar *arg, gchar **use_arg);
 E2_Action* e2_action_check (gchar *command);
-gboolean e2_action_run_simple_from (gchar *name, gpointer arg, gpointer from);
 #ifdef E2_VFS
 gboolean e2_action_inaction (gpointer from, E2_ActionRuntime *art);
 #endif
 E2_ActionState e2_action_get_current_state (gpointer from);
 //void e2_action_run_cb (GtkWidget *from, E2_ActionRuntime *rt);
 //#define e2_action_run_cb e2_action_run
+gboolean e2_action_run_simple_from (const gchar *name, gpointer arg, gpointer from);
 gboolean e2_action_run (gpointer from, E2_ActionRuntime *rt);
 E2_ActionRuntime *e2_action_pack_runtime (E2_Action *action, gpointer data,
-	void(*data_free)(gpointer));
+	void(*data_free)(gpointer)) G_GNUC_MALLOC;
 void e2_action_free_runtime (E2_ActionRuntime *rt);
 gboolean e2_action_start_hover_cb (GtkWidget *widget, GdkEventCrossing *event,
 		E2HoverData *data);
 gboolean e2_action_end_hover_cb (GtkWidget *widget, GdkEventCrossing *event,
 		E2HoverData *data);
 
-//void e2_actions_clean ();
 //initialization things
 void e2_action_setup_labels(void);
 void e2_actions_init (void);
