@@ -1,6 +1,6 @@
-/* $Id: e2_cache.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_cache.c 2828 2013-10-23 07:54:48Z tpgww $
 
-Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 Copyright (C) 2004 Florian Zaehringer (flo.zaehringer@web.de)
 
 This file is part of emelFM2.
@@ -58,129 +58,10 @@ static gint _e2_cache_str_compare (E2_Cache *cache_element, const gchar *name)
 	return strcmp (name, cache_element->name);
 }
 
-#ifdef E2_IMAGECACHE
-/**
-@brief cleanup helper for icon hash
-
-@param data pointer to E2_Image struct for an icon
-*/
-static void _e2_cache_iconhash_remove (E2_Image *data)
-{
-	g_object_unref (G_OBJECT (data->pixbuf));
-	DEMALLOCATE (E2_Image, data);
-}
-
   /******************/
  /***** public *****/
 /******************/
 
-/**
-@brief setup for icon caching
-The destroy func assumes that pixbufs have refcount 1 then
-when this is called
-*/
-void e2_cache_icons_init (void)
-{
-	app.icons = g_hash_table_new_full (g_str_hash, g_str_equal,
-		g_free, (GDestroyNotify) _e2_cache_iconhash_remove);
-	e2_utils_init_icon_sizes ();
-}
-/**
-@brief helper func to clear cached icon pixbuf
-@param key UNUSED key of hash table item being processed
-@param value value of hash table item being processed
-@param user_data UNUSED data specified when foreach was initiated
-@return
-*/
-/*static void _e2_cache_image_clear (gpointer key,
-	E2_Image *value, gpointer user_data)
-{
-	value->refcount--;
-//	g_object_unref (G_OBJECT (value->pixbuf));
-} */
-/**
-@brief clear all cached icon pixbufs from X memory
-@return
-*/
-void e2_cache_image_clearall (void)
-{
-//	g_hash_table_foreach (app.icons, (GHFunc) _e2_cache_image_clear, NULL);
-	g_hash_table_destroy (app.icons);
-}
-/**
-@brief get cached image for icon named @a name with size @a size
-If not already cached, the relevant image will be created and added to
-the cache
-@param name gtk-stock-item name, or NULL for missing image icon, or localised custom-icon filename with or without path
-@param size icon size enumerator
-@return pointer to E2_Image struct for the image, or NULL if problem occurred
-*/
-E2_Image *e2_cache_image_get (const gchar *name, GtkIconSize size)
-{
-	if (name == NULL)
-		name = GTK_STOCK_MISSING_IMAGE;	//revert to default icon image
-	gint psize = e2_utils_get_icon_size (size);
-	//make a unique cache key
-	gchar *cachename = g_strdup_printf ("%s%d", name, psize);
-	E2_Image *cached = g_hash_table_lookup (app.icons, cachename);
-	if (cached == NULL)
-	{	//need to cache this icon
-		cached = MALLOCATE (E2_Image);	//too small for slice
-		CHECKALLOCATEDWARN (cached, return NULL;)
-		cached->size = psize;
-//		cached->refcount = 1;
-		if (e2_utils_check_stock_icon (name))
-		{	//stock image
-	//		cached->type = E2_IMAGE_BMP;
-#ifdef USE_GTK3_0
-			cached->pixbuf = gtk_icon_set_render_icon_pixbuf (
-				gtk_icon_factory_lookup_default (name),
-				gtk_widget_get_style_context (app.main_window),
-				size);
-#else
-			cached->pixbuf = gtk_icon_set_render_icon (
-				gtk_icon_factory_lookup_default (name),
-				gtk_rc_get_style (app.main_window),
-				gtk_widget_get_default_direction (),
-				GTK_STATE_NORMAL, size, NULL, NULL);
-#endif
-		}
-		else
-		{	//custom icon file
-			gchar *fullname;
-			if (g_path_is_absolute (name))
-				fullname = (gchar *)name;
-			else
-			{
-				gchar *freeme = e2_utils_get_icons_path (TRUE);
-				fullname = e2_utils_strcat (freeme, name);
-				g_free (freeme);
-			}
-
-			cached->pixbuf = gdk_pixbuf_new_from_file_at_scale
-					(fullname, psize, psize, FALSE, NULL);
-
-			if (fullname != name)
-				g_free (fullname);
-
-			if (cached->pixbuf == NULL)
-			{	//can't do the custom image, revert to 'missing'
-				DEMALLOCATE (E2_Image, cached);
-				g_free (cachename);
-				return e2_cache_image_get (NULL, size);	//recurse
-			}
-		}
-		g_hash_table_insert (app.icons, cachename, cached);
-	}
-	else
-	{
-		g_free (cachename);
-//		cached->refcount++;
-//		g_object_ref (G_OBJECT (cached->pixbuf));
-	}
-	return cached;
-}
-#endif
 /**
 @brief check whether cache entry @a name exists
 @param name cache identifier string
@@ -301,7 +182,7 @@ void e2_cache_time_register (gchar *name, time_t *value, time_t def)
 			g_free (str);
 		unknown_list = g_list_remove_link (unknown_list, found);
 		g_list_free (found);
-		g_free (unknown);
+		DEALLOCATE (E2_Cache, unknown);
 	}
 	cache->data = (gpointer) value;
 	cache_list = g_list_append (cache_list, cache);
@@ -351,7 +232,7 @@ void e2_cache_double_register (gchar *name, gdouble *value, gdouble def)
 			g_free (str);
 		unknown_list = g_list_remove_link (unknown_list, found);
 		g_list_free (found);
-		g_free (unknown);
+		DEALLOCATE (E2_Cache, unknown);
 	}
 	cache->data = (gpointer *) value;
 	cache_list = g_list_append (cache_list, cache);
@@ -393,7 +274,7 @@ void e2_cache_str_register (gchar *name, gchar **str, gchar *def)
 		*str = (gchar *) unknown->data;
 		unknown_list = g_list_remove_link (unknown_list, found);
 		g_list_free (found);
-		g_free (unknown);
+		DEALLOCATE (E2_Cache, unknown);
 	}
 	cache->data = (gpointer *) str;
 	cache_list = g_list_append (cache_list, cache);
@@ -486,7 +367,7 @@ E2_Cache *e2_cache_list_register (gchar *name, GList **list)
 		*list = (GList *) unknown->data; //the backup records list start, not its store
 		unknown_list = g_list_remove_link (unknown_list, found);
 		g_list_free (found);
-		g_free (unknown);
+		DEALLOCATE (E2_Cache, unknown);
 	}
 	cache->data = (gpointer *) list;
 	cache_list = g_list_append (cache_list, cache);
@@ -569,7 +450,7 @@ void e2_cache_array_register (gchar *name, guint size, gint *values, gint *defs)
 		g_free (str);
 		unknown_list = g_list_remove_link (unknown_list, found);
 		g_list_free (found);
-		g_free (unknown);
+		DEALLOCATE (E2_Cache, unknown);
 	}
 	cache->data = (gpointer) values;
 	cache->sync_data = GINT_TO_POINTER (size);
@@ -710,6 +591,8 @@ Expects BGL to be on/closed
 */
 void e2_cache_file_write (void)
 {
+	if (e2_cl_options.original)
+		return;
 #ifdef E2_VFS
 	VPATH ddata;
 	VPATH tdata;
@@ -848,7 +731,7 @@ extern gint real_height;
 							for (node = (GList *) *cache->data; node != NULL; node = node->next)
 							{
 								//escape anything that would seem like a tree-option start or end
-								if (//*((gchar *)tmp->data) == '<'//ascii check is ok here
+								if (node->data && //*((gchar *)tmp->data) == '<'//ascii check is ok here
 									//||
 									*((gchar *)node->data) == '>')
 										e2_fs_file_write (f, "\\");
@@ -947,7 +830,7 @@ extern gint real_height;
 		ddata.path = local;
 		ddata.spacedata = NULL;
 #endif
-		gdk_threads_leave ();	//downstream error messages invoke local mutex management
+		OPENBGL	//downstream error messages invoke local mutex management
 #ifdef E2_VFS
 		e2_task_backend_rename (&tdata, &ddata);
 		e2_fs_chmod (&ddata, 0600 E2_ERR_NONE());
@@ -955,20 +838,20 @@ extern gint real_height;
 		e2_task_backend_rename (tempname, local);
 		e2_fs_chmod (local, 0600 E2_ERR_NONE());
 #endif
-		gdk_threads_enter ();
+		CLOSEBGL
 		goto cleanup;
 	}
 error_handler:
 	if (f != NULL)
 	{
 		e2_fs_close_stream (f);
-		gdk_threads_leave ();
+		OPENBGL
 #ifdef E2_VFS
 		e2_task_backend_delete (&tdata);
 #else
 		e2_task_backend_delete (tempname);
 #endif
-		gdk_threads_enter ();
+		CLOSEBGL
 	}
 	gchar *msg = g_strdup_printf (_("Cannot write cache file %s - %s"),
 		utfpath, g_strerror (errno));	//ok for native-only config file
