@@ -1,6 +1,6 @@
-/* $Id: e2_option.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_option.c 3051 2014-02-08 21:59:43Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2014 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -180,9 +180,9 @@ void e2_option_set_trash_dir (void)
 		{
 			gchar *message = g_strdup_printf (_("Cannot create trash directory %s"),
 				e2_cl_options.trash_dir);
-			gdk_threads_enter ();
+			CLOSEBGL
 			e2_output_print_error (message, TRUE);
-			gdk_threads_leave ();
+			OPENBGL
 		}
 	}
 	//it's there, but can we use it ?
@@ -281,12 +281,11 @@ void e2_option_refresh (gboolean reload, gboolean recreate)
 	if (reload)
 	{
 		printd (INFO, "reloading config due to external change");
-		e2_option_file_read ();
+		e2_option_file_read (NULL);
 	}
 
 	e2_option_tree_install_defaults ();
-
-	e2_plugins_load_all (); // load plugins (if any)
+	e2_plugins_load_configured (); // load plugins (if any)
 
 	//re-initialise things that are not done in normal 'recreate' functions
 	e2_pane_create_option_data (&app.pane1);
@@ -471,10 +470,10 @@ gboolean e2_option_check_config_files (gpointer user_data)
 	{
 		//CHECKME e2_option_refresh() probably needs BGL closed
 //		printd (DEBUG, "At e2_option_check_config_files 1, goto e2_option_refresh");
-		gdk_threads_enter ();
+		CLOSEBGL
 		e2_option_refresh (TRUE, TRUE);
 		e2_output_print (&app.tab, _("Configuration data re-loaded"), NULL, TRUE, NULL);
-		gdk_threads_leave ();
+		OPENBGL
 		g_atomic_int_set (&cfgdirty, 0);
 		last_work_time = time (NULL);
 	}
@@ -694,7 +693,7 @@ void e2_option_attach_value_changed_simple (E2_OptionSet *set, GtkWidget *widget
 	duo->a = hook;
 	duo->b = &set->hook_value_changed;
 	g_object_set_data_full (G_OBJECT (widget), "e2-option-attach-value-changed",
-		duo, (GDestroyNotify) e2_hook_unattach_cb);
+		duo, (GDestroyNotify) e2_hook_unattach);
 }
 
 void e2_option_connect (GtkWidget *controller, gboolean active)
@@ -780,6 +779,8 @@ Expects BGL to be on/closed
 */
 void e2_option_file_write (const gchar *utfpath)
 {
+	if (e2_cl_options.original)
+		return;
 #ifdef E2_VFS
 	VPATH ddata;
 	VPATH tdata;
@@ -873,7 +874,7 @@ void e2_option_file_write (const gchar *utfpath)
 		ddata.path = local;
 		ddata.spacedata = NULL;
 #endif
-		gdk_threads_leave ();	//downstream errors invoke local mutex locking
+		OPENBGL	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
 		e2_task_backend_rename (&tdata, &ddata);
 		e2_fs_chmod (&ddata, 0600 E2_ERR_NONE());
@@ -881,7 +882,7 @@ void e2_option_file_write (const gchar *utfpath)
 		e2_task_backend_rename (tempname, local);
 		e2_fs_chmod (local, 0600 E2_ERR_NONE());
 #endif
-		gdk_threads_enter ();
+		CLOSEBGL
 		printd (DEBUG, "after renaming replacement config file, resume monitoring");
 		e2_option_enable_config_checks ();	//resume monitoring
 
@@ -891,13 +892,13 @@ error_handler:
 	if (f != NULL)
 	{
 		e2_fs_close_stream (f);
-		gdk_threads_leave ();	//downstream errors invoke local mutex locking
+		OPENBGL	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
 		e2_task_backend_delete (&tdata);
 #else
 		e2_task_backend_delete (tempname);
 #endif
-		gdk_threads_enter ();
+		CLOSEBGL
 	}
 	gchar *msg = g_strdup_printf (_("Cannot write config file %s - %s"),
 		cfg_file, g_strerror (errno));	//ok for native-only config file
@@ -1044,14 +1045,15 @@ void e2_option_read_array (gchar *f[])
 @brief read config file into memory at @a contents
 It's intended that a config file be native, not virtual.
 If the file is read successfully, log the post-read config dir timestamp
+@param config_dir absolute path of directory containing config files, UTF-8 string
 @param contents pointer to place to store the loaded file content
 
 @return TRUE if the file was read
 */
-static gboolean _e2_option_config_file_read (gpointer *contents)
+static gboolean _e2_option_config_file_read (const gchar *config_dir, gpointer *contents)
 {
 	//find absolute path to config file
-	gchar *cfg_file = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
+	gchar *cfg_file = g_build_filename (config_dir, default_config_file, NULL);
 	gchar *local = F_FILENAME_TO_LOCALE (cfg_file);
 #ifdef E2_VFS
 	VPATH ddata = { local, NULL };	//only local config data
@@ -1062,7 +1064,7 @@ static gboolean _e2_option_config_file_read (gpointer *contents)
 	{	//try for the old-style name
 		g_free (cfg_file);
 		F_FREE (local, cfg_file);
-		cfg_file = g_build_filename (e2_cl_options.config_dir, "config", NULL);
+		cfg_file = g_build_filename (config_dir, "config", NULL);
 		local = F_FILENAME_TO_LOCALE (cfg_file);
 #ifdef E2_VFS
 		ddata.path = local;
@@ -1124,12 +1126,16 @@ First, the configuration is read, then its content is parsed with
 e2_option_read_array(). If tree options have no configuration,
 they are initialized with a default one.
 (All other options have been intialized when they were registered.)
+@param config_dir absolute path of dir containing config files, UTF-8 string,
+ or NULL to use default
 @return TRUE if the config file was read
 */
-gboolean e2_option_file_read (void)
+gboolean e2_option_file_read (const gchar *config_dir)
 {
 	gpointer contents;
-	if (_e2_option_config_file_read (&contents))
+	if (config_dir == NULL)
+		config_dir = e2_cl_options.config_dir;
+	if (_e2_option_config_file_read (config_dir, &contents))
 	{
 		gboolean retval;
 		//everything goes into an array
@@ -1271,10 +1277,6 @@ writing */
 #ifndef E2_FAM
 	e2_fs_FAM_config_stamp ();	//set baseline time, for config refreshing
 #endif
-
-#ifdef E2_IMAGECACHE
-	e2_cache_icons_init ();
-#endif
 }
 /**
 @brief setup array of i18n config dialog labels
