@@ -1,6 +1,6 @@
-/* $Id: e2_option__default.c 2344 2011-04-11 22:46:24Z tpgww $
+/* $Id: e2_option__default.c 3058 2014-02-14 23:38:00Z tpgww $
 
-Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -143,6 +143,7 @@ void e2_option_default_register (void)
 
 	e2_keybinding_options_register ();	//here to make nicer order in tab
 
+#ifndef USE_GTK3_10
 	e2_option_int_register ("submenu-up-delay", group_name, _("menu popup delay (ms)"),
 		_("The delay (in milliseconds) after the mouse pointer arrives at a menu item, before any submenu will pop up"),
 		NULL, 50, 0, 10000,
@@ -151,6 +152,13 @@ void e2_option_default_register (void)
 		_("The delay (in milliseconds) after the mouse pointer leaves a menu item, before popping down an activated submenu"),
 		NULL, 400, 0, 10000,
 		E2_OPTION_FLAG_ADVANCED); //no rebuild
+#endif
+	const gchar *opt_window_title[] =
+		{_("fixed"), _("pane1 directory"), _("pane2 directory"), _("active pane directory"), NULL};
+	e2_option_sel_register ("title-type", group_name, _("main window title"),
+		_("Show the application name only or append a pane directory"),
+		NULL, 0, opt_window_title,
+		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDALL);
     //option-values for this must conform to GtkCornerType enum, but for scrolled
 	//windows, the enum determines where the child is positioned, the opposite of
 	//where the bar(s) are positioned
@@ -214,8 +222,8 @@ void e2_option_default_register (void)
 		NULL, FALSE,
 		E2_OPTION_FLAG_ADVANCED);	//no rebuild
 	e2_option_bool_register ("cache-history", group_name,
-		_("persistent opened-directories history"),
-		_("Save opened-directories data for use in the next session"),
+		_("persistent goto history"),
+		_("Save \"goto buttons\" opened-directories data for use in the next session"),
 		NULL, TRUE,
 		E2_OPTION_FLAG_ADVANCED);	//no rebuild, and if changed in-session,
 									//will not affect cacheing of that session
@@ -233,9 +241,21 @@ void e2_option_default_register (void)
 		{_("colored headers"), _("bold name header"),_("sensitivity"),NULL};
 	e2_option_sel_register ("active-pane-signal", group_name, _("type of indicator for active pane"),
 		_("This determines how the active pane is indicated on-screen. Some GTK themes do not allow column-header re-coloring"),
-		NULL, 0, opt_menu_activeflag,
+#ifdef E2_SMALLSCREEN
+		"!active-pane-tools",
+#else
+		NULL,
+#endif
+		0, opt_menu_activeflag,
 		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDPANES);
 
+#ifdef E2_SMALLSCREEN
+		e2_option_bool_register ("active-pane-tools", group_name, _("single toolbar"),
+		_("Show toolbar in active pane only"),
+		NULL, FALSE,
+		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDPANES);
+#endif
+
 	e2_option_bool_register ("panes-hinted", group_name, _("banded background"),
 		_("If activated, lines in file lists will alternate background color"),
 		NULL, FALSE,
@@ -390,6 +410,7 @@ tags not supported now
 //	e2_search_dialog_options_register ();
 	e2_view_dialog_options_register ();
 	e2_edit_dialog_options_register ();
+	e2_file_info_dialog_options_register ();
 // keybindings have a separate data structure too
 //above	e2_keybinding_init(); //this also does some other stuff
 //above	e2_filetype_options_register();
