@@ -1,4 +1,4 @@
-/* $Id: e2_option_str.c 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_option_str.c 2903 2013-10-31 22:32:24Z tpgww $
 
 Copyright (C) 2004-2010 tooar <tooar@emelfm2.net>
 
@@ -28,6 +28,8 @@ E2_OptionSet *e2_option_str_register (gchar *name, gchar *group, gchar *desc,
 	E2_OptionSet *set = e2_option_register (E2_OPTION_TYPE_STR, name, group,
 		desc, tip, depends, flags);
 	set->ival = -1;
+	if (value == NULL)
+		value = "";
 	set->sval = g_strdup (value);
 	return set;
 }
@@ -55,5 +57,7 @@ gchar *e2_option_str_get_direct (E2_OptionSet *set)
 void e2_option_str_set_direct (E2_OptionSet *set, gchar *value)
 {
 	g_free (set->sval);
+	if (value == NULL)
+		value = "";
 	set->sval = g_strdup (value);
 }
