@@ -1,6 +1,6 @@
-/* $Id: e2_option_tree_context_menu.c 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_option_tree_context_menu.c 3030 2014-01-23 19:30:30Z tpgww $
 
-Copyright (C) 2003-2010 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 Portions copyright (C) 2004 Florian Zaehringer <flo.zaehringer@web.de>
 
 This file is part of emelFM2.
@@ -30,6 +30,8 @@ This file is #included in e2_option_tree.c
 //prevent building of separate object file
 #ifdef INCLUDED_IN_PARENT
 
+#include "e2_plugins.h"
+
   /*******************/
  /***** private *****/
 /*******************/
@@ -69,12 +71,14 @@ static const gchar *_e2_option_tree_menu_get_signature (GtkTreeView *treeview)
 */
 static void _e2_option_tree_expand_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
+	NEEDCLOSEBGL
 	gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
 	//get the paired item
 	gpointer m = g_object_get_data (G_OBJECT (widget), "e2-menu-pair");
 	gtk_widget_set_sensitive (GTK_WIDGET (m), TRUE);
 
 	gtk_tree_view_expand_all (treeview);
+	NEEDOPENBGL
 }
 /**
 @brief tree collapse-all callback
@@ -86,12 +90,14 @@ static void _e2_option_tree_expand_all_cb (GtkMenuItem *widget, GtkTreeView *tre
 */
 static void _e2_option_tree_collapse_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
+	NEEDCLOSEBGL
 	gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
 	//get the paired button
 	gpointer m = g_object_get_data (G_OBJECT (widget), "e2-menu-pair");
 	gtk_widget_set_sensitive (GTK_WIDGET (m), TRUE);
 
 	gtk_tree_view_collapse_all (treeview);
+	NEEDOPENBGL
 }
 /**
 @brief context-menu copy callback
@@ -114,6 +120,7 @@ The table may contain entries for more than 1 treeview
 static void _e2_option_tree_menu_copy_cb (GtkMenuItem *menu_item, GtkTreeView *treeview)
 {
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
+	NEEDCLOSEBGL
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
 		GList *rowscopied = e2_tree_copy (treeview);
@@ -122,6 +129,7 @@ static void _e2_option_tree_menu_copy_cb (GtkMenuItem *menu_item, GtkTreeView *t
 			g_list_length (rowscopied));
 		g_hash_table_insert (tree_view_buffer_hash, (gchar*)id_string, rowscopied); //NOT replace
 	}
+	NEEDOPENBGL
 }
 /**
 @brief context-menu paste callback
@@ -140,6 +148,7 @@ static void _e2_option_tree_menu_paste_cb (GtkMenuItem *menu_item, GtkTreeView *
 {
 	//we only paste after the 1st selected row
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
+	NEEDCLOSEBGL
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
 		//get data for the current page/view
@@ -154,6 +163,7 @@ static void _e2_option_tree_menu_paste_cb (GtkMenuItem *menu_item, GtkTreeView *
 			e2_option_tree_flag_change (set);
 		}
 	}
+	NEEDOPENBGL
 }
 /**
 @brief context-menu cut callback
@@ -166,17 +176,90 @@ static void _e2_option_tree_menu_paste_cb (GtkMenuItem *menu_item, GtkTreeView *
 static void _e2_option_tree_menu_cut_cb (GtkMenuItem *menu_item, GtkTreeView *treeview)
 {
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
+	NEEDCLOSEBGL
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
+		NEEDOPENBGL
 		_e2_option_tree_menu_copy_cb (menu_item, treeview);
+		NEEDCLOSEBGL
 		e2_tree_delete (treeview);
 		GtkTreeModel *mdl = gtk_tree_view_get_model (treeview);
 		E2_OptionSet *set = g_object_get_data (G_OBJECT (mdl), "e2-option-set");
 		e2_option_tree_flag_change (set);
 	}
+	NEEDOPENBGL
 }
 /**
-@brief add items to treev-option context menu
+@brief helper to [un]select all values of a boolean field in config store for @a set
+@param set config data for the set
+@param colnum treestore column enum value
+@param val the new value, T or F
+@return
+*/
+static void _e2_option_tree_update_all_bool (E2_OptionSet *set, guint colnum, gboolean val)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *mdl = set->ex.tree.model;
+	if (gtk_tree_model_get_iter_first (mdl, &iter))
+	{
+		NEEDCLOSEBGL
+		do
+		{
+			gtk_tree_store_set (GTK_TREE_STORE(mdl), &iter, colnum, val, -1);
+		} while (gtk_tree_model_iter_next (mdl, &iter));
+		NEEDOPENBGL
+	}
+}
+/**
+@brief plugins context-menu load all callback
+
+@param menu_item the activated menu widget
+@param set the plugins config data set
+
+@return 
+*/
+static void _e2_option_tree_loadall_cb (GtkMenuItem *menu_item, E2_OptionSet *set)
+{
+	_e2_option_tree_update_all_bool (set, LOAD_COL, TRUE);
+}
+/**
+@brief plugins context-menu unload all callback
+
+@param menu_item the activated menu widget
+@param set the plugins config data set
+
+@return 
+*/
+static void _e2_option_tree_unloadall_cb (GtkMenuItem *menu_item, E2_OptionSet *set)
+{
+	_e2_option_tree_update_all_bool (set, LOAD_COL, FALSE);
+}
+/**
+@brief plugins context-menu all on menu callback
+
+@param menu_item the activated menu widget
+@param set the plugins config data set
+
+@return 
+*/
+static void _e2_option_tree_menuall_cb (GtkMenuItem *menu_item, E2_OptionSet *set)
+{
+	_e2_option_tree_update_all_bool (set, MENU_COL, TRUE);
+}
+/**
+@brief plugins context-menu none on menu callback
+
+@param menu_item the activated menu widget
+@param set the plugins config data set
+
+@return 
+*/
+static void _e2_option_tree_unmenuall_cb (GtkMenuItem *menu_item, E2_OptionSet *set)
+{
+	_e2_option_tree_update_all_bool (set, MENU_COL, FALSE);
+}
+/**
+@brief add items to tree-option context menu
 
 @param menu the menu widget to which the items are to be added
 @param treeview where the action is to occur
@@ -193,51 +276,74 @@ static void _e2_option_tree_populate_menu (GtkWidget *menu,
 		(GDestroyNotify) e2_utils_nontet_destroy);
 	if (n == NULL)
 		return;
-	//menu items which modify store content need set for callback data,
-	//other items
+	//menu items which modify store content need set for callback data
 	if ((flags & E2_TREE_CONTEXT_CP_PST) || (flags & E2_TREE_CONTEXT_DEFAULT))
 	{
-		n->a = e2_menu_add (menu, _("Cu_t"), GTK_STOCK_CUT,
+		n->a = e2_menu_add (menu, _("Cu_t"), STOCK_NAME_CUT,
 			_("Cut selected row and any descendant(s)"),
 			_e2_option_tree_menu_cut_cb, treeview);
 
-		n->b = e2_menu_add (menu, _("_Copy"), GTK_STOCK_COPY,
-			_("Copy selected row and any descendant(s)"),
-			_e2_option_tree_menu_copy_cb, treeview);
+		if (e2_option_get_simple ("plugins") != set)
+			n->b = e2_menu_add (menu, _("_Copy"), STOCK_NAME_COPY,
+				_("Copy selected row and any descendant(s)"),
+				_e2_option_tree_menu_copy_cb, treeview);
 
-		n->c = e2_menu_add (menu, _("_Paste"), GTK_STOCK_PASTE,
+		n->c = e2_menu_add (menu, _("_Paste"), STOCK_NAME_PASTE,
 			_("Paste previously copied or cut row(s) after current row"),
 			_e2_option_tree_menu_paste_cb, treeview);
 
 		e2_menu_add_separator (menu);
 	}
-	if (flags & E2_TREE_CONTEXT_EXP_COL)
+	if (e2_option_get_simple ("plugins") != set)
 	{
-		n->d = e2_menu_add (menu, _("_Expand"), GTK_STOCK_ZOOM_IN,
-				_("Expand all rows on this page"),
-				_e2_option_tree_expand_all_cb, treeview);
-		gtk_widget_set_sensitive (GTK_WIDGET(n->d), FALSE);	//widget starts with rows expanded
-		n->e = e2_menu_add (menu, _("C_ollapse"), GTK_STOCK_ZOOM_OUT,
-				_("Collapse all rows on this page"),
-				_e2_option_tree_collapse_all_cb, treeview);
-		//set pointers to the paired buttons
-		g_object_set_data (G_OBJECT (n->d), "e2-menu-pair", n->e);
-		g_object_set_data (G_OBJECT (n->e), "e2-menu-pair", n->d);
+		if (flags & E2_TREE_CONTEXT_EXP_COL)
+		{
+			n->d = e2_menu_add (menu, _("_Expand"), STOCK_NAME_ZOOM_IN,
+					_("Expand all rows on this page"),
+					_e2_option_tree_expand_all_cb, treeview);
+			gtk_widget_set_sensitive (GTK_WIDGET(n->d), FALSE);	//widget starts with rows expanded
+			n->e = e2_menu_add (menu, _("C_ollapse"), STOCK_NAME_ZOOM_OUT,
+					_("Collapse all rows on this page"),
+					_e2_option_tree_collapse_all_cb, treeview);
+			//set pointers to the paired buttons
+			g_object_set_data (G_OBJECT (n->d), "e2-menu-pair", n->e);
+			g_object_set_data (G_OBJECT (n->e), "e2-menu-pair", n->d);
 
-		e2_menu_add_separator (menu); //more items added in the calling function
+			e2_menu_add_separator (menu); //more items added in the calling function
+		}
+
+		n->f = e2_menu_add (menu, _("_Add"), STOCK_NAME_ADD,
+			_("Add a row after the current one"), (void(*)()) _e2_option_tree_add_below_cb, set);
+
+		if (flags & E2_TREE_CONTEXT_EXP_COL)
+			n->g = e2_menu_add (menu, _("Add c_hild"), STOCK_NAME_INDENT,
+				_("Add a child to the selected row"), (void(*)()) _e2_option_tree_add_child_cb, set);
+	}
+	else //this menu is for the plugins config page
+	{
+		n->d = e2_menu_add (menu, _("_Load all"),
+#ifdef USE_GTK2_10
+		STOCK_NAME_SELECT_ALL,
+#else
+		STOCK_NAME_APPLY,
+#endif
+			_("Mark all listed plugins to be loaded"), (void(*)()) _e2_option_tree_loadall_cb, set);
+		n->e = e2_menu_add (menu, _("Load _none"), STOCK_NAME_CLEAR,
+			_("Mark all listed plugins to not be loaded"), (void(*)()) _e2_option_tree_unloadall_cb, set);
+		n->f = e2_menu_add (menu, _("_Menu all"), STOCK_NAME_YES,
+			_("Show all listed plugins in menu"), (void(*)()) _e2_option_tree_menuall_cb, set);
+		n->g = e2_menu_add (menu, _("M_enu none"), STOCK_NAME_NO,
+			_("Hide all listed plugins from menu"), (void(*)()) _e2_option_tree_unmenuall_cb, set);
+		e2_menu_add_separator (menu);
+		e2_menu_add (menu, _("_Select"), STOCK_NAME_INDEX,
+			_("Select one or more plugins to be added"), (void(*)()) e2_confdlg_choose_plugins_cb, set);
 	}
-	n->f = e2_menu_add (menu, _("_Add"), GTK_STOCK_ADD,
-		_("Add a row after the current one"), (void(*)()) _e2_option_tree_add_below_cb, set);
-
-	if (flags & E2_TREE_CONTEXT_EXP_COL)
-		n->g = e2_menu_add (menu, _("Add c_hild"), GTK_STOCK_INDENT,
-			_("Add a child to the selected row"), (void(*)()) _e2_option_tree_add_child_cb, set);
 
 	e2_menu_add_separator (menu);
-	n->h = e2_menu_add (menu, _("_Up"), GTK_STOCK_GO_UP,
+	n->h = e2_menu_add (menu, _("_Up"), STOCK_NAME_GO_UP,
 		_("Move selected row up"), (void(*)()) _e2_option_tree_move_up_cb, set);
 
-	n->i = e2_menu_add (menu, _("_Down"), GTK_STOCK_GO_DOWN,
+	n->i = e2_menu_add (menu, _("_Down"), STOCK_NAME_GO_DOWN,
 		_("Move selected row down"), (void(*)()) _e2_option_tree_move_down_cb, set);
 
 }
@@ -332,9 +438,11 @@ void e2_option_tree_menu_set_sensitive (GtkWidget *menu, GtkWidget *treeview)
 /**
 @brief free memory used by tree-option context menu hash table entry
 
-Each entry in the hash buffer is a glist of value arrays, one array for each
-row copied to the buffer
-The first item in an array is the the row's source-view path
+Each entry in the hash buffer is a glist of value arrays, one array for each row
+copied to the buffer.
+The first member of each array records the row's (allocated) treeview-path.
+Other members are taken from treemodel column data.
+See _e2_tree_copy_branch() 
 
 @param list  glist to be freed
 
@@ -342,15 +450,22 @@ The first item in an array is the the row's source-view path
 */
 void e2_option_tree_menu_hash_clean (GList *list)
 {
-	GList *tmp;
-	for (tmp = list; tmp != NULL; tmp = tmp->next)
+	GList *member;
+	for (member = list; member != NULL; member = member->next)
 	{
-		//clear treepath at start of each array
-		GValueArray *value_array = tmp->data;
-		GValue *value = g_value_array_get_nth (value_array, 0);
-		GtkTreePath *path = (GtkTreePath *) g_value_get_pointer (value);
+		GArray *value_array = member->data;
+		//clear treepath at start of array
+		GValue *valptr = (GValue*)value_array->data; //&g_array_index (value_array, GValue, 0);
+		GtkTreePath *path = (GtkTreePath *) g_value_get_pointer (valptr);
 		gtk_tree_path_free (path);
-		g_value_array_free (value_array);
+		//free other resources in the array, hope for no leaks! 
+		guint i;
+		for (i = 0; i < value_array->len; i++, valptr++)
+		{
+			if (G_VALUE_TYPE (valptr) != 0)
+				g_value_unset (valptr);
+		}
+		g_array_free (value_array, TRUE);
 	}
 	g_list_free (list);
 }
