@@ -1,6 +1,6 @@
-/* $Id: e2_config_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_config_dialog.c 2988 2013-12-03 07:38:31Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -30,6 +30,7 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #ifdef E2_MOUSECUSTOM
 # include "e2_mousebinding.h"
 #endif
+#include "e2_icons.h"
 
 //define to apply scrolledwindow to each page of the config notebeok, allowing
 //omission of bars for whole book, which cause nested bars for tree-set data
@@ -89,7 +90,9 @@ static void _e2_cd1_cancel (E2_SpecificConfDialogRuntime *rt, gboolean flushtype
 /* handled in default response
 static void _e2_cd1_close_cb (GtkWidget *widget, E2_SpecificConfDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	_e2_cd1_cancel (rt, TRUE);
+	NEEDOPENBGL
 } */
 /**
 @brief close the dialog when esc key is pressed
@@ -107,7 +110,9 @@ static gboolean _e2_cd1_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 {
 	if (event->keyval == GDK_Escape)
 	{  //restore tree data and clean up
+		NEEDCLOSEBGL
 		_e2_cd1_cancel (rt, TRUE);
+		NEEDOPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -127,6 +132,7 @@ Note: applies only to tree options
 static void _e2_cd1_response_cb (GtkDialog *dialog, gint response,
 	E2_SpecificConfDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	switch (response)
 	{
 		case E2_RESPONSE_APPLY:
@@ -155,6 +161,7 @@ static void _e2_cd1_response_cb (GtkDialog *dialog, gint response,
 			_e2_cd1_cancel (rt, TRUE);
 			break;
 	}
+	NEEDOPENBGL
 }
 /**
 @brief create single-page config dialog for a tree-type option set
@@ -286,10 +293,10 @@ void e2_confdlg_menu_set_position (GtkMenu *menu,
 static void _e2_confdlg_show_context_menu (GtkWidget *treeview, guint event_button,
 	guint32 event_time, gpointer data)
 {
-	GtkWidget *menu = gtk_menu_new ();
-	e2_menu_add (menu, _("_Expand"), GTK_STOCK_ZOOM_IN,
+	GtkWidget *menu = e2_menu_get ();
+	e2_menu_add (menu, _("_Expand"), STOCK_NAME_ZOOM_IN,
 		_("Expand all rows"), e2_tree_expand_all_cb, treeview);
-	e2_menu_add (menu, _("C_ollapse"), GTK_STOCK_ZOOM_OUT,
+	e2_menu_add (menu, _("C_ollapse"), STOCK_NAME_ZOOM_OUT,
 		_("Collapse all rows"), e2_tree_collapse_all_cb, treeview);
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 		G_CALLBACK (e2_menu_selection_done_cb), NULL);
@@ -332,8 +339,10 @@ static void _e2_hashfree (gpointer data)
 */
 static void _e2_confdlg_show_cb (GtkWidget *dialog, GtkWidget *sw)
 {
+	NEEDCLOSEBGL
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	NEEDOPENBGL
 }
 #endif
 /**
@@ -360,12 +369,14 @@ static void _e2_confdlg_show_cb2 (GtkWidget *dialog, E2_ConfigDialogRuntime *rt)
 static gboolean _e2_confdlg_negdepends_cb (GtkWidget *button,
 	GtkWidget *dependant)
 {
+	NEEDCLOSEBGL
 	gtk_widget_set_sensitive (dependant,
 #ifdef USE_GTK2_14
 		!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
 #else
 		!GTK_TOGGLE_BUTTON (button)->active);
 #endif
+	NEEDOPENBGL
 	return TRUE;
 }
 /**
@@ -519,15 +530,15 @@ static gboolean _e2_confdlg_apply (gboolean close, E2_ConfigDialogRuntime *rt)
 #endif
 	}
 #endif
-#ifdef E2_IMAGECACHE
+#ifdef E2_ICONCACHE
 	if (buildflags & E2_OPTION_FLAG_BUILDICONS)
 	{
-		e2_cache_image_clearall ();
-		e2_cache_icons_init ();
+		e2_icons_cache_clear ();
+		e2_icons_cache_init ();
 	}
 #endif
 	if (buildflags & E2_OPTION_FLAG_BUILDPLUGS)
-		e2_plugins_update ();
+		e2_plugins_update_configured ();
 
 	if (buildflags & E2_OPTION_FLAG_BUILDSAMEBARS)
 	{	//user wants toolbar buttons conformed
@@ -752,6 +763,7 @@ static void _e2_confdlg_cancel_cb (void)
 	printd (DEBUG, "config dialog cancel cb");
 	if (config_dialog != NULL)
 	{
+		NEEDCLOSEBGL
 		//NOTE this triggers an "edited" callback for any
 		//treeview cell that is being edited at this time !
 		_e2_confdlg_clean ();
@@ -759,6 +771,7 @@ static void _e2_confdlg_cancel_cb (void)
 		//after the clean
 		e2_option_tree_restore_all ();
 //		gtk_widget_grab_focus (curr_view->treeview);
+		NEEDOPENBGL
 	}
 }
 /**
@@ -773,6 +786,7 @@ static void _e2_confdlg_cancel_cb (void)
 static void _e2_confdlg_response_cb (GtkDialog *dialog, gint response,
 	E2_ConfigDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	switch (response)
 	{
 		case E2_RESPONSE_APPLY:
@@ -807,11 +821,13 @@ static void _e2_confdlg_response_cb (GtkDialog *dialog, gint response,
 			e2_config_dialog_create (NULL);
 			break;
 		default:
+			NEEDOPENBGL
 			_e2_confdlg_cancel_cb ();
+			NEEDCLOSEBGL
 			DEALLOCATE (E2_ConfigDialogRuntime, rt);
 			break;
 	}
-	return;
+	NEEDOPENBGL
 }
 /**
 @brief close the dialog if the esc key is pressed
@@ -826,14 +842,22 @@ static void _e2_confdlg_response_cb (GtkDialog *dialog, gint response,
 gboolean e2_confdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	gpointer data)
 {
-//	if (event->keyval == GDK_Return
-//	 || event->keyval == GDK_KP_Enter
-//	 || event->keyval == GDK_ISO_Enter
-//	 || event->keyval == GDK_3270_Enter)  return activates any focused button
-//		return e2_confdlg_ok_cb (FALSE);
+/ *	if (event->keyval == GDK_Return
+	 || event->keyval == GDK_KP_Enter
+	 || event->keyval == GDK_ISO_Enter
+	 || event->keyval == GDK_3270_Enter)  return activates any focused button
+	{
+//		NEEDOPENBGL
+		gboolean ret = e2_confdlg_ok_cb (FALSE);
+//		NEEDCLOSEBGL
+		return ret;
+	}
+* /
 	if (event->keyval == GDK_Escape)
 	{
+//		NEEDOPENBGL
 		_e2_confdlg_cancel_cb ();
+//		NEEDCLOSEBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -867,8 +891,10 @@ static gboolean _e2_confdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	{
 		if (event->keyval == GDK_Escape)
 		{
+			NEEDCLOSEBGL
 			gtk_entry_set_text (GTK_ENTRY (set->widget), set->sval);
 			gtk_editable_set_position (GTK_EDITABLE (set->widget), -1);
+			NEEDOPENBGL
 		}
 		//revert <Esc> key handling when cell editing is finsished
 		g_signal_handlers_unblock_by_func (G_OBJECT (config_dialog),
@@ -918,7 +944,9 @@ static gboolean _e2_confdlg_button_press_cb (GtkWidget *treeview,
 #endif
 		)
 	{
+		NEEDCLOSEBGL
 		_e2_confdlg_show_context_menu (treeview, 3, event->time, data);
+		NEEDOPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -934,7 +962,9 @@ static gboolean _e2_confdlg_button_press_cb (GtkWidget *treeview,
 static gboolean _e2p_confdlg_popup_menu_cb (GtkWidget *treeview, gpointer data)
 {
 	guint32 event_time = gtk_get_current_event_time ();
+	NEEDCLOSEBGL
 	_e2_confdlg_show_context_menu (treeview, 0, event_time, data);
+	NEEDOPENBGL
 	return TRUE;
 }
 /**
@@ -942,36 +972,86 @@ static gboolean _e2p_confdlg_popup_menu_cb (GtkWidget *treeview, gpointer data)
 
 @param dialog the dialog where the response was triggered
 @param response the response assigned to the activated button widget
-@param data ptr to plugins option data struct
+@param set ptr to plugins option set
 
 @return
 */
 static void _e2_confdlg_plugpick_response_cb (GtkDialog *dialog, gint response,
-	E2_TreeCfgData *data)
+	E2_OptionSet *set)
 {
-	gchar *plocal;
+	GSList *selected;
+	NEEDCLOSEBGL
 	switch (response)
 	{
 		case E2_RESPONSE_USER1:	//toggle hidden items display
 			break;
 		case GTK_RESPONSE_OK:
 		case E2_RESPONSE_MORE:
-			//returns localized (not utf) string, no trailer unless the user deliberately enters one
-			plocal = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-			if (plocal != NULL)
+			//returns localized (not UTF8) strings
+			selected = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (dialog));
+			if (selected)
 			{
-				Plugin *p = e2_plugins_open1 (plocal);
-				if (p != NULL)
+				GSList *member;
+				GtkTreeIter first, sibling;
+				GtkTreeIter *pf = NULL, *ps = NULL;
+
+				if (gtk_tree_model_get_iter_first (set->ex.tree.model, &first))
 				{
-					GtkTreeIter iter;
-					gtk_tree_model_get_iter (data->set->ex.tree.model, &iter, data->tpath);
-					//assume the user wants to use the selected plugin, so TRUE menu etc flags
-					e2_plugins_store_data (data->set->ex.tree.model, &iter, p, TRUE, plocal);
-					gtk_tree_view_expand_row (GTK_TREE_VIEW (data->set->widget), data->tpath, FALSE);
-					e2_option_tree_flag_change (data->set);
-					DEALLOCATE (Plugin, p);	//no leaks provided all embedded strings are constant
+					pf = &first;
+
+					GtkTreeSelection *sel = gtk_tree_view_get_selection
+						(GTK_TREE_VIEW (set->widget));
+					if (gtk_tree_selection_get_selected (sel, NULL, &sibling))
+						ps = &sibling;
+					else
+					{
+						gint count = gtk_tree_model_iter_n_children
+							(set->ex.tree.model, NULL);
+						if (count > 1)
+						{
+							gtk_tree_model_iter_nth_child (set->ex.tree.model,
+								&sibling, NULL, count-1);
+							ps = &sibling;
+						}
+						else
+							ps = pf;
+					}
 				}
-				g_free (plocal);
+
+				for (member = selected; member != NULL; member = member->next)
+				{
+					E2P_InitData pdata;
+					gchar *plocal = member->data;
+					if (e2_plugins_open_module (plocal, &pdata))
+					{
+						Plugin *p = (*pdata.init) (E2P_UIDATA);
+						if ((p->flags & E2P_NOCFG) == 0)
+						{
+							if (pf != NULL)
+							{
+								GtkTreeIter work = first;
+								//absolute filepath will always have at least 1 separator
+								gchar *sep = strrchr (plocal, G_DIR_SEPARATOR);
+								if (e2_tree_find_iter_from_str_simple (set->ex.tree.model,
+									FILE_COL, sep + 1, &work, FALSE))
+								{
+									//duplicate entry, dump it
+									p->module = pdata.module;
+									e2_plugins_unload1 (p, TRUE);
+									continue;
+								}
+							}
+							e2_plugins_store_config_data (set->ex.tree.model,
+								ps, p, TRUE, plocal);
+							//if we started with an empty store, can't be duplication, so don't start matching
+							e2_option_tree_flag_change (set);
+							p->module = pdata.module;
+							e2_plugins_unload1 (p, TRUE); //cleanup then dump
+						}
+					}
+					g_free (plocal);
+				}
+				g_slist_free (selected);
 			}
 			else
 			{	//conversion error
@@ -981,11 +1061,10 @@ static void _e2_confdlg_plugpick_response_cb (GtkDialog *dialog, gint response,
 				break;
 		default:
 			gtk_widget_destroy (GTK_WIDGET (dialog));
-			gtk_tree_path_free (data->tpath);
-			gtk_widget_grab_focus (data->set->widget);
-			DEMALLOCATE (E2_TreeCfgData, data);	//too small for slice
+			gtk_widget_grab_focus (set->widget);
 			break;
 	}
+	NEEDOPENBGL
 }
 /**
 @brief bring up a system find-file window to choose a plugin
@@ -995,90 +1074,70 @@ static void _e2_confdlg_plugpick_response_cb (GtkDialog *dialog, gint response,
 
 @return
 */
-void e2_confdlg_choose_plugin_cb (GtkButton *button, E2_OptionSet *set)
+void e2_confdlg_choose_plugins_cb (GtkButton *button, E2_OptionSet *set)
 {
+	gchar *ppath;
 	GtkTreeView *tvw = GTK_TREE_VIEW (set->widget);
 	GtkTreePath *tpath;
-	gtk_tree_view_get_cursor (tvw, &tpath, NULL);
-	if (tpath == NULL)
-		return;
 
-	E2_TreeCfgData *data = MALLOCATE (E2_TreeCfgData);	//too small for slice
-	CHECKALLOCATEDWARN (data, return;);
-	data->set = set;
-	data->tpath = tpath;
+	NEEDCLOSEBGL
+	gtk_tree_view_get_cursor (tvw, &tpath, NULL); //NULL tpath (no plugins) is unlikely
 
-	GtkTreeIter iter;
-	gchar *pname, *ppath;
-	gtk_tree_model_get_iter (set->ex.tree.model, &iter, tpath);
-	gtk_tree_model_get (set->ex.tree.model, &iter, 5, &pname, 6, &ppath, -1);
-	if (*pname == '\0')
+	if (tpath != NULL)
 	{
-		g_free (ppath);
-		ppath = NULL;	//we'll just set the dir in the opened dialog
-	}
-	else
-	{
-		if (*ppath == '\0')
+		GtkTreeIter iter;
+		gchar *pname;
+
+		gtk_tree_model_get_iter (set->ex.tree.model, &iter, tpath);
+		gtk_tree_model_get (set->ex.tree.model, &iter, 5, &pname, 6, &ppath, -1);
+		if (*pname == '\0')
 		{
 			g_free (ppath);
-			ppath = g_strconcat (PLUGINS_DIR G_DIR_SEPARATOR_S, pname, NULL);	//mixed ascii & utf
+			ppath = NULL;	//we'll just set the dir in the opened dialog
 		}
 		else
 		{
-			gchar *freeme = ppath;
-			ppath = g_build_filename (ppath, pname, NULL);
-			g_free (freeme);
+			if (*ppath == '\0')
+			{
+				g_free (ppath);
+				ppath = g_strconcat (PLUGINS_DIR G_DIR_SEPARATOR_S, pname, NULL);	//mixed ascii & utf
+			}
+			else
+			{
+				gchar *freeme = ppath;
+				ppath = g_build_filename (ppath, pname, NULL);
+				g_free (freeme);
+			}
 		}
+		g_free (pname);
 	}
-	g_free (pname);
+	else
+		ppath = NULL;
 
 	//no need for vfs support, local plugins only
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
-		GTK_WINDOW (config_dialog), GTK_FILE_CHOOSER_ACTION_OPEN,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-		NULL);
+		GTK_WINDOW (config_dialog), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL);
 
-/*	//hide the dialog's standard 'open' button
-	GtkContainer *bbox = GTK_CONTAINER (
-#ifdef USE_GTK2_14
-		gtk_dialog_get_action_area (GTK_DIALOG (dialog));
-#else
-		GTK_DIALOG (dialog)->action_area);
-#endif
-	GList* children = gtk_container_get_children (bbox);
-	GtkWidget *btn = children->data;
-	gtk_widget_hide (btn);
-	g_list_free (children);
-	//add 2 buttons that we want
-	e2_dialog_add_defined_button (dialog, &E2_BUTTON_MORE);
-	e2_dialog_add_defined_button (dialog, &E2_BUTTON_APPLY);
-*/
 	E2_Button more_btn = E2_BUTTON_MORE;
 	more_btn.tip = _("Select and continue");
 	more_btn.showflags |= E2_BTN_TIPPED;
-	GtkWidget *btn = e2_dialog_add_defined_button (dialog, &more_btn);
-	GtkWidget *bbox =
-#ifdef USE_GTK2_14
-		gtk_dialog_get_action_area (GTK_DIALOG (dialog));
-#else
-		GTK_DIALOG (dialog)->action_area;
-#endif
-	gtk_box_reorder_child (GTK_BOX (bbox), btn, 0);
+	e2_dialog_add_defined_button (dialog, &more_btn);
 
 	e2_dialog_setup_chooser (dialog,
 		_("choose plugin"),
 		ppath,	//full path or NULL
 		GTK_FILE_CHOOSER_ACTION_OPEN,
 		TRUE,	//show hidden
-		FALSE,	//single-selection
-		GTK_RESPONSE_OK);	//default response
+		TRUE,	//multi-selection
+		GTK_RESPONSE_OK,	//default response
+		STOCK_NAME_CANCEL, GTK_RESPONSE_CANCEL,
+		STOCK_NAME_OPEN, GTK_RESPONSE_OK,
+		NULL);
 
-	if (ppath == NULL)
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), PLUGINS_DIR);//localised
-	else
+	if (ppath != NULL)
 		g_free (ppath);
+	else
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), PLUGINS_DIR);//localised
 
 	GtkFileFilter *filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (GTK_FILE_FILTER (filter), _("plugin"));
@@ -1086,34 +1145,88 @@ void e2_confdlg_choose_plugin_cb (GtkButton *button, E2_OptionSet *set)
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
 
 	g_signal_connect (G_OBJECT (dialog), "response",
-		G_CALLBACK (_e2_confdlg_plugpick_response_cb), data);
+		G_CALLBACK (_e2_confdlg_plugpick_response_cb), set);
 
 	e2_dialog_setup (dialog, config_dialog);
 	gtk_widget_show (dialog);
+	NEEDOPENBGL
 }
 
   /**************************/
  /***** font callbacks *****/
 /**************************/
 
+#ifdef USE_GTK3_2
+/**
+@brief callback for signal issued when displayed font is activated
+
+@param chooser UNUSED interface object
+@param font_str the name of the activated font
+@param entry config dialog set-widget
+
+@return
+*/
+static void _e2_confdlg_font_activate_cb (GtkFontChooser *chooser, gchar *font_str,
+	GtkWidget *entry)
+{
+	if (font_str == NULL)
+		font_str = "";
+	NEEDCLOSEBGL
+	gtk_entry_set_text (GTK_ENTRY (entry), font_str);
+	NEEDOPENBGL
+}
+/**
+@brief font selection dialog response callback
+
+@param dialog the dialog
+@param response enumerator of type of re
+@param entry config dialog set-widget
+
+@return
+*/
+static void _e2_confdlg_font_response_cb (GtkDialog *dialog, gint response,
+	GtkWidget *entry)
+{
+	NEEDCLOSEBGL
+	gtk_widget_hide (GTK_WIDGET (dialog));
+	if (response == GTK_RESPONSE_OK)
+	{
+		gchar *font_str = gtk_font_chooser_get_font (GTK_FONT_CHOOSER (dialog));
+		if (font_str != NULL)
+		{
+			gtk_entry_set_text (GTK_ENTRY (entry), font_str);
+			g_free (font_str);
+		}
+	}
+	else if (response == GTK_RESPONSE_DELETE_EVENT)
+	{
+		const gchar *title = gtk_window_get_title (GTK_WINDOW (dialog));
+		g_object_set_data (G_OBJECT (config_dialog), title, NULL);
+	}
+	NEEDOPENBGL
+}
+#else
 /**
 @brief accept font selection
 
 @param button
-@param entry
+@param entry config dialog set-widget
 
 @return
 */
 static void _e2_confdlg_font_select_ok_cb (GtkWidget *button, GtkWidget *entry)
 {
 	GtkWidget *dialog = g_object_get_data (G_OBJECT (button), "dialog_widget");
+	NEEDCLOSEBGL
 	const gchar *title = gtk_window_get_title (GTK_WINDOW (dialog));
 	gchar *font_str = gtk_font_selection_dialog_get_font_name
 		(GTK_FONT_SELECTION_DIALOG (dialog));
 	gtk_entry_set_text (GTK_ENTRY (entry), font_str);
+	NEEDOPENBGL
 	g_free (font_str);
 	g_object_set_data (G_OBJECT (config_dialog), title, NULL);
 }
+#endif
 /**
 @brief cancel font selection
 
@@ -1124,7 +1237,9 @@ static void _e2_confdlg_font_select_ok_cb (GtkWidget *button, GtkWidget *entry)
 */
 static void _e2_confdlg_font_select_cancel_cb (GtkWidget *button, GtkWidget *dialog)
 {
+	NEEDCLOSEBGL
 	const gchar *title = gtk_window_get_title (GTK_WINDOW (dialog));
+	NEEDOPENBGL
 	g_object_set_data (G_OBJECT (config_dialog), title, NULL);
 }
 /**
@@ -1137,57 +1252,74 @@ static void _e2_confdlg_font_select_cancel_cb (GtkWidget *button, GtkWidget *dia
 */
 static void _e2_confdlg_font_select_cb (GtkButton *button, E2_OptionSet *set)
 {
+#ifdef USE_GTK3_2
+	GtkFontChooserDialog *dialog;
+#else
+	GtkFontSelectionDialog *dialog;
+#endif
 	gchar *title = g_strdup_printf (_("Choose font: %s"), set->desc);
-	GtkFontSelectionDialog *dialog = g_object_get_data (G_OBJECT (config_dialog), title);
+	dialog = g_object_get_data (G_OBJECT (config_dialog), title);
+	NEEDCLOSEBGL
 	if (dialog != NULL)
 		gtk_window_present (GTK_WINDOW (dialog));
 	else
 	{
-		//CHECKME the normal dialog properties like close on Esc press ?
-		dialog = GTK_FONT_SELECTION_DIALOG (gtk_font_selection_dialog_new (title));
-#ifndef USE_GTK3_0
-		gtk_widget_destroy (
-# ifdef USE_GTK2_14
-			gtk_font_selection_dialog_get_apply_button (dialog)
-# else
-			dialog->apply_button
-# endif
-		);
-#endif
-		g_object_set_data (
-#ifdef USE_GTK2_14
-			G_OBJECT (gtk_font_selection_dialog_get_ok_button (dialog)),
+#ifdef USE_GTK3_2
+		dialog = (GtkFontChooserDialog*) gtk_font_chooser_dialog_new (title, GTK_WINDOW (config_dialog));
+		gtk_font_chooser_set_font (GTK_FONT_CHOOSER (dialog), gtk_entry_get_text (GTK_ENTRY (set->widget)));
+		gtk_font_chooser_set_show_preview_entry (GTK_FONT_CHOOSER (dialog), FALSE);
+		g_signal_connect (G_OBJECT (dialog), "font-activated",
+			G_CALLBACK (_e2_confdlg_font_activate_cb), set->widget);
+		g_signal_connect (G_OBJECT (dialog), "response",
+			G_CALLBACK (_e2_confdlg_font_response_cb), set->widget);
 #else
+		dialog = (GtkFontSelectionDialog*) gtk_font_selection_dialog_new (title);
+# ifndef USE_GTK3_0
+		gtk_widget_destroy (
+#  ifdef USE_GTK2_14
+			gtk_font_selection_dialog_get_apply_button (dialog)
+#  else
+			dialog->apply_button
+#  endif
+		);
+# endif
+		g_object_set_data (
+# ifdef USE_GTK2_14
+			G_OBJECT (gtk_font_selection_dialog_get_ok_button (dialog)),
+# else
 			G_OBJECT (dialog->ok_button),
-#endif
+# endif
 			"dialog_widget", GTK_WIDGET (dialog));
 		gtk_font_selection_dialog_set_font_name (dialog,
 			gtk_entry_get_text (GTK_ENTRY (set->widget)));
-#ifdef E2_COMPOSIT
+# ifdef E2_COMPOSIT
 		e2_window_set_opacity (GTK_WIDGET (dialog), DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
-#endif
+# endif
 		g_signal_connect (
-#ifdef USE_GTK2_14
+# ifdef USE_GTK2_14
 			G_OBJECT (gtk_font_selection_dialog_get_ok_button (dialog)),
-#else
+# else
 			G_OBJECT (dialog->ok_button),
-#endif
+# endif
 			"clicked",
 			G_CALLBACK (_e2_confdlg_font_select_ok_cb), set->widget);
 		g_signal_connect (
-#ifdef USE_GTK2_14
+# ifdef USE_GTK2_14
 			G_OBJECT (gtk_font_selection_dialog_get_cancel_button (dialog)),
-#else
+# else
 			G_OBJECT (dialog->cancel_button),
-#endif
+# endif
 			"clicked",
 			G_CALLBACK (_e2_confdlg_font_select_cancel_cb), dialog);
+#endif //use GTK3_2
 		g_signal_connect (G_OBJECT (dialog), "delete-event",
 			G_CALLBACK (_e2_confdlg_font_select_cancel_cb), NULL);
 		g_object_set_data_full (G_OBJECT (config_dialog), title, dialog,
 			(gpointer) gtk_widget_destroy);
+		//CHECKME the normal dialog properties like close on Esc press ?
 		e2_dialog_show (GTK_WIDGET (dialog), config_dialog, 0, NULL);
 	}
+	NEEDOPENBGL
 	g_free (title);
 }
 /**
@@ -1200,6 +1332,7 @@ static void _e2_confdlg_font_select_cb (GtkButton *button, E2_OptionSet *set)
 */
 static void _e2_confdlg_font_entry_cb (GtkEditable *entry, GtkWidget *label)
 {
+	NEEDCLOSEBGL
 	gchar *font_str = gtk_editable_get_chars (entry, 0, -1);
 	if ((font_str != NULL) && (*font_str != '\0') && (strlen (font_str) > 2))
 	{
@@ -1208,6 +1341,7 @@ static void _e2_confdlg_font_entry_cb (GtkEditable *entry, GtkWidget *label)
 		gtk_label_set_markup (GTK_LABEL (label), label_text);
 		g_free (label_text);
 	}
+	NEEDOPENBGL
 	g_free (font_str);
 }
 
@@ -1217,7 +1351,7 @@ static void _e2_confdlg_font_entry_cb (GtkEditable *entry, GtkWidget *label)
 
 #ifdef E2_RAINBOW
 /**
-@brief response callback for extension-color selection dialog
+@brief response callback for filetype-color selection dialog
 
 @param dialog the dialog where the response was triggered
 @param response the response assigned to the activated button widget
@@ -1228,32 +1362,48 @@ static void _e2_confdlg_font_entry_cb (GtkEditable *entry, GtkWidget *label)
 static void _e2_confdlg_colorpick_response_cb (GtkDialog *dialog, gint response,
 	E2_TreeCfgData *data)
 {
+#ifdef USE_GTK3_4
+	GdkRGBA color;
+	gchar color_str[8];
+#else
 	GdkColor color;
+#endif
 	GtkTreeIter iter;
+
+	NEEDCLOSEBGL
 	switch (response)
 	{
+#ifdef USE_GTK3_4
+		case GTK_RESPONSE_OK:
+			gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (dialog), &color);
+			snprintf (color_str, sizeof(color_str), "#%.2X%.2X%.2X",
+				(gint)(color.red * 255), (gint)(color.green * 255), (gint)(color.blue * 255));
+#else
 		case E2_RESPONSE_APPLY:
 			gtk_color_selection_get_current_color (
-#ifdef USE_GTK2_14
+# ifdef USE_GTK2_14
 				GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection
 					(GTK_COLOR_SELECTION_DIALOG (dialog))),
-#else
+# else
 				GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel),
-#endif
+# endif
 				&color);
 			gchar *color_str = e2_utils_color2str (&color);
-
+#endif
 			gtk_tree_model_get_iter (data->set->ex.tree.model, &iter, data->tpath);
 			gtk_tree_store_set (GTK_TREE_STORE (data->set->ex.tree.model),
 				&iter, 2, color_str, -1);
 			e2_option_tree_flag_change (data->set);
+#ifndef USE_GTK3_4
 			g_free (color_str);
+#endif
 		default:
 			gtk_widget_destroy (GTK_WIDGET (dialog));
 			gtk_tree_path_free (data->tpath);
 			g_free (data);
 			break;
 	}
+	NEEDOPENBGL
 }
 /**
 @brief callback for color-selection button in filetypes page
@@ -1301,9 +1451,11 @@ void e2_confdlg_extcolorpick_cb (GtkButton *button, E2_OptionSet *set)
 		default:
 		break;
 	}
+	NEEDCLOSEBGL
 	if (!valid)
 	{
 		e2_output_print_error (_("Color data are not stored there"), FALSE);
+		NEEDOPENBGL
 		gtk_tree_path_free (path);
 		return;
 	}
@@ -1316,6 +1468,16 @@ void e2_confdlg_extcolorpick_cb (GtkButton *button, E2_OptionSet *set)
 	data->tpath = path;
 	gtk_tree_model_get_iter (set->ex.tree.model, &iter, path);
 	gtk_tree_model_get (set->ex.tree.model, &iter, 2, &current, -1);
+#ifdef USE_GTK3_4
+	GdkRGBA color;
+	if (*current == '\0')
+		gdk_rgba_parse (&color, "black");
+	else if (!gdk_rgba_parse (&color, current))
+	{
+		e2_output_print_error (_("The current color descriptor is not valid"), FALSE);
+		gdk_rgba_parse (&color, "black");
+	}
+#else
 	GdkColor color;
 	if (*current == '\0')
 		gdk_color_parse ("black", &color);
@@ -1324,45 +1486,97 @@ void e2_confdlg_extcolorpick_cb (GtkButton *button, E2_OptionSet *set)
 		e2_output_print_error (_("The current color descriptor is not valid"), FALSE);
 		gdk_color_parse ("black", &color);
 	}
+#endif
 	g_free (current);
 
-	//CHECKME the normal dialog properties like close on Esc press ?
+#ifdef USE_GTK3_4
+	GtkWidget *dialog = gtk_color_chooser_dialog_new (_("Set filetype color"),
+      GTK_WINDOW(config_dialog));
+	GtkColorChooser *gcc = GTK_COLOR_CHOOSER (dialog);
+	gtk_color_chooser_set_use_alpha (gcc, FALSE);
+	gtk_color_chooser_set_rgba (gcc, &color);
+	g_object_set (G_OBJECT(dialog), "show-editor", TRUE, NULL);
+#else
 	GtkWidget *dialog = gtk_color_selection_dialog_new
 		(_("Set filetype color"));
 	gtk_color_selection_set_current_color (
-#ifdef USE_GTK2_14
+# ifdef USE_GTK2_14
 		GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection
 				(GTK_COLOR_SELECTION_DIALOG (dialog))),
-#else
+# else
 		GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel),
-#endif
+# endif
 		&color);
-#ifdef E2_COMPOSIT
+# ifdef E2_COMPOSIT
 	e2_window_set_opacity (dialog, DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
-#endif
+# endif
 	//substitute our own button
 	E2_Button yes_btn;
 	e2_button_derive (&yes_btn, &E2_BUTTON_APPLY, BTN_YES_CONTINUE);
 	GtkWidget *bbox =
-#ifdef USE_GTK2_14
+# ifdef USE_GTK2_14
 		gtk_dialog_get_action_area (GTK_DIALOG (dialog));
-#else
+# else
 		GTK_DIALOG (dialog)->action_area;
-#endif
+# endif
 
 	GList *btns = gtk_container_get_children (GTK_CONTAINER (bbox));
 	gtk_container_remove (GTK_CONTAINER (bbox), btns->next->data);
 	g_list_free (btns);
 	GtkWidget *btn = e2_dialog_add_defined_button (dialog, &yes_btn);
-
+#endif
 	g_signal_connect (G_OBJECT (dialog), "response",
 		G_CALLBACK (_e2_confdlg_colorpick_response_cb), data);
 
 	e2_dialog_setup (dialog, config_dialog);
+#ifndef USE_GTK3_4
 	gtk_widget_show_all (btn);
+#endif
 	gtk_widget_show (dialog);
+	NEEDOPENBGL
 }
 #endif
+
+#ifdef USE_GTK3_4
+/**
+@brief non-filetypes color-selection-dialog response callback
+@param dialog the dialog where the response was generated
+@param response the generated response number
+@param set pointer to option data struct
+@return
+*/
+static void _e2_confdlg_color_response_cb (GtkDialog *dialog, gint response, E2_OptionSet *set)
+{
+	if (GTK_IS_WIDGET (dialog))
+	{
+		NEEDCLOSEBGL
+		switch (response)
+		{
+			case GTK_RESPONSE_OK:
+			{
+				GdkRGBA color;
+				gchar color_str[8];
+
+				gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (dialog), &color);
+				snprintf (color_str, sizeof(color_str), "#%.2X%.2X%.2X",
+					(gint)(color.red * 255), (gint)(color.green * 255), (gint)(color.blue * 255));
+				gtk_entry_set_text (GTK_ENTRY (set->widget), color_str);
+			}
+			//no break here
+			default:
+			{
+				//kill reference to the dialog
+				const gchar *title = gtk_window_get_title (GTK_WINDOW (dialog));
+				g_object_set_data (G_OBJECT (config_dialog), title, NULL);
+				if (response != GTK_RESPONSE_DELETE_EVENT)
+					gtk_widget_destroy ((GtkWidget*)dialog);
+			}
+			break;
+		}
+		NEEDOPENBGL
+	}
+}
+#else //not USE_GTK3_4
 /**
 @brief
 
@@ -1373,19 +1587,23 @@ void e2_confdlg_extcolorpick_cb (GtkButton *button, E2_OptionSet *set)
 */
 static void _e2_confdlg_color_select_ok_cb (GtkWidget *button, GtkWidget *entry)
 {
-	GtkWidget *dialog = gtk_widget_get_toplevel (button);
 	GdkColor color;
+	NEEDCLOSEBGL
+	GtkWidget *dialog = gtk_widget_get_toplevel (button);
 	gtk_color_selection_get_current_color (
-#ifdef USE_GTK2_14
+# ifdef USE_GTK2_14
 		GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (GTK_COLOR_SELECTION_DIALOG (dialog))),
-#else
+# else
 		GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel),
-#endif
+# endif
 		&color);
 	gchar *color_str = e2_utils_color2str (&color);
+
 	gtk_entry_set_text (GTK_ENTRY (entry), color_str);
 	g_free (color_str);
+
 	const gchar *title = gtk_window_get_title (GTK_WINDOW (dialog));
+	NEEDOPENBGL
 	g_object_set_data (G_OBJECT (config_dialog), title, NULL);
 }
 /**
@@ -1398,9 +1616,12 @@ static void _e2_confdlg_color_select_ok_cb (GtkWidget *button, GtkWidget *entry)
 */
 static void _e2_confdlg_color_select_cancel_cb (GtkWidget *button, GtkWidget *dialog)
 {
+	NEEDCLOSEBGL
 	const gchar *title = gtk_window_get_title (GTK_WINDOW (dialog));
+	NEEDOPENBGL
 	g_object_set_data (G_OBJECT (config_dialog), title, NULL);
 }
+#endif //def USE_GTK3_4
 /**
 @brief callback for color-selection button in non-filetypes page
 
@@ -1412,57 +1633,73 @@ static void _e2_confdlg_color_select_cancel_cb (GtkWidget *button, GtkWidget *di
 static void _e2_confdlg_color_select_cb (GtkButton *button, E2_OptionSet *set)
 {
 	gchar *title = g_strdup_printf (_("Choose color: %s"), set->desc);
-	GtkColorSelectionDialog *dialog = g_object_get_data (G_OBJECT (config_dialog), title);
+	GtkWidget *dialog = g_object_get_data (G_OBJECT (config_dialog), title);
+	NEEDCLOSEBGL
 	if (dialog != NULL)
 		gtk_window_present (GTK_WINDOW (dialog));
 	else
 	{
 		//CHECKME close on Esc press ?
-		dialog = GTK_COLOR_SELECTION_DIALOG (gtk_color_selection_dialog_new (title));
-#ifdef USE_GTK2_20
+#ifdef USE_GTK3_4
+		GdkRGBA color;
+		dialog = gtk_color_chooser_dialog_new (title, GTK_WINDOW(config_dialog));
+		GtkColorChooser *gcc = GTK_COLOR_CHOOSER (dialog);
+		gtk_color_chooser_set_use_alpha (gcc, FALSE);
+		gchar *color_str = gtk_editable_get_chars (GTK_EDITABLE (set->widget), 0, -1);
+		gdk_rgba_parse (&color, color_str);
+		g_free (color_str);
+		gtk_color_chooser_set_rgba (gcc, &color);
+		g_object_set (G_OBJECT (dialog), "show-editor", TRUE, NULL);
+		g_signal_connect (G_OBJECT (dialog), "response",
+			G_CALLBACK(_e2_confdlg_color_response_cb), set);
+#else
+		GdkColor color;
+		dialog = gtk_color_selection_dialog_new (title);
+# ifdef USE_GTK2_20
 		GtkWidget *btn;
 		g_object_get (G_OBJECT (dialog), "help-button", &btn, NULL);
 		gtk_widget_destroy (btn);
-#else
-		gtk_widget_destroy (dialog->help_button);
-#endif
-
-		GdkColor color;
+# else
+		gtk_widget_destroy ((GTK_COLOR_SELECTION_DIALOG(dialog))->help_button);
+# endif
 		gchar *color_str = gtk_editable_get_chars (GTK_EDITABLE (set->widget), 0, -1);
 		gdk_color_parse (color_str, &color);
-		gtk_color_selection_set_current_color (
-#ifdef USE_GTK2_14
-			GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (dialog)),
-#else
-			GTK_COLOR_SELECTION (dialog->colorsel),
-#endif
-			&color);
 		g_free (color_str);
+		gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (
+# ifdef USE_GTK2_14
+			gtk_color_selection_dialog_get_color_selection (GTK_COLOR_SELECTION_DIALOG(dialog))
+# else
+			(GTK_COLOR_SELECTION_DIALOG(dialog))->colorsel
+# endif
+			), &color);
 
-#ifdef USE_GTK2_20
+# ifdef USE_GTK2_20
 		g_object_get (G_OBJECT (dialog), "ok-button", &btn, NULL);
 		g_signal_connect (G_OBJECT (btn),
-#else
-		g_signal_connect (G_OBJECT (dialog->ok_button),
-#endif
+# else
+		g_signal_connect (G_OBJECT ((GTK_COLOR_SELECTION_DIALOG(dialog))->ok_button),
+# endif
 			"clicked",
 			G_CALLBACK (_e2_confdlg_color_select_ok_cb), set->widget);
 
-#ifdef USE_GTK2_20
+# ifdef USE_GTK2_20
 		g_object_get (G_OBJECT (dialog), "cancel-button", &btn, NULL);
 		g_signal_connect (G_OBJECT (btn),
-#else
-		g_signal_connect (G_OBJECT (dialog->cancel_button),
-#endif
+# else
+		g_signal_connect (G_OBJECT ((GTK_COLOR_SELECTION_DIALOG(dialog))->cancel_button),
+# endif
 			"clicked",
 			G_CALLBACK (_e2_confdlg_color_select_cancel_cb), dialog);
 		g_signal_connect (G_OBJECT (dialog),
 			"delete-event", G_CALLBACK (_e2_confdlg_color_select_cancel_cb), dialog);
+#endif //def USE_GTK3_4
+		//make the dialog discoverable
 		g_object_set_data_full (G_OBJECT (config_dialog), title, dialog,
 			(void *) gtk_widget_destroy);
 		gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
 		e2_dialog_show (GTK_WIDGET (dialog), config_dialog, 0, NULL);
 	}
+	NEEDOPENBGL
 	g_free (title);
 }
 /**
@@ -1475,6 +1712,7 @@ static void _e2_confdlg_color_select_cb (GtkButton *button, E2_OptionSet *set)
 */
 static void _e2_confdlg_color_entry_cb (GtkEditable *entry, GtkWidget *label)
 {
+	NEEDCLOSEBGL
 	gchar *color_str = gtk_editable_get_chars (entry, 0, -1);
 	GdkColor color;
 	if (gdk_color_parse (color_str, &color))
@@ -1486,6 +1724,7 @@ static void _e2_confdlg_color_entry_cb (GtkEditable *entry, GtkWidget *label)
 		gtk_label_set_markup (GTK_LABEL (label), label_text);
 		g_free (label_text);
 	}
+	NEEDOPENBGL
 	g_free (color_str);
 }
 /**
@@ -1496,13 +1735,14 @@ static void _e2_confdlg_color_entry_cb (GtkEditable *entry, GtkWidget *label)
 
 @return TRUE if there is a selected category
 */
-static gboolean _e2_confdlg_category_selected_cb
-	(GtkTreeSelection *selection, E2_ConfigDialogRuntime *rt)
+static gboolean _e2_confdlg_category_selected_cb (
+	GtkTreeSelection *selection, E2_ConfigDialogRuntime *rt)
 {
 	printd (DEBUG, "callback: category selected");
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 
+	NEEDCLOSEBGL
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))
 	{
 		g_free (page_last_name);
@@ -1511,8 +1751,10 @@ static gboolean _e2_confdlg_category_selected_cb
 
 		gtk_notebook_set_current_page (rt->notebook, page);
 		gtk_widget_grab_focus (rt->treeview);
+		NEEDOPENBGL
 		return TRUE;
 	}
+	NEEDOPENBGL
 	return FALSE;
 }
 /**
@@ -1575,7 +1817,11 @@ static GtkWidget *_e2_confdlg_add_page (GtkTreeIter *parent_iter,
 	{
 		GtkWidget *sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC,
 			GTK_POLICY_AUTOMATIC, GTK_SHADOW_NONE);
+#ifdef USE_GTK3_0
+		vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, E2_PADDING);
+#else
 		vbox = gtk_vbox_new (FALSE, E2_PADDING);
+#endif
 		e2_widget_sw_add_with_viewport (sw, vbox);
 		gtk_widget_show_all (sw);
 		gtk_notebook_append_page (rt->notebook, sw, NULL);
@@ -1584,7 +1830,11 @@ static GtkWidget *_e2_confdlg_add_page (GtkTreeIter *parent_iter,
 	{
 #endif
 		//page vbox containing the headline and config widgets
+#ifdef USE_GTK3_0
+		vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, E2_PADDING);
+#else
 		vbox = gtk_vbox_new (FALSE, E2_PADDING);
+#endif
 		gtk_widget_show (vbox);
 		gtk_notebook_append_page (rt->notebook, vbox, NULL);
 #ifdef NEWSCROLL
@@ -1630,7 +1880,11 @@ static GtkWidget *_e2_confdlg_get_page (E2_OptionSet *set, E2_ConfigDialogRuntim
 			gchar *frame_title = e2_utils_str_stretch (frame + 1);
 			GtkWidget *frame_widget = gtk_frame_new (frame_title);
 			g_free (frame_title);
+#ifdef USE_GTK3_0
+			GtkWidget *vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 			GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
+#endif
 			gtk_container_add (GTK_CONTAINER (frame_widget), vbox);
 			gtk_container_set_border_width (GTK_CONTAINER (vbox), E2_PADDING_SMALL);
 			gtk_box_pack_start (GTK_BOX (frame_box), frame_widget, FALSE, FALSE, 0);
@@ -1711,7 +1965,11 @@ void e2_config_dialog_create (gchar *page)
 	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 
 	//the main hpane - contains the categories treeview and the frame
+#ifdef USE_GTK3_2
+	GtkWidget *hpane = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+#else
 	GtkWidget *hpane = gtk_hpaned_new ();
+#endif
 	gtk_box_pack_start (GTK_BOX (dialog_vbox), hpane, TRUE, TRUE, 0);
 
 	//scrolled window for the categories treeview
@@ -1828,7 +2086,11 @@ void e2_config_dialog_create (gchar *page)
 				//gtk_size_group_add_widget (size_group, button);
 				break;
 			case E2_OPTION_TYPE_STR:
+#ifdef USE_GTK3_0
+				hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, E2_PADDING);
+#else
 				hbox = gtk_hbox_new (FALSE, E2_PADDING);
+#endif
 				gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, E2_PADDING);
 				label = e2_widget_add_mid_label (hbox, set->desc, 0.0, FALSE, 0);
 				gtk_size_group_add_widget (size_group, label);
@@ -1845,7 +2107,11 @@ void e2_config_dialog_create (gchar *page)
 				break;
 			case E2_OPTION_TYPE_FONT:
 				//hbox for option label, font entry and font dialog button
+#ifdef USE_GTK3_0
+				hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, E2_PADDING_SMALL);
+#else
 				hbox = gtk_hbox_new (FALSE, E2_PADDING_SMALL);
+#endif
 				gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, E2_PADDING_SMALL);
 				//option label
 				label_text = g_strconcat (set->desc, ": ", NULL);
@@ -1859,20 +2125,26 @@ void e2_config_dialog_create (gchar *page)
 #endif
 				gtk_entry_set_text (GTK_ENTRY (set->widget), set->sval);
 				//button for font select dialog
-				e2_button_add (hbox, FALSE, 0, _("change"), GTK_STOCK_SELECT_FONT,
+				e2_button_add (hbox, FALSE, 0, _("change"), STOCK_NAME_SELECT_FONT,
 					_("Click to open a font select dialog"), _e2_confdlg_font_select_cb, set);
 				//label for font example
 				label = e2_widget_add_mid_label (box, "", 0.0, FALSE, 0);
 				gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
 				//set label contents
+				NEEDOPENBGL
 				_e2_confdlg_font_entry_cb (GTK_EDITABLE (set->widget), label);
+				NEEDCLOSEBGL
 				//react on changes to the font name
 				g_signal_connect (GTK_EDITABLE (set->widget), "changed",
 					G_CALLBACK (_e2_confdlg_font_entry_cb), label);
 				break;
 			case E2_OPTION_TYPE_COLOR:
 				//hbox for option label, color entry and color dialog button
+#ifdef USE_GTK3_0
+				hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, E2_PADDING_SMALL);
+#else
 				hbox = gtk_hbox_new (FALSE, E2_PADDING_SMALL);
+#endif
 				gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, E2_PADDING_SMALL);
 				//option label
 				label_text = g_strconcat (set->desc, ": ", NULL);
@@ -1886,14 +2158,16 @@ void e2_config_dialog_create (gchar *page)
 #endif
 				gtk_entry_set_text (GTK_ENTRY (set->widget), set->sval);
 				//button for color select dialog
-				e2_button_add (hbox, FALSE, 0, _("change"), GTK_STOCK_SELECT_COLOR,
+				e2_button_add (hbox, FALSE, 0, _("change"), STOCK_NAME_SELECT_COLOR,
 					_("Click to open a color selection dialog"),
 					_e2_confdlg_color_select_cb, set);
 				//label for color example
 				label = e2_widget_add_mid_label (box, "", 0.0, FALSE, 0);
 				gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
 				//set label contents
+				NEEDOPENBGL
 				_e2_confdlg_color_entry_cb (GTK_EDITABLE (set->widget), label);
+				NEEDCLOSEBGL
 				//react on changes to the color name/color hex value
 				g_signal_connect (GTK_EDITABLE (set->widget), "changed",
 					G_CALLBACK (_e2_confdlg_color_entry_cb), label);
@@ -2000,20 +2274,20 @@ void e2_config_dialog_create (gchar *page)
 		G_CALLBACK (_e2_confdlg_show_cb), sw);
 #endif
 	e2_dialog_add_custom_button_full (config_dialog, FALSE,
-		E2_RESPONSE_USER1, _("De_fault"), GTK_STOCK_CLEAR,
+		E2_RESPONSE_USER1, _("De_fault"), STOCK_NAME_CLEAR,
 		_("Revert all options to their default settings"),
 		NULL, NULL);
 	if (e2_option_bool_get ("advanced-config"))
 	{
 		e2_dialog_add_custom_button_full (config_dialog, FALSE,
-			E2_RESPONSE_USER2, _("_Basic"), GTK_STOCK_REDO,
+			E2_RESPONSE_USER2, _("_Basic"), STOCK_NAME_REDO,
 			_("Display only the basic configuration options"),
 			NULL, NULL);
 	}
 	else
 	{
 		e2_dialog_add_custom_button_full (config_dialog, FALSE,
-			E2_RESPONSE_USER2, _("Ad_vanced"), GTK_STOCK_REDO,
+			E2_RESPONSE_USER2, _("Ad_vanced"), STOCK_NAME_REDO,
 			_("Display all configuration options"),
 			NULL, NULL);
 	}
