@@ -1,6 +1,6 @@
-/* $Id: e2_date_filter_dialog.c 2373 2011-06-06 08:04:35Z tpgww $
+/* $Id: e2_date_filter_dialog.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -163,9 +163,11 @@ time_t e2_date_filter_dialog_get_time (const gchar *datestring,
 {
 	gulong handler = g_signal_handler_find (menu_item, G_SIGNAL_MATCH_FUNC,
 		0, 0, NULL, e2_date_filter_dialog_create_cb, NULL);
+	NEEDCLOSEBGL
 	g_signal_handler_block (menu_item, handler);
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), !itemstate);
 	g_signal_handler_unblock (menu_item, handler);
+	NEEDOPENBGL
 }
 */
 static void _e2_date_filter_dialog_ok (E2_DtFltDlgData *data)
@@ -227,9 +229,9 @@ static void _e2_date_filter_dialog_ok (E2_DtFltDlgData *data)
 #ifdef E2_STATUS_DEMAND
 	    if (data->view == curr_view)
 		{
-BGL?		gdk_threads_leave ();
+BGL?		OPENBGL
 			e2_window_update_status_bar (NULL);
-			gdk_threads_enter ();
+			CLOSEBGL
 		}
 #endif
 	}
@@ -264,6 +266,8 @@ static void _e2_date_filter_dialog_destroy_cb (
 static void _e2_date_filter_dialog_response_cb (GtkDialog *dialog,
 	gint response, E2_DtFltDlgData *rt)
 {
+	NEEDCLOSEBGL
+
 	switch (response)
 	{
 		case E2_RESPONSE_APPLY:
@@ -280,9 +284,9 @@ static void _e2_date_filter_dialog_response_cb (GtkDialog *dialog,
 #ifdef E2_STATUS_DEMAND
 		    if (rt->view == curr_view)
             {
-			    gdk_threads_leave ();
+			    OPENBGL
 			    e2_window_update_status_bar (NULL);
-			    gdk_threads_enter ();
+			    CLOSEBGL
             }
 #endif
 			break;
@@ -293,6 +297,9 @@ static void _e2_date_filter_dialog_response_cb (GtkDialog *dialog,
 	g_signal_handlers_disconnect_by_func ((gpointer)rt->view->treeview,
 		_e2_date_filter_dialog_destroy_cb, rt);
 	gtk_widget_destroy (rt->dialog);
+	
+	NEEDOPENBGL
+
 	DEALLOCATE (E2_DtFltDlgData, rt);
 }
 /**
@@ -335,6 +342,8 @@ void e2_date_filter_dialog_create_cb (GtkCheckMenuItem *item, ViewInfo *view)
 //	rt->menu_item = item;
 //	rt->itemstate = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM (item));
 
+	NEEDCLOSEBGL
+
 	rt->dialog = e2_dialog_create (NULL, _("Display only the items:"),
 		_("date filter"),
 		(ResponseFunc)_e2_date_filter_dialog_response_cb, rt);
@@ -408,4 +417,5 @@ void e2_date_filter_dialog_create_cb (GtkCheckMenuItem *item, ViewInfo *view)
 
 	e2_dialog_show (rt->dialog, app.main_window, 0,
 		&no_btn, &E2_BUTTON_APPLY, NULL);
+	NEEDOPENBGL
 }
