@@ -1,6 +1,6 @@
-/* $Id: e2_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_dialog.c 3003 2014-01-17 23:17:53Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2014 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -30,13 +30,14 @@ ToDo - describe how dialogs work
 */
 
 #include "emelfm2.h"
+#include <string.h>
 #include "e2_dialog.h"
 #include "e2_option.h"
 #include "e2_filelist.h"
 #ifdef E2_MOUSECUSTOM
 # include "e2_mousebinding.h"
 #endif
-#include <string.h>
+#include "e2_icons.h"
 
 /**
 @brief remove dialog's bindings when it's being closed
@@ -68,8 +69,10 @@ Turns on automatic scrollbars for @a scrolled
 */
 void e2_dialog_show_cb (GtkWidget *dialog, GtkScrolledWindow *scrolled)
 {
+	NEEDCLOSEBGL
 	gtk_scrolled_window_set_policy (scrolled, GTK_POLICY_AUTOMATIC,
 		GTK_POLICY_AUTOMATIC);
+	NEEDOPENBGL
 }
 /**
 @brief "show" callback for a dialog with notebook
@@ -102,7 +105,9 @@ This is convenient because the buttons in the dialog are typically not focused
 static void _e2_dialog_activated_cb (GtkEntry *entry, GtkWidget *dialog)
 {
 	//mimic yes or apply button
+//	NEEDOPENBGL
 	g_signal_emit_by_name (G_OBJECT(dialog), "response", GTK_RESPONSE_YES);
+//	NEEDCLOSEBGL
 }
 /**
 @brief default callback to handle responses from @a dialog
@@ -118,14 +123,20 @@ static void _e2_dialog_response_cb (GtkDialog *dialog, gint response,
 {
 	gint negresponse = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog),
 		"e2-negative-response"));
-	if (response == negresponse)	// || response == GTK_RESPONSE_DELETE_EVENT)
+	if (response == negresponse) // || response == GTK_RESPONSE_DELETE_EVENT)
+	{
+//		NEEDOPENBGL
 		e2_dialog_cancel_cb (NULL, GTK_WIDGET (dialog));
+//		NEEDCLOSEBGL
+	}
 }
 
 static void _e2_dialog_wait_response2_cb (GtkDialog *dialog, gint response, E2_Wait *w)
 {
 	w->choice = response;
+	NEEDCLOSEBGL
 	e2_main_loop_quit (w->loopdata);
+	NEEDOPENBGL
 }
 /**
 @brief blocked-dialog response callback
@@ -139,7 +150,9 @@ static void _e2_dialog_wait_response_cb (GtkDialog *dialog, gint response, E2_Wa
 {
 	if (w->choice != IGNORE)
 		w->choice = e2_dialog_response_decode (response);
+	NEEDCLOSEBGL
 	e2_main_loop_quit (w->loopdata);
+	NEEDOPENBGL
 }
 
   /******************/
@@ -148,16 +161,16 @@ static void _e2_dialog_wait_response_cb (GtkDialog *dialog, gint response, E2_Wa
 
 /**
 @brief initiate a mainloop pending the user's choice in @a dialog
-This is instead of nested gtk_main();
+This is a replacement for a nested gtk_main(), to allow interruption
 @param dialog the dialog where the response will be generated
-@param locked TRUE if we arrive here with BGL closed
+@param lockednow TRUE if we arrive here with BGL closed
 @param maincontext TRUE to create mainloop in default context
 @param multi TRUE when this dialog applies to multiple selected items
 @param getresult TRUE to decode the original response from @a dialog, FALSE just returns a default value (suitable to cancel any operation)
 @return enumerator corresponding to the user's choice
 */
 DialogButtons e2_dialog_wait (GtkWidget *dialog,
-	gboolean locked, gboolean maincontext, gboolean multi, gboolean getresult)
+	gboolean lockednow, gboolean maincontext, gboolean multi, gboolean getresult)
 {
 	E2_Wait wdata;
 
@@ -169,21 +182,56 @@ DialogButtons e2_dialog_wait (GtkWidget *dialog,
 			wdata.choice = (multi) ? NO_TO_ALL : CANCEL;
 		else
 			wdata.choice = IGNORE;
-		//now we can assign the "real" response cb
+		//replace the default response handler, which fails to clean up after cancellation
+		g_signal_handlers_disconnect_matched ((gpointer)dialog,
+			G_SIGNAL_MATCH_FUNC, 0, 0, NULL, _e2_dialog_response_cb, NULL);
 		g_signal_connect (G_OBJECT (dialog), "response",
 			G_CALLBACK (_e2_dialog_wait_response_cb), &wdata);
 
-		//allow for aborting this operation (FIXME can't be conditional)
-//		pthread_cleanup_push ((gpointer)gdk_threads_leave, NULL);
-		if (!locked)
-			gdk_threads_enter ();
-		gtk_widget_queue_draw (dialog); //FIXME doesn't always work
-		gdk_flush ();
-		WAIT_FOR_EVENTS
-		e2_main_loop_run (wdata.loopdata);
-		if (!locked)
-			gdk_threads_leave ();
-//		pthread_cleanup_pop (0);
+#if 1
+		GET_EVENTS_CONTEXT
+#endif
+
+		if (lockednow)
+		{
+#if 1
+			gtk_widget_queue_draw (dialog); //FIXME doesn't always work
+			EXTRA_WAIT_FOR_EVENTS
+#else
+			OPENBGL
+			gtk_widget_queue_draw (dialog); //refresh dialog at idle-time
+			EXTRA_WAIT_FOR_EVENTS_UNLOCKED
+			CLOSEBGL
+#endif
+//			gdk_flush (); //also in e2_main_loop_run();
+			e2_main_loop_run (wdata.loopdata);
+		}
+		else
+		{
+			//allow for aborting the process using this loop
+			//this is a macro, it can't be conditional by itself
+			pthread_cleanup_push ((gpointer)OPENBGL_NAME,
+#ifdef DEBUG_MESSAGES
+				NULL
+#else
+				&display_mutex
+#endif
+			);
+#if 1
+			CLOSEBGL
+			gtk_widget_queue_draw (dialog);
+			EXTRA_WAIT_FOR_EVENTS
+#else
+			gtk_widget_queue_draw (dialog); //refresh view at idle-time
+			EXTRA_WAIT_FOR_EVENTS_UNLOCKED
+			CLOSEBGL
+#endif
+//			gdk_flush (); //also in e2_main_loop_run();
+			e2_main_loop_run (wdata.loopdata);
+			OPENBGL
+			pthread_cleanup_pop (0); //another macro
+		}
+
 		//prevent fatal multiple-callbacks if this func is repeatedly called
 		//on the same dialog e.g. inside a loop
 		if (GTK_IS_DIALOG(dialog))
@@ -228,9 +276,11 @@ DialogButtons e2_dialog_response_decode (gint response)
 */
 gboolean e2_dialog_cancel_cb (GtkWidget *widget, GtkWidget *dialog)
 {
+	NEEDCLOSEBGL
 	gtk_widget_destroy (dialog);
 	gtk_widget_set_sensitive (app.main_window, TRUE);
 //	gtk_widget_grab_focus (curr_view->treeview);
+	NEEDOPENBGL
 	return TRUE;
 }
 /* *
@@ -302,7 +352,7 @@ void e2_dialog_setup (GtkWidget *dialog, GtkWidget *parent)
 	if (parent != NULL)
 		gtk_window_set_transient_for (thiswindow, GTK_WINDOW (parent));
 	gtk_window_set_destroy_with_parent (thiswindow, TRUE);
-//	gtk_window_set_role (thiswindow, "dialog");
+	gtk_window_set_role (thiswindow, "dialog");
 	gtk_window_set_position (thiswindow, e2_option_int_get ("dialog-position"));
 	gtk_window_set_resizable (thiswindow, TRUE);
 }
@@ -333,10 +383,10 @@ DialogButtons e2_dialog_show (GtkWidget *dialog, GtkWidget *parent,
 		va_end (args);
 	}
 	if (flags & E2_DIALOG_CLOSELOCK) //or else BGL is assumed already closed
-		gdk_threads_enter ();
+		CLOSEBGL
 	e2_dialog_setup (dialog, parent);
 	if (flags & E2_DIALOG_CLOSELOCK)
-		gdk_threads_leave ();
+		OPENBGL
 	return e2_dialog_run (dialog, parent, flags);
 }
 /**
@@ -357,7 +407,7 @@ DialogButtons e2_dialog_run (GtkWidget *dialog, GtkWidget *parent, E2_DialogFlag
 	gboolean lock = flags & E2_DIALOG_CLOSELOCK;
 
 	if (lock) //or else BGL is assumed already closed
-		gdk_threads_enter ();
+		CLOSEBGL
 
 	if (flags & E2_DIALOG_DONT_SHOW_ALL)
 		gtk_widget_show (dialog);	//this duplicates effect in gtk_dialog_run()
@@ -394,7 +444,7 @@ DialogButtons e2_dialog_run (GtkWidget *dialog, GtkWidget *parent, E2_DialogFlag
 	}
 
 	if (lock)
-		gdk_threads_leave ();
+		OPENBGL
 
 	return ret;
 }
@@ -484,14 +534,18 @@ GtkWidget *e2_dialog_create (const gchar *stock, const gchar *label_text,
 
 	if (img != NULL || label_text != NULL)
 	{
+#ifdef USE_GTK3_0
+		GtkWidget *hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, E2_PADDING);
+#else
 		GtkWidget *hbox = gtk_hbox_new (FALSE, E2_PADDING);
+#endif
 		gtk_container_set_border_width (GTK_CONTAINER (hbox), E2_PADDING);
 		g_object_set_data (G_OBJECT (dialog), "e2-dialog-hbox", hbox);
 		if (img != NULL)
 		{
 //			gtk_misc_set_alignment (GTK_MISC (img), 0.5, 0.0);	//no effect
 //			gtk_box_pack_start (GTK_BOX (hbox), img, FALSE, FALSE, E2_PADDING);
-			gint isize = e2_utils_get_icon_size (GTK_ICON_SIZE_DIALOG);
+			gint isize = e2_icons_get_pixsize (GTK_ICON_SIZE_DIALOG);
 			GtkWidget *align = gtk_alignment_new (1.0, 0.0, 0.0, 0.0);
 			gtk_widget_set_size_request (align, isize*1.2, isize);
 			gtk_container_add (GTK_CONTAINER (align), img);
@@ -583,7 +637,11 @@ GtkWidget *e2_dialog_add_sw (GtkWidget *dialog)
 	g_signal_connect_after (G_OBJECT (GTK_DIALOG (dialog)), "show",
 		G_CALLBACK (e2_dialog_show_cb), scrolled);
 	GtkWidget *vbox = e2_widget_get_box (TRUE, FALSE, 0);
+#ifdef USE_GTK3_8
+	gtk_container_add (GTK_CONTAINER (scrolled), vbox);
+#else
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled), vbox);
+#endif
 	gtk_viewport_set_shadow_type (
 #ifdef USE_GTK2_14
 		GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (scrolled))),
@@ -626,13 +684,13 @@ static const gchar *_e2_dialog_button_image_check (const gchar *stock)
 Any custom tooltip will be cleared (and maybe freed) after button is added
 @param dialog the widget to hold the button
 @param button_data ptr to data struct holding dialog-button parameters
-@return the created button widget
+@return the (unshown) new button widget
 */
 GtkWidget *e2_dialog_add_defined_button (GtkWidget *dialog, E2_Button *button_data)
 {
 	gchar *tip = (button_data->showflags & E2_BTN_TIPPED) ? button_data->tip : NULL;
 	GtkWidget *button = e2_button_get (button_data->label,
-		_e2_dialog_button_image_check (button_data->stock), tip, NULL, NULL);
+		_e2_dialog_button_image_check (button_data->name), tip, NULL, NULL);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, button_data->response);
 	if (button_data->showflags & E2_BTN_DEFAULT)
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), button_data->response);
@@ -645,7 +703,7 @@ GtkWidget *e2_dialog_add_defined_button (GtkWidget *dialog, E2_Button *button_da
 /**
 @brief pack a simple custom button at the end of the 'action-area' of @a dialog
 @param dialog the widget to hold the button
-@param stock string with icon name, or NULL
+@param stock string with icon (file)name, or NULL
 @param label string with button label
 @param response the dialog response to be assigned to the button
 @return the created button widget
@@ -653,8 +711,9 @@ GtkWidget *e2_dialog_add_defined_button (GtkWidget *dialog, E2_Button *button_da
 GtkWidget *e2_dialog_add_simple_button
 	(GtkWidget *dialog, const gchar *stock, const gchar *label, gint response)
 {
-	GtkWidget *button = e2_button_get (label,
-		_e2_dialog_button_image_check (stock), NULL, NULL, NULL);
+	GtkWidget *button = e2_button_get_full (label,
+		_e2_dialog_button_image_check (stock), GTK_ICON_SIZE_BUTTON,
+		NULL, NULL, NULL, E2_BUTTON_CAN_DEFAULT | E2_BUTTON_CAN_FOCUS);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, response);
 	return button;
 }
@@ -676,7 +735,7 @@ GtkWidget *e2_dialog_add_custom_button
 		tip = button->tip;
 	return e2_dialog_add_custom_button_full
 		(dialog, is_default, button->response, button->label,
-		_e2_dialog_button_image_check (button->stock), tip,
+		_e2_dialog_button_image_check (button->name), tip,
 		callback, data);
 }
 /**
@@ -815,11 +874,13 @@ void e2_dialog_set_cursor (GtkWidget *dialog, GdkCursorType type)
 */
 static void _e2_dialog_toggglehidden_cb (GtkButton *button, GtkFileChooser *chooser)
 {
+	NEEDCLOSEBGL
 	//toggle hidden-items display & button-image
 	gboolean nowhidden = gtk_file_chooser_get_show_hidden (chooser);
 	gtk_file_chooser_set_show_hidden (chooser, ! nowhidden);
 	e2_button_set_image (GTK_WIDGET (button), (nowhidden) ?
 		"hidden_show"E2ICONTB : "hidden_noshow"E2ICONTB);
+	NEEDOPENBGL
 }
 /**
 @brief setup a file-chooser dialog
@@ -833,72 +894,67 @@ Expects BGL closed, probably
 @param showhidden TRUE to start with hidden items displayed
 @param multi TRUE if selection of multiple items is allowed
 @param defresponse the default response for the dialog
+@param first_button stock-item name of first button to include
+@param varargs NULL-terminated series of args: response, stockname, response ....
 
 @return the filechooser widget that's part of the dialog
 */
 void e2_dialog_setup_chooser (GtkWidget *dialog, const gchar *title,
 	const gchar *startpath, GtkFileChooserAction action, gboolean showhidden,
-	gboolean multi, gint defresponse)
+	gboolean multi, gint defresponse, const gchar *first_button, ...)
 {
-/* FIXME make something like this work
-    va_list argpointer;
-    va_start (argpointer, defresponse);
-	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL, GTK_WINDOW (parent),
-		action, argpointer, NULL);
-	va_end (argpointer);
-*/
-	//FIXME vfs
+	va_list btnargs;
+	const gchar *name;
 	gchar *local, *s;
 	GtkFileChooser *chooser = GTK_FILE_CHOOSER (dialog);
 
-/*	GtkWidget *dialog = e2_dialog_create (NULL, NULL, title,
-		_e2_some_response_cb, somedata);
-	GtkWidget *dialog_vbox =
-#ifdef USE_GTK2_14
-		gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-#else
-		GTK_DIALOG (dialog)->vbox;
-#endif
-	gtk_box_pack_start (GTK_BOX (dialog_vbox), wchooser, TRUE, TRUE, 0);
-	e2_dialog_setup (dialog, parent);
-*/
-	e2_window_set_title (dialog, title);
-#ifdef E2_COMPOSIT
-	e2_window_set_opacity (dialog, DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
-#endif
-
-/*	//hide the dialog's standard 'open' button
-	GtkContainer *bbox = GTK_CONTAINER (
-#ifdef USE_GTK2_14
-		gtk_dialog_get_action_area (GTK_DIALOG (dialog)));
-#else
-		GTK_DIALOG (dialog)->action_area);
-#endif
-	GList* children = gtk_container_get_children (bbox);
-	gtk_widget_hide ((GtkWidget *)children->data);
-	g_list_free (children);
-*/
-	GtkWidget *hidebtn = e2_dialog_add_custom_button_full
+	e2_dialog_add_custom_button_full
 		(dialog, FALSE, E2_RESPONSE_USER1, _("_Hidden"),
 		(showhidden) ? "hidden_noshow"E2ICONTB : "hidden_show"E2ICONTB,
 		_("Toggle display of hidden items"),
 		_e2_dialog_toggglehidden_cb, chooser);
-	GtkWidget *bbox =
-#ifdef USE_GTK2_14
-		gtk_dialog_get_action_area (GTK_DIALOG (dialog));
-#else
-		GTK_DIALOG (dialog)->action_area;
+
+	if (first_button != NULL)
+	{
+		name = first_button;
+		va_start (btnargs, first_button);
+		while (name)
+		{
+			GtkWidget *btn;
+			const gchar *label = e2_icons_get_stock_label (name);
+			gint response = va_arg (btnargs, gint);
+			if (label != NULL)
+				btn = e2_dialog_add_simple_button (dialog, name, label, response);
+			else
+			{
+				//const gchar *stock = STOCK_NAME_MISSING_IMAGE; TODO
+				if (strncmp (name, "gtk-", 4) == 0)
+					name += 4;
+				btn = e2_dialog_add_simple_button (dialog, /*stock*/NULL, name, response);
+			}
+			gtk_widget_show_all (btn);
+			name = va_arg (btnargs, const gchar *);
+		}
+		va_end (btnargs);
+
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), defresponse);
+	}
+	//the dialog may not have a CANCEL button, but the response will be decoded anyway
+	e2_dialog_set_negative_response (dialog, GTK_RESPONSE_CANCEL);
+
+	e2_window_set_title (dialog, title);
+#ifdef E2_COMPOSIT
+	e2_window_set_opacity (dialog, DIALOG_OPACITY_LEVEL);	//constant opacity for dialogs
 #endif
-	gtk_box_reorder_child (GTK_BOX (bbox), hidebtn, 0);
 
 #ifdef USE_GTK2_8
 	if ((action == GTK_FILE_CHOOSER_ACTION_SAVE
 		|| action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER
 		|| action == GTK_FILE_CHOOSER_ACTION_OPEN)
 		&& e2_option_bool_get ("confirm-overwrite"))
-		gtk_file_chooser_set_do_overwrite_confirmation (chooser, TRUE);
+			gtk_file_chooser_set_do_overwrite_confirmation (chooser, TRUE);
 #endif
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), defresponse);
+
 	gtk_file_chooser_set_select_multiple (chooser, multi);
 	if (showhidden)
 		gtk_file_chooser_set_show_hidden (chooser, TRUE);
@@ -943,9 +999,6 @@ void e2_dialog_setup_chooser (GtkWidget *dialog, const gchar *title,
 //	g_signal_connect (G_OBJECT (dialog), "map", //"show",
 //		G_CALLBACK (_e2_dialog_show_cb), NULL);
 
-	//the dialog may not have a CANCEL button, but the response will be decoded anyway
-	e2_dialog_set_negative_response (dialog, GTK_RESPONSE_CANCEL);
-
 	//highest priority - arrange for key-translations from locale
 #ifdef USE_GTK3_0
 	g_signal_connect (G_OBJECT (dialog), "key-press-event",
@@ -1007,7 +1060,7 @@ static GtkWidget *_e2_dialog_setup_line (gchar* window_title, gchar *prompt,
 	e2_dialog_set_title (dialog, title);
 */
 	gchar *title = (window_title != NULL) ? window_title : _("user input");
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
+	GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION,
 #ifdef LATESHOW
 		"",
 #else
@@ -1053,6 +1106,8 @@ static GtkWidget *_e2_dialog_setup_line (gchar* window_title, gchar *prompt,
 	}
 	*retentry = dialogentry;
 
+	OPENBGL
+
 #ifdef E2_ASSISTED
 	GtkWidget *label = (GtkWidget *) g_object_get_data (G_OBJECT (dialog),
 		"e2-dialog-label");
@@ -1129,6 +1184,7 @@ static GtkWidget *_e2_dialog_setup_line (gchar* window_title, gchar *prompt,
 	}
 #endif
 
+	CLOSEBGL
 	return dialog;
 }
 /**
@@ -1162,7 +1218,11 @@ Button sensitivities in @a dialog are also adjusted, so @a dialog must be 'compl
 */
 static void _e2_dialog_setup_auth (GtkWidget *dialog, gboolean multi)
 {
+#ifdef USE_GTK3_0
+	GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	GtkWidget *box = gtk_hbox_new (FALSE, 0);
+#endif
 	gtk_container_set_border_width (GTK_CONTAINER (box), E2_PADDING);
 	gchar *s = g_strconcat ("<b>",_("You don't have authority to do this"), "</b>", NULL);
 	e2_widget_add_mid_label (box, s, 0.5, TRUE, 0);
@@ -1259,7 +1319,7 @@ DialogButtons e2_dialog_positioned_input (gchar* window_title, gchar *prompt,
 	E2_DialogFlags flags = 0;
 	GtkWidget *dialogentry;
 
-	gdk_threads_enter ();
+	CLOSEBGL
 
 	GtkWidget *dialog = _e2_dialog_setup_line (window_title, prompt,
 		suggestion, extras,	select_text,// tag PASSWORDINPUT FALSE,
@@ -1269,7 +1329,7 @@ DialogButtons e2_dialog_positioned_input (gchar* window_title, gchar *prompt,
 		_e2_dialog_setup_auth (dialog, (flags & E2_DIALOG_MULTI));
 
 	e2_dialog_setup (dialog, app.main_window);
-	e2_dialog_run (dialog, NULL, 0);
+	gtk_widget_show_all (dialog);
 	if (*horz >= 0 && *vert >= 0)
 		gtk_window_move (GTK_WINDOW (dialog), *horz, *vert);
 	//refreshing is always temporarily enabled downstream, while waiting
@@ -1292,7 +1352,7 @@ DialogButtons e2_dialog_positioned_input (gchar* window_title, gchar *prompt,
 		gtk_window_get_position (GTK_WINDOW (dialog), horz, vert); //FIXME position if closed
 		gtk_widget_destroy (dialog);
 	}
-	gdk_threads_leave ();
+	OPENBGL
 
 	return retval;
 }
@@ -1331,7 +1391,7 @@ DialogButtons e2_dialog_combo_input (gchar* window_title, gchar *prompt,
 			retval = CANCEL;
 		}
 		else
-			e2_list_update_history (*input, history_list, NULL, 10, FALSE);
+			e2_list_update_history (history_list, *input, NULL, 10, FALSE);
 	}
 
 	gtk_widget_destroy (dialog);
@@ -1361,10 +1421,10 @@ DialogButtons e2_dialog_delete_check (VPATH *localpath, gboolean multi,
 	F_FREE (s2, s1);
 	s1 = g_strdup_printf(_("Are you sure you want to delete <b>%s</b>?"), s3);
 	g_free (s3);
-	gdk_threads_enter ();
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION, s1,
+	CLOSEBGL
+	GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION, s1,
 		_("confirm"), DUMMY_RESPONSE_CB, NULL);
-	gdk_threads_leave ();
+	OPENBGL
 	g_free (s1);
 
 	E2_DialogFlags flags = 0;
@@ -1373,7 +1433,7 @@ DialogButtons e2_dialog_delete_check (VPATH *localpath, gboolean multi,
 	stop_btn = E2_BUTTON_CANCEL;
 	E2_Button all_btn;
 	all_btn = E2_BUTTON_APPLYTOALL;
-	all_btn.stock = GTK_STOCK_DELETE;
+	all_btn.name = STOCK_NAME_DELETE;
 
 	if (multi)
 	{
@@ -1408,7 +1468,7 @@ DialogButtons e2_dialog_delete_check (VPATH *localpath, gboolean multi,
 	e2_dialog_add_defined_button (dialog, &stop_btn);
 
 	e2_button_derive (&stop_btn, &E2_BUTTON_YES, BTN_YES_DELETE);
-	stop_btn.stock = GTK_STOCK_DELETE;
+	stop_btn.name = STOCK_NAME_DELETE;
 	stop_btn.showflags &= ~E2_BTN_DEFAULT;
 	e2_dialog_add_defined_button (dialog, &stop_btn);
 
@@ -1417,9 +1477,9 @@ DialogButtons e2_dialog_delete_check (VPATH *localpath, gboolean multi,
 
 //tag DEBUGfreeze2;
 	//maybe this helps gtk 2.16 crashes ?
-//	gdk_threads_leave ();
+//	OPENBGL
 
-	gdk_threads_enter ();
+	CLOSEBGL
 	e2_dialog_setup (dialog, app.main_window);
 	e2_dialog_run (dialog, NULL, flags);
 	if (*horz >= 0 && *vert >= 0)
@@ -1432,7 +1492,7 @@ DialogButtons e2_dialog_delete_check (VPATH *localpath, gboolean multi,
 		gtk_window_get_position (GTK_WINDOW (dialog), horz, vert);
 		gtk_widget_destroy (dialog);
 	}
-	gdk_threads_leave ();
+	OPENBGL
 
 	return choice;
 }
@@ -1451,7 +1511,7 @@ DialogButtons e2_dialog_ow_check (VPATH *slocal, VPATH *dlocal, OW_ButtonFlags e
 
 //tag gboolean DEBUGfreeze;
 	//maybe this helps with gtk 2.16 flakiness
-//	gdk_threads_enter ();
+//	CLOSEBGL
 
 	GString *dialog_prompt = g_string_sized_new (NAME_MAX + 64);	//pick a reasonable initial size
 	gchar *utf = F_DISPLAYNAME_FROM_LOCALE (VPSTR(dlocal));
@@ -1502,10 +1562,10 @@ DialogButtons e2_dialog_ow_check (VPATH *slocal, VPATH *dlocal, OW_ButtonFlags e
 		g_free (dpath);
 	}
 
-	gdk_threads_enter ();
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_WARNING,
+	CLOSEBGL
+	GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_WARNING,
 		dialog_prompt->str, _("confirm"), DUMMY_RESPONSE_CB, NULL);
-	gdk_threads_leave ();
+	OPENBGL
 
 	F_FREE (utf, VPSTR(dlocal));
 	g_free (public);
@@ -1514,7 +1574,7 @@ DialogButtons e2_dialog_ow_check (VPATH *slocal, VPATH *dlocal, OW_ButtonFlags e
 	E2_Button all_btn;
 	E2_Button stop_btn;
 	all_btn = E2_BUTTON_APPLYTOALL;
-	all_btn.stock = GTK_STOCK_OK;
+	all_btn.name = STOCK_NAME_OK;
 	stop_btn = E2_BUTTON_CANCEL;
 	if (extras & NOALL)
 	{
@@ -1543,7 +1603,7 @@ DialogButtons e2_dialog_ow_check (VPATH *slocal, VPATH *dlocal, OW_ButtonFlags e
 	stop_btn = E2_BUTTON_YES;
 	stop_btn.label = _("_Replace");
 	stop_btn.showflags &= ~E2_BTN_DEFAULT;
-	stop_btn.stock = GTK_STOCK_OK;
+	stop_btn.name = STOCK_NAME_OK;
 	e2_dialog_add_defined_button (dialog, &stop_btn);
 
 	//block until user responds
@@ -1558,7 +1618,7 @@ DialogButtons e2_dialog_ow_check (VPATH *slocal, VPATH *dlocal, OW_ButtonFlags e
 
 //tag DEBUGfreeze2;
 	//maybe this helps
-//	gdk_threads_leave ();
+//	OPENBGL
 
 	return (e2_dialog_show (dialog, app.main_window, flags, NULL));
 }
@@ -1572,7 +1632,7 @@ Assumes BGL is on/closed
 */
 DialogButtons e2_dialog_warning (gchar *prompt, const gchar *yes_label)
 {
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_WARNING, prompt,
+	GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_WARNING, prompt,
 		_("confirm"), DUMMY_RESPONSE_CB, NULL);
 
 	E2_Button no_btn;
@@ -1589,7 +1649,7 @@ DialogButtons e2_dialog_warning (gchar *prompt, const gchar *yes_label)
 	else
 	{
 		e2_button_derive (&yes_btn, &E2_BUTTON_YES, BTN_YES_COMMIT);
-		yes_btn.stock = E2_BUTTON_APPLY.stock;
+		yes_btn.name = E2_BUTTON_APPLY.name;
 	}
 	yes_btn.showflags &= ~E2_BTN_DEFAULT;
 	//block until user responds
@@ -1616,7 +1676,7 @@ GtkWidget *e2_dialog_slow (gchar *prompt_type, gchar *tip_type,
 {
 	gchar *prompt = g_strdup_printf (
 		_("%s is taking a long time. Continue waiting ?"), prompt_type);
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION, prompt,
+	GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION, prompt,
 		_("confirm"), response_func, data);
 	g_free (prompt);
 
