@@ -1,6 +1,6 @@
-/* $Id: e2_dialog.h 2377 2011-06-11 00:16:34Z tpgww $
+/* $Id: e2_dialog.h 2831 2013-10-23 07:59:22Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -28,6 +28,7 @@ This file is the header for the dialog utility functions.
 #define __E2_DIALOG_H__
 
 #include "emelfm2.h"
+#include <pwd.h>
 #include "e2_about_dialog.h"
 #include "e2_config_dialog.h"
 //#include "e2_mkdir_dialog.h"
@@ -120,10 +121,14 @@ DialogButtons e2_dialog_delete_check (VPATH *localpath, gboolean multi,
 DialogButtons e2_dialog_ow_check (VPATH *slocal, VPATH *dlocal, OW_ButtonFlags extras);
 DialogButtons e2_dialog_warning (gchar *prompt, const gchar *yes_label);
 GtkWidget *e2_dialog_slow (gchar *prompt_type, gchar *tip_type,
-	void(*response_func) (GtkDialog*,gint,gpointer), gpointer data);
+	void(*response_func) (GtkDialog*,gint,gpointer), gpointer data) G_GNUC_MALLOC;
 void e2_dialog_setup_chooser (GtkWidget *dialog, const gchar *title,
 	const gchar *startpath, GtkFileChooserAction action, gboolean showhidden,
-	gboolean multi, gint defresponse);
+	gboolean multi, gint defresponse, const gchar *first_button, ...)
+#ifdef G_GNUC_NULL_TERMINATED
+    G_GNUC_NULL_TERMINATED
+#endif
+	;
 
 void e2_dialog_setup_auth (GtkWidget *box);
 void e2_dialog_setup (GtkWidget *dialog, GtkWidget *parent);
@@ -132,21 +137,21 @@ gint e2_dialog_show (GtkWidget *dialog, GtkWidget *parent, E2_DialogFlags flags,
 gint e2_dialog_run (GtkWidget *dialog, GtkWidget *parent, E2_DialogFlags flags);
 //GtkWidget *get_dialog (GtkWidget *parent, GtkMessageType type, gchar *label_text);
 GtkWidget *e2_dialog_create (const gchar *stock, const gchar *label_text,
-	const gchar *title,	void (*callback) (GtkDialog*,gint,gpointer), gpointer data);
+	const gchar *title,	void (*callback) (GtkDialog*,gint,gpointer), gpointer data) G_GNUC_MALLOC;
 GtkWidget *e2_dialog_add_sw (GtkWidget *dialog);
-GtkWidget *e2_dialog_add_defined_button (GtkWidget *dialog, E2_Button *button);
+GtkWidget *e2_dialog_add_defined_button (GtkWidget *dialog, E2_Button *button) G_GNUC_MALLOC;
 GtkWidget *e2_dialog_add_simple_button (GtkWidget *dialog, const gchar *stock,
-	const gchar *label, gint response);
+	const gchar *label, gint response) G_GNUC_MALLOC;
 GtkWidget *e2_dialog_add_custom_button (GtkWidget *dialog, E2_Button *button,
 	gboolean is_default, const gchar *tip, void *callback, void *data);
 GtkWidget *e2_dialog_add_custom_button_full
 	(GtkWidget *dialog, gboolean is_default, guint response,
 	  const gchar *label, const gchar *stock, const gchar *tip,
-	  void (*callback)(/*GtkButton*,gpointer*/), gpointer data);
+	  void (*callback)(/*GtkButton*,gpointer*/), gpointer data) G_GNUC_MALLOC;
 GtkWidget *e2_dialog_add_toggle_button (GtkWidget *dialog, gboolean value,
-	gchar *label, gchar *tip, gint response);
+	gchar *label, gchar *tip, gint response) G_GNUC_MALLOC;
 GtkWidget *e2_dialog_add_check_button (GtkWidget *dialog, gboolean value,
-	gchar *label, gchar *tip, gint response);
+	gchar *label, gchar *tip, gint response) G_GNUC_MALLOC;
 //void e2_dialog_set_positive_response (GtkWidget *dialog, gint response);
 void e2_dialog_set_negative_response (GtkWidget *dialog, gint response);
 //void e2_dialog_set_responses (GtkWidget *dialog, gint pos, gint neg);
@@ -164,6 +169,7 @@ void e2_select_items_byname_dialog_create (GtkMenuItem *item);
 gint e2_filetype_dialog_create (VPATH *localpath, gboolean text,
 	gboolean ambig, gboolean newtype);
 DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi);
+void e2_file_info_dialog_options_register (void);
 gint e2_dialog_run_simple (GtkWidget *dialog, GtkWidget *parent);
 void e2_filetype_dialog_edit_create (gchar *category);
 void e2_opendir_dialog_create (ViewInfo *view, gchar *oldpath, gchar **newpath);
