@@ -1,12 +1,11 @@
-/* $Id: e2_edit_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_edit_dialog.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2006-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2006-2013 tooar <tooar@emelfm2.net>
 
-This file is part of emelFM2.
-emelFM2 is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+This file is part of emelFM2, which is free software. You can redistribute it
+and/or modify it under the terms of the GNU General Public License as published
+by the Free Software Foundation - either version 3, or (at your option) any
+later version.
 
 emelFM2 is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,6 +28,7 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #ifdef E2_MOUSECUSTOM
 # include "e2_mousebinding.h"
 #endif
+#include "e2_icons.h"
 
 //rates around the default blink rate will never actually blink! dunno why ...
 //so: MULTIPLIER >= 1.5, MULTIPLIER/DIVIDER <= 0.5
@@ -93,6 +93,7 @@ static void _e2_edit_dialog_cancel_spell (E2_ViewDialogRuntime *rt)
 /**
 @brief save text selected in @a buffer
 This func is also used to save output pane selection
+Expects BGL closed
 @param buffer text buffer from which the text will be saved
 #ifdef E2_VFS
 @param spacedata pointer to namspace data
@@ -115,11 +116,7 @@ void e2_edit_dialog_save_selected (GtkTextBuffer *buffer,
 	if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end))
 	{
 		GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
-			GTK_WINDOW (parent), GTK_FILE_CHOOSER_ACTION_SAVE,
-			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
-			NULL);
-
+			GTK_WINDOW (parent), GTK_FILE_CHOOSER_ACTION_SAVE, NULL, NULL);
 		e2_dialog_setup_chooser (dialog,
 			_("file save as"),
 			//API doco is silent on whether g_get_home_dir() gets a localised string
@@ -127,7 +124,10 @@ void e2_edit_dialog_save_selected (GtkTextBuffer *buffer,
 			GTK_FILE_CHOOSER_ACTION_SAVE,
 			FALSE,	//hide-hidden
 			FALSE,	//single-selection
-			GTK_RESPONSE_OK);	//default response
+			GTK_RESPONSE_OK,	//default response
+			STOCK_NAME_CANCEL, GTK_RESPONSE_CANCEL,
+			STOCK_NAME_SAVE, GTK_RESPONSE_OK,
+			NULL);
 
 		gint response;
 		while ((response = e2_dialog_run_simple (dialog, parent)) == E2_RESPONSE_USER1)
@@ -148,7 +148,7 @@ void e2_edit_dialog_save_selected (GtkTextBuffer *buffer,
 				&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
 #endif
 			{
-				gdk_threads_leave ();
+				OPENBGL
 				DialogButtons choice = e2_dialog_ow_check (NULL,
 #ifdef E2_VFS
 					&ddata,
@@ -156,7 +156,7 @@ void e2_edit_dialog_save_selected (GtkTextBuffer *buffer,
 					local,
 #endif
 					NONE);
-				gdk_threads_enter ();
+				CLOSEBGL
 				if (choice != OK)
 				{
 					gtk_widget_destroy (dialog);
@@ -191,11 +191,13 @@ void e2_edit_dialog_save_selected (GtkTextBuffer *buffer,
 static void _e2_edit_dialog_savesel_cb (GtkMenuItem *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	e2_edit_dialog_save_selected (rt->textbuffer,
 #ifdef E2_VFS
-	rt->spacedata,
+		rt->spacedata,
 #endif
-	rt->dialog);
+		rt->dialog);
+	NEEDOPENBGL
 }
 /**
 @brief save file
@@ -239,6 +241,7 @@ static void _e2_edit_dialog_save_cb (GtkMenuItem *menuitem,
 			gchar *msg;
 			if (newtext != NULL)
 				g_free (newtext);
+			NEEDCLOSEBGL
 			switch (error->code)
 			{
 				case G_CONVERT_ERROR_ILLEGAL_SEQUENCE:
@@ -255,15 +258,16 @@ static void _e2_edit_dialog_save_cb (GtkMenuItem *menuitem,
 					e2_output_print_error (error->message, FALSE);
 					g_error_free (error);
 					rt->saved_ok = FALSE;
+					NEEDOPENBGL
 					return;
 			}
+			NEEDOPENBGL
 		}
 	}
 
 	const gchar *usepath;
 	//rt->localpath may be a replacement name, when "saving as"
 	usepath = (rt->saveas) ? rt->newlocalpath : rt->localpath;
-
 #ifdef E2_VFS
 	VPATH ddata = { usepath, rt->spacedata };
 #endif
@@ -280,13 +284,19 @@ static void _e2_edit_dialog_save_cb (GtkMenuItem *menuitem,
 #ifdef E2_VFS
 		VPATH sdata = { saved, rt->spacedata };
 #endif
-		gdk_threads_leave ();	//downstream errors invoke local mutex locking
+#ifndef LOCAL_BGL
+//		NEEDCLOSEBGL
+		OPENBGL	//downstream errors invoke local mutex locking
+#endif
 #ifdef E2_VFS
 		e2_task_backend_rename (&ddata, &sdata);	//ignore any warning !!
 #else
 		e2_task_backend_rename (usepath, saved);	//ignore any warning !!
 #endif
-		gdk_threads_enter ();
+#ifndef LOCAL_BGL
+		CLOSEBGL
+//		NEEDOPENBGL
+#endif
 		g_free (saved);
 	}
 
@@ -321,6 +331,7 @@ static void _e2_edit_dialog_save_cb (GtkMenuItem *menuitem,
 
 		if (!strcmp (rt->charset, "UNKNOWN"))
 		{
+			NEEDCLOSEBGL
 			e2_fs_error_simple (
 			_("Original encoding of '%s' is unknown, now saved as UTF-8"),
 #ifdef E2_VFS
@@ -328,6 +339,7 @@ static void _e2_edit_dialog_save_cb (GtkMenuItem *menuitem,
 #else
 			usepath);
 #endif
+			NEEDOPENBGL
 		}
 		gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (rt->textbuffer), FALSE);
 	}
@@ -346,12 +358,10 @@ static void _e2_edit_dialog_saveas_cb (GtkMenuItem *menuitem,
 #ifdef E2_VFSTMP
 	check assumption that selector only works for current namespace
 #endif
-	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
-		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SAVE,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_SAVE, GTK_RESPONSE_OK,
-		NULL);
+	NEEDCLOSEBGL
 
+	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
+		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SAVE, NULL, NULL);
 	gchar *utf = F_FILENAME_FROM_LOCALE (rt->localpath);
 	e2_dialog_setup_chooser (dialog,
 		_("file save as"),
@@ -359,7 +369,10 @@ static void _e2_edit_dialog_saveas_cb (GtkMenuItem *menuitem,
 		GTK_FILE_CHOOSER_ACTION_SAVE,
 		FALSE, 	//hide-hidden
 		FALSE,	//single-selection
-		GTK_RESPONSE_OK);	//default response
+		GTK_RESPONSE_OK,	//default response
+		STOCK_NAME_CANCEL, GTK_RESPONSE_CANCEL,
+		STOCK_NAME_SAVE, GTK_RESPONSE_OK,
+		NULL);
 	F_FREE (utf, rt->localpath);
 
 	gint response;
@@ -381,7 +394,7 @@ static void _e2_edit_dialog_saveas_cb (GtkMenuItem *menuitem,
 			&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
 #endif
 		{
-			gdk_threads_leave ();
+			OPENBGL
 			DialogButtons choice = e2_dialog_ow_check (NULL,
 #ifdef E2_VFS
 				&ddata,
@@ -389,7 +402,7 @@ static void _e2_edit_dialog_saveas_cb (GtkMenuItem *menuitem,
 				local,
 #endif
 				NONE);
-			gdk_threads_enter ();
+			CLOSEBGL
 			if (choice != OK)
 			{
 				gtk_widget_destroy (dialog);
@@ -400,7 +413,9 @@ static void _e2_edit_dialog_saveas_cb (GtkMenuItem *menuitem,
 #endif
 		rt->saveas = TRUE;
 		rt->newlocalpath = local;
+		NEEDOPENBGL
 		_e2_edit_dialog_save_cb (NULL, rt);
+		NEEDCLOSEBGL
 		rt->saveas = FALSE;
 
 		if (rt->saved_ok)
@@ -415,6 +430,7 @@ static void _e2_edit_dialog_saveas_cb (GtkMenuItem *menuitem,
 		}
 	}
 	gtk_widget_destroy (dialog);
+	NEEDOPENBGL
 }
 /**
 @brief reload edited file
@@ -429,6 +445,7 @@ static void _e2_edit_dialog_refresh_cb (GtkMenuItem *menuitem,
 {
 	if (!gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (rt->textbuffer)))
 		return;	//no point in reloading
+	NEEDCLOSEBGL
 	DialogButtons choice = e2_dialog_warning (
 	_("Reverting to saved version cannot be undone"), _("Re_vert"));
 	if (choice == OK)
@@ -444,6 +461,7 @@ static void _e2_edit_dialog_refresh_cb (GtkMenuItem *menuitem,
 		gtk_text_buffer_set_modified (rt->textbuffer, FALSE);
 	}
 	gtk_widget_grab_focus (rt->textview);
+	NEEDOPENBGL
 }
 /**
 @brief undo change
@@ -456,7 +474,9 @@ static void _e2_edit_dialog_refresh_cb (GtkMenuItem *menuitem,
 static void _e2_edit_dialog_undo_cb (GtkMenuItem *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	_e2edtdlg_undo_undo_one (rt);
+	NEEDOPENBGL
 }
 #ifdef E2_REDO_ENABLED
 /**
@@ -470,7 +490,9 @@ static void _e2_edit_dialog_undo_cb (GtkMenuItem *menuitem,
 static void _e2_edit_dialog_redo_cb (GtkMenuItem *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	_e2edtdlg_undo_redo_one (rt);
+	NEEDOPENBGL
 }
 #endif
 #ifdef E2_SPELLCHECK
@@ -494,7 +516,9 @@ static void _e2_edit_dialog_spellall_cb (GtkMenuItem *menuitem,
 	{
 		gchar *msg = g_strdup_printf (
 			_("Cannot initialize spell checking: %s"), error->message);
+		NEEDCLOSEBGL
 		e2_output_print_error (msg, TRUE);
+		NEEDOPENBGL
 		g_error_free (error);
 	}
 	else
@@ -515,6 +539,7 @@ static void _e2_edit_dialog_spellall_cb (GtkMenuItem *menuitem,
 static void _e2_edit_dialog_unspellall_cb (GtkMenuItem *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+//	NEEDOPENBGL
 	gtkspell_detach (rt->spelldata);	//this destroys the GtkSpell
 	rt->spelldata = NULL;
 }
@@ -540,6 +565,7 @@ static void _e2_edit_dialog_spell_language_cb (GtkMenuItem *menuitem,
 	else
 		lang = (gchar *)lang_history->data;
 
+	NEEDCLOSEBGL
 	DialogButtons choice = e2_dialog_combo_input (_("choose language"),
 		_("Enter the spell-checker language (like en_CA):"), lang,
 		0, &lang_history, &lang);	//re-use for entered value
@@ -557,6 +583,7 @@ static void _e2_edit_dialog_spell_language_cb (GtkMenuItem *menuitem,
 		}
 		g_free (lang);
 	}
+	NEEDOPENBGL
 }
 #endif
 /**
@@ -571,8 +598,10 @@ static void _e2_edit_dialog_wrap_cb (GtkMenuItem *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
 	rt->textwrap = !rt->textwrap;
+	NEEDCLOSEBGL
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
 		rt->textwrap ? GTK_WRAP_WORD: GTK_WRAP_NONE);
+	NEEDOPENBGL
 }
 /**
 @brief hide search/replace bar
@@ -585,6 +614,7 @@ static void _e2_edit_dialog_wrap_cb (GtkMenuItem *menuitem,
 static void _e2_edit_dialog_hide_cb (GtkWidget *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	gtk_widget_hide (rt->panel);
 	rt->is_hidden = TRUE;
 	gtk_widget_hide (rt->info_label);
@@ -598,10 +628,10 @@ static void _e2_edit_dialog_hide_cb (GtkWidget *menuitem,
 		GTK_BIN (rt->combo)->child;
 #endif
 	const gchar *choice = gtk_entry_get_text (GTK_ENTRY (entry));
-	if (*choice != '\0')
+	if (choice != NULL && *choice != '\0')
 	{
-		e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
-		e2_list_update_history (choice, &find_history, NULL, 30, FALSE);
+		e2_combobox_prepend_history (rt->combo, choice, 20, FALSE);
+		e2_list_update_history (&find_history, choice, NULL, 20, FALSE);
 	}
 #ifdef E2_MARK_FINDS
 	//turn off any highlihts
@@ -619,13 +649,14 @@ static void _e2_edit_dialog_hide_cb (GtkWidget *menuitem,
 		GTK_BIN (rt->combo2)->child;
 #endif
 	choice = gtk_entry_get_text (GTK_ENTRY (entry));
-	if (*choice != '\0')
+	if (choice != NULL && *choice != '\0')
 	{
-		e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
-		e2_list_update_history (choice, &replace_history, NULL, 30, FALSE);
+		e2_combobox_prepend_history (rt->combo2, choice, 20, FALSE);
+		e2_list_update_history (&replace_history, choice, NULL, 20, FALSE);
 	}
 	//focus on the text
 	gtk_widget_grab_focus (rt->textview);
+	NEEDOPENBGL
 }
 /**
 @brief perform find
@@ -638,6 +669,7 @@ This is a menu-selection callback, and can also be called directly
 static void _e2_edit_dialog_find_cb (GtkWidget *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	if (!rt->is_hidden) //search panel is visible
 	{
 		//if a mod key is pressed, treat it specially
@@ -697,6 +729,7 @@ static void _e2_edit_dialog_find_cb (GtkWidget *menuitem,
 		//focus on the text entry
 		gtk_widget_grab_focus (child);
 	}
+	NEEDOPENBGL
 }
 /**
 @brief perform replacement
@@ -709,6 +742,7 @@ Assumes BGL closed
 static void _e2_edit_dialog_replace_cb (GtkWidget *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 #ifdef USE_GTK2_18
 	if (!gtk_widget_get_visible (rt->replacebtn))
 		gtk_widget_show (rt->replacebtn);
@@ -748,7 +782,7 @@ static void _e2_edit_dialog_replace_cb (GtkWidget *menuitem,
 			DialogButtons choice;
 			if (confirm_before)
 			{
-				dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
+				dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION,
 					_("Replace this one ?"), _("confirm"), DUMMY_RESPONSE_CB, NULL);
 				e2_dialog_add_defined_button (dialog, &E2_BUTTON_CANCEL);
 
@@ -822,6 +856,7 @@ static void _e2_edit_dialog_replace_cb (GtkWidget *menuitem,
 		//focus on the text entry
 		gtk_widget_grab_focus (child);
 	}
+	NEEDOPENBGL
 }
 /**
 @brief construct and pop up destroyable context-menu for the edit-dialog
@@ -837,7 +872,7 @@ static void _e2_edit_dialog_show_context_menu (GtkWidget *textview,
 	guint event_button, gint event_time, E2_ViewDialogRuntime *rt)
 {
 	gchar *item_name;
-	GtkWidget *item, *menu = gtk_menu_new ();
+	GtkWidget *item, *menu = e2_menu_get ();
 #ifdef E2_SPELLCHECK
 	if (rt->spelldata != NULL)
 	{	//decide if we need a spellfix item in the menu
@@ -881,10 +916,10 @@ static void _e2_edit_dialog_show_context_menu (GtkWidget *textview,
 	}
 #endif
 	item =
-	e2_menu_add (menu, _("_Save"), GTK_STOCK_SAVE,
+	e2_menu_add (menu, _("_Save"), STOCK_NAME_SAVE,
 		_("Save the file"), _e2_edit_dialog_save_cb, rt);
 	gtk_widget_set_sensitive (item, rt->is_dirty);
-	e2_menu_add (menu, _("Save as.."), GTK_STOCK_SAVE_AS,	//no available valid mnemonic
+	e2_menu_add (menu, _("Save as.."), STOCK_NAME_SAVE_AS,	//no available valid mnemonic
 		_("Save the file with a new name"), _e2_edit_dialog_saveas_cb, rt);
 	item =
 	e2_menu_add (menu, _("Save se_lection.."), "save_selection"E2ICOND,
@@ -894,44 +929,44 @@ static void _e2_edit_dialog_show_context_menu (GtkWidget *textview,
 #ifdef USE_GTK2_10
 	gchar *tip = (gtk_text_buffer_get_has_selection (rt->textbuffer)) ?
 		_("Print selected text") : _("Print file");
-	e2_menu_add (menu, _("_Print.."), GTK_STOCK_PRINT, tip,
+	e2_menu_add (menu, _("_Print.."), STOCK_NAME_PRINT, tip,
 		e2_dialog_print_cb, rt);
 #endif
 	item =
-	e2_menu_add (menu, _("Re_vert"), GTK_STOCK_REVERT_TO_SAVED,
+	e2_menu_add (menu, _("Re_vert"), STOCK_NAME_REVERT_TO_SAVED,
 		_("Reload the file being edited"), _e2_edit_dialog_refresh_cb, rt);
 	gtk_widget_set_sensitive (item, rt->is_dirty);
 	e2_menu_add_separator (menu);
-	e2_menu_add (menu, _("_Find.."), GTK_STOCK_FIND,
+	e2_menu_add (menu, _("_Find.."), STOCK_NAME_FIND,
 		_("Find matching text"), _e2_edit_dialog_find_cb, rt);
-	e2_menu_add (menu, _("_Replace.."), GTK_STOCK_FIND_AND_REPLACE,
+	e2_menu_add (menu, _("_Replace.."), STOCK_NAME_FIND_AND_REPLACE,
 		_("Find and replace matching text"), _e2_edit_dialog_replace_cb, rt);
 	item =
-	e2_menu_add (menu, _("_Hide"), GTK_STOCK_ZOOM_FIT,
+	e2_menu_add (menu, _("_Hide"), STOCK_NAME_ZOOM_FIT,
 		_("Hide the search options bar"), _e2_edit_dialog_hide_cb, rt);
 	gtk_widget_set_sensitive (item, !rt->is_hidden);
 	item =
-	e2_menu_add (menu, _("_Undo"), GTK_STOCK_UNDO,
+	e2_menu_add (menu, _("_Undo"), STOCK_NAME_UNDO,
 		_("Undo last change"), _e2_edit_dialog_undo_cb, rt);
 	gtk_widget_set_sensitive (item, rt->undo_enabled);
 #ifdef E2_REDO_ENABLED
 	item =
-	e2_menu_add (menu, _("Re_do"), GTK_STOCK_REDO,
+	e2_menu_add (menu, _("Re_do"), STOCK_NAME_REDO,
 		_("Reverse last undo"), _e2_edit_dialog_redo_cb, rt);
 	gtk_widget_set_sensitive (item, rt->redo_enabled);
 #endif
 #ifdef E2_SPELLCHECK
 	if (rt->spelldata == NULL)
-		e2_menu_add (menu, _("_Check spelling"), GTK_STOCK_SPELL_CHECK,
+		e2_menu_add (menu, _("_Check spelling"), STOCK_NAME_SPELL_CHECK,
 			_("Flag mis-spelt words"), _e2_edit_dialog_spellall_cb, rt);
 	else
-		e2_menu_add (menu, _("_Clear spellcheck"), GTK_STOCK_SPELL_CHECK,
+		e2_menu_add (menu, _("_Clear spellcheck"), STOCK_NAME_SPELL_CHECK,
 			_("Remove all spell-check flags"), _e2_edit_dialog_unspellall_cb, rt);
 #endif
 	e2_menu_add_separator (menu);
 #ifdef E2_SPELLCHECK
 	item =
-	e2_menu_add (menu, _("_Language.."), GTK_STOCK_INDEX,
+	e2_menu_add (menu, _("_Language.."), STOCK_NAME_INDEX,
 		_("Set spell-checker language"), _e2_edit_dialog_spell_language_cb, rt);
 	gtk_widget_set_sensitive (item, (rt->spelldata != NULL));
 #endif
@@ -941,8 +976,8 @@ static void _e2_edit_dialog_show_context_menu (GtkWidget *textview,
 	e2_widget_set_safetip (item,
 		_("If activated, text in the window will be word-wrapped"));
 
-	item_name = g_strconcat (_A(3),".",_A(32),NULL);
-	e2_menu_add_action (menu, _("Se_ttings"), GTK_STOCK_PREFERENCES,
+	item_name = g_strconcat (_A(3),".",_A(34),NULL);
+	e2_menu_add_action (menu, _("Se_ttings"), STOCK_NAME_PREFERENCES,
 		_("Open the configuration dialog at the options page"), item_name,
 		_C(11)); //_("dialogs")
 	g_free(item_name);
@@ -969,7 +1004,8 @@ static void _e2_edit_dialog_destroy (E2_ViewDialogRuntime *rt, gboolean forced)
 {
 	if (rt->is_dirty)
 	{
-		GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
+		NEEDCLOSEBGL
+		GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION,
 			_("Save modified file ?"), _("confirm"), DUMMY_RESPONSE_CB, NULL);
 
 		//arrange for a specific retval if the user presses <Esc>
@@ -977,7 +1013,7 @@ static void _e2_edit_dialog_destroy (E2_ViewDialogRuntime *rt, gboolean forced)
 
 		E2_Button resume_btn;
 		resume_btn.label = _("_Edit");
-		resume_btn.stock = GTK_STOCK_EDIT;
+		resume_btn.name = STOCK_NAME_EDIT;
 		resume_btn.tip = _("Resume editing");
 		resume_btn.showflags = E2_BTN_TIPPED;
 		resume_btn.response = E2_RESPONSE_NOTOALL;	//must be understood by the response-decoder
@@ -988,7 +1024,7 @@ static void _e2_edit_dialog_destroy (E2_ViewDialogRuntime *rt, gboolean forced)
 
 		E2_Button yes_btn;
 		yes_btn.label = _("_Save");
-		yes_btn.stock = GTK_STOCK_SAVE;
+		yes_btn.name = STOCK_NAME_SAVE;
 		yes_btn.showflags = E2_BTN_DEFAULT;
 		yes_btn.response = GTK_RESPONSE_YES;
 
@@ -997,6 +1033,7 @@ static void _e2_edit_dialog_destroy (E2_ViewDialogRuntime *rt, gboolean forced)
 			E2_DIALOG_BLOCKED | E2_DIALOG_FREE,
 			&resume_btn, &discard_btn, &yes_btn, NULL);
 
+		NEEDOPENBGL
 		switch (choice)
 		{
 			case OK:	//save
@@ -1021,6 +1058,7 @@ static void _e2_edit_dialog_destroy (E2_ViewDialogRuntime *rt, gboolean forced)
 		rt->blink_id = 0;
 	}
 
+	NEEDCLOSEBGL
 	GtkWidget *entry =
 #ifdef USE_GTK2_14
 		gtk_bin_get_child (GTK_BIN (rt->combo2));
@@ -1028,10 +1066,10 @@ static void _e2_edit_dialog_destroy (E2_ViewDialogRuntime *rt, gboolean forced)
 		GTK_BIN (rt->combo2)->child;
 #endif
 	const gchar *repl = gtk_entry_get_text (GTK_ENTRY (entry));
-	if (*repl != '\0')
+	if (repl != NULL && *repl != '\0')
 	{
-		e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
-		e2_list_update_history (repl, &replace_history, NULL, 30, FALSE);
+		e2_combobox_prepend_history (rt->combo2, repl, 20, FALSE);
+		e2_list_update_history (&replace_history, repl, NULL, 20, FALSE);
 	}
 	//if editing an output tab, need some specific cleanup
 	GList *tmp;
@@ -1063,6 +1101,7 @@ static void _e2_edit_dialog_destroy (E2_ViewDialogRuntime *rt, gboolean forced)
 #endif
 
 	e2_view_dialog_destroy (rt);
+	NEEDOPENBGL
 }
 /**
 @brief toggle cursor visibility
@@ -1075,7 +1114,7 @@ This is a timer callback
 static gboolean _e2_edit_dialog_blink (E2_ViewDialogRuntime *rt)
 {
 //	printd (DEBUG, "edit dialog cursor visibility toggle cb");
-//	gdk_threads_enter ();
+//	CLOSEBGL
 //	gboolean blinkme = TRUE;	//GTK_WIDGET_HAS_FOCUS (rt->dialog);
 //	if (blinkme)
 //	{
@@ -1094,7 +1133,7 @@ static gboolean _e2_edit_dialog_blink (E2_ViewDialogRuntime *rt)
 			(GSourceFunc) _e2_edit_dialog_blink, rt, NULL);
 	}
 //	}
-//	gdk_threads_leave ();
+//	OPENBGL
 //	return !blinkme; //remove ourself if we're done
 //	return TRUE;	//already removed
 	return FALSE;	//remove ourself
@@ -1152,7 +1191,9 @@ static void _e2_edit_dialog_dirty_change_cb (GtkTextBuffer *textbuffer,
 	E2_ViewDialogRuntime *rt)
 {
 	rt->is_dirty = !rt->is_dirty;
+	NEEDCLOSEBGL
 	gtk_widget_set_sensitive (rt->savebtn, rt->is_dirty);
+	NEEDOPENBGL
 }
 /**
 @brief menu-button press callback
@@ -1167,7 +1208,9 @@ static gboolean _e2_edit_dialog_popup_menu_cb (GtkWidget *textview,
 {
 	_e2_edit_dialog_blink_stop (rt);
 	guint32 event_time = gtk_get_current_event_time ();
+	NEEDCLOSEBGL
 	_e2_edit_dialog_show_context_menu (textview, 0, event_time, rt);
+	NEEDOPENBGL
 	return TRUE;
 }
 /**
@@ -1182,6 +1225,8 @@ static gboolean _e2_edit_dialog_popup_menu_cb (GtkWidget *textview,
 gboolean e2_edit_dialog_key_press_cb (GtkWidget *textview,
 	GdkEventKey *event, E2_ViewDialogRuntime *rt)
 {
+	gboolean retval = FALSE;
+	NEEDCLOSEBGL
 	_e2_edit_dialog_blink_stop (rt);
 	guint mask = event->state & gtk_accelerator_get_default_mod_mask (); //SHIFT check used
 	if (//g_ascii_isalpha (event->keyval) //&& (event->keyval < 0xF000 || event->keyval > 0xFFFF) //&&
@@ -1195,6 +1240,7 @@ gboolean e2_edit_dialog_key_press_cb (GtkWidget *textview,
 			asciicode = lower;
 		if (lower == save_keycode) //|| asciicode == GDK_s)
 		{
+			NEEDOPENBGL
 			if (rt->is_dirty)
 				_e2_edit_dialog_save_cb (NULL, rt);
 			return TRUE;
@@ -1207,6 +1253,7 @@ gboolean e2_edit_dialog_key_press_cb (GtkWidget *textview,
 #endif
 			rt->release_blocked = TRUE;	//block anomalous key-release-event searches
 			//trick cb into doing a from-start|end search for Ctrl f
+			NEEDOPENBGL
 			_e2_edit_dialog_find_cb (
 				(lower == find_keycode) ? NULL : GINT_TO_POINTER (1), rt);
 			return TRUE;
@@ -1217,11 +1264,13 @@ gboolean e2_edit_dialog_key_press_cb (GtkWidget *textview,
 			if (rt->spelldata != NULL)
 				_e2_edit_dialog_cancel_spell (rt);
 #endif
+			NEEDOPENBGL
 			_e2_edit_dialog_replace_cb (NULL, rt);
 			return TRUE;
 		}
 		else if (lower == hide_keycode)
 		{
+			NEEDOPENBGL
 			_e2_edit_dialog_hide_cb (NULL, rt);
 			return TRUE;
 		}
@@ -1229,7 +1278,7 @@ gboolean e2_edit_dialog_key_press_cb (GtkWidget *textview,
 		{
 			if (rt->undo_enabled)
 				_e2edtdlg_undo_undo_one (rt);
-			return TRUE;
+			retval = TRUE;
 		}
 #ifdef E2_REDO_ENABLED
 		else if (lower == redo_keycode
@@ -1237,23 +1286,25 @@ gboolean e2_edit_dialog_key_press_cb (GtkWidget *textview,
 		{
 			if (rt->redo_enabled)
 				_e2edtdlg_undo_redo_one (rt);
-			return TRUE;
+			retval = TRUE;
 		}
 #endif
 	}
 	else if (event->keyval == GDK_Insert)
 	{
-	//GTK_TEXT_VIEW (rt->textview)->overwrite_mode can get overwrite_mode state
+		//GTK_TEXT_VIEW (rt->textview)->overwrite_mode can get overwrite_mode state
 		rt->ow_mode = !gtk_text_view_get_overwrite (GTK_TEXT_VIEW (rt->textview));
 		gtk_text_view_set_overwrite (GTK_TEXT_VIEW (rt->textview), rt->ow_mode);
-		return TRUE;
+		retval = TRUE;
 	}
-//	else if (lower == GDK_Delete)
-//	{	//workaround gtk behaviour
-//		gtk_text_buffer_set_modified (rt->textbuffer, TRUE);
-//	}
+/*	else if (lower == GDK_Delete)
+	{	//workaround gtk behaviour
+		gtk_text_buffer_set_modified (rt->textbuffer, TRUE);
+	}
+*/
+	NEEDOPENBGL
 	rt->keyval = event->keyval;	//save, for undo processing
-	return FALSE;
+	return retval;
 }
 /**
 @brief replace combo-box key press callback
@@ -1284,7 +1335,10 @@ static gboolean _e2_edit_dialog_combokey_cb (GtkWidget *entry,
 	 || lower == findnext_keycode
 	 || lower == replace_keycode
 	 || lower == hide_keycode)
+	{
+//		NEEDOPENBGL
 		return (e2_edit_dialog_key_press_cb (rt->textview, event, rt));
+	}
 	else
 		return FALSE;
 }
@@ -1300,6 +1354,7 @@ static gboolean _e2_edit_dialog_combokey_cb (GtkWidget *entry,
 static gboolean _e2_edit_dialog_button_press_cb (GtkWidget *textview,
 	GdkEventButton *event, E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	_e2_edit_dialog_blink_stop (rt);
 	if (event->button == 3
 #ifdef E2_MOUSECUSTOM
@@ -1308,8 +1363,10 @@ static gboolean _e2_edit_dialog_button_press_cb (GtkWidget *textview,
 		)
 	{
 		_e2_edit_dialog_show_context_menu (textview, 3, event->time, rt);
+		NEEDOPENBGL
 		return TRUE;
 	}
+	NEEDOPENBGL
 	return FALSE;
 }
 /**
@@ -1325,6 +1382,7 @@ static void _e2_edit_dialog_response_cb (GtkDialog *dialog, gint response,
 	E2_ViewDialogRuntime *rt)
 {
 	printd (DEBUG, "response_cb (dialog:_,response:%d,rt:_)", response);
+	NEEDCLOSEBGL
 	switch (response)
 	{
 		case E2_RESPONSE_FIND:
@@ -1332,18 +1390,21 @@ static void _e2_edit_dialog_response_cb (GtkDialog *dialog, gint response,
 			if (rt->spelldata != NULL)
 				_e2_edit_dialog_cancel_spell (rt);
 #endif
+			NEEDOPENBGL
 			_e2_edit_dialog_find_cb (NULL, rt);
-			break;
+			return;
 		case E2_RESPONSE_USER1:	//replace
 #ifdef E2_SPELLCHECK
 			if (rt->spelldata != NULL)
 				_e2_edit_dialog_cancel_spell (rt);
 #endif
+			NEEDOPENBGL
 			_e2_edit_dialog_replace_cb (NULL, rt);
-			break;
+			return;
 		case E2_RESPONSE_USER2:	//save
+			NEEDOPENBGL
 			_e2_edit_dialog_save_cb (NULL, rt);
-			break;
+			return;
 #ifdef E2_SPELLCHECK
 		case E2_RESPONSE_USER3:	//cancel spelling
 			_e2_edit_dialog_cancel_spell (rt);
@@ -1355,6 +1416,7 @@ static void _e2_edit_dialog_response_cb (GtkDialog *dialog, gint response,
 			_e2_edit_dialog_destroy (rt, (response != GTK_RESPONSE_CLOSE));
 			break;
 	}
+	NEEDOPENBGL
 }
 
 #ifdef E2_TRANSIENTBINDINGS
@@ -1374,27 +1436,26 @@ static void _e2_edit_dialog_keybindings (E2_OptionSet *set)
 	//treestore-iter of the full category name
 //	g_strconcat(_C(17),"||||",NULL),  //_("general"
 //	g_strconcat("\t",_C(11),"||||",NULL),  //_("dialogs"
-	g_strconcat("\t\t",_A(43),"||||",NULL),  //_("edit"
-	g_strconcat("\t\t\t|<Control>u","||","echo|\"up\"", NULL),//_A(124),".",_A(125),"|<Control>Home",NULL),
-	g_strconcat("\t\t\t|<Control>n","||","echo|\"down\"", NULL),//_A(124),".",_A(125),"|<Control>End",NULL),
+	g_strconcat("\t\t",_A(46),"||||",NULL),  //_("edit"
+	g_strconcat("\t\t\t|<Control>u","||","echo|\"up\"", NULL),//_A(127),".",_A(128),"|<Control>Home",NULL),
+	g_strconcat("\t\t\t|<Control>n","||","echo|\"down\"", NULL),//_A(127),".",_A(128),"|<Control>End",NULL),
 	g_strdup(">"),
 	NULL);
 }
 # ifdef E2_MOUSECUSTOM
 static void _e2_edit_dialog_mousebindings (E2_OptionSet *set)
 {
-	//the button name strings are parsed by gtk, and no translation is possible
-	//if releases handled, lines must have one extra (=6) separator
 	e2_option_tree_setup_defaults (set,
 	g_strdup("mousebuttons=<"),  //internal name
-	//the column-0 category string(s) here need to match at least the lowest
-	//treestore-iter of the full category name
-//	g_strconcat(_C(17),"|||||",NULL),  //_("general"
-//	g_strconcat("\t",_C(11),"|||||",NULL),  //_("dialogs"
-	g_strconcat("\t\t",_A(43),"|||||",NULL),  //_("edit"
+/*	these lines go into category general.dialogs, so at least 2 leading tabs
+	the button name strings are parsed by gtk, and no translation is possible, button codes may be 1 ... whatever
+	columns: 0 cat, 1 button, 2 dblclick, 3 trplclick, [4 release,] 5 action, 6 action_data
+	if releases handled (#ifdef MCOL_REL), lines must have one extra (=6) separator
+*/
+	g_strconcat("\t\t",_A(46),"|||||",NULL),  //_("edit"
 #ifdef WITH_KEYFAKE
-	g_strconcat("\t\t\t|<Control>4","|||",_A(124),".",_A(125),"|<Control>Home",NULL),
-	g_strconcat("\t\t\t|<Control>5","|||",_A(124),".",_A(125),"|<Control>End",NULL),
+	g_strconcat("\t\t\t|<Control>4","|||",_A(127),".",_A(128),"|<Control>Home",NULL),
+	g_strconcat("\t\t\t|<Control>5","|||",_A(127),".",_A(128),"|<Control>End",NULL),
 #endif
 	//MORE HERE
 	g_strdup(">"),
@@ -1409,10 +1470,10 @@ static void _e2_edit_dialog_mousegestures (E2_OptionSet *set)
 	//treestore-iter of the full category name
 //	g_strconcat(_C(17),"||||",NULL),  //_("general"
 //	g_strconcat("\t",_C(11),"||||",NULL),  //_("dialogs"
-	g_strconcat("\t\t",_A(43),"||||",NULL),  //_("edit"
+	g_strconcat("\t\t",_A(46),"||||",NULL),  //_("edit"
 #ifdef WITH_KEYFAKE
-	g_strconcat("\t\t\t|<Alt>3|^|8,2|",_A(124),".",_A(125),"|Page_Up",NULL), //"key.fake"
-	g_strconcat("\t\t\t|<Alt>3||2,8|",_A(124),".",_A(125),"|Page_Down",NULL), //"key.fake"
+	g_strconcat("\t\t\t|<Alt>3|^|8,2|",_A(127),".",_A(128),"|Page_Up",NULL), //"key.fake"
+	g_strconcat("\t\t\t|<Alt>3||2,8|",_A(127),".",_A(128),"|Page_Down",NULL), //"key.fake"
 #endif
 	//MORE HERE
 	g_strdup(">"),
@@ -1465,7 +1526,7 @@ static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_Vie
 	else
 	{	//edit output pane buffer
 		const gchar *home = g_get_home_dir ();
-		gchar *name = g_strconcat (_C(28), "-", _A(43), NULL);	//_("output-edit")
+		gchar *name = g_strconcat (_C(28), "-", _A(46), NULL);	//_("output-edit")
 		rt->localpath = g_build_filename (home, name, NULL);
 		rt->charset = "UTF-8";	//the text is from our buffer
 		g_free (name);
@@ -1558,14 +1619,21 @@ static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_Vie
 	gtk_box_reorder_child (GTK_BOX (dialog_vbox), rt->panel, 1);
 
 	//add handlebox
+#ifdef USE_GTK3_4
+	GtkWidget *hndlbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	//if context menu wanted, add event box c.f. toolbar
+#else
 	GtkWidget *hndlbox = gtk_handle_box_new ();
-	gtk_handle_box_set_shadow_type (GTK_HANDLE_BOX (hndlbox),
-		GTK_SHADOW_NONE);
-	gtk_handle_box_set_handle_position (GTK_HANDLE_BOX (hndlbox),
-		GTK_POS_LEFT);
+	gtk_handle_box_set_shadow_type (GTK_HANDLE_BOX (hndlbox), GTK_SHADOW_NONE);
+	gtk_handle_box_set_handle_position ((GtkHandleBox*)hndlbox, GTK_POS_LEFT);
+#endif
 	gtk_container_add (GTK_CONTAINER (rt->panel), hndlbox);
 
+#ifdef USE_GTK3_0
+	GtkWidget *vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
+#endif
 	gtk_container_add (GTK_CONTAINER (hndlbox), vbox);
 	//create search bar
 	rt->sgroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
@@ -1575,7 +1643,11 @@ static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_Vie
 
 	//create (hidden) replace bar, with buttons-area same size as buttons in search bar
 	//(so that the corresponding expandable entries are the same size)
+#ifdef USE_GTK3_0
+	rt->replacebar = hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	rt->replacebar = hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	//add replace-string combo
 	//cuz E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, downstream calls
 	//gtk_widget_set_name() which MAY? need BGL closed
@@ -1634,7 +1706,7 @@ static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_Vie
 	if (asciicode != 0)
 		replace_keycode = asciicode;
 	rt->replacebtn =
-	e2_dialog_add_simple_button (rt->dialog, GTK_STOCK_CONVERT, labeltext, E2_RESPONSE_USER1);
+	e2_dialog_add_simple_button (rt->dialog, STOCK_NAME_CONVERT, labeltext, E2_RESPONSE_USER1);
 	e2_widget_set_safetip (rt->replacebtn, _("Replace this match"));
 	labeltext = _("_Find");
 	find_keycode = e2_utils_get_mnemonic_keycode (labeltext);
@@ -1643,8 +1715,11 @@ static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_Vie
 	if (asciicode != 0)
 		find_keycode = asciicode;
 	rt->findbtn =
-	e2_dialog_add_simple_button (rt->dialog, GTK_STOCK_FIND, labeltext, E2_RESPONSE_FIND);
+	e2_dialog_add_simple_button (rt->dialog, STOCK_NAME_FIND, labeltext, E2_RESPONSE_FIND);
 	e2_widget_set_safetip (rt->findbtn, _("Find the next match"));
+
+	e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_CLOSE);
+
 	labeltext = _("_Save");
 	save_keycode = e2_utils_get_mnemonic_keycode (labeltext);
 	asciicode = GPOINTER_TO_UINT (g_hash_table_lookup (app.keysnative,
@@ -1652,10 +1727,10 @@ static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_Vie
 	if (asciicode != 0)
 		save_keycode = asciicode;
 	rt->savebtn =
-	e2_dialog_add_simple_button (rt->dialog, GTK_STOCK_SAVE, labeltext, E2_RESPONSE_USER2);
+	e2_dialog_add_simple_button (rt->dialog, STOCK_NAME_SAVE, labeltext, E2_RESPONSE_USER2);
 	//allow saving of a fresh output buffer
 	gtk_widget_set_sensitive (rt->savebtn, !fileedit);
-	e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_CLOSE);
+
 //	e2_dialog_set_responses (rt->dialog, E2_RESPONSE_FIND, GTK_RESPONSE_CLOSE);
 	//make sure we always exit cleanly
 	e2_dialog_set_negative_response (rt->dialog, GTK_RESPONSE_CLOSE);
@@ -1714,7 +1789,7 @@ static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_Vie
 	//add dialog-specific key bindings, before the key-press callback
 	//see also dialog-general
 	//group name (must be freeable)
-	gchar *category = g_strconcat (_C(17),".",_C(11),".",_A(43),NULL);	//_(general.dialogs.edit
+	gchar *category = g_strconcat (_C(17),".",_C(11),".",_A(46),NULL);	//_(general.dialogs.edit
 	e2_keybinding_enrol (GTK_WIDGET (rt->textview), category, _e2_edit_dialog_keybindings);
 # ifdef E2_MOUSECUSTOM
 	e2_mousebinding_enrol (rt->dialog, category, _e2_edit_dialog_mousebindings);
@@ -1724,16 +1799,15 @@ static void _e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf, E2_Vie
 # endif
 	g_free (category);
 #endif
-
-	g_signal_connect (G_OBJECT (rt->textbuffer), "modified-changed",
-		G_CALLBACK (_e2_edit_dialog_dirty_change_cb), rt);
-	g_signal_connect (G_OBJECT (rt->textview), "popup-menu",
-		G_CALLBACK (_e2_edit_dialog_popup_menu_cb), rt);
 	//this cb does generic stuff and is needed with or without E2_MOUSECUSTOM
 	g_signal_connect (G_OBJECT (rt->textview), "button-press-event",
 		G_CALLBACK (_e2_edit_dialog_button_press_cb), rt);
+	g_signal_connect (G_OBJECT (rt->textview), "popup-menu",
+		G_CALLBACK (_e2_edit_dialog_popup_menu_cb), rt);
 	g_signal_connect (G_OBJECT (rt->textview), "key-press-event",
 		G_CALLBACK (e2_edit_dialog_key_press_cb), rt);
+	g_signal_connect (G_OBJECT (rt->textbuffer), "modified-changed",
+		G_CALLBACK (_e2_edit_dialog_dirty_change_cb), rt);
 
 	gtk_widget_show_all (dialog_vbox);
 	gtk_widget_hide (rt->panel);
@@ -1786,13 +1860,13 @@ static gboolean _e2_edit_dialog_reeditQ (E2_ActionTaskData *qed)
 				command = g_strconcat (editor, " ", qp, NULL);
 				g_free (qp);
 			}
-			gdk_threads_enter ();
+			CLOSEBGL
 			gint res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT, curr_view->treeview
 #ifdef E2_COMMANDQ
 			, TRUE
 #endif
 			);
-			gdk_threads_leave ();
+			OPENBGL
 			F_FREE (utf, localpath);
 			g_free (command);
 			retval = (res == 0);
@@ -1807,7 +1881,7 @@ static gboolean _e2_edit_dialog_reeditQ (E2_ActionTaskData *qed)
 		VPATH ddata = { localpath, qed->currspace };
 #endif
 		E2_ViewDialogRuntime *vrt = NULL;
-		gdk_threads_enter ();
+		CLOSEBGL
 #ifdef E2_VFS
 		_e2_edit_dialog_create (&ddata, NULL, &vrt); //no error check for a reload
 #else
@@ -1827,7 +1901,7 @@ static gboolean _e2_edit_dialog_reeditQ (E2_ActionTaskData *qed)
 		}
 		else
 			retval = FALSE;
-		gdk_threads_leave ();
+		OPENBGL
 	}
 	g_free (localpath);
 	return retval;
@@ -1874,7 +1948,7 @@ gboolean e2_edit_dialog_create (VPATH *localpath, GtkTextBuffer *buf)
 void e2_edit_dialog_actions_register (void)
 {
 	E2_Action action =
-	{g_strconcat(_A(6),".",_A(44),NULL),_e2_edit_dialog_reopen,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	{g_strconcat(_A(6),".",_A(47),NULL),_e2_edit_dialog_reopen,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
 	e2_action_register (&action);
 #ifdef E2_SPELLCHECK
 	//HACK while we're at it, init some session parameters
@@ -1887,7 +1961,7 @@ void e2_edit_dialog_actions_register (void)
 */
 void e2_edit_dialog_options_register (void)
 {
-	gchar *group_name = g_strconcat(_C(11),":",_A(43),NULL); //_("dialogs:edit"
+	gchar *group_name = g_strconcat(_C(11),":",_A(46),NULL); //_("dialogs:edit"
 	//first some options that may, but probably won't, change during the session
 	e2_option_bool_register ("edit-save-backup", group_name, _("backup when saving"),
 		_("When saving an edited file, an existing file with the same name will be renamed"),
