@@ -1,6 +1,6 @@
-/* $Id: e2_file_info_dialog.c 2309 2011-01-17 00:12:22Z tpgww $
+/* $Id: e2_file_info_dialog.c 2993 2014-01-10 02:55:17Z tpgww $
 
-Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -30,6 +30,7 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 # include <dlfcn.h>
 #endif
 #include "e2_dialog.h"
+#include "e2_icons.h"
 
 //max length of the file type string
 #define FILE_TYPE_LENGTH 128
@@ -37,7 +38,6 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 typedef struct _E2_FileInfoRuntime
 {
 	GtkWidget *dialog;
-	gchar *localpath; //path of item being displayed
 	GtkWidget *description;
 	GtkWidget *target;
 	GtkWidget *mime;
@@ -48,40 +48,55 @@ typedef struct _E2_FileInfoRuntime
 	GtkWidget *accessdate;
 	GtkWidget *moddate;
 	GtkWidget *changedate;
+	const gchar *localpath; //path of item being displayed
 } E2_FileInfoRuntime;
 
 /**
-@brief add thousand-separators to the number-string contained in @a number
-@param number Gstring containing the number to be processed
-@return
+@brief convert @a number to string with thousand-separators
+@param number the number to be processed
+@return allocated string, UTF-8 or compatible
 */
-static void _e2_file_info_dialog_prettify_number (GString *number)
+static gchar *_e2_file_info_dialog_prettify_number (guint64 number)
 {
-	//expand number to add separator char(s)
-	const gchar *comma = nl_langinfo (THOUSEP);
+	guint64 whole;
+	gchar *result;
+	gchar tmp[16];
+
+	gboolean expanded = FALSE;
+	const gchar *comma = nl_langinfo (THOUSEP); //this may need conversion to UTF-8
 	if (comma == NULL || *comma == '\0')
 		comma = ",";
-	guint len = number->len;
-	guint ths = len-1;  //0-based index
-	guint i;
-	while (ths > 2 && len < number->allocated_len)
+
+	GString *buf = g_string_new (NULL);
+	while ((whole = number / 1000) > 0)
 	{
-		for (i = len-1; i > ths-3; i--)
-			number->str[i+1] = number->str[i];
-		number->str[i+1] = *comma;
-		number->str[++len] = '\0';
-		ths = i;
+		snprintf (tmp, 16, "%s%u", comma, (guint)number % 1000);
+		buf = g_string_prepend (buf, tmp);
+		number = whole;
+		expanded = TRUE;
 	}
-	number->len = len;
+	snprintf (tmp, 16, "%u", (guint)number);
+	buf = g_string_prepend (buf, tmp);
+
+	if (expanded)
+	{
+		result = e2_utf8_from_locale (buf->str);
+		g_string_free (buf, TRUE);
+	}
+	else
+		result = g_string_free (buf, FALSE);
+
+	return result;
 }
 /**
-@brief get description and mimetype of @a filepath, using the 'file' command
+@brief get description and mimetype of @a filepath, using libmagic and/or xdg-mime command
 @a readable_type and/or @a mime_type are set to NULL if nothing valid is available
-@param localpath data for item to be processed, including ulocalised path string
-@param readable_type store for pointer to determined description string
+Assumes BGL closed
+@param localpath data for item to be processed, including localised path string
+@param readable_type store for pointer to item-description, allocated localised string
 @param mime_type store for pointer to determined mime (and for text files, charset too) string
 
-@return TRUE if both description and mume commands are executed successfully (which happens if file options are bad)
+@return TRUE if both description and mime strings are populated successfully (which happens if file options are bad)
 */
 static gboolean _e2_file_info_dialog_get_file_type (VPATH *localpath,
 	gchar **readable_type, gchar **mime_type)
@@ -92,6 +107,8 @@ static gboolean _e2_file_info_dialog_get_file_type (VPATH *localpath,
 #endif
 #ifdef E2_MAGIC
 		MagicIface ifc;
+		*readable_type = NULL;
+		*mime_type = NULL;
 
 		if (e2_utils_fill_magic_iface (&ifc))
 		{
@@ -108,7 +125,10 @@ static gboolean _e2_file_info_dialog_get_file_type (VPATH *localpath,
 				{
 					results = g_strdup (msg);
 					g_strstrip (results);
-					*readable_type = results;
+					if (*results != '\0')
+						*readable_type = results;
+					else
+						g_free (results);
 				}
 				else
 				{
@@ -129,7 +149,7 @@ static gboolean _e2_file_info_dialog_get_file_type (VPATH *localpath,
 					{
 						results = g_strdup (msg);
 						g_strstrip (results);	//get rid of \n etc
-						if (*(gchar *)results != '\0')
+						if (*results != '\0')
 						{
 							g_free (*mime_type);
 							*mime_type = results;
@@ -151,8 +171,6 @@ static gboolean _e2_file_info_dialog_get_file_type (VPATH *localpath,
 			dlclose (ifc.libhandle);
 		}
 //		e2_output_print_error ("?", FALSE);
-		*readable_type = NULL;
-		*mime_type = NULL;
 		return FALSE;
 #else
 //tag E2_BADQUOTES
@@ -253,6 +271,9 @@ static void _e2_file_info_dialog_copy_cb (GtkMenuItem *menuitem,
 	//some format massaging is needed ...
 	gchar *utf = F_FILENAME_FROM_LOCALE (rt->localpath);
 	const gchar *target;
+
+	NEEDCLOSEBGL
+
 	if (rt->target != NULL)
 		target = gtk_label_get_text (GTK_LABEL(rt->target));
 	else
@@ -282,6 +303,9 @@ static void _e2_file_info_dialog_copy_cb (GtkMenuItem *menuitem,
 			g_free (freeme);
 		}
 	}
+
+	NEEDOPENBGL
+
 	g_string_append_printf (text,
  		"%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n" "%s %s\n",
 		_("Size:"), gtk_label_get_text (GTK_LABEL(rt->size)),
@@ -292,6 +316,7 @@ static void _e2_file_info_dialog_copy_cb (GtkMenuItem *menuitem,
 		_("Content modified:"), gtk_label_get_text (GTK_LABEL(rt->moddate)),
 		_("Property changed:"), gtk_label_get_text (GTK_LABEL(rt->changedate))
 	);
+
 	GtkClipboard *cb = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 	gtk_clipboard_set_text (cb, text->str, text->len);
 	F_FREE (utf, rt->localpath);
@@ -312,8 +337,8 @@ static void _e2_file_info_dialog_copy_cb (GtkMenuItem *menuitem,
 static void _e2_file_info_dialog_show_context_menu (GtkWidget *dialog,
 	guint event_button, guint32 event_time, E2_FileInfoRuntime *rt)
 {
-	GtkWidget *menu = gtk_menu_new ();
-	e2_menu_add (menu, _("_Copy"), GTK_STOCK_COPY,
+	GtkWidget *menu = e2_menu_get ();
+	e2_menu_add (menu, _("_Copy"), STOCK_NAME_COPY,
 		_("Copy displayed data"), _e2_file_info_dialog_copy_cb, rt);
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 		G_CALLBACK (e2_menu_selection_done_cb), NULL);
@@ -338,7 +363,9 @@ static gboolean _e2_file_info_dialog_popup_menu_cb (GtkWidget *dialog,
 	E2_FileInfoRuntime *rt)
 {
 	guint32 event_time = gtk_get_current_event_time ();
+	NEEDCLOSEBGL
 	_e2_file_info_dialog_show_context_menu (dialog, 0, event_time, rt);
+	NEEDOPENBGL
 	return TRUE;
 }
 /**
@@ -359,7 +386,9 @@ static gboolean _e2_file_info_dialog_button_press_cb (GtkWidget *dialog,
 #endif
 		)
 	{
+		NEEDCLOSEBGL
 		_e2_file_info_dialog_show_context_menu (dialog, 3, event->time, rt);
+		NEEDOPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -454,7 +483,7 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	GString *label_text = g_string_sized_new (NAME_MAX + 32);  //this is plenty big for all uses here
 	gchar *s1, *s2;
 	gchar *mime;
-	gchar date_string[32];
+	gchar date_string[48];
     struct stat statbuf;
 	struct tm *tm_ptr;
 	struct passwd *pw_buf;
@@ -464,21 +493,22 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 //tag DEBUGfreeze;
 	//maybe locking gtk out of X during dialog setup will help
 	//CHECKME effects of child-signals received due to file-command pipe ?
-//	gdk_threads_enter ();
+//	CLOSEBGL
 
 	if (e2_fs_lstat (localpath, &statbuf E2_ERR_NONE()))
 	{
-//tag DEBUGfreeze;
-		gdk_threads_leave ();
+		printd (ERROR, "Cannot stat item %s", VPCSTR (localpath));
 		//CHECKME explain to user ?
+//tag DEBUGfreeze;
+//		OPENBGL
 		return CANCEL;
 	}
 
 	//must prevent the default dialog response, it stuffs up Q cleanups
-	gdk_threads_enter ();
+	CLOSEBGL
 	rt.dialog = e2_dialog_create (NULL, NULL, _("file info"),
 		DUMMY_RESPONSE_CB, NULL);
-	gdk_threads_leave ();
+	OPENBGL
 	dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (rt.dialog));
@@ -527,7 +557,7 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 		if (mime != NULL && *mime != '\0')
 		{
 			gchar *m2 = g_strdup (mime);
-			//mime string returned by 'find' is of the form "text/plain; <stuff>"
+			//mime string returned by libmagic is of the form "text/plain; <stuff>"
 			//from freedesktop.org, icon name is of the form media-subtype (e.g. text-plain).
 			//If no specific icon is found, fallback to media-x-generic (e.g. text-x-generic).
 			s1 = strchr (m2, ';');
@@ -537,17 +567,20 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 			if (s1 != NULL)
 				*s1 = '-';
 			GtkIconTheme *icon_theme = gtk_icon_theme_get_for_screen
-				(gtk_widget_get_screen (app.main_window));	//app.main_window->screen;
-			pixbuf = gtk_icon_theme_load_icon (icon_theme, m2,
-				GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL);
+				(gtk_widget_get_screen (app.main_window));
+			gint psize = e2_icons_get_pixsize (GTK_ICON_SIZE_LARGE_TOOLBAR);
+			pixbuf = gtk_icon_theme_load_icon (icon_theme, m2, psize,
+				GTK_ICON_LOOKUP_GENERIC_FALLBACK | GTK_ICON_LOOKUP_USE_BUILTIN,
+				NULL);
 			if (pixbuf == NULL)
 			{
 				if (s1 != NULL)
 				{
 					*s1 = '\0';
 					gchar *inm = g_strconcat (m2, "-x-generic", NULL);
-					pixbuf = gtk_icon_theme_load_icon (icon_theme, inm,
-						GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL);
+					pixbuf = gtk_icon_theme_load_icon (icon_theme, inm, psize,
+						GTK_ICON_LOOKUP_GENERIC_FALLBACK | GTK_ICON_LOOKUP_USE_BUILTIN,
+						NULL);
 					g_free (inm);
 					if (pixbuf == NULL)
 					{	//fallback to anything that looks half-reasonable !
@@ -560,8 +593,9 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 		//					printd (DEBUG, "candidate name = %s", inm);
 							if (strstr (inm, s1) != NULL)
 							{
-								pixbuf = gtk_icon_theme_load_icon (icon_theme, inm,
-									GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL);
+								pixbuf = gtk_icon_theme_load_icon (icon_theme, inm, psize,
+									GTK_ICON_LOOKUP_GENERIC_FALLBACK | GTK_ICON_LOOKUP_USE_BUILTIN,
+									NULL);
 								break;
 							}
 						}
@@ -599,27 +633,25 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	else if (S_ISDIR (statbuf.st_mode))
 	{
 		guint64 total;
+		const gchar *dir =  _("Directory");
 
 		total = 0;
 		e2_fs_tw (localpath, _e2_file_info_dialog_count_twcb, &total, 1, E2TW_QT | E2TW_PHYS E2_ERR_NONE());
-		s1 =  _("Directory");
 		if (total > 1)
 		{
-			g_string_printf (label_text, "%"PRIu64, total - 1);
-			_e2_file_info_dialog_prettify_number (label_text);
-			gchar *countstr = g_strdup (label_text->str);
+			gchar *countstr = _e2_file_info_dialog_prettify_number (total);
 			total = 0;
 			e2_fs_tw (localpath, _e2_file_info_dialog_usage_twcb, &total, -1, E2TW_PHYS E2_ERR_NONE());
-			g_string_printf (label_text, "%"PRIu64, total - statbuf.st_size);
-			_e2_file_info_dialog_prettify_number (label_text);
+			s1 = _e2_file_info_dialog_prettify_number (total);
 			s2 = g_strdup_printf (
-				_("%s containing %s item(s) which occupy %s byte(s)"), s1, countstr, label_text->str);
+				_("%s containing %s item(s) which occupy %s byte(s)"), dir, countstr, s1);
 			g_free (countstr);
+			g_free (s1);
 		}
 		else
-			s2 = g_strdup_printf ("%s - %s", s1, _("empty"));
+			s2 = g_strdup_printf ("%s - %s", dir, _("empty"));
 
-		GtkWidget *image = e2_widget_get_icon (GTK_STOCK_DIRECTORY,
+		GtkWidget *image = e2_widget_get_icon (STOCK_NAME_DIRECTORY,
 						GTK_ICON_SIZE_LARGE_TOOLBAR);
 		if (image != NULL)
 		{
@@ -769,8 +801,11 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 		rt.target = NULL;	//don't use this label when copying
 
 	table = e2_widget_add_table (dialog_vbox, 10, 2, FALSE, TRUE, E2_PADDING);
+#ifdef USE_GTK3_2
+	gtk_grid_set_column_spacing (GTK_GRID (table), E2_PADDING);
+#else
 	gtk_table_set_col_spacings (GTK_TABLE (table), E2_PADDING);
-
+#endif
 	if (mime != NULL)
 	{
 		if (g_str_has_prefix (mime, "text/"))
@@ -779,6 +814,8 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 			if (s2 != NULL)
 			{
 				s1 = g_utf8_strup (s2 + 9, -1);	//uppercase encoding
+				if (*(s2-1) == ';')
+					s2--;
 				*s2 = '\0';
 				s2 = g_strconcat (mime, "\n", s1, NULL);
 				g_free (s1);
@@ -795,8 +832,8 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 			s1 = _("Mime:");
 			s2 = mime;
 		}
-		e2_widget_add_mid_label_to_table (table, s1, 0, 0,1,0,1);
-		rt.mime = e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,0,1);
+		e2_widget_add_mid_label_to_table (table, s1, 0.0, 0,1,0,1);
+		rt.mime = e2_widget_add_mid_label_to_table (table, s2, 0.0, 1,2,0,1);
 		if (s2 != mime)
 		{
 			g_free (s1);
@@ -806,69 +843,78 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	else
 		rt.mime = NULL;
 
-	e2_widget_add_mid_label_to_table (table, _("Size:"), 0, 0,1,1,2);
+	e2_widget_add_mid_label_to_table (table, _("Size:"), 0.0, 0,1,1,2);
+
+	s1 = _e2_file_info_dialog_prettify_number (statbuf.st_size);
+	g_string_printf (label_text, _("%s bytes"), s1);
+	g_free (s1);
 
-	g_string_printf (label_text, "%"PRIu64, statbuf.st_size);
-	_e2_file_info_dialog_prettify_number (label_text);
-	label_text = g_string_append_c (label_text, ' ');
-	label_text = g_string_append (label_text, _("bytes"));
 	rt.size =
-	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,1,2);
+	e2_widget_add_mid_label_to_table (table, label_text->str, 0.0, 1,2,1,2);
 
+#ifdef USE_GTK3_2
+	GtkWidget *sep = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+	gtk_grid_attach (GTK_GRID (table), sep, 0,2,2,1);
+#else
 	GtkWidget *sep = gtk_hseparator_new ();
 	gtk_table_attach_defaults (GTK_TABLE (table), sep, 0,2,2,3);
-	e2_widget_add_mid_label_to_table (table, _("User:"), 0, 0,1,3,4);
+#endif
+	e2_widget_add_mid_label_to_table (table, _("User:"), 0.0, 0,1,3,4);
 	if ((pw_buf = getpwuid (statbuf.st_uid)) != NULL)
 		g_string_printf (label_text, "%s", pw_buf->pw_name);
 	else
 		g_string_printf (label_text, "%d", (guint) statbuf.st_uid);
 	rt.user =
-	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,3,4);
+	e2_widget_add_mid_label_to_table (table, label_text->str, 0.0, 1,2,3,4);
 
-	e2_widget_add_mid_label_to_table (table, _("Group:"), 0, 0,1,4,5);
+	e2_widget_add_mid_label_to_table (table, _("Group:"), 0.0, 0,1,4,5);
 	if ((grp_buf = getgrgid (statbuf.st_gid)) != NULL)
 		g_string_printf (label_text, "%s", grp_buf->gr_name);
 	else
 		g_string_printf (label_text, "%d", (guint) statbuf.st_gid);
 	rt.group =
-	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,4,5);
+	e2_widget_add_mid_label_to_table (table, label_text->str, 0.0, 1,2,4,5);
 	gchar *perm_string = e2_fs_get_perm_string (statbuf.st_mode);
 	label_text = g_string_assign (label_text, g_utf8_next_char (perm_string));	//skip the "type code" in 1st byte of string
 	g_free (perm_string);
 	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 6), ' ');
 	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 3), ' ');
-	e2_widget_add_mid_label_to_table (table, _("Permissions:"), 0, 0,1,5,6);
+	e2_widget_add_mid_label_to_table (table, _("Permissions:"), 0.0, 0,1,5,6);
 	rt.permissions =
-	e2_widget_add_mid_label_to_table (table, label_text->str, 0, 1,2,5,6);
+	e2_widget_add_mid_label_to_table (table, label_text->str, 0.0, 1,2,5,6);
 
+#ifdef USE_GTK3_2
+	sep = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+	gtk_grid_attach (GTK_GRID (table), sep, 0,6,2,1);
+#else
 	sep = gtk_hseparator_new ();
 	gtk_table_attach_defaults (GTK_TABLE (table), sep, 0,2,6,7);
-
-	s1 = "%a %Od %b %EY %OI:%OM %p";
+#endif
+	s1 = e2_option_str_get ("info-date-format");
 	tm_ptr = localtime (&statbuf.st_atime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Accessed:"), 0, 0,1,7,8);
+	e2_widget_add_mid_label_to_table (table, _("Accessed:"), 0.0, 0,1,7,8);
 	rt.accessdate =
-	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,7,8);
+	e2_widget_add_mid_label_to_table (table, s2, 0.0, 1,2,7,8);
 	e2_widget_set_safetip (rt.accessdate, _("Item was last opened, run, read etc"));
 	g_free (s2);
 
 	tm_ptr = localtime (&statbuf.st_mtime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Content modified:"), 0, 0,1,8,9);
+	e2_widget_add_mid_label_to_table (table, _("Content modified:"), 0.0, 0,1,8,9);
 	rt.moddate =
-	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,8,9);
+	e2_widget_add_mid_label_to_table (table, s2, 0.0, 1,2,8,9);
 	e2_widget_set_safetip (rt.moddate, _("Item content was last changed"));
 	g_free (s2);
 
 	tm_ptr = localtime (&statbuf.st_ctime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Property changed:"), 0, 0,1,9,10);
+	e2_widget_add_mid_label_to_table (table, _("Property changed:"), 0.0, 0,1,9,10);
 	rt.changedate =
-	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,9,10);
+	e2_widget_add_mid_label_to_table (table, s2, 0.0, 1,2,9,10);
 	e2_widget_set_safetip (rt.changedate,
 		_("Item name, permission, owner, etc was last changed"));
 	g_free (s2);
@@ -878,19 +924,15 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 		g_free (mime);
 
 //tag DEBUGfreeze2;
-//	gdk_threads_leave ();
-
-	//block until user responds
-	E2_DialogFlags flags = E2_DIALOG_BLOCKED | E2_DIALOG_CLOSELOCK | E2_DIALOG_FREE;
+//	OPENBGL
 
 	E2_Button local_btn;
 	if (multi)
 	{
-		e2_dialog_set_negative_response (rt.dialog, E2_RESPONSE_NOTOALL);
 		local_btn = E2_BUTTON_CANCEL;
 		local_btn.showflags &= ~E2_BTN_DEFAULT;
 		e2_dialog_add_defined_button (rt.dialog, &local_btn);
-		flags |= E2_DIALOG_MULTI;
+		e2_dialog_set_negative_response (rt.dialog, E2_RESPONSE_NOTOALL);
 	}
 	local_btn = E2_BUTTON_CLOSE;
 	local_btn.showflags |= E2_BTN_DEFAULT;
@@ -902,9 +944,27 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 		G_CALLBACK (_e2_file_info_dialog_button_press_cb), &rt);
 
 //tag DEBUGfreeze
-	gdk_threads_enter ();
+	CLOSEBGL
 	e2_dialog_setup (rt.dialog, app.main_window);
-	gdk_threads_leave ();
+	OPENBGL
+
+	//block until user responds
+	E2_DialogFlags flags = E2_DIALOG_BLOCKED | E2_DIALOG_CLOSELOCK | E2_DIALOG_FREE;
+	if (multi)
+		flags |= E2_DIALOG_MULTI;
 
 	return (e2_dialog_run (rt.dialog, NULL, flags));
 }
+
+/**
+@brief register config option related to file-info dialog
+@return
+*/
+void e2_file_info_dialog_options_register (void)
+{
+	gchar *group_name = g_strconcat(_C(11),":",_("file information"),NULL); //_("dialogs:file information"
+	e2_option_str_register ("info-date-format", group_name, _("displayed date/time format"),
+		_("Pattern provided to strftime() to format date/time strings in the dialog"),
+		NULL, "%a %d %b %Y %I:%M %p",
+		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP);
+}
