@@ -1,6 +1,6 @@
-/* $Id: e2_filetype_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_filetype_dialog.c 2911 2013-11-13 01:01:29Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -22,6 +22,7 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #include "e2_dialog.h"
 #include "e2_task.h"
 #include "e2_filetype.h"
+#include "e2_icons.h"
 
 #ifndef OLDFTDLG
 //no translation of this
@@ -330,15 +331,18 @@ static void _e2_edftdlg_update_category (GtkTreePath *path,
 static void _e2_edftdlg_cell_edited_cb (GtkCellRendererText *renderer,
 	gchar *path_string, gchar *new_text, E2_FileTypeDlgRuntime *rt)
 {
-	if (new_text == NULL) return;	//the user did not finish editing a new cell
 	GtkTreeIter iter;
+	if (new_text == NULL)
+		return;	//the user did not finish editing a new cell
 	gtk_tree_model_get_iter_from_string
 		(GTK_TREE_MODEL (rt->stores[rt->currentID]), &iter, path_string);
 	gpointer colptr = g_object_get_data (G_OBJECT (renderer), "col_num");
 	gint colnum = GPOINTER_TO_INT (colptr);
 	if (colnum == -1)	//this is the categories renderer
 		colnum = 0;
+	NEEDCLOSEBGL
 	gtk_tree_store_set (rt->stores[rt->currentID], &iter, colnum, new_text, -1);
+	NEEDOPENBGL
 }
 /**
 @brief category treeview selection-changed callback
@@ -352,6 +356,7 @@ static void _e2_edftdlg_cat_sel_change_cb (GtkTreeSelection *treeselection,
 	E2_FileTypeDlgRuntime *rt)
 {
 	GtkTreePath *path;
+	NEEDCLOSEBGL
 	gtk_tree_view_get_cursor (GTK_TREE_VIEW (rt->views[E2_CATPANE]), &path, NULL);
 	if (path != NULL)
 	{
@@ -359,6 +364,7 @@ static void _e2_edftdlg_cat_sel_change_cb (GtkTreeSelection *treeselection,
 		_e2_edftdlg_update_category (path, rt);
 		gtk_tree_path_free (path);
 	}
+	NEEDOPENBGL
 }
 /**
 @brief determine which pane's store is current, and the first selected iter
@@ -403,13 +409,15 @@ This is a "focus-in-event" callback, applied to each treeview
 static gboolean _e2_edftdlg_focus_in_cb (GtkWidget *treeview,
 	GdkEventFocus *event, E2_FileTypeDlgRuntime *rt)
 {
+	GtkTreeIter iter;
 	if (treeview == rt->views[E2_CATPANE])
 		rt->currentID = E2_CATPANE;
 	else if (treeview == rt->views[E2_EXTPANE])
 		rt->currentID = E2_EXTPANE;
 	else
 		rt->currentID = E2_CMDPANE;
-	GtkTreeIter iter;
+	
+	NEEDCLOSEBGL
 	if (!_e2_edftdlg_find_selected (rt, &iter))
 	{
 		GtkTreePath*path = gtk_tree_path_new_first ();
@@ -418,6 +426,7 @@ static gboolean _e2_edftdlg_focus_in_cb (GtkWidget *treeview,
 		gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, NULL, TRUE);
 		gtk_tree_path_free (path);
 	}
+	NEEDOPENBGL
 	return FALSE;
 }
 /**
@@ -483,13 +492,14 @@ static void _e2_edftdlg_response_cb (GtkDialog *dialog, gint response,
 	E2_FileTypeDlgRuntime *rt)
 {
 	GtkTreeIter iter;
+	NEEDCLOSEBGL
 	switch (response)
 	{
 		case E2_RESPONSE_APPLY:
 			_e2_edftdlg_save_current_category (rt); //this might be dirty
 			e2_option_tree_unbackup (rt->typeset, FALSE);	//get rid of old backup data
 			g_hash_table_destroy (rt->buffer_hash);
-			g_free (rt);
+			DEALLOCATE (E2_FileTypeDlgRuntime, rt);
 			gtk_widget_destroy (filetypes_dialog);
 			filetypes_dialog = NULL;
 			e2_filetype_apply_allnew ();
@@ -573,10 +583,11 @@ static void _e2_edftdlg_response_cb (GtkDialog *dialog, gint response,
 				e2_tree_delete (GTK_TREE_VIEW (rt->views[rt->currentID]));
 			break;
 		default:
+			NEEDOPENBGL
 			_e2_edftdlg_cancel_cb (NULL, rt);
-			break;
+			return;
 	}
-	return;
+	NEEDOPENBGL
 }
 /**
 series of context-menu callback functions
@@ -590,6 +601,7 @@ static void _e2_edftdlg_menu_copy_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *
 {
 	GtkTreeView *treeview = GTK_TREE_VIEW (rt->views[rt->currentID]);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
+	NEEDCLOSEBGL
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
 		GList *copied = e2_tree_copy (treeview);
@@ -598,19 +610,25 @@ static void _e2_edftdlg_menu_copy_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *
 			g_list_length (copied));
 		g_hash_table_replace (rt->buffer_hash, id_string, copied);
 	}
+	NEEDOPENBGL
 }
 static void _e2_edftdlg_menu_cut_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	GtkTreeView *treeview = GTK_TREE_VIEW (rt->views[rt->currentID]);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
+	NEEDCLOSEBGL
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
+		NEEDOPENBGL
 		_e2_edftdlg_menu_copy_cb (item, rt);
+		NEEDCLOSEBGL
 		e2_tree_delete (treeview);
 	}
+	NEEDOPENBGL
 }
 static void _e2_edftdlg_menu_paste_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
+	NEEDCLOSEBGL
 	//as a special case, if the store is empty, setup to add the data
 	gboolean empty = (gtk_tree_model_iter_n_children
 		(GTK_TREE_MODEL (rt->stores[rt->currentID]), NULL) == 0);
@@ -632,18 +650,28 @@ static void _e2_edftdlg_menu_paste_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime
 
 	if (empty)
 		e2_tree_delete (treeview);
+	NEEDOPENBGL
 }
+
 static void _e2_edftdlg_menu_add_below_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
+	NEEDCLOSEBGL
 	_e2_edftdlg_insert_row (rt);
+	NEEDOPENBGL
 }
+
 static void _e2_edftdlg_menu_move_up_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
+//	NEEDOPENBGL
 	_e2_edftdlg_response_cb (GTK_DIALOG (filetypes_dialog), E2_RESPONSE_USER1, rt);
+//	NEEDCLOSEBGL
 }
+
 static void _e2_edftdlg_menu_move_down_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
+//	NEEDOPENBGL
 	_e2_edftdlg_response_cb (GTK_DIALOG (filetypes_dialog), E2_RESPONSE_USER2, rt);
+//	NEEDCLOSEBGL
 }
 /**
 @brief set popup menu position
@@ -689,26 +717,26 @@ static void _e2_edftdlg_set_menu_position (GtkMenu *menu,
 static void _e2_edftdlg_menu_create (GtkWidget *treeview,
 	guint button, guint32 time, E2_FileTypeDlgRuntime *rt)
 {
-	GtkWidget *menu = gtk_menu_new ();
-	e2_menu_add (menu, _("Cu_t"), GTK_STOCK_CUT,
+	GtkWidget *menu = e2_menu_get ();
+	e2_menu_add (menu, _("Cu_t"), STOCK_NAME_CUT,
 		_("Cut selected row(s)"), _e2_edftdlg_menu_cut_cb, rt);
-	e2_menu_add (menu, _("_Copy"), GTK_STOCK_COPY,
+	e2_menu_add (menu, _("_Copy"), STOCK_NAME_COPY,
 		_("Copy selected row(s)"), _e2_edftdlg_menu_copy_cb, rt);
 	//check for valid stored content
 	gchar *id_string = g_strdup_printf (E2_FTDLG_SIG, rt->currentID);
 	gboolean state = (g_hash_table_lookup (rt->buffer_hash, id_string) != NULL);
 	g_free (id_string);
-	GtkWidget *item = e2_menu_add (menu, _("_Paste"), GTK_STOCK_PASTE,
+	GtkWidget *item = e2_menu_add (menu, _("_Paste"), STOCK_NAME_PASTE,
 		_("Paste previously copied or cut row(s) after current row"),
 		_e2_edftdlg_menu_paste_cb, rt);
 	gtk_widget_set_sensitive (item, state);
 	e2_menu_add_separator (menu);
-	e2_menu_add (menu, _("_Add"), GTK_STOCK_ADD,
+	e2_menu_add (menu, _("_Add"), STOCK_NAME_ADD,
 		_("Add a row after the current one"), _e2_edftdlg_menu_add_below_cb, rt);
 	e2_menu_add_separator (menu);
-	e2_menu_add (menu, _("_Up"), GTK_STOCK_GO_UP,
+	e2_menu_add (menu, _("_Up"), STOCK_NAME_GO_UP,
 		_("Move selected row up"), _e2_edftdlg_menu_move_up_cb, rt);
-	e2_menu_add (menu, _("_Down"), GTK_STOCK_GO_DOWN,
+	e2_menu_add (menu, _("_Down"), STOCK_NAME_GO_DOWN,
 		_("Move selected row down"), _e2_edftdlg_menu_move_down_cb, rt);
 
 	g_signal_connect (G_OBJECT (menu), "selection-done",
@@ -734,7 +762,9 @@ static gboolean _e2_edftdlg_popup_menu_cb (GtkWidget *treeview,
 	E2_FileTypeDlgRuntime *rt)
 {
 	guint32 event_time = gtk_get_current_event_time ();
+	NEEDCLOSEBGL
 	_e2_edftdlg_menu_create (treeview, 0, event_time, rt);
+	NEEDOPENBGL
 	return TRUE;
 }
 /**
@@ -749,7 +779,8 @@ static gboolean _e2_edftdlg_popup_menu_cb (GtkWidget *treeview,
 static gboolean _e2_edftdlg_button_press_cb (GtkWidget *treeview,
 	GdkEventButton *event, E2_FileTypeDlgRuntime *rt)
 {
-//FIXME find a way to cancel editing of a renderer that is clicked again
+	//FIXME find a way to cancel editing of a renderer that is clicked again
+	NEEDCLOSEBGL
 	if (event->button == 1 && event->type == GDK_BUTTON_PRESS //no multi-clicks
 #ifdef E2_MOUSECUSTOM
 		&& (event->state & E2_MODIFIER_MASK) == 0
@@ -782,8 +813,10 @@ static gboolean _e2_edftdlg_button_press_cb (GtkWidget *treeview,
 		)
 	{
 		_e2_edftdlg_menu_create (treeview, 3, event->time, rt);
+		NEEDOPENBGL
 		return TRUE;
 	}
+	NEEDOPENBGL
 	return FALSE;
 }
 /**
@@ -796,11 +829,14 @@ static gboolean _e2_edftdlg_button_press_cb (GtkWidget *treeview,
 */
 static gboolean _e2_edftdlg_cancel_cb (GtkWidget *widget, E2_FileTypeDlgRuntime *rt)
 {
+	NEEDCLOSEBGL
 	gtk_widget_destroy (filetypes_dialog);
-	filetypes_dialog = NULL;
 	e2_option_tree_unbackup (rt->typeset, TRUE);	//undo everything
-	g_free (rt);
 	gtk_widget_grab_focus (curr_view->treeview);
+	NEEDOPENBGL
+	filetypes_dialog = NULL;
+	DEALLOCATE (E2_FileTypeDlgRuntime, rt);
+
 	return TRUE;
 }
 /**
@@ -816,9 +852,11 @@ static gboolean _e2_edftdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	E2_FileTypeDlgRuntime *rt)
 {
 	if (event->keyval == GDK_Escape)
-	  //restore tree data and clean up
+	{
+//		NEEDOPENBGL
+		//restore tree data and clean up
 		return (_e2_edftdlg_cancel_cb (NULL, rt));
-
+	}
 	return FALSE;
 }
 /**
@@ -860,10 +898,18 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 		GTK_DIALOG (filetypes_dialog)->vbox,
 #endif
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, E2_PADDING_SMALL);
-	gtk_widget_set_size_request (sw, 0, 400);
+	gtk_widget_set_size_request (sw, 1, 400);
 
+#ifdef USE_GTK3_0
+	GtkWidget *hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
+#endif
+#ifdef USE_GTK3_8
+	gtk_container_add (GTK_CONTAINER (sw), hbox);
+#else
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), hbox);
+#endif
 	gtk_viewport_set_shadow_type (
 #ifdef USE_GTK2_14
 		GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (sw))),
@@ -901,7 +947,7 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 	gtk_tree_view_column_set_visible (expander_col, FALSE);
 #endif
 	gtk_container_add (GTK_CONTAINER (cat_sw), rt->views[E2_CATPANE]);
-	gtk_widget_set_size_request (cat_sw, 150, 0);
+	gtk_widget_set_size_request (cat_sw, 150, 1);
 
 	//extensions scrolled window
 	GtkWidget *ext_sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC,
@@ -922,7 +968,7 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 	gtk_tree_view_insert_column_with_attributes
 		(GTK_TREE_VIEW (rt->views[E2_EXTPANE]), -1, _("Extensions"), renderer, "text", 0, NULL);
 	gtk_container_add (GTK_CONTAINER (ext_sw), rt->views[E2_EXTPANE]);
-	gtk_widget_set_size_request (ext_sw, 100, 0);
+	gtk_widget_set_size_request (ext_sw, 100, 1);
 	GtkTreeSelection *selection = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW (rt->views[E2_EXTPANE]));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
@@ -958,7 +1004,7 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 	g_object_set (G_OBJECT (column), "resizable", TRUE, NULL);
 
 	gtk_container_add (GTK_CONTAINER (cmd_sw), rt->views[E2_CMDPANE]);
-	gtk_widget_set_size_request (cmd_sw, 400, 0);
+	gtk_widget_set_size_request (cmd_sw, 400, 1);
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (rt->views[E2_CMDPANE]));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
@@ -1016,19 +1062,19 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 		G_CALLBACK (_e2_edftdlg_focus_in_cb), rt);
 
 	//add buttons, custom and standard
-	E2_Button local_btn = { _("_Up"), GTK_STOCK_GO_UP,
+	E2_Button local_btn = { _("_Up"), STOCK_NAME_GO_UP,
 		_("Move the selected row one place up"),
 		E2_BTN_TIPPED, E2_BTN_TIPPED, E2_RESPONSE_USER1 };
 	e2_dialog_add_defined_button (filetypes_dialog, &local_btn);
 
 	local_btn.label = _("_Down");
-	local_btn.stock = GTK_STOCK_GO_DOWN;
+	local_btn.name = STOCK_NAME_GO_DOWN;
 	local_btn.tip = _("Move the selected row one place down");
 	local_btn.response = E2_RESPONSE_USER2;
 	e2_dialog_add_defined_button (filetypes_dialog, &local_btn);
 
 	local_btn.label = _("_Add");
-	local_btn.stock = GTK_STOCK_ADD;
+	local_btn.name = STOCK_NAME_ADD;
 	local_btn.tip = _("Add a row after the currently selected one");
 	local_btn.response = E2_RESPONSE_USER3;
 	e2_dialog_add_defined_button (filetypes_dialog, &local_btn);
@@ -1067,6 +1113,8 @@ typedef struct _E2_FileTypeDlg2Runtime
 static void _e2_filetype_dialog_response_cb (GtkDialog *dialog, gint response,
 	gint *store)
 {
+//	NEEDOPENBGL
+//	NEEDCLOSEBGL
 	printd (DEBUG, "response_cb (dialog:_,response:%d,rt:_)", response);
 	*store = response;
 }
@@ -1114,7 +1162,7 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 	base = g_path_get_basename (public);
 	message = g_strdup_printf (
 		_("What would you like to do with\n<b>%s</b>\nin %s ?"), base, dir);
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION, message, title,
+	GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION, message, title,
 		(ResponseFunc)_e2_filetype_dialog_response_cb, &response);
 	g_free (public);
 	g_free (dir);
@@ -1139,7 +1187,7 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 		{
 			if (sb.st_size == 0)
 				rt.view_btn = e2_dialog_add_custom_button_full (dialog, FALSE,
-					E2_RESPONSE_USER1, _("_Edit"), GTK_STOCK_EDIT,
+					E2_RESPONSE_USER1, _("_Edit"), STOCK_NAME_EDIT,
 					_("Edit the file"), NULL, NULL);
 			else if (text)
 				rt.view_btn = e2_dialog_add_custom_button_full (dialog, FALSE,
@@ -1149,10 +1197,10 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 		if (ambig)
 		{
 			e2_dialog_add_custom_button_full (dialog, FALSE,
-				E2_RESPONSE_USER5, _("_Run"), GTK_STOCK_EXECUTE,
+				E2_RESPONSE_USER5, _("_Run"), STOCK_NAME_EXECUTE,
 					_("Execute the item"), NULL, NULL);
 			rt.open_btn = e2_dialog_add_custom_button_full (dialog, TRUE,
-				E2_RESPONSE_USER6, _("_Open"), GTK_STOCK_OPEN,	//FIXME better icon
+				E2_RESPONSE_USER6, _("_Open"), STOCK_NAME_OPEN,	//FIXME better icon
 					_("Open with the default application"), NULL, NULL);
 		}
 		else
@@ -1171,9 +1219,10 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	gtk_widget_grab_focus (curr_view->treeview);
 
-	gchar *action;
 //tag E2_BADQUOTES CHECKME used for action arg
 	gchar *item = e2_utils_quote_string (rt.utfpath);
+	gchar *action = NULL;
+
 	switch (response)
 	{
 	/* these callbacks handle a specific filename, so are usable for output
@@ -1184,19 +1233,13 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 			e2_filetype_config_show (NULL);
 			break;
 		case E2_RESPONSE_USER1:	//edit
-			action = g_strconcat (_A(6),".",_A(43), NULL);
-			e2_action_run_simple_from (action, item, dialog);
-			g_free (action);
+			action = g_strconcat (_A(6),".",_A(46), NULL);
 			break;
 		case E2_RESPONSE_USER2:	//view
-			action = g_strconcat (_A(6),".",_A(106), NULL);
-			e2_action_run_simple_from (action, item, dialog);
-			g_free (action);
+			action = g_strconcat (_A(6),".",_A(109), NULL);
 			break;
 		case E2_RESPONSE_USER3:	//open with
-			action = g_strconcat (_A(6),".",_A(66), NULL);	//Q'd action
-			e2_action_run_simple_from (action, item, dialog);	//don't know if user cancelled this
-			g_free (action);
+			action = g_strconcat (_A(6),".",_A(69), NULL);	//Q'd action
 			break;
 		case E2_RESPONSE_USER5:	//execute
 			//CHECKME no quotes in this case ?
@@ -1208,6 +1251,14 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 		default:
 			break;
 	}
+
+	if (action != NULL)
+	{
+		OPENBGL
+		e2_action_run_simple_from (action, item, dialog);
+		CLOSEBGL
+		g_free (action);
+	}
 	g_free (item);
 
 	//send back "handled" type flag to caller
