@@ -1,6 +1,6 @@
-/* $Id: e2_mkdir_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_mkdir_dialog.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -24,6 +24,7 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #include "e2_mkdir_dialog.h"
 #include "e2_task.h"
 #include "e2_filelist.h"
+#include "e2_icons.h"
 
 //option-pointers shared by all mkdir dialogs in the session
 //FIXME make sure these are NULL'd if optionsets are ever renewed
@@ -578,10 +579,10 @@ static gboolean _e2_mkdirdlg_change_dir_hook (gchar *path, E2_MkdirDialogRuntime
 {
 	g_free (rt->path);
 	rt->path = g_strdup (path);
-	gdk_threads_enter ();
+	CLOSEBGL
 	_e2_mkdirdlg_update_status (rt);
 	_e2_mkdirdlg_update_dialog_size (rt);
-	gdk_threads_leave ();
+	OPENBGL
 	return TRUE;
 }
 /**
@@ -601,10 +602,10 @@ static gboolean _e2_mkdirdlg_change_focus_hook (E2_PaneRuntime *pane_rt, E2_Mkdi
 	rt->path = g_strdup (pane_rt->path);
 	LISTS_UNLOCK
 #endif
-	gdk_threads_enter ();
+	CLOSEBGL
 	_e2_mkdirdlg_update_status (rt);
 	_e2_mkdirdlg_update_dialog_size (rt);
-	gdk_threads_leave ();
+	OPENBGL
 	return TRUE;
 }
 /**
@@ -687,14 +688,14 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry, E2_MkdirDialogRuntime *rt
 		if (e2_fs_mkdir (local, 0777 E2_ERR_PTR()))	//FIXME vfs
 #endif
 		{
-			gdk_threads_leave (); //downstream does local mutex management
+			OPENBGL //downstream does local mutex management
 			e2_fs_error_local (_("Cannot create directory %s"),
 #ifdef E2_VFS
 				&ddata E2_ERR_MSGL());
 #else
 				local E2_ERR_MSGL());
 #endif
-			gdk_threads_enter ();
+			CLOSEBGL
 			E2_ERR_CLEAR
 
 			if (!close)
@@ -729,18 +730,18 @@ static void _e2_mkdirdlg_create_dir (GtkWidget *entry, E2_MkdirDialogRuntime *rt
 #else
 		e2_filelist_request_refresh (rt->path, TRUE);
 #endif
-		e2_list_update_history ((gchar *) dir, &mkdir_history, NULL, 30, FALSE);
+		e2_list_update_history (&mkdir_history, dir, NULL, 30, FALSE);
 	}
 	else	//mkdir failed
 	{
-		gdk_threads_leave (); //downstream does local mutex management
+		OPENBGL //downstream does local mutex management
 		e2_fs_error_local (_("Cannot create directory %s"),
 #ifdef E2_VFS
 			&ddata E2_ERR_MSGL());
 #else
 			local E2_ERR_MSGL());
 #endif
-		gdk_threads_enter ();
+		CLOSEBGL
 		E2_ERR_CLEAR
 
 		if (!close)
@@ -767,6 +768,7 @@ static void _e2_mkdirdlg_response_cb (GtkDialog *dialog, gint response,
 {
 	printd (DEBUG, "mkdir dialog response_cb (dialog:_,response:%d,rt:_)", response);
 	GtkWidget *entry;
+
 	switch (response)
 	{
 		case GTK_RESPONSE_YES:
@@ -779,8 +781,9 @@ static void _e2_mkdirdlg_response_cb (GtkDialog *dialog, gint response,
 #else
 				GTK_BIN (rt->combo)->child;
 #endif
+			NEEDCLOSEBGL
 			const gchar *dir = gtk_entry_get_text (GTK_ENTRY (entry));
-			if (*dir != '\0')
+			if (dir != NULL && *dir != '\0')
 			{
 				if (strchr (dir, '%') != NULL)
 				{
@@ -789,12 +792,13 @@ static void _e2_mkdirdlg_response_cb (GtkDialog *dialog, gint response,
 					{
 						gtk_entry_set_text (GTK_ENTRY (entry), freeme);
 						g_free (freeme);
+						dir = gtk_entry_get_text (GTK_ENTRY (entry));
 					}
 				}
 //				*rt->status = E2_TASK_RUNNING;
 				if (response == E2_RESPONSE_MORE)
 				{
-					e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
+					e2_combobox_prepend_history (rt->combo, dir, 20, FALSE);
 					_e2_mkdirdlg_create_dir (entry, rt, FALSE);
 //					*rt->status = E2_TASK_PAUSED;
 					gtk_widget_grab_focus (entry);
@@ -804,6 +808,7 @@ static void _e2_mkdirdlg_response_cb (GtkDialog *dialog, gint response,
 				else
 					_e2_mkdirdlg_create_dir (entry, rt, TRUE);
 			}
+			NEEDOPENBGL
 		default:
 			break;
 	}
@@ -818,10 +823,10 @@ static gboolean _e2_mkdirdlg_deferred_change_cb (E2_MkdirDialogRuntime *rt)
 	rt->idle_id = 0;	//allow sequential changes ASAP
 	if (GTK_IS_DIALOG (rt->dialog))
 	{
-		gdk_threads_enter ();
+		CLOSEBGL
 		_e2_mkdirdlg_update_status (rt);
 		_e2_mkdirdlg_update_dialog_size (rt);
-		gdk_threads_leave ();
+		OPENBGL
 	}
 	return FALSE;
 }
@@ -836,6 +841,8 @@ name-entry. So GUI update is deferred to idle time, to help eliminate keypress l
 static void _e2_mkdirdlg_changed_cb (GtkEditable *editable, E2_MkdirDialogRuntime *rt)
 {
 //	printd (DEBUG, "mkdir dialog changed cb");
+//	NEEDCLOSEBGL
+//	NEEDOPENBGL
 	if (rt->idle_id == 0)	//don't want re-entrance
 		rt->idle_id = g_idle_add ((GSourceFunc)_e2_mkdirdlg_deferred_change_cb, rt);
 //		rt->idle_id = g_idle_add_full (G_PRIORITY_HIGH,
@@ -859,7 +866,9 @@ static gboolean _e2_mkdirdlg_button_press_cb (GtkWidget *dialog,
 #endif
 		)
 	{
+		NEEDCLOSEBGL
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
+		NEEDOPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -874,7 +883,9 @@ This simply triggers a 'create' type response
 static void _e2_mkdirdlg_activated_cb (GtkWidget *entry, E2_MkdirDialogRuntime *rt)
 {
 	printd (DEBUG, "mkdir dialog activate cb");
+//	NEEDOPENBGL
 	g_signal_emit_by_name (G_OBJECT (rt->dialog), "response", GTK_RESPONSE_YES, rt); //trigger all callbacks
+//	NEEDCLOSEBGL
 }
 /**
 @brief callback for context menu item for toggling "follow active pane"
@@ -887,7 +898,9 @@ static void _e2_mkdirdlg_toggled1_cb (GtkWidget *menu_item, E2_MkdirDialogRuntim
 	printd (DEBUG, "mkdir dialog follow pane toggle");
 	gboolean state = gtk_check_menu_item_get_active
 		(GTK_CHECK_MENU_ITEM (menu_item));
+	NEEDCLOSEBGL
 	_e2_mkdirdlg_update_follow_dir (rt, state);
+	NEEDOPENBGL
 	gpointer p = g_object_get_data (G_OBJECT (rt->dialog), "e2-controller-blocked");
 	if (!GPOINTER_TO_INT (p))
 		e2_option_bool_set_direct (follow_pane, state);
@@ -903,8 +916,10 @@ static void _e2_mkdirdlg_toggled2_cb (GtkWidget *menu_item, E2_MkdirDialogRuntim
 	printd (DEBUG, "mkdir dialog show last entry toggle");
 	rt->opt_show_last = gtk_check_menu_item_get_active
 		(GTK_CHECK_MENU_ITEM (menu_item));
+	NEEDCLOSEBGL
 	_e2_mkdirdlg_update_name (rt);
 	_e2_mkdirdlg_update_status (rt);
+	NEEDOPENBGL
 	gpointer p = g_object_get_data (G_OBJECT (rt->dialog), "e2-controller-blocked");
 	if (!GPOINTER_TO_INT (p))
 		e2_option_bool_set_direct (show_last, rt->opt_show_last);
@@ -920,8 +935,10 @@ static void _e2_mkdirdlg_toggled3_cb (GtkWidget *menu_item, E2_MkdirDialogRuntim
 	printd (DEBUG, "mkdir dialog suggest dir toggle");
 	rt->opt_suggest_dir = gtk_check_menu_item_get_active
 		(GTK_CHECK_MENU_ITEM (menu_item));
+	NEEDCLOSEBGL
 	_e2_mkdirdlg_update_name (rt);
 	_e2_mkdirdlg_update_status (rt);
+	NEEDOPENBGL
 	gpointer p = g_object_get_data (G_OBJECT (rt->dialog), "e2-controller-blocked");
 	if (!GPOINTER_TO_INT (p))
 		e2_option_bool_set_direct (suggest_dir, rt->opt_suggest_dir);
@@ -937,6 +954,7 @@ static void _e2_mkdirdlg_expander_toggled_cb (GtkWidget *expander,
 	GParamSpec *spec, E2_MkdirDialogRuntime *rt)
 {
 //	printd (DEBUG, "mkdir dialog expander toggle");
+	NEEDCLOSEBGL
 	gboolean value = gtk_expander_get_expanded (GTK_EXPANDER (expander));
 	if (value)
 	{
@@ -957,6 +975,8 @@ static void _e2_mkdirdlg_expander_toggled_cb (GtkWidget *expander,
 
 	_e2_mkdirdlg_update_dialog_size (rt);
 
+	NEEDOPENBGL
+
 	GtkWidget *controller = g_object_get_data (G_OBJECT (expander),
 			"e2-controller-widget");
 	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
@@ -1022,8 +1042,8 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 //tag DEBUGfreeze;
 
 	//create dialog
-	gdk_threads_enter ();
-	rt.dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
+	CLOSEBGL
+	rt.dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION,
 		_("What is the new directory's name?"), _("create directory"),
 		(ResponseFunc)_e2_mkdirdlg_response_cb, &rt);
 
@@ -1034,7 +1054,9 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 	rt.combo = e2_combobox_add (vbox, FALSE, E2_PADDING,
 		(ActivateFunc)_e2_mkdirdlg_activated_cb, &rt, &mkdir_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
-	gdk_threads_leave ();
+#ifndef USE_GTK3_0
+	OPENBGL
+#endif
 
 	rt.scrolled = gtk_widget_get_ancestor (vbox, GTK_TYPE_SCROLLED_WINDOW);
 	rt.info_expander = gtk_expander_new (_("info"));
@@ -1080,7 +1102,9 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 	if (connected == NULL)
 		connected = e2_option_get ("dialog-mkdir-connected");
 
-	gdk_threads_enter ();
+#ifndef USE_GTK3_0
+	CLOSEBGL
+#endif
 	//CHECKME seems that this needs BGL closed
 	rt.menu = e2_menu_create_options_menu (rt.dialog, NULL,
 		follow_pane, _e2_mkdirdlg_toggled1_cb, &rt,
@@ -1088,8 +1112,9 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 		show_last, _e2_mkdirdlg_toggled2_cb, &rt,
 		connected, e2_menu_control_cb, rt.dialog,
 		NULL);
-	gdk_threads_leave ();
-
+#ifndef USE_GTK3_0
+	OPENBGL
+#endif
 	//arrange menu cleanup
 	g_object_set_data_full (G_OBJECT (rt.dialog), "menu", rt.menu,
 		(GDestroyNotify) e2_menu_selection_done_cb);
@@ -1099,20 +1124,22 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 
 //do this after create button is created ?
 //tag DEBUGfreeze
-	gdk_threads_enter ();
+#ifndef USE_GTK3_0
+	CLOSEBGL
+#endif
 	//this updates status but not the suggested dir name
 	_e2_mkdirdlg_update_follow_dir (&rt, e2_option_bool_get_direct (follow_pane));
 //tag DEBUGfreeze
-	gdk_threads_leave ();
+	OPENBGL
 	e2_option_connect (rt.dialog, e2_option_bool_get_direct (connected));
 	rt.opt_show_last = e2_option_bool_get_direct (show_last);
 	rt.opt_suggest_dir = e2_option_bool_get_direct (suggest_dir);
 
 //tag DEBUGfreeze
-	gdk_threads_enter ();
+	CLOSEBGL
 	_e2_mkdirdlg_update_name (&rt);
 //tag DEBUGfreeze
-	gdk_threads_leave ();
+	OPENBGL
 
 	E2_Button local_btn;
 	local_btn = E2_BUTTON_MORE;
@@ -1126,7 +1153,7 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 	e2_dialog_add_defined_button (rt.dialog, &local_btn);
 
 	local_btn.label = _("C_reate");	//this appears with "_Cancel"
-	local_btn.stock = GTK_STOCK_DIRECTORY;
+	local_btn.name = STOCK_NAME_DIRECTORY;
 	local_btn.showflags = 0;
 	local_btn.response = GTK_RESPONSE_YES;
 	e2_dialog_add_defined_button (rt.dialog, &local_btn);
@@ -1141,15 +1168,16 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 	if (!e2_option_bool_get_direct (infoshow))
 		gtk_widget_hide (rt.info_box);
 
+//	NEEDOPENBGL
 	_e2_mkdirdlg_deferred_change_cb (&rt);	//init dialog widgets
-
-	gdk_threads_enter ();
+//	NEEDCLOSEBGL
+	CLOSEBGL
 #ifdef GTK3_COMBO_FIX
 	gtk_widget_grab_focus (gtk_bin_get_child (GTK_BIN(rt.combo)));
 #endif
 	e2_dialog_setup (rt.dialog, app.main_window);
 	e2_dialog_run (rt.dialog, NULL, E2_DIALOG_DONT_SHOW_ALL);
-	gdk_threads_leave ();
+	OPENBGL
 
 	//connect signal after showing, to reduce repetition during setup
 	GtkWidget *entry =
@@ -1167,9 +1195,9 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 
 	if (GTK_IS_DIALOG (rt.dialog))
 	{	//not explicitly closed by the user
-		gdk_threads_enter ();
+		CLOSEBGL
 		gtk_widget_destroy (rt.dialog);
-		gdk_threads_leave ();
+		OPENBGL
 	}
 
 	g_free (rt.path);
@@ -1188,7 +1216,7 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 void e2_mkdir_dialog_actions_register ()
 {
 	E2_Action action =
-	{g_strconcat(_A(1),".",_A(60),NULL),_e2_mkdir_dialog_create,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	{g_strconcat(_A(1),".",_A(63),NULL),_e2_mkdir_dialog_create,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
 	e2_action_register (&action);
 	//while we're at it, init some session parameters
 	e2_cache_list_register ("mkdir-history", &mkdir_history);
