@@ -1,6 +1,6 @@
-/* $Id: e2_ownership_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_ownership_dialog.c 2934 2013-11-15 08:08:22Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -47,7 +47,6 @@ typedef struct _E2_OwnersDldRuntime
 static uid_t _e2_ownership_dialog_get_user_id (E2_OwnersDldRuntime *rt)
 {
 	gchar *user;
-	struct passwd *pw_buf;
 	uid_t user_id;
 
 	user = (gchar *) gtk_entry_get_text (
@@ -60,6 +59,8 @@ static uid_t _e2_ownership_dialog_get_user_id (E2_OwnersDldRuntime *rt)
 	user = e2_utf8_to_locale (user);
 	if (user != NULL)
 	{
+		struct passwd *pw_buf;
+
 		if ((pw_buf = getpwnam (user)) != NULL)
 			user_id = pw_buf->pw_uid;
 		else
@@ -127,15 +128,17 @@ This only applies if change is permitted.
 
 @return
 */
-static void _e2_ownership_dialog_toggle_recurse_button_cb
-	(GtkToggleButton *togglebutton, E2_OwnersDldRuntime *rt)
+static void _e2_ownership_dialog_toggle_recurse_button_cb (
+	GtkToggleButton *togglebutton, E2_OwnersDldRuntime *rt)
 {
+//	NEEDCLOSEBGL
 	rt->recurse =
 #ifdef USE_GTK2_14
 		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->recurse_dirs_button));
 #else
 		GTK_TOGGLE_BUTTON (rt->recurse_dirs_button)->active;
 #endif
+//	NEEDOPENBGL
 }
 /**
 @brief change widget focus after the user-combobox entry, or possibly group-entry, is activated
@@ -149,7 +152,9 @@ static void _e2_ownership_dialog_activate_cb (GtkEntry *entry, GtkWidget *focus)
 {
 	//prevent activation of any newly-focused widget too
 	g_signal_stop_emission_by_name ((gpointer)entry, "activate");
+	NEEDCLOSEBGL
 	gtk_widget_grab_focus (focus);
+	NEEDOPENBGL
 }
 /**
 @brief apply current settings after the group-combobox entry is activated for a non-directory item
@@ -162,7 +167,9 @@ static void _e2_ownership_dialog_activate_cb (GtkEntry *entry, GtkWidget *focus)
 static void _e2_ownership_dialog_activate_cb2 (GtkEntry *entry, GtkWidget *dialog)
 {
 	g_signal_stop_emission_by_name ((gpointer)entry, "activate");
+//	NEEDOPENBGL
 	g_signal_emit_by_name (G_OBJECT (dialog), "response", E2_RESPONSE_APPLY, NULL);
+//	NEEDCLOSEBGL
 }
 /**
 @brief create and run a dialog for changing item permissions
@@ -185,7 +192,6 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 {
 	GtkWidget *ownership_dialog;
 	GtkWidget *dialog_vbox;
-	GtkWidget *action_area;
 	GtkWidget *table, *hbox;
 
 //	E2_OwnersDldRuntime *rt = ALLOCATE (E2_OwnersDldRuntime);
@@ -229,6 +235,7 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 				current = utf;
 		}
 	    endpwent();
+
 		if ((pw_buf = getpwuid (src_uid)) == NULL)
 		{
 			userlist = g_list_prepend (userlist,
@@ -314,27 +321,21 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 
 //tag DEBUGfreeze;
 	//maybe this helps gtk 2.16 crashes ?
-//	gdk_threads_enter ();
+//	CLOSEBGL
 
 	GString *label_text = g_string_sized_new (NAME_MAX+20);
 	gboolean thisis_dir = e2_fs_is_dir3 (localpath E2_ERR_NONE());
 
-	gdk_threads_enter ();
+	CLOSEBGL
 	ownership_dialog = e2_dialog_create (NULL, NULL, _("ownership"),
 		DUMMY_RESPONSE_CB, NULL);
-	gdk_threads_leave ();
+	OPENBGL
 
 	dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (ownership_dialog));
 #else
 		GTK_DIALOG (ownership_dialog)->vbox;
-#endif
-	action_area =
-#ifdef USE_GTK2_14
-		gtk_dialog_get_action_area (GTK_DIALOG (ownership_dialog));
-#else
-		GTK_DIALOG (ownership_dialog)->action_area; //this is for the buttons
 #endif
 	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 
@@ -342,7 +343,11 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 	gchar *name = g_filename_display_basename (VPSTR (localpath));
 	g_string_printf (label_text, "%s: <b>%s</b>", label, name);
 	g_free (name);
+#ifdef USE_GTK3_0
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, E2_PADDING); //top, bottom padding
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING); //L, R padding
 	gtk_widget_show (hbox);
@@ -355,7 +360,11 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 	g_string_insert_c (label_text, e2_utils_get_byte_position (label_text->str, 1), ' ');
 	g_string_prepend (label_text, ": ");
 	g_string_prepend (label_text, _("Permissions"));
+#ifdef USE_GTK3_0
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
 	gtk_widget_show (hbox);
@@ -364,32 +373,49 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 
 	table = e2_widget_add_table (dialog_vbox, 2, 3, TRUE, TRUE, 0);  //2 rows, 3 cols, homogen, fill, no pad
 	gtk_container_set_border_width (GTK_CONTAINER (table), E2_PADDING);
+#ifdef USE_GTK3_2
+	gtk_grid_set_row_spacing (GTK_GRID (table), E2_PADDING);
+#else
 	gtk_table_set_row_spacings (GTK_TABLE (table), E2_PADDING);
-
-//	gdk_threads_enter (); //block entry-cursor blinking for now, to avoid sporadic bad keypress-latency
+#endif
+//	CLOSEBGL //block entry-cursor blinking for now, to avoid sporadic bad keypress-latency
 
 	//comboboxes for user & group
 	tmp = e2_utils_strcat (_("User"), ": ");
-	e2_widget_add_mid_label_to_table (table, tmp, 0, 0, 1, 0, 1);
+	e2_widget_add_mid_label_to_table (table, tmp, 0.1, 0,1,0,1);
 	g_free (tmp);
-	//on gtk2, this func MAY need BGL closed
+#ifndef USE_GTK3_0
+	CLOSEBGL
+#endif
+	//on gtk2 at least, combobox seems to work better if this func is called with BGL closed
 	rt.user_combo = e2_combobox_get ((ActivateFunc)NULL, NULL,
 		(userlist == NULL) ? NULL : &userlist,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_NO_AUTO_HISTORY | E2_COMBOBOX_CYCLE_HISTORY);
+#ifdef GTK3_COMBO_FIX
+	CLOSEBGL
+#endif
 	e2_combobox_set_active (rt.user_combo, user_index);
 	e2_combobox_block_changed (rt.user_combo); //prevent "activate" signal emission upon change
-	gtk_table_attach_defaults (GTK_TABLE (table), rt.user_combo, 1, 3, 0, 1);  // gint left col, gint right col, gint top row, gint bottom row)
+#ifdef USE_GTK3_2
+	gtk_grid_attach (GTK_GRID (table), rt.user_combo, 1, 0, 2, 1);
+#else
+	gtk_table_attach_defaults (GTK_TABLE (table), rt.user_combo, 1, 3, 0, 1);
+#endif
 	e2_list_free_with_data (&userlist);
 
 	tmp = e2_utils_strcat (_("Group"), ": ");
-	e2_widget_add_mid_label_to_table (table, tmp, 0, 0, 1, 1, 2);
+	e2_widget_add_mid_label_to_table (table, tmp, 0.1, 0,1,1,2);
 	g_free (tmp);
 	rt.group_combo = e2_combobox_get ((ActivateFunc)NULL, NULL,
 		(grouplist == NULL) ? NULL : &grouplist,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_NO_AUTO_HISTORY | E2_COMBOBOX_CYCLE_HISTORY);
 	e2_combobox_set_active (rt.group_combo, grp_index);
 	e2_combobox_block_changed (rt.group_combo); //prevent "activate" signal emission upon change
+#ifdef USE_GTK3_2
+	gtk_grid_attach (GTK_GRID (table), rt.group_combo, 1, 1, 2, 1);
+#else
 	gtk_table_attach_defaults (GTK_TABLE (table), rt.group_combo, 1, 3, 1, 2);
+#endif
 	e2_list_free_with_data (&grouplist);
 
 	if (permitted)
@@ -398,7 +424,11 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 		if (thisis_dir)  //note: no recurse button now if multiple-selection
 		{
 			table = e2_widget_add_table (dialog_vbox, 1, 3, TRUE, TRUE, 0); //1 row, 3 cols, homogen, fill, no pad
-			gtk_table_set_row_spacing (GTK_TABLE(table), 0, E2_PADDING);
+#ifdef USE_GTK3_2
+			gtk_grid_set_row_spacing (GTK_GRID (table), E2_PADDING);
+#else
+			gtk_table_set_row_spacings (GTK_TABLE(table), E2_PADDING);
+#endif
 			rt.recurse_dirs_button = e2_button_add_toggle_to_table (table, _("_Recurse"),
 				rt.recurse, _e2_ownership_dialog_toggle_recurse_button_cb, &rt, //rt,
 				1, 2, 0, 1 ); // gint left, gint right, gint top, gint bottom
@@ -474,7 +504,6 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 				G_CALLBACK(_e2_ownership_dialog_activate_cb2), ownership_dialog);
 	}
 
-	gdk_threads_enter ();
 #ifdef GTK3_COMBO_FIX
 	gtk_widget_grab_focus (gtk_bin_get_child (GTK_BIN(rt.user_combo)));
 #endif
@@ -485,7 +514,7 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 	//refreshing is enabled downstream, while waiting
 	//block until the user selects
 	result = e2_dialog_wait (ownership_dialog, TRUE, TRUE, multi, TRUE);//CHECKME more reliable if TRUE maincontext ?
-	gdk_threads_leave ();
+	OPENBGL
 
 	if (result == OK || result == YES_TO_ALL)
 	{
@@ -498,11 +527,11 @@ DialogButtons e2_ownership_dialog_run (VPATH *localpath,
 	}
 	if (GTK_IS_DIALOG(ownership_dialog)) //not explicitly closed by the user
 	{
-		gdk_threads_enter ();
+		CLOSEBGL
 		gtk_window_get_position (GTK_WINDOW (ownership_dialog), winx_ret, winy_ret); //irrelevant if closed
 		//now it's safe to destroy widgets
 		gtk_widget_destroy (ownership_dialog);
-		gdk_threads_leave ();
+		OPENBGL
 	}
 
 	return result;
