@@ -1,6 +1,6 @@
-/* $Id: e2_password_dialog.c 2309 2011-01-17 00:12:22Z tpgww $
+/* $Id: e2_password_dialog.c 2864 2013-10-27 09:04:44Z tpgww $
 
-Copyright (C) 2007-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2007-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -23,6 +23,9 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "e2_password_dialog.h"
+#include "e2_button.h"
+#include "e2_icons.h"
+
 //#include <pthread.h>
 #define E2_HINT_MSEC 600
 
@@ -51,6 +54,7 @@ static gboolean _e2_pwdlg_key_press_cb (GtkWidget *entry, GdkEventKey *event,
 {
 	if (event->state & GDK_CONTROL_MASK)
 	{
+		NEEDCLOSEBGL
 		if (event->keyval == GDK_p || event->keyval == GDK_P)
 		{
 			rt->plain = !rt->plain;
@@ -88,6 +92,7 @@ static gboolean _e2_pwdlg_key_press_cb (GtkWidget *entry, GdkEventKey *event,
 			return TRUE;
 		}
 #endif
+		NEEDOPENBGL
 	}
 	return FALSE;
 }
@@ -104,7 +109,8 @@ static void _e2_pwdlg_activate_cb (GtkEntry *entry, E2_PWDataRuntime *rt)
 	//prevent activation of any newly-focused widget too
 	g_signal_stop_emission_by_name ((gpointer)entry, "activate");
 
-	if (entry == GTK_ENTRY(rt->pwentry1))
+	NEEDCLOSEBGL
+	if (entry == GTK_ENTRY (rt->pwentry1))
 	{
 		if (rt->confirm &&
 #ifdef USE_GTK2_18
@@ -129,12 +135,17 @@ static void _e2_pwdlg_activate_cb (GtkEntry *entry, E2_PWDataRuntime *rt)
 	{
 process:
 		if (rt->dialog != NULL)
+		{
+			NEEDOPENBGL
 			//assumes that E2_BUTTON_YES or some derivative is in the dialog
 			g_signal_emit_by_name (G_OBJECT (rt->dialog), "response",
 				GTK_RESPONSE_YES);
+			NEEDCLOSEBGL
+		}
 		else if (rt->focus != NULL)
 			gtk_widget_grab_focus (rt->focus);
 	}
+	NEEDOPENBGL
 }
 /**
 @brief during entry destruction, zap plaintext password in gtk's buffer
@@ -151,26 +162,30 @@ static void _e2_pwdlg_clear_entry (GtkWidget *entry, gpointer user_data)
 }
 /**
 @brief setup password-related widgets to get user input with obfuscated text display
+
 One or two sets of centred labels and entries are consecutively added to the end of @a box.
 The respective entry(ies)' contents are zapped when the widgets are destroyed.
 Probably best if BGL is closed upon arrival, to avoid entry-keypress latency.
-Member(s) of the returned data struct (dialog, focus) might need to be set by
-the calling code.
+The returned struct is used as callback data, and so must persist while the
+widget(s) remain in use. Member(s) of the returned data struct (dialog, focus)
+might need to be set by the calling code, especially to enable focusing and/or
+response processing.
+
 @a pw may hold a pointer to a suggested value.
 @param box the widget into which things will be packed
 @param confirm TRUE to get 2 matched copies of entered password
 @param mainprompt custom prompt string, or NULL for default
 @param pw store for password pointer, or NULL
 
-@return pointer to data struct, NULL upon error
+@return pointer to allocated data struct, NULL upon error
 */
 E2_PWDataRuntime *e2_password_dialog_setup (GtkWidget *box, gboolean confirm,
-	gchar *mainprompt, gchar **pw)
+	const gchar *mainprompt, gchar **pw)
 {
 	E2_PWDataRuntime *rt = ALLOCATE0 (E2_PWDataRuntime);
 	CHECKALLOCATEDWARN (rt, return NULL;)
 
-	gchar *realprompt = (mainprompt != NULL) ? mainprompt : _("Enter password:");
+	const gchar *realprompt = (mainprompt != NULL) ? mainprompt : _("Enter password:");
 	gchar *realpw = (pw != NULL && *pw != NULL && **pw != '\0') ? *pw : NULL;
 
 /*	if (tabular)
@@ -324,7 +339,7 @@ gboolean e2_password_dialog_confirm (E2_PWDataRuntime *rt)
 		{
 			gtk_widget_grab_focus (rt->pwentry2);
 
-			GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_ERROR,
+			GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_ERROR,
 			_("Entered passwords are different"), NULL, DUMMY_RESPONSE_CB, NULL);
 			GtkWidget *parent = gtk_widget_get_toplevel (rt->pwentry1);
 			if (parent == NULL)
