@@ -1,6 +1,6 @@
-/* $Id: e2_permissions_dialog.c 2358 2011-05-31 13:48:41Z tpgww $
+/* $Id: e2_permissions_dialog.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -108,10 +108,10 @@ static void _e2_permissions_dialog_get_mode_string (E2_ChmodType *op_ret,
 @brief add a check button for a current permission
 
 @param table the table to which the button will be added
-@param left left position for attaching to @a table
-@param right right position for attaching to @a table
-@param top top position for attaching to @a table
-@param bottom bottom position for attaching to @a table
+@param left index of column to left of attach-position in @a table
+@param right index of column to right of attach-position in @a table
+@param top index or row above attach-position in @a table
+@param bottom index of row below attach-position in @a table
 @param label label to add to the button, or NULL
 @param state state of the button
 @param i buttons-array index of added button
@@ -132,6 +132,9 @@ static void _e2_permissions_dialog_add_chmod_button (
 		GtkWidget *aligner =  gtk_alignment_new (0.1, 0.5, 0, 0 );
 		GtkWidget *check_button = gtk_check_button_new ();
 		gtk_container_add (GTK_CONTAINER (aligner), check_button);
+#ifdef USE_GTK3_2
+		gtk_grid_attach (GTK_GRID(table), aligner, left, top, right-left, bottom-top);
+#else
 		gtk_table_attach (GTK_TABLE (table), aligner,
 			left, right, top, bottom,
 			GTK_FILL,  //GtkAttachOptions xoptions
@@ -139,6 +142,7 @@ static void _e2_permissions_dialog_add_chmod_button (
 			0, //xpadding
 			0 //ypadding
 			);
+#endif
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button), state);
 		rt->chmod_buttons[i] = check_button;
 	}
@@ -161,8 +165,14 @@ static void _e2_permissions_dialog_clear_chmod_buttons_cb (GtkToggleButton *butt
 	E2_PermsDldRuntime *rt)
 {
 	gint i;
+
+	NEEDCLOSEBGL
 	for (i = 0; i < PERM_COUNT; i++)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i]), FALSE);
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i]),
+			FALSE);
+	}
+	NEEDOPENBGL
 }
 /**
 @brief "toggled" signal callback for set_perms_button
@@ -177,11 +187,14 @@ static void _e2_permissions_dialog_reset_chmod_buttons_cb (GtkToggleButton *butt
 {
 	mode_t mode = rt->mode;
 	gint i;
+
+	NEEDCLOSEBGL
 	for (i = 0; i < PERM_COUNT; i++)
 	{
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->chmod_buttons[i]),
 			(mode & mask[i]));
 	}
+	NEEDOPENBGL
 }
 /**
 @brief "toggled" signal callback for recurse_button
@@ -202,8 +215,10 @@ static void _e2_permissions_dialog_toggle_recurse_button_cb (GtkToggleButton *bu
 #else
 			button->active;
 #endif
+		NEEDCLOSEBGL
 		gtk_widget_set_sensitive (rt->recurse_dirs_button, flag);
 		gtk_widget_set_sensitive (rt->recurse_other_button, flag);
+		NEEDOPENBGL
 	}
 }
 /**
@@ -226,6 +241,7 @@ static void _e2_permissions_dialog_toggle_recurse_type_cb (GtkToggleButton *butt
 #endif
 	)
 	{
+		NEEDCLOSEBGL
 		if (button == GTK_TOGGLE_BUTTON (rt->recurse_dirs_button))
 		{
 			if (!
@@ -248,6 +264,7 @@ static void _e2_permissions_dialog_toggle_recurse_type_cb (GtkToggleButton *butt
 			)
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rt->recurse_dirs_button),TRUE);
 		}
+		NEEDOPENBGL
 	}
 }
 /**
@@ -272,7 +289,6 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 	GtkWidget *dialog_vbox, *sub_vbox;
 	GtkWidget *hbox;
 	GtkWidget *frame;
-	GtkWidget *action_area;
 	GtkWidget *table;
 
 	struct stat statbuf;
@@ -286,28 +302,22 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 
 //tag DEBUGfreeze;
 	//maybe this helps gtk 2.16 crashes ?
-//	gdk_threads_enter ();
+//	CLOSEBGL
 
 	rt.permission = permitted;
 
 	GString *label_text = g_string_sized_new (NAME_MAX+20);
 	gboolean thisis_dir = e2_fs_is_dir3 (localpath E2_ERR_NONE());
-	gdk_threads_enter ();
+	CLOSEBGL
 	permissions_dialog = e2_dialog_create (NULL, NULL, _("permissions"),
 		DUMMY_RESPONSE_CB, NULL);
-	gdk_threads_leave ();
+	OPENBGL
 
 	dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (permissions_dialog));
 #else
 		GTK_DIALOG (permissions_dialog)->vbox;
-#endif
-	action_area =
-#ifdef USE_GTK2_14
-		gtk_dialog_get_action_area (GTK_DIALOG (permissions_dialog));
-#else
-		GTK_DIALOG (permissions_dialog)->action_area;
 #endif
 	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 #ifndef USE_GTK3_0
@@ -318,7 +328,11 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 	g_string_printf (label_text, "%s: <b>%s</b>", label, name);
 	g_free (name);
 
+#ifdef USE_GTK3_0
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, E2_PADDING); //top, bottom padding
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING); //L, R padding
 
@@ -335,7 +349,11 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 	else
 		g_string_printf (label_text, "%s: %d", label, (guint) statbuf.st_uid);
 
+#ifdef USE_GTK3_0
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
 
@@ -352,7 +370,11 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 	else
 		g_string_printf (label_text, "%s: %d", label, (guint) statbuf.st_gid);
 
+#ifdef USE_GTK3_0
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
 
@@ -364,19 +386,27 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 	g_string_insert_c (label_text, 0, ' ');
 	label_text = g_string_prepend (label_text, _("currently"));
 
+#ifdef USE_GTK3_0
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	e2_widget_add_mid_label (hbox, label_text->str, 0, TRUE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 0);
 
 	frame = gtk_frame_new (_("Permissions"));
 	gtk_box_pack_start (GTK_BOX (dialog_vbox), frame, TRUE, TRUE, E2_PADDING);
 
+#ifdef USE_GTK3_0
+	sub_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	sub_vbox = gtk_vbox_new (FALSE, 0);
+#endif
 	gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
 
 	table = e2_widget_add_table (sub_vbox, 4, 5, TRUE, TRUE, 0);  //4 rows, 5 cols, homogen, fill, no pad
-	e2_widget_add_mid_label_to_table (table, _("Read"), 0,		1,2,0,1);  //gint left, gint right, gint top, gint bottom)
-	e2_widget_add_mid_label_to_table (table, _("Write"), 0,		2,3,0,1);
+	e2_widget_add_mid_label_to_table (table, _("Read"), 0.0,	1,2,0,1);  //gint left, gint right, gint top, gint bottom)
+	e2_widget_add_mid_label_to_table (table, _("Write"), 0.0,	2,3,0,1);
 	e2_widget_add_mid_label_to_table (table, _("Exec"), 0.05,	3,4,0,1);
 	e2_widget_add_mid_label_to_table (table, _("Special"), 0.2,	4,5,0,1);
 
@@ -423,7 +453,11 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 		frame = gtk_frame_new (_("Action"));
 		gtk_box_pack_start (GTK_BOX (dialog_vbox), frame, TRUE, TRUE, E2_PADDING);
 
+#ifdef USE_GTK3_0
+		sub_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 		sub_vbox = gtk_vbox_new (FALSE, 0);
+#endif
 		gtk_container_add (GTK_CONTAINER (frame), sub_vbox);
 
 		table = e2_widget_add_table (sub_vbox, 1, 5, TRUE, TRUE, 0);  //1 rows, 5 cols, homogen, fill, no pad
@@ -494,7 +528,7 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 	if (!permitted)
 		gtk_widget_set_sensitive (btn, FALSE);
 
-	gdk_threads_enter ();
+	CLOSEBGL
 	e2_dialog_setup (permissions_dialog, app.main_window);
 	e2_dialog_run (permissions_dialog, NULL, flags);
 	if (*winx_ret >= 0 && *winy_ret >= 0)
@@ -502,7 +536,7 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 	//block until the user selects
 	//refreshing is enabled downstream, while waiting
 	result = e2_dialog_wait (permissions_dialog, TRUE, TRUE, multi, TRUE);//CHECKME more reliable if TRUE maincontext ?
-	gdk_threads_leave ();
+	OPENBGL
 
 	//defaults
 	*op_ret = E2_CHMOD_NONE;
@@ -545,11 +579,11 @@ DialogButtons e2_permissions_dialog_run (VPATH *localpath,
 
 	if (GTK_IS_DIALOG (permissions_dialog)) //not explicitly closed by the user
 	{
-		gdk_threads_enter ();
+		CLOSEBGL
 		gtk_window_get_position (GTK_WINDOW (permissions_dialog), winx_ret, winy_ret);
 		//now it's safe to destroy the window
 		gtk_widget_destroy (permissions_dialog);
-		gdk_threads_leave ();
+		OPENBGL
 	}
 
 	return result;
