@@ -1,6 +1,6 @@
-/* $Id: e2_select_image_dialog.c 2377 2011-06-11 00:16:34Z tpgww $
+/* $Id: e2_select_image_dialog.c 3002 2014-01-17 23:17:19Z tpgww $
 
-Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2014 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -31,6 +31,7 @@ page in the main configuration dialog
 #include "e2_dialog.h"
 #include "e2_select_image_dialog.h"
 #include "e2_config_dialog.h"
+#include "e2_icons.h"
 
 enum
 {
@@ -66,6 +67,8 @@ static void _e2_sidlg_response_cb (GtkDialog *dialog, gint response, E2_SID_Runt
 	GtkTreePath *tpath;
 	GList *selected;
 
+	NEEDCLOSEBGL
+
 	selected = (rt->page == 0) ?
 		gtk_icon_view_get_selected_items (rt->customview):
 		gtk_icon_view_get_selected_items (rt->stockview);
@@ -126,6 +129,8 @@ static void _e2_sidlg_response_cb (GtkDialog *dialog, gint response, E2_SID_Runt
 	}
 
 	gtk_widget_set_sensitive (rt->rem_btn, (response != E2_RESPONSE_REMOVE));
+
+	NEEDOPENBGL
 }
 /**
 @brief issue ok response for @a dialog
@@ -140,7 +145,9 @@ This is a callback for item-activated signal in stock-icons view
 static void _e2_sidlg_activated_cb (GtkIconView *iconview, GtkTreePath *path,
 	GtkWidget *dialog)
 {
+//	NEEDOPENBGL
 	gtk_dialog_response (GTK_DIALOG (dialog), E2_RESPONSE_APPLY);
+//	NEEDCLOSEBGL
 }
 /**
 @brief notebook page-switch callback
@@ -161,7 +168,9 @@ static void _e2_sidlg_page_switch_cb (GtkNotebook *notebook,
 	guint page_num, E2_SID_Runtime *rt)
 {
 	rt->page = page_num;
+	NEEDCLOSEBGL
 	gtk_widget_set_sensitive (rt->dir_chooser, (page_num == 0));
+	NEEDOPENBGL
 }
 /**
 @brief cleanup sid-dialog data
@@ -173,6 +182,8 @@ static void _e2_sidlg_page_switch_cb (GtkNotebook *notebook,
 */
 static void _e2_sidlg_destroy_cb (GtkWidget *dialog, E2_SID_Runtime *rt)
 {
+//	NEEDCLOSEBGL
+//	NEEDOPENBGL
 	g_object_set_data (G_OBJECT (rt->parent), rt->name, NULL);
 	g_free (rt->name);
 	g_free (rt->icon);
@@ -205,6 +216,7 @@ static void _e2_sidlg_dir_change_cb (GtkFileChooser *chooser, E2_SID_Runtime *rt
 		gchar *dirpath = g_filename_from_uri (uri, NULL, NULL);
 		if (dirpath != NULL)
 		{
+			NEEDCLOSEBGL
 			gtk_list_store_clear (GTK_LIST_STORE (rt->custommodel));
 #ifdef E2_VFSTMP
 			VPATH data;
@@ -216,6 +228,8 @@ static void _e2_sidlg_dir_change_cb (GtkFileChooser *chooser, E2_SID_Runtime *rt
 #endif
 			//show the relevant icon, if any
 			_e2_sidlg_show_current (FALSE, rt);
+
+			NEEDOPENBGL
 			//remember for next session
 			if (VPSTR (initial_dir) != NULL)
 				g_free (VPSTR (initial_dir));
@@ -247,7 +261,7 @@ static gchar *_e2_sidlg_get_icon_dir (E2_SID_Runtime *rt)
 	gpointer dtype = g_object_get_data (G_OBJECT (rt->parent), "dialog-form");
 	if (GPOINTER_TO_INT (dtype) == E2_CFGDLG_SINGLE)
 	{	//doing a single-page dialog
-		iconpath = e2_utils_get_icons_path (TRUE);
+		iconpath = e2_icons_get_custom_path (TRUE);
 	}
 	else
 	{	//doing a full config dialog
@@ -302,10 +316,14 @@ static void	_e2_sidlg_fill_custom_store (VPATH *localpath, E2_SID_Runtime *rt)
 	{
 		GList *member;
 		gint w, h;
+#ifdef USE_GTK3_10
+		if (!gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &w, &h))
+#else
 		GtkSettings *s = gtk_settings_get_default ();
 		if (!gtk_icon_size_lookup_for_settings (s, GTK_ICON_SIZE_LARGE_TOOLBAR, &w, &h))
+#endif
 		{
-			w = h = 18;	//can't find useful size, use this default
+			w = h = 24;	//can't find useful size, use this default
 		}
 
 		entries = g_list_sort (entries, (GCompareFunc) strcmp);
@@ -368,14 +386,296 @@ static GtkListStore *_e2_sidlg_create_custom_store (E2_SID_Runtime *rt)
 	return store;
 }
 
-/**
-@brief idle callback to populate stock icons view
-@param store object in which to save icons' data
+#ifdef E2_ADD_STOCKS
 
-@return FALSE to terminate the source
-*/
-static gboolean _e2_sidlg_fill_stock_store (GtkListStore *store)
+#ifndef USE_GLIB2_16
+typedef struct _E2_StockHashData
 {
+	GtkListStore *store;
+	GtkStyle *style;
+	GtkTextDirection dir;
+} E2_StockHashData;
+
+static void _e2_sidlg_stockhash_each (gpointer key, gpointer value, E2_StockHashData *data)
+{
+	const gchar *name = (const gchar*) key;
+	GtkIconSet *iset = gtk_style_lookup_icon_set (data->style, name);
+	if (iset != NULL)
+	{
+		GdkPixbuf *pxb = gtk_icon_set_render_icon (iset, style, dir,
+			GTK_STATE_NORMAL, GTK_ICON_SIZE_LARGE_TOOLBAR, NULL, NULL);
+		if (pxb != NULL)
+		{
+			GtkTreeIter iter;
+
+			if (strncmp (name, "gtk-", 4) == 0)	//skip leading "gtk-"
+				name += 4;
+			gtk_list_store_insert_with_values (data->store, &iter, -1,
+				PIXBUF_COL, pxb, TEXT_COL, name, -1);
+			g_object_unref (G_OBJECT (pxb));
+		}
+		else
+		{
+			printd (DEBUG, "No themed pixbuf created for %s", name);
+		}
+	}
+	else
+	{
+		printd (DEBUG, "No themed iconset found for %s", name);
+	}
+}
+#endif //ndef USE_GLIB2_16
+
+/**
+@brief populate stock icons view
+Assumes BGL closed, app.main_window exists (for gtk 3.10+)
+
+@param store empty liststore widget in which to save icons' data
+*/
+static void _e2_sidlg_fill_stock_store (GtkListStore *store)
+{
+	extern GHashTable *cached_stocks;
+#ifdef USE_GLIB2_16
+	gpointer value;
+	GHashTableIter hashiter;
+#endif
+
+#ifdef USE_GTK3_10
+	gint psize = 24;
+	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &psize, NULL);
+	GtkIconTheme *thm =	gtk_icon_theme_get_for_screen
+		(gtk_widget_get_screen (app.main_window));
+#elif defined (USE_GTK3_0)
+	GtkStyleContext *context = gtk_widget_get_style_context (app.main_window);
+#else
+	GtkStyle *style = gtk_widget_get_style (app.main_window);
+	GtkTextDirection dir = gtk_widget_get_direction (app.main_window);
+#endif
+
+	e2_icons_cache_stocks (); //freshen cache (SLOW!)
+
+	GtkTreeSortable *sort = GTK_TREE_SORTABLE (store);
+//	CLOSEBGL if idle callback
+	gtk_tree_sortable_set_sort_column_id (sort,
+		GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
+
+#ifdef USE_GLIB2_16
+
+	g_hash_table_iter_init (&hashiter, cached_stocks);
+	while (g_hash_table_iter_next (&hashiter, NULL, &value))
+	{
+		E2_Stock *data = (E2_Stock *) value;
+		gchar *stock = data->stock;
+# ifdef USE_GTK3_10
+		gchar *iname = data->name;
+		if (iname == NULL)
+			iname = data->stock;
+		GtkIconInfo *inf = gtk_icon_theme_lookup_icon (thm, iname, psize,
+			GTK_ICON_LOOKUP_GENERIC_FALLBACK | GTK_ICON_LOOKUP_USE_BUILTIN);
+# elif defined(USE_GTK3_0)
+		GtkIconSet *iset = gtk_style_context_lookup_icon_set (context, stock);
+# else
+		GtkIconSet *iset = gtk_style_lookup_icon_set (style, stock);
+# endif
+
+# ifdef USE_GTK3_10
+		if (inf != NULL)
+# else
+		if (iset != NULL)
+#endif
+		{
+			GdkPixbuf *pxb;
+# ifdef USE_GTK3_10
+			pxb = gtk_icon_info_load_icon (inf, NULL);
+# elif defined(USE_GTK3_0)
+			pxb = gtk_icon_set_render_icon_pixbuf (iset, context,
+				GTK_ICON_SIZE_LARGE_TOOLBAR);
+# else
+			pxb = gtk_icon_set_render_icon (iset, style, dir, GTK_STATE_NORMAL,
+				GTK_ICON_SIZE_LARGE_TOOLBAR, NULL, NULL);
+# endif
+			if (pxb != NULL)
+			{
+				GtkTreeIter iter;
+
+				if (strncmp (stock, "gtk-", 4) == 0)	//skip leading "gtk-"
+					stock += 4;
+				gtk_list_store_insert_with_values (store, &iter, -1,
+					PIXBUF_COL, pxb, TEXT_COL, stock, -1);
+				g_object_unref (G_OBJECT (pxb));
+			}
+			else
+			{
+				printd (DEBUG, "No themed pixbuf created for %s", stock);
+			}
+# ifdef USE_GTK3_10
+			g_object_unref (inf);
+# endif
+		}
+		else
+		{
+			printd (DEBUG, "No themed iconset found for %s", stock);
+		}
+	}
+
+#else //ndef USE_GLIB2_16
+
+	E2_StockHashData data = { store, style, dir };
+	g_hash_table_foreach (cached_stocks, (GHFunc)_e2_sidlg_stockhash_each, &data);
+
+#endif //ndef USE_GLIB2_16
+
+	gtk_tree_sortable_set_sort_column_id (sort, TEXT_COL, GTK_SORT_ASCENDING);
+//	OPENBGL if idle callback
+
+//	return FALSE; ditto
+}
+
+#else //ndef E2_ADD_STOCKS
+
+static void _e2_sidlg_fill_stock_store (GtkListStore *store)
+{
+#ifdef USE_GTK3_10
+	const gchar *stocks[] =
+	{
+		STOCK_NAME_ABOUT,
+		STOCK_NAME_ADD,
+		STOCK_NAME_APPLY,
+		STOCK_NAME_BOLD,
+		STOCK_NAME_CANCEL,
+		STOCK_NAME_CAPS_LOCK_WARNING,
+		STOCK_NAME_CDROM,
+		STOCK_NAME_CLEAR,
+		STOCK_NAME_CLOSE,
+		STOCK_NAME_COLOR_PICKER,
+		STOCK_NAME_CONNECT,
+		STOCK_NAME_CONVERT,
+		STOCK_NAME_COPY,
+		STOCK_NAME_CUT,
+		STOCK_NAME_DELETE,
+		STOCK_NAME_DIALOG_AUTHENTICATION,
+		STOCK_NAME_DIALOG_ERROR,
+		STOCK_NAME_DIALOG_INFO,
+		STOCK_NAME_DIALOG_QUESTION,
+		STOCK_NAME_DIALOG_WARNING,
+		STOCK_NAME_DIRECTORY,
+		STOCK_NAME_DISCARD,
+		STOCK_NAME_DISCONNECT,
+		STOCK_NAME_DND,
+		STOCK_NAME_DND_MULTIPLE,
+		STOCK_NAME_EDIT,
+		STOCK_NAME_EXECUTE,
+		STOCK_NAME_FILE,
+		STOCK_NAME_FIND_AND_REPLACE,
+		STOCK_NAME_FIND,
+		STOCK_NAME_FLOPPY,
+		STOCK_NAME_FULLSCREEN,
+		STOCK_NAME_GO_BACK,
+		STOCK_NAME_GO_DOWN,
+		STOCK_NAME_GO_FORWARD,
+		STOCK_NAME_GOTO_BOTTOM,
+		STOCK_NAME_GOTO_FIRST,
+		STOCK_NAME_GOTO_LAST,
+		STOCK_NAME_GOTO_TOP,
+		STOCK_NAME_GO_UP,
+		STOCK_NAME_HARDDISK,
+		STOCK_NAME_HELP,
+		STOCK_NAME_HOME,
+		STOCK_NAME_INDENT,
+		STOCK_NAME_INDEX,
+		STOCK_NAME_INFO,
+		STOCK_NAME_ITALIC,
+		STOCK_NAME_JUMP_TO,
+		STOCK_NAME_JUSTIFY_CENTER,
+		STOCK_NAME_JUSTIFY_FILL,
+		STOCK_NAME_JUSTIFY_LEFT,
+		STOCK_NAME_JUSTIFY_RIGHT,
+		STOCK_NAME_LEAVE_FULLSCREEN,
+		STOCK_NAME_MEDIA_FORWARD,
+		STOCK_NAME_MEDIA_NEXT,
+		STOCK_NAME_MEDIA_PAUSE,
+		STOCK_NAME_MEDIA_PLAY,
+		STOCK_NAME_MEDIA_PREVIOUS,
+		STOCK_NAME_MEDIA_RECORD,
+		STOCK_NAME_MEDIA_REWIND,
+		STOCK_NAME_MEDIA_STOP,
+		STOCK_NAME_MISSING_IMAGE,
+		STOCK_NAME_NETWORK,
+		STOCK_NAME_NEW,
+		STOCK_NAME_NO,
+		STOCK_NAME_OK,
+		STOCK_NAME_OPEN,
+		STOCK_NAME_ORIENTATION_LANDSCAPE,
+		STOCK_NAME_ORIENTATION_PORTRAIT,
+		STOCK_NAME_ORIENTATION_REVERSE_LANDSCAPE,
+		STOCK_NAME_ORIENTATION_REVERSE_PORTRAIT,
+		STOCK_NAME_PAGE_SETUP,
+		STOCK_NAME_PASTE,
+		STOCK_NAME_PREFERENCES,
+		STOCK_NAME_PRINT_ERROR,
+		STOCK_NAME_PRINT,
+		STOCK_NAME_PRINT_PAUSED,
+		STOCK_NAME_PRINT_PREVIEW,
+		STOCK_NAME_PRINT_REPORT,
+		STOCK_NAME_PRINT_WARNING,
+		STOCK_NAME_PROPERTIES,
+		STOCK_NAME_QUIT,
+		STOCK_NAME_REDO,
+		STOCK_NAME_REFRESH,
+		STOCK_NAME_REMOVE,
+		STOCK_NAME_REVERT_TO_SAVED,
+		STOCK_NAME_SAVE_AS,
+		STOCK_NAME_SAVE,
+		STOCK_NAME_SELECT_ALL,
+		STOCK_NAME_SELECT_COLOR,
+		STOCK_NAME_SELECT_FONT,
+		STOCK_NAME_SORT_ASCENDING,
+		STOCK_NAME_SORT_DESCENDING,
+		STOCK_NAME_SPELL_CHECK,
+		STOCK_NAME_STOP,
+		STOCK_NAME_STRIKETHROUGH,
+		STOCK_NAME_UNDELETE,
+		STOCK_NAME_UNDERLINE,
+		STOCK_NAME_UNDO,
+		STOCK_NAME_UNINDENT,
+		STOCK_NAME_YES,
+		STOCK_NAME_ZOOM_100,
+		STOCK_NAME_ZOOM_FIT,
+		STOCK_NAME_ZOOM_IN,
+		STOCK_NAME_ZOOM_OUT
+	};
+	gint psize = 24;
+	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &psize, NULL);
+	GtkIconTheme *thm =	gtk_icon_theme_get_for_screen
+		(gtk_widget_get_screen (app.main_window));
+
+	gint i, c = sizeof (stocks) / sizeof (stocks[0]);
+	for (i = 0; i < c; i++)
+	{
+		const gchar *iname = stocks[i];
+		GtkIconInfo *inf = gtk_icon_theme_lookup_icon (thm, iname, psize,
+			GTK_ICON_LOOKUP_GENERIC_FALLBACK | GTK_ICON_LOOKUP_USE_BUILTIN);
+		if (inf != NULL)
+		{
+			GdkPixbuf *pxb = gtk_icon_info_load_icon (inf, NULL);
+			if (pxb != NULL)
+			{
+				GtkTreeIter iter;
+
+				if (strncmp (iname, "gtk-", 4) == 0)	//skip leading "gtk-"
+					iname += 4;
+				gtk_list_store_insert_with_values (store, &iter, -1,
+					PIXBUF_COL, pxb, TEXT_COL, iname, -1);
+				g_object_unref (G_OBJECT (pxb));
+			}
+			else
+				printd (DEBUG, "No themed pixbuf created for %s", iname);
+			g_object_unref (inf);
+		}
+		else
+			printd (DEBUG, "No registered icon for %s", iname);
+	}
+#else //ndef USE_GTK3_10
 	GSList *ids = gtk_stock_list_ids ();
 	if (ids != NULL)
 	{
@@ -390,22 +690,19 @@ static gboolean _e2_sidlg_fill_stock_store (GtkListStore *store)
 		ids = g_slist_sort (ids, (GCompareFunc) strcmp);
 		for (member = ids; member != NULL; member = g_slist_next (member))
 		{
-			GtkIconSet *iset;
 #ifdef USE_GTK3_0
-			iset = gtk_style_context_lookup_icon_set (context,
-									(const gchar*) member->data);
+			GtkIconSet *iset = gtk_style_context_lookup_icon_set (context, (const gchar*) member->data);
 #else
-			iset = gtk_style_lookup_icon_set (style, (const gchar*) member->data);
+			GtkIconSet *iset = gtk_style_lookup_icon_set (style, (const gchar*) member->data);
 #endif
 			if (iset != NULL)
 			{
-				GdkPixbuf *pxb;
 #ifdef USE_GTK3_0
-				pxb = gtk_icon_set_render_icon_pixbuf (iset, context,
+				GdkPixbuf *pxb = gtk_icon_set_render_icon_pixbuf (iset, context,
 					GTK_ICON_SIZE_LARGE_TOOLBAR);
 #else
-				pxb = gtk_icon_set_render_icon (iset, style, dir, GTK_STATE_NORMAL,
-					GTK_ICON_SIZE_LARGE_TOOLBAR, NULL, NULL);
+				GdkPixbuf *pxb = gtk_icon_set_render_icon (iset, style, dir,
+					GTK_STATE_NORMAL, GTK_ICON_SIZE_LARGE_TOOLBAR, NULL, NULL);
 #endif
 				if (pxb != NULL)
 				{
@@ -421,22 +718,10 @@ static gboolean _e2_sidlg_fill_stock_store (GtkListStore *store)
 		}
 		g_slist_free (ids);
 	}
-	return FALSE;
+#endif //ndef USE_GTK3_10
 }
 
-/**
-@brief create empty liststore for all gtk stock icons
-
-@return the store
-*/
-static GtkListStore *_e2_sidlg_create_stock_store (void)
-{
-	//store with pixbuf and title
-	//conform to column-number enum
-	GtkListStore *store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
-
-	return store;
-}
+#endif //ndef E2_ADD_STOCKS
 
 /**
 @brief create scrolled window showing stock icons
@@ -447,7 +732,12 @@ static GtkListStore *_e2_sidlg_create_stock_store (void)
 */
 static GtkWidget *_e2_sidlg_create_stock_icon_browser (E2_SID_Runtime *rt)
 {
-	GtkListStore *store = _e2_sidlg_create_stock_store ();
+	//store with pixbuf and title, conforms to column-number enum
+	GtkListStore *store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
+	//FIXME populate this lot of icons at idle, if the current icon is not stock
+	//g_idle_add ((GSourceFunc)_e2_sidlg_fill_stock_store, store);
+	_e2_sidlg_fill_stock_store (store);
+
 	rt->stockmodel = GTK_TREE_MODEL (store);
 	rt->stockview = GTK_ICON_VIEW (gtk_icon_view_new_with_model (rt->stockmodel));
 	g_object_unref (G_OBJECT (store));
@@ -492,9 +782,6 @@ static GtkWidget *_e2_sidlg_create_stock_icon_browser (E2_SID_Runtime *rt)
 		GTK_POLICY_AUTOMATIC);
 
 	gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (rt->stockview));
-	//FIXME populate this lot of icons at idle, if the current icon is not stock
-	//g_idle_add ((GSourceFunc)_e2_sidlg_fill_stock_store, store);
-	_e2_sidlg_fill_stock_store (store);
 
 	return sw;
 }
@@ -677,8 +964,15 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon, Gdk
 		rt->parent = parent;
 	}
 
-	//create dialog widgets
+#ifdef USE_GTK3_10
+	//icons population can take a while!
+	GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
+	gdk_window_set_cursor (gtk_widget_get_window (parent), cursor);
+	g_object_unref (G_OBJECT (cursor));
+	gdk_flush ();
+#endif
 
+	//create dialog widgets
 	rt->dialog = e2_dialog_create (NULL, NULL, rt->name,
 		(ResponseFunc)_e2_sidlg_response_cb, rt);
 	GtkWidget *dialog_vbox =
@@ -765,8 +1059,13 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon, Gdk
 //						"button-spacing", &button_spacing,
 						"action-area-border", &action_area_border,
 						NULL);
+#ifdef USE_GTK3_0
+	GtkWidget *bbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	GtkWidget *bbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	GtkWidget *bbox = gtk_hbox_new (FALSE, 0);
 	GtkWidget *bbox2 = gtk_vbox_new (FALSE, 0);
+#endif
 	GtkWidget *action_area =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_action_area (GTK_DIALOG (rt->dialog));
@@ -800,5 +1099,13 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon, Gdk
 	if (page != 0)
 		gtk_notebook_set_current_page (rt->notebook, page);
 
+#ifdef USE_GTK3_10
+	//icons population can take a while!
+	cursor = gdk_cursor_new (GDK_LEFT_PTR);
+	gdk_window_set_cursor (gtk_widget_get_window (parent), cursor);
+	g_object_unref (G_OBJECT (cursor));
+	gdk_flush ();
+#endif
+
 	return rt->dialog;
 }
