@@ -1,4 +1,4 @@
-/* $Id: e2_select_image_dialog.h 1663 2009-05-09 09:59:37Z tpgww $
+/* $Id: e2_select_image_dialog.h 2377 2011-06-11 00:16:34Z tpgww $
 
 Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
 
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #ifndef __E2_SELECT_IMAGE_DIALOG_H__
@@ -39,6 +38,6 @@ typedef struct _E2_SID_Runtime
 	GtkWidget *rem_btn;
 } E2_SID_Runtime;
 
-GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon);
+GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon, GdkEventButton *event);
 
 #endif //ndef __E2_SELECT_IMAGE_DIALOG_H__
