@@ -1,6 +1,6 @@
-/* $Id: e2_tree_dialog.c 2404 2011-10-30 23:55:23Z tpgww $
+/* $Id: e2_tree_dialog.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2007-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2007-2013 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1996-2007 Steve Baker <ice@mama.indstate.edu>
 
 This file is part of emelFM2.
@@ -19,6 +19,7 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "e2_tree_dialog.h"
+#include "e2_icons.h"
 
 #ifdef E2_TREEDIALOG
 
@@ -946,7 +947,7 @@ static void	_e2_treedlg_fill_store (gchar *startpath, E2_TreeDialogRuntime *rt)
 	//FIXME vfs, and other namespaces
 # endif
 	gpointer result;
-	gdk_threads_leave ();
+	OPENBGL
 	if (pthread_create (&rt->threadID, NULL, _e2_treedlg_do_full_scan, rt) == 0)
 	{
 		pthread_join (thisID, &result);
@@ -959,7 +960,7 @@ static void	_e2_treedlg_fill_store (gchar *startpath, E2_TreeDialogRuntime *rt)
 		result = NULL;
 		printd (WARN, "Failed to create tree-scan thread");
 	}
-	gdk_threads_enter ();
+	CLOSEBGL
 	if (result == NULL)
 	{
 		//FIXME do what ?
@@ -981,7 +982,9 @@ static void	_e2_treedlg_fill_store (gchar *startpath, E2_TreeDialogRuntime *rt)
 # endif
 		G_DIR_SEPARATOR_S, rt->dirhash);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (rt->store), &iter, tp);
+	NEEDOPENBGL
 	_e2_treedlg_row_expanded_cb (GTK_TREE_VIEW (rt->treeview), &iter, tp, rt);
+	NEEDCLOSEBGL
 
 	gchar *showpath = (startpath == NULL) ? rt->view->dir : startpath;
 	gchar **split = g_strsplit (showpath, G_DIR_SEPARATOR_S, -1);
@@ -1003,7 +1006,9 @@ static void	_e2_treedlg_fill_store (gchar *startpath, E2_TreeDialogRuntime *rt)
 				if (tp != NULL)
 				{
 					gtk_tree_model_get_iter (GTK_TREE_MODEL (rt->store), &iter, tp);
+					NEEDOPENBGL
 					_e2_treedlg_row_expanded_cb (GTK_TREE_VIEW (rt->treeview), &iter, tp, rt);
+					NEEDCLOSEBGL
 					gtk_tree_path_free (tp);
 				}
 				else
@@ -1088,7 +1093,9 @@ static void _e2_treedlg_copy_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
+	NEEDCLOSEBGL
 	gtk_tree_selection_get_selected (sel, &model, &iter);
+	NEEDOPENBGL
 	GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
 	gchar *str = _e2_treedlg_make_path (model, path);
 	if (str != NULL)
@@ -1109,9 +1116,12 @@ static void _e2_treedlg_copy_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 */
 static void _e2_treedlg_collapse_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
+	GtkTreePath *tp;
+	NEEDCLOSEBGL
 	gtk_tree_view_collapse_all (treeview);
-	GtkTreePath *tp = gtk_tree_path_new_first ();
+	tp = gtk_tree_path_new_first ();
 	gtk_tree_view_expand_row (treeview, tp, FALSE);
+	NEEDOPENBGL
 	gtk_tree_path_free (tp);
 }
 /**
@@ -1127,6 +1137,7 @@ static void _e2_treedlg_refresh_cb (GtkMenuItem *widget, E2_TreeDialogRuntime *r
 	//FIXME use filter-model and just re-filter ?
 	GtkTreeModel *model;
 	GtkTreeIter iter;
+	NEEDCLOSEBGL
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (rt->treeview));
 	if (gtk_tree_selection_get_selected (sel, &model, &iter))
 	{
@@ -1147,6 +1158,7 @@ static void _e2_treedlg_refresh_cb (GtkMenuItem *widget, E2_TreeDialogRuntime *r
 		gtk_tree_path_free (tp);
 		g_free (pathstr);
 	}
+	NEEDOPENBGL
 }
 #ifndef STRICT_HIDE
 /**
@@ -1159,7 +1171,9 @@ This is performed when the context-menu item 'strict hiding' is selected
 */
 static void _e2_treedlg_toggle_strict_cb (GtkMenuItem *menuitem, E2_TreeDialogRuntime *rt)
 {
+//	NEEDCLOSEBGL
 	rt->strict_hide = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem));
+//	NEEDOPENBGL
 	_e2_treedlg_refresh_cb (NULL, rt);
 }
 #endif
@@ -1178,16 +1192,16 @@ static void _e2_treedlg_show_context_menu (GtkWidget *treeview,
 {
 	GtkTreeView *tvw = GTK_TREE_VIEW (treeview);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (tvw);
-	GtkWidget *menu = gtk_menu_new ();
+	GtkWidget *menu = e2_menu_get ();
 
 	GtkWidget *item =
-	e2_menu_add (menu, _("_Copy"), GTK_STOCK_COPY, _("Copy selected path"),
+	e2_menu_add (menu, _("_Copy"), STOCK_NAME_COPY, _("Copy selected path"),
 		_e2_treedlg_copy_cb, tvw);
 	if (!gtk_tree_selection_get_selected (sel, NULL, NULL))
 		gtk_widget_set_sensitive (item, FALSE);
-	e2_menu_add (menu, _("C_ollapse"), GTK_STOCK_ZOOM_OUT, _("Collapse all paths"),
+	e2_menu_add (menu, _("C_ollapse"), STOCK_NAME_ZOOM_OUT, _("Collapse all paths"),
 		_e2_treedlg_collapse_all_cb, tvw);
-	e2_menu_add (menu, _("_Refresh"), GTK_STOCK_REFRESH, NULL,
+	e2_menu_add (menu, _("_Refresh"), STOCK_NAME_REFRESH, NULL,
 		_e2_treedlg_refresh_cb, rt);
 #ifndef STRICT_HIDE
 	item = e2_menu_add_check (menu, _("_Strict hiding"), rt->strict_hide,
@@ -1233,7 +1247,9 @@ static gboolean _e2_treedlg_button_press_cb (GtkWidget *treeview,
 #endif
 		)
 	{
+		NEEDCLOSEBGL
 		_e2_treedlg_show_context_menu (treeview, 3, event->time, rt);
+		NEEDOPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -1250,6 +1266,8 @@ static gboolean _e2_treedlg_button_press_cb (GtkWidget *treeview,
 /*static gboolean _e2_treedlg_key_press_cb (GtkWidget *treeview,
 	GdkEventKey *event, E2_TreeDialogRuntime *rt)
 {
+//	NEEDCLOSEBGL
+//	NEEDOPENBGL
 	printd (DEBUG, "callback: _e2_treedlg key press");
 	return FALSE;
 } */
@@ -1265,7 +1283,9 @@ static gboolean _e2_treedlg_popup_menu_cb (GtkWidget *treeview,
 	E2_TreeDialogRuntime *rt)
 {
 	guint32 event_time = gtk_get_current_event_time ();
+	NEEDCLOSEBGL
 	_e2_treedlg_show_context_menu (treeview, 0, event_time, rt);
+	NEEDOPENBGL
 	return TRUE;
 }
 /**
@@ -1296,7 +1316,9 @@ static void _e2_treedlg_row_activated_cb (
 		{
 			//show that in associated filepane
 			E2_PaneRuntime *rt = (E2_PaneRuntime *)view;
+//			NEEDCLOSEBGL
 			e2_pane_change_dir (rt, newpath);
+//			NEEDOPENBGL
 			g_free (newpath);
 		}
 	}
@@ -1323,6 +1345,7 @@ static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *ite
 	gtk_tree_model_get (model, iter, DIRFLAGS, &iterflags, -1);
 	if (!(iterflags & E2_TRDLG_GRANDSCANNED))
 	{	//expanded row is not 'ready' to be expanded
+		NEEDCLOSEBGL
 //#ifdef USE_GTK2_14
 //		if (gtk_widget_get_window (rt->dialog) != NULL)
 //#else
@@ -1357,7 +1380,7 @@ static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *ite
 #endif
 		rt->opendir = thisdir;
 		gpointer result;
-		gdk_threads_leave ();
+		OPENBGL
 		if (pthread_create (&rt->threadID, NULL,
 			(void*(*)(void*))_e2_treedlg_do_shallow_scan, rt) == 0)
 		{
@@ -1371,7 +1394,7 @@ static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *ite
 			result = NULL;
 			printd (WARN, "Failed to create tree-scan thread");
 		}
-		gdk_threads_enter ();
+		CLOSEBGL
 		if (result == NULL)
 		{
 			//FIXME do what ?
@@ -1422,6 +1445,8 @@ static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *ite
 //		if (GDK_IS_WINDOW (rt->dialog->window))
 //#endif
 			e2_dialog_set_cursor (rt->dialog, GDK_LEFT_PTR);
+
+		NEEDOPENBGL
 	}
 }
 #endif //ndef ENABLE_FULL_TREE
@@ -1441,7 +1466,9 @@ static void _e2_treedlg_destroy_cb (
 {
 	g_signal_handlers_disconnect_by_func ((gpointer)rt->dialog,
 		_e2_treedlg_response_cb, rt); //no double-handling
+	NEEDOPENBGL
 	_e2_treedlg_response_cb (GTK_DIALOG (rt->dialog), 0, rt);
+	NEEDCLOSEBGL
 }
 /**
 @brief handle button click, window-close etc for directory-tree dialog
@@ -1455,11 +1482,14 @@ This is the callback for response signals emitted from @a dialog
 static void _e2_treedlg_response_cb (GtkDialog *dialog, gint response,
 	E2_TreeDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	switch (response)
 	{
 		case E2_RESPONSE_USER1: //toggle display of hidden items
 			rt->show_hidden = !rt->show_hidden;
+			NEEDOPENBGL
 			_e2_treedlg_refresh_cb (NULL, rt);	//do the content before changing button
+			NEEDCLOSEBGL
 			e2_button_set_image (rt->hiddenbtn, (rt->show_hidden) ?
 				"hidden_noshow"E2ICONTB : "hidden_show"E2ICONTB);
 			break;
@@ -1496,6 +1526,7 @@ static void _e2_treedlg_response_cb (GtkDialog *dialog, gint response,
 			DEALLOCATE (E2_TreeDialogRuntime, rt);
 			break;
 	}
+	NEEDOPENBGL
 }
 
 /**
@@ -1518,10 +1549,10 @@ static gpointer _e2_tree_dialog_run (ViewInfo *view)
 //	memset (rt->itable, 0, sizeof(rt->itable));
 
 	gchar *title = (view == &app.pane1.view) ? _("pane 1 navigator") : _("pane 2 navigator") ;
-	gdk_threads_enter ();
+	CLOSEBGL
 	rt->dialog = e2_dialog_create (NULL, NULL, title,
 		(ResponseFunc)_e2_treedlg_response_cb, rt);
-	gdk_threads_leave ();
+	OPENBGL
 	//scrolled window for the treeview
 	GtkWidget *sw = e2_widget_add_sw (
 #ifdef USE_GTK2_14
@@ -1626,14 +1657,14 @@ static gpointer _e2_tree_dialog_run (ViewInfo *view)
 		_("Toggle display of hidden directories"), NULL, NULL);
 	E2_BUTTON_CLOSE.showflags |= E2_BTN_DEFAULT; //set default button
 
-	gdk_threads_enter ();
+	CLOSEBGL
 	e2_dialog_show (rt->dialog, app.main_window, 0,	&E2_BUTTON_CLOSE, NULL);
 
 	//get the dir names
 	_e2_treedlg_fill_store (NULL, rt);
 	//show and select the startup row corresponding to displayed dir
 	_e2_treedlg_show_path (rt->view->dir, TRUE, rt);
-	gdk_threads_leave ();
+	OPENBGL
 
 	return NULL;
 }
