@@ -1,12 +1,11 @@
-/* $Id: e2_view_dialog.c 2478 2012-03-25 22:23:11Z tpgww $
+/* $Id: e2_view_dialog.c 2901 2013-10-31 22:31:16Z tpgww $
 
-Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
-This file is part of emelFM2.
-emelFM2 is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+This file is part of emelFM2, which is free software. You can redistribute it
+and/or modify it under the terms of the GNU General Public License as published
+by the Free Software Foundation - either version 3, or (at your option) any
+later version.
 
 emelFM2 is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,9 +26,12 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #ifdef E2_MOUSECUSTOM
 # include "e2_mousebinding.h"
 #endif
+#include "e2_icons.h"
 
 //enable user to specify file encoding(s) to test before opening
 //#define CHOOSE_CODING
+//enable encoding check for start of file before using external encoder
+//#define SNIFF_CODING //must also manually enable e2_fs_get_file_start()
 
 //stores for session-static and/or cached variables
 gboolean case_sensitive;
@@ -71,7 +73,9 @@ This is to turn off all highlighting of matched strings
 static void _e2_view_dialog_buffer_changed_cb (GtkTextBuffer *textbuffer,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	e2_view_dialog_clear_hilites (rt);
+	NEEDOPENBGL
 }
 /**
 @brief setup to highlight all occurrences of a string in the buffer
@@ -310,8 +314,8 @@ gboolean e2_view_dialog_search (gboolean first, gboolean incremental,
 #ifdef E2_MARK_FINDS
 		_e2_view_dialog_set_hilites (find, rt);
 #endif
-		e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
-		e2_list_update_history (find, &find_history, NULL, 30, FALSE);
+		e2_combobox_prepend_history (rt->combo, find, 20, FALSE);
+		e2_list_update_history (&find_history, find, NULL, 20, FALSE);
 	}
 #ifdef E2_MARK_FINDS
 	else
@@ -476,14 +480,14 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 	if (e2_fs_access (localpath, R_OK E2_ERR_PTR()))	//traverse a link
 #endif
 	{
-		gdk_threads_leave ();
+		OPENBGL
 		e2_fs_error_local (_("Cannot read '%s'"),
 #ifdef E2_VFS
 		&ddata E2_ERR_MSGL());
 #else
 		localpath E2_ERR_MSGL());
 #endif
-		gdk_threads_enter ();
+		CLOSEBGL
 		E2_ERR_CLEAR
 		g_free (localpath);
 		return FALSE;
@@ -513,28 +517,35 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 		utfconverter = e2_option_str_get ("command-encoder");
 		if (*utfconverter != '\0')
 		{
-			gchar *command;
-#ifdef E2_VFSTMP
-	//can't sniff non-local files
-#endif
+#ifdef SNIFF_CODING
 			//sniff the file to check if conversion is really needed
-			//this is not too good for files with mixed encoding !
-//tag E2_BADQUOTES
-			gchar *qp = e2_utils_quote_string (localpath);
-			command = e2_utils_strcat ("cat ", qp);
-			if (e2_fs_sniff_command_output (command, &contents, 1024))
+			//this is bad for files with mixed encoding !
+			OPENBGL
+			if (!e2_fs_get_file_start (
+# ifdef E2_VFS
+				&data,
+# else
+				localpath,
+# endif
+				&contents, 1024, TRUE E2_ERR_MSGL()))
 			{
-				rt->charset = e2_utf8_detect_charset ((guchar *)contents, &usable);
-#ifdef USE_GLIB2_10
-				g_slice_free1 (1024, contents);
-#else
-				g_free (contents);
-#endif
+				e2_fs_error_local (_("Cannot read %s"), localpath E2_ERR_MSGL());
+				CLOSEBGL
+				E2_ERR_CLEAR
+				g_free (localpath);
+				return FALSE;
 			}
-			else
-				usable = FALSE;
-			g_free (command);
+			CLOSEBGL
 
+			if (*(guchar *)contents != '\0')
+				rt->charset = e2_utf8_detect_charset ((guchar *)contents, &usable);
+			else
+				usable = TRUE;
+# ifdef USE_GLIB2_10
+			g_slice_free1 (1024, contents);
+# else
+			g_free (contents);
+# endif
 			if (usable)
 			{
 				utfconverter = NULL;	//signal no external conversion needed
@@ -543,7 +554,9 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 			}
 			else
 			{	//we need to do a conversion
+#endif //def SNIFF_CODING
 				gchar *utfpath = F_FILENAME_FROM_LOCALE (localpath);
+				//CHECKME don't quote utfpath, the macro should handle that
 				//substitute file for any name/path macro in command string, get language etc
 				gchar *utfcmd = e2_utils_expand_macros (utfconverter, utfpath);
 				F_FREE (utfpath, localpath);
@@ -551,15 +564,17 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 				if (utfcmd == NULL || utfcmd == GINT_TO_POINTER(1))
 				{
 					g_free (localpath);
-					g_free (qp);
 					return FALSE;
 				}
 				else
 				{
+					gchar *command;
 					if (!strcmp (utfcmd, utfconverter))
 					{	//macro expansion didn't do anything
 						gchar *tmp2 = F_FILENAME_TO_LOCALE (utfconverter);
+						gchar *qp = e2_utils_quote_string (localpath);
 						command = g_strconcat (tmp2, " ", qp, NULL);
+						g_free (qp);
 						F_FREE (tmp2, utfconverter);
 					}
 					else
@@ -576,9 +591,9 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 					g_free (utfcmd);
 					g_free (command);
 				}
+#ifdef SNIFF_CODING
 			}
-
-			g_free (qp);
+#endif
 		}
 		else
 			utfconverter = NULL;
@@ -673,8 +688,12 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 								button = gtk_check_button_new_with_label (coding_names [row][col]);
 								g_signal_connect (G_OBJECT(button), "toggled",
 									G_CALLBACK (_e2_view_dialog_lang_toggle), &drt);
+#ifdef USE_GTK3_2
+								gtk_grid_attach (GTK_GRID (table), button, row, col, 1, 1);
+#else
 								gtk_table_attach_defaults (GTK_TABLE (table), button,
 									row, row+1, col, col+1); //swapped r,c
+#endif
 								drt.chk_btns[row][col] = button;
 							}
 						}
@@ -688,12 +707,12 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 					//buttons
 					drt.test_btn = e2_dialog_add_custom_button_full
 					  ( dialog, FALSE, E2_BUTTON_APPLYTOALL.response,
-						_("_Test"), GTK_STOCK_EXECUTE, NULL, NULL, NULL);
+						_("_Test"), STOCK_NAME_EXECUTE, NULL, NULL, NULL);
 //					gtk_widget_set_sensitive (drt.test_btn, FALSE);
 					e2_dialog_add_defined_button (dialog, &E2_BUTTON_CANCEL);
 					drt.open_btn = e2_dialog_add_custom_button_full
 					  ( dialog, FALSE, E2_BUTTON_APPLY.response,
-						_("_Open"), GTK_STOCK_OK, NULL, NULL, NULL);
+						_("_Open"), STOCK_NAME_OK, NULL, NULL, NULL);
 //					gtk_widget_set_sensitive (drt.open_btn, FALSE);
 					e2_dialog_show (dialog, app.main_window, E2_DIALOG_CLOSELOCK, NULL);
 rewait:
@@ -860,8 +879,12 @@ rewait:
 	rt->textbuffer = gtk_text_buffer_new (NULL); //always create a buffer
 	if (contents)
 	{
+		GtkTextIter iter;
 		gtk_text_buffer_set_text (rt->textbuffer, contents, -1);
 		g_free (contents);
+		//cursor normally set to end of buffer
+		gtk_text_buffer_get_start_iter (rt->textbuffer, &iter);
+        gtk_text_buffer_place_cursor (rt->textbuffer, &iter);
 		printd (DEBUG, "Read text file charset is %s", rt->charset);
 	}
 
@@ -949,8 +972,10 @@ void e2_view_dialog_set_menu_position (GtkWidget *menu,
 static void _e2_view_dialog_copy_cb (GtkMenuItem *menuitem,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	GtkClipboard *cb = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 	gtk_text_buffer_copy_clipboard (rt->textbuffer, cb);
+	NEEDOPENBGL
 }
 /**
 @brief construct and pop up destroyable context-menu for the view-dialog
@@ -966,9 +991,9 @@ static void _e2_view_dialog_show_context_menu (GtkWidget *textview,
 	guint event_button, guint32 event_time, E2_ViewDialogRuntime *rt)
 {
 	gchar *item_name;
-	GtkWidget *menu = gtk_menu_new ();
+	GtkWidget *menu = e2_menu_get ();
 
-/*	GtkWidget *item = e2_menu_add (menu, _("_Find"), GTK_STOCK_FIND,
+/*	GtkWidget *item = e2_menu_add (menu, _("_Find"), STOCK_NAME_FIND,
 		NULL,  _e2_view_dialog_menu_find_cb, rt);
 #ifdef USE_GTK2_12TIPS
 	e2_widget_set_toggletip (
@@ -978,7 +1003,7 @@ static void _e2_view_dialog_show_context_menu (GtkWidget *textview,
 		item,
 		_("Show the search options bar"), _("Find the first/next occurrence"));
 
-	GtkWidget *submenu = e2_menu_add_submenu (menu, _("_Settings"), GTK_STOCK_PREFERENCES);
+	GtkWidget *submenu = e2_menu_add_submenu (menu, _("_Settings"), STOCK_NAME_PREFERENCES);
 	e2_option_create_menu (GTK_WIDGET (textview), submenu,
 		opt_view_wrap, NULL, NULL,
  etc
@@ -986,19 +1011,19 @@ static void _e2_view_dialog_show_context_menu (GtkWidget *textview,
 		app.output.opt_show_on_focus_in, NULL, NULL,
 		NULL);
 */
-	GtkWidget *item = e2_menu_add (menu, _("_Copy"), GTK_STOCK_COPY,
+	GtkWidget *item = e2_menu_add (menu, _("_Copy"), STOCK_NAME_COPY,
 		_("Copy selected text"), _e2_view_dialog_copy_cb, rt);
 	gtk_widget_set_sensitive (item,
 		gtk_text_buffer_get_selection_bounds (rt->textbuffer, NULL, NULL));
 #ifdef USE_GTK2_10
 	gchar *tip = (gtk_text_buffer_get_has_selection (rt->textbuffer)) ?
 		_("Print selected text") : _("Print file");
-	e2_menu_add (menu, _("_Print.."), GTK_STOCK_PRINT, tip,
+	e2_menu_add (menu, _("_Print.."), STOCK_NAME_PRINT, tip,
 		e2_dialog_print_cb, rt);
 #endif
 	e2_menu_add_separator (menu);
-	item_name = g_strconcat (_A(3),".",_A(32),NULL);
-	e2_menu_add_action (menu, _("_Settings"), GTK_STOCK_PREFERENCES,
+	item_name = g_strconcat (_A(3),".",_A(34),NULL);
+	e2_menu_add_action (menu, _("_Settings"), STOCK_NAME_PREFERENCES,
 		_("Open the configuration dialog at the options page"), item_name,
 		_C(11)); //_("dialogs")
 	g_free(item_name);
@@ -1027,7 +1052,11 @@ GtkWidget *e2_view_dialog_create_searchbar (E2_ViewDialogRuntime *rt)
 	rt->is_hidden = TRUE;	//searchbar is not displayed until a search is started
 	rt->release_blocked = FALSE;
 
+#ifdef USE_GTK3_0
+	GtkWidget *hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	//add search-string combo
 	//CHECKME close BGL to cut latency ?
 	//cuz E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, downstream calls
@@ -1139,10 +1168,10 @@ void e2_view_dialog_destroy (E2_ViewDialogRuntime *rt)
 			GTK_BIN (rt->combo)->child;
 #endif
 		const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
-		if (*find != '\0')
+		if (find != NULL && *find != '\0')
 		{
-			e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
-			e2_list_update_history (find, &find_history, NULL, 30, FALSE);
+			e2_combobox_prepend_history (rt->combo, find, 20, FALSE);
+			e2_list_update_history (&find_history, find, NULL, 20, FALSE);
 		}
 	}
 	//backup some dialog-specific options
@@ -1230,7 +1259,9 @@ static gboolean _e2_view_dialog_popup_menu_cb (GtkWidget *textview,
 	E2_ViewDialogRuntime *rt)
 {
 	guint32 event_time = gtk_get_current_event_time ();
+	NEEDCLOSEBGL
 	_e2_view_dialog_show_context_menu (textview, 0, event_time, rt);
+	NEEDOPENBGL
 	return TRUE;
 }
 /**
@@ -1251,7 +1282,9 @@ static gboolean _e2_view_dialog_button_press_cb (GtkWidget *textview,
 #endif
 		)
 	{
+		NEEDCLOSEBGL
 		_e2_view_dialog_show_context_menu (textview, 3, event->time, rt);
+		NEEDOPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -1266,6 +1299,8 @@ static gboolean _e2_view_dialog_button_press_cb (GtkWidget *textview,
 */
 void e2_view_dialog_toggled_cb (GtkToggleButton *button, gboolean *store)
 {
+//	NEEDCLOSEBGL
+//	NEEDOPENBGL
 	*store =
 #ifdef USE_GTK2_14
 		gtk_toggle_button_get_active (button);
@@ -1314,22 +1349,25 @@ static gboolean _e2_view_dialog_key_press_cb (GtkWidget *widget, GdkEventKey *ev
 	if (//g_ascii_isalpha (event->keyval) //&& (event->keyval < 0xF000 || event->keyval > 0xFFFF) //&&
 		(mask & GDK_CONTROL_MASK || mask == GDK_MOD1_MASK))
 	{	//the key is a letter and the modifier is Alt or a modifier is Ctrl (CHECKME support Shift+Ctrl to reverse direction?)
+//		NEEDCLOSEBGL
 		guint lower = (gdk_keyval_is_upper (event->keyval)) ?
 			gdk_keyval_to_lower (event->keyval) : event->keyval;
 		if (lower == find_keycode || lower == findnext_keycode)
 		{
 			rt->release_blocked = TRUE;	//block anomalous key-release-event searches
-
+//			NEEDOPENBGL
 			_e2_view_dialog_response_cb (GTK_DIALOG (rt->dialog),
 				(lower == find_keycode) ? E2_RESPONSE_FIND : E2_RESPONSE_USER2, rt);
 			return TRUE;
 		}
 		else if (lower == hide_keycode)
 		{
+//			NEEDOPENBGL
 			_e2_view_dialog_response_cb (GTK_DIALOG (rt->dialog),
 				E2_RESPONSE_USER3, rt);
 			return TRUE;
 		}
+//		NEEDOPENBGL
 	}
 //	printd (DEBUG, "_e2_view_dialog_key_press_cb returns FALSE");
 	return FALSE;
@@ -1349,6 +1387,7 @@ special cases are detected.
 gboolean e2_view_dialog_combokey_cb (GtkWidget *entry, GdkEventKey *event,
 	E2_ViewDialogRuntime *rt)
 {
+	NEEDCLOSEBGL
 	if (event->keyval == GDK_BackSpace || event->keyval == GDK_Delete)
 		e2_view_dialog_search (FALSE, TRUE, rt);
 	else
@@ -1381,11 +1420,13 @@ gboolean e2_view_dialog_combokey_cb (GtkWidget *entry, GdkEventKey *event,
 					if (rt->release_blocked)	//this is an event after a textview keypress
 					{
 						rt->release_blocked = FALSE;
+						NEEDOPENBGL
 						return FALSE;
 					}
 					else
 					{
 						printd (DEBUG, "e2_view_dialog_combokey_cb - going to response cb");
+						NEEDOPENBGL
 						_e2_view_dialog_response_cb (GTK_DIALOG (rt->dialog),
 							(lower == find_keycode) ? E2_RESPONSE_FIND : E2_RESPONSE_USER2,
 							rt);
@@ -1395,9 +1436,13 @@ gboolean e2_view_dialog_combokey_cb (GtkWidget *entry, GdkEventKey *event,
 				else if (lower == hide_keycode)
 				{
 					if (rt->replacebar != NULL)	//actually in edit mode
+					{
+						NEEDOPENBGL
 						return (e2_edit_dialog_key_press_cb (rt->textview, event, rt));
+					}
 					else
 					{
+						NEEDOPENBGL
 						_e2_view_dialog_response_cb (GTK_DIALOG (rt->dialog),
 							E2_RESPONSE_USER3, rt);
 						return TRUE;
@@ -1406,13 +1451,17 @@ gboolean e2_view_dialog_combokey_cb (GtkWidget *entry, GdkEventKey *event,
 				else if (lower == replace_keycode)
 				{	//this combo is used for the edit dialog too
 					if (rt->replacebar != NULL)	//actually in edit mode
+					{
+						NEEDOPENBGL
 						return (e2_edit_dialog_key_press_cb (rt->textview, event, rt));
+					}
 				}
 			}
 			// by default, do an incremental search
 			e2_view_dialog_search (FALSE, TRUE, rt);
 		}
 	}
+	NEEDOPENBGL
 	return FALSE;
 }
 /**
@@ -1429,115 +1478,117 @@ static void _e2_view_dialog_response_cb (GtkDialog *dialog, gint response,
 	E2_ViewDialogRuntime *rt)
 {
 	printd (DEBUG, "_e2_view_dialog_response_cb (dialog:_,response:%d,rt:_)", response);
+	NEEDCLOSEBGL
 	switch (response)
 	{
-		case E2_RESPONSE_USER1:  //text wrap
-			rt->textwrap = !rt->textwrap;
-			gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
-				rt->textwrap ? GTK_WRAP_WORD: GTK_WRAP_NONE);
-			break;
-		case E2_RESPONSE_FIND:
-		case E2_RESPONSE_USER2:	//find next (after <Ctrl>g press)
-			if (!rt->is_hidden) //search panel is visible
-			{
-				//these tests are irrelevant for an actual response callback,
-				//as no callback happens then
-				//if a mod key is pressed, treat it specially
-				//Ctrl = first, Shift = temp reverse
-				GdkModifierType mask =
+	  case E2_RESPONSE_USER1:  //text wrap
+		rt->textwrap = !rt->textwrap;
+		gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
+			rt->textwrap ? GTK_WRAP_WORD: GTK_WRAP_NONE);
+		break;
+	  case E2_RESPONSE_FIND:
+	  case E2_RESPONSE_USER2:	//find next (after <Ctrl>g press)
+		if (!rt->is_hidden) //search panel is visible
+		{
+			//these tests are irrelevant for an actual response callback,
+			//as no callback happens then
+			//if a mod key is pressed, treat it specially
+			//Ctrl = first, Shift = temp reverse
+			GdkModifierType mask =
 #ifdef USE_GTK3_0
-					e2_utils_get_savedstate (rt->dialog)
+				e2_utils_get_savedstate (rt->dialog)
 #else
-					e2_utils_get_modifiers ()
+				e2_utils_get_modifiers ()
 #endif
-					& gtk_accelerator_get_default_mod_mask ();
-				if (mask & GDK_SHIFT_MASK)
-				{	//temporary direction change
-					rt->search_backward = !rt->search_backward;
-					e2_view_dialog_search (response == E2_RESPONSE_FIND, FALSE, rt);
-					rt->search_backward = !rt->search_backward;
-				}
-				else
-					//perform non-incremental search, from start|end if <Control> is pressed
-					e2_view_dialog_search (response == E2_RESPONSE_FIND, FALSE, rt);
+				& gtk_accelerator_get_default_mod_mask ();
+			if (mask & GDK_SHIFT_MASK)
+			{	//temporary direction change
+				rt->search_backward = !rt->search_backward;
+				e2_view_dialog_search (response == E2_RESPONSE_FIND, FALSE, rt);
+				rt->search_backward = !rt->search_backward;
 			}
 			else
-			{	//show the search panel
-				gtk_widget_show (rt->panel);
-				rt->is_hidden = FALSE;
-				if (rt->hidebtn != NULL)
-					gtk_widget_show (rt->hidebtn);
+				//perform non-incremental search, from start|end if <Control> is pressed
+				e2_view_dialog_search (response == E2_RESPONSE_FIND, FALSE, rt);
+		}
+		else
+		{	//show the search panel
+			gtk_widget_show (rt->panel);
+			rt->is_hidden = FALSE;
+			if (rt->hidebtn != NULL)
+				gtk_widget_show (rt->hidebtn);
 
-				GtkWidget *entry =
-#ifdef USE_GTK2_14
-					gtk_bin_get_child (GTK_BIN (rt->combo));
-#else
-					GTK_BIN (rt->combo)->child;
-#endif
-				GtkTextIter start_iter, end_iter;
-				if (gtk_text_buffer_get_selection_bounds (rt->textbuffer,
-						&start_iter, &end_iter))
-				{
-					gchar *find_string = gtk_text_buffer_get_text
-						(rt->textbuffer, &start_iter, &end_iter, FALSE);
-					gtk_entry_set_text (GTK_ENTRY (entry), find_string);
-					gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
-					e2_view_dialog_search (response == E2_RESPONSE_FIND, FALSE, rt); //CHECKME never search for first when first showing the bar
-					g_free (find_string);
-				}
-				else
-					e2_view_dialog_update_combo (rt->combo);
-
-				//focus on the text entry
-				gtk_widget_grab_focus (entry);
-				//swap find btn tip
-				if (rt->findbtn != NULL)
-					e2_widget_swap_tooltip (rt->findbtn);
-			}
-			break;
-/*		case E2_RESPONSE_USER2:	//find next
-			_e2_view_dialog_search (FALSE, FALSE, rt);
-			break; */
-		case E2_RESPONSE_USER3:	//hide
-		{
-			//in case we last performed an incremental search ...
 			GtkWidget *entry =
 #ifdef USE_GTK2_14
 				gtk_bin_get_child (GTK_BIN (rt->combo));
 #else
 				GTK_BIN (rt->combo)->child;
 #endif
-			const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
-			if (*find != '\0')
+			GtkTextIter start_iter, end_iter;
+			if (gtk_text_buffer_get_selection_bounds (rt->textbuffer,
+					&start_iter, &end_iter))
 			{
-				e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
-				e2_list_update_history (find, &find_history, NULL, 30, FALSE);
+				gchar *find_string = gtk_text_buffer_get_text
+					(rt->textbuffer, &start_iter, &end_iter, FALSE);
+				gtk_entry_set_text (GTK_ENTRY (entry), find_string);
+				gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
+				e2_view_dialog_search (response == E2_RESPONSE_FIND, FALSE, rt); //CHECKME never search for first when first showing the bar
+				g_free (find_string);
 			}
-			gtk_widget_hide (rt->panel);
-			rt->is_hidden = TRUE;
-//			gtk_widget_hide (rt->nextbtn);
-			if (rt->hidebtn != NULL)	//in view mode
-				gtk_widget_hide (rt->hidebtn);
-			gtk_widget_hide (rt->info_label);
-//			gtk_widget_show (rt->refreshbtn);
-#ifdef E2_MARK_FINDS
-			//turn off any highlihts
-			e2_view_dialog_clear_hilites (rt);
-#endif
-			//hide any selection
-			GtkTextIter iter;
-			if (gtk_text_buffer_get_selection_bounds (rt->textbuffer, &iter, NULL))
-				gtk_text_buffer_select_range (rt->textbuffer, &iter, &iter);
-			//revert find btn tip
+			else
+				e2_view_dialog_update_combo (rt->combo);
+
+			//focus on the text entry
+			gtk_widget_grab_focus (entry);
+			//swap find btn tip
 			if (rt->findbtn != NULL)
 				e2_widget_swap_tooltip (rt->findbtn);
-			gtk_widget_grab_focus (rt->textview);
 		}
-			break;
-		default:
-			e2_view_dialog_destroy (rt);
-			break;
+		break;
+/*	  case E2_RESPONSE_USER2:	//find next
+		_e2_view_dialog_search (FALSE, FALSE, rt);
+		break; */
+	  case E2_RESPONSE_USER3:	//hide
+	  {
+		//in case we last performed an incremental search ...
+		GtkWidget *entry =
+#ifdef USE_GTK2_14
+			gtk_bin_get_child (GTK_BIN (rt->combo));
+#else
+			GTK_BIN (rt->combo)->child;
+#endif
+		const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
+		if (find != NULL && *find != '\0')
+		{
+			e2_combobox_prepend_history (rt->combo, find, 20, FALSE);
+			e2_list_update_history (&find_history, find, NULL, 20, FALSE);
+		}
+		gtk_widget_hide (rt->panel);
+		rt->is_hidden = TRUE;
+//			gtk_widget_hide (rt->nextbtn);
+		if (rt->hidebtn != NULL)	//in view mode
+			gtk_widget_hide (rt->hidebtn);
+		gtk_widget_hide (rt->info_label);
+//			gtk_widget_show (rt->refreshbtn);
+#ifdef E2_MARK_FINDS
+		//turn off any highlihts
+		e2_view_dialog_clear_hilites (rt);
+#endif
+		//hide any selection
+		GtkTextIter iter;
+		if (gtk_text_buffer_get_selection_bounds (rt->textbuffer, &iter, NULL))
+			gtk_text_buffer_select_range (rt->textbuffer, &iter, &iter);
+		//revert find btn tip
+		if (rt->findbtn != NULL)
+			e2_widget_swap_tooltip (rt->findbtn);
+		gtk_widget_grab_focus (rt->textview);
+	  }
+		break;
+	  default:
+		e2_view_dialog_destroy (rt);
+		break;
 	}
+	NEEDOPENBGL
 }
 
 #ifdef E2_TRANSIENTBINDINGS
@@ -1557,27 +1608,26 @@ static void _e2_view_dialog_keybindings (E2_OptionSet *set)
 	//treestore-iter of the full category name
 //	g_strconcat(_C(17),"||||",NULL),  //_("general"
 //	g_strconcat("\t",_C(11),"||||",NULL),  //_("dialogs"
-	g_strconcat("\t\t",_A(106),"||||",NULL),  //_("view"
-	g_strconcat("\t\t\t|<Control>j","||","echo|\"up\"", NULL),//_A(124),".",_A(125),"|<Control>a",NULL),
-	g_strconcat("\t\t\t|<Control>k","||","echo|\"down\"", NULL),//_A(124),".",_A(125),"|<Control>c",NULL),
+	g_strconcat("\t\t",_A(109),"||||",NULL),  //_("view"
+	g_strconcat("\t\t\t|<Control>j","||","echo|\"up\"", NULL),//_A(127),".",_A(128),"|<Control>a",NULL),
+	g_strconcat("\t\t\t|<Control>k","||","echo|\"down\"", NULL),//_A(127),".",_A(128),"|<Control>c",NULL),
 	g_strdup(">"),
 	NULL);
 }
 # ifdef E2_MOUSECUSTOM
 static void _e2_view_dialog_mousebindings (E2_OptionSet *set)
 {
-	//the key name strings are parsed by gtk, and no translation is possible
-	//if releases handled, lines must have one extra (=6) separator
 	e2_option_tree_setup_defaults (set,
 	g_strdup("mousebuttons=<"),  //internal name
-	//the column-0 category string(s) here need to match at least the lowest
-	//treestore-iter of the full category name
-//	g_strconcat(_C(17),"|||||",NULL),  //_("general"
-//	g_strconcat("\t",_C(11),"|||||",NULL),  //_("dialogs"
-	g_strconcat("\t\t",_A(106),"|||||",NULL),  //_("view"
+/*	these lines go into category general.dialogs, so at least 2 leading tabs
+	the button name strings are parsed by gtk, and no translation is possible, button codes may be 1 ... whatever
+	columns: 0 cat, 1 button, 2 dblclick, 3 trplclick, [4 release,] 5 action, 6 action_data
+	if releases handled (#ifdef MCOL_REL), lines must have one extra (=6) separator
+*/
+	g_strconcat("\t\t",_A(109),"|||||",NULL),  //_("view"
 #ifdef WITH_KEYFAKE
-	g_strconcat("\t\t\t|<Control>6","|||",_A(124),".",_A(125),"|<Control>a",NULL),
-	g_strconcat("\t\t\t|<Control>7","|||",_A(124),".",_A(125),"|<Control>c",NULL),
+	g_strconcat("\t\t\t|<Control>6","|||",_A(127),".",_A(128),"|<Control>a",NULL),
+	g_strconcat("\t\t\t|<Control>7","|||",_A(127),".",_A(128),"|<Control>c",NULL),
 #endif
 	//MORE HERE
 	g_strdup(">"),
@@ -1592,10 +1642,10 @@ static void _e2_view_dialog_mousegestures (E2_OptionSet *set)
 	//treestore-iter of the full category name
 //	g_strconcat(_C(17),"||||",NULL),  //_("general"
 //	g_strconcat("\t",_C(11),"||||",NULL),  //_("dialogs"
-	g_strconcat("\t\t",_A(106),"||||",NULL),  //_("view"
+	g_strconcat("\t\t",_A(109),"||||",NULL),  //_("view"
 #ifdef WITH_KEYFAKE
-	g_strconcat("\t\t\t|<Alt>3|^|8,2|",_A(124),".",_A(125),"|Page_Up",NULL), //"key.fake"
-	g_strconcat("\t\t\t|<Alt>3||2,8|",_A(124),".",_A(125),"|Page_Down",NULL), //"key.fake"
+	g_strconcat("\t\t\t|<Alt>3|^|8,2|",_A(127),".",_A(128),"|Page_Up",NULL), //"key.fake"
+	g_strconcat("\t\t\t|<Alt>3||2,8|",_A(127),".",_A(128),"|Page_Down",NULL), //"key.fake"
 #endif
 	//MORE HERE
 	g_strdup(">"),
@@ -1705,11 +1755,14 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 	gtk_box_reorder_child (GTK_BOX (dialog_vbox), rt->panel, 1);
 
 	//add handlebox
+#ifdef USE_GTK3_4
+	GtkWidget *hndlbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	//if context menu wanted, add event box c.f. toolbar
+#else
 	GtkWidget *hndlbox = gtk_handle_box_new ();
-	gtk_handle_box_set_shadow_type (GTK_HANDLE_BOX (hndlbox),
-		GTK_SHADOW_NONE);
-	gtk_handle_box_set_handle_position (GTK_HANDLE_BOX (hndlbox),
-		GTK_POS_LEFT);
+	gtk_handle_box_set_shadow_type (GTK_HANDLE_BOX (hndlbox), GTK_SHADOW_NONE);
+	gtk_handle_box_set_handle_position ((GtkHandleBox*)hndlbox, GTK_POS_LEFT);
+#endif
 	gtk_container_add (GTK_CONTAINER (rt->panel), hndlbox);
 
 	GtkWidget *hbox = e2_view_dialog_create_searchbar (rt);
@@ -1736,7 +1789,7 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 		hide_keycode = asciicode;
 
 	rt->hidebtn = e2_dialog_add_simple_button
-		(rt->dialog, GTK_STOCK_ZOOM_FIT, labeltext, E2_RESPONSE_USER3);
+		(rt->dialog, STOCK_NAME_ZOOM_FIT, labeltext, E2_RESPONSE_USER3);
 	e2_widget_set_safetip (rt->hidebtn, _("Hide the search options bar"));
 
 	e2_dialog_add_check_button (rt->dialog, rt->textwrap, _("_wrap"),
@@ -1751,7 +1804,7 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 		find_keycode = asciicode;
 
 	rt->findbtn = e2_dialog_add_simple_button
-		(rt->dialog, GTK_STOCK_FIND, labeltext, E2_RESPONSE_FIND);
+		(rt->dialog, STOCK_NAME_FIND, labeltext, E2_RESPONSE_FIND);
 #ifdef USE_GTK2_12TIPS
 	e2_widget_set_toggletip (
 #else
@@ -1781,7 +1834,7 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 	//add dialog-specific key bindings, before the key-press callback
 	//see also dialog-general
 	//group name (must be freeable)
-	gchar *category = g_strconcat (_C(17),".",_C(11),".",_A(106),NULL);	//_(general.dialogs.view
+	gchar *category = g_strconcat (_C(17),".",_C(11),".",_A(109),NULL);	//_(general.dialogs.view
 	e2_keybinding_enrol (GTK_WIDGET (rt->textview), category, _e2_view_dialog_keybindings);
 # ifdef E2_MOUSECUSTOM
 	e2_mousebinding_enrol (rt->dialog, category, _e2_view_dialog_mousebindings);
@@ -1791,12 +1844,11 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 # endif
 	g_free (category);
 #endif
-
-	g_signal_connect (G_OBJECT (rt->textview), "popup-menu",
-		G_CALLBACK (_e2_view_dialog_popup_menu_cb), rt);
 	//this cb does generic stuff and is needed with or without E2_MOUSECUSTOM
 	g_signal_connect (G_OBJECT (rt->textview), "button-press-event",
 		G_CALLBACK (_e2_view_dialog_button_press_cb), rt);
+	g_signal_connect (G_OBJECT (rt->textview), "popup-menu",
+		G_CALLBACK (_e2_view_dialog_popup_menu_cb), rt);
 	g_signal_connect (G_OBJECT (rt->textview), "key-press-event",
 		G_CALLBACK (_e2_view_dialog_key_press_cb), rt);
 
@@ -1875,7 +1927,7 @@ static gboolean _e2_view_dialog_reviewQ (E2_ActionTaskData *qed)
 #endif
 		E2_ViewDialogRuntime *vrt;
 
-		gdk_threads_enter ();
+		CLOSEBGL
 		GtkWidget *dialog = _e2_view_dialog_create
 #ifdef E2_VFS
 			(&ddata, &vrt);
@@ -1895,7 +1947,7 @@ static gboolean _e2_view_dialog_reviewQ (E2_ActionTaskData *qed)
 		}
 		else
 			retval = FALSE;
-		gdk_threads_leave ();
+		OPENBGL
 	}
 	g_free (localpath);
 	return retval;
@@ -1929,14 +1981,14 @@ static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 	VPATH ddata = { view_this, qed->currspace };
 #endif
 	E2_ViewDialogRuntime *vrt;
-	gdk_threads_enter ();	//dialog setup wants BGL closed
+	CLOSEBGL	//dialog setup wants BGL closed
 	GtkWidget *dialog = _e2_view_dialog_create
 #ifdef E2_VFS
  		(&ddata, &vrt);
 #else
 		(view_this, &vrt);
 #endif
-	gdk_threads_leave ();
+	OPENBGL
 
 	if (target != NULL)
 		*target = ' ';	//revert the argument, ready for next time
@@ -1957,7 +2009,7 @@ static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 		}
 		else
 			found = FALSE;
-		gdk_threads_enter ();
+		CLOSEBGL
 //		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
 		e2_dialog_setup (dialog, app.main_window);
 //		gtk_window_present (GTK_WINDOW (dialog));
@@ -1974,7 +2026,7 @@ static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 				0.0, TRUE, 0.0, 0.5);
 //			WAIT_FOR_EVENTS
 		}
-		gdk_threads_leave ();
+		OPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -2029,9 +2081,9 @@ gboolean e2_view_dialog_create_immediate (VPATH *view_this)
 #ifdef E2_VFS
 	qed.currspace = view_this->spacedata;
 #endif
-	gdk_threads_leave ();	//turn off the BGL
+	OPENBGL	//turn off the BGL
 	gboolean retval = _e2_view_dialog_viewatQ (&qed);
-	gdk_threads_enter ();
+	CLOSEBGL
 	return retval;
 }
 /**
@@ -2042,8 +2094,8 @@ void e2_view_dialog_actions_register (void)
 {
 	E2_Action actions[] =
 	{
-	{g_strconcat(_A(6),".",_A(107),NULL),_e2_view_dialog_reopen, FALSE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
-	{g_strconcat(_A(6),".",_A(108),NULL),_e2_view_dialog_openat, TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
+	{g_strconcat(_A(6),".",_A(110),NULL),_e2_view_dialog_reopen, FALSE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
+	{g_strconcat(_A(6),".",_A(111),NULL),_e2_view_dialog_openat, TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
 	};
 	guint i;
 	for (i = 0; i < 2; i++)
