@@ -1,6 +1,6 @@
-/* $Id: e2_bookmark.c 2309 2011-01-17 00:12:22Z tpgww $
+/* $Id: e2_bookmark.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2004-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -20,13 +20,13 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 /**
 @file src/e2_bookmark.c
 @brief bookmark functions
-
 */
 
 #include "e2_bookmark.h"
 #include <string.h>
 #include "e2_dialog.h"
 #include "e2_task.h"
+#include "e2_icons.h"
 #include "e2_option_tree.h"
 
 static E2_OptionSet *bookmarks_set;
@@ -90,8 +90,8 @@ Config dialog page buttons are de-sensitized if the row is the toolbar 'parent'
 @param set data struct for the keybings option
 @return TRUE always (the row is always selectable)
 */
-static gboolean _e2_bookmark_tree_selection_check_cb
-	(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path,
+static gboolean _e2_bookmark_tree_selection_check_cb (GtkTreeSelection *selection,
+	GtkTreeModel *model, GtkTreePath *path,
 	gboolean path_currently_selected, E2_OptionSet *set)
 {
 	GtkTreeIter iter;
@@ -102,7 +102,9 @@ static gboolean _e2_bookmark_tree_selection_check_cb
 		gboolean result = strcmp (label, _C(41));
 		g_free (label);
 		GtkTreeView *view = gtk_tree_selection_get_tree_view (selection);
+		NEEDCLOSEBGL
 		e2_option_tree_adjust_buttons (view, result);
+		NEEDOPENBGL
 	}
 	return TRUE;
 }
@@ -121,6 +123,8 @@ Checks whether @a iter in @a model has the name of the taskbar
 static gboolean _e2_bookmark_visible_check_cb (GtkTreeModel *model,
 	GtkTreeIter *iter, GtkCellRenderer *cell, gpointer data)
 {
+//	NEEDCLOSEBGL
+//	NEEDOPENBGL
 	gchar *label;
 	gtk_tree_model_get (model, iter, 0, &label, -1);
 	gboolean result = strcmp (label, _C(41));
@@ -136,9 +140,11 @@ If so, the row is not draggable
 @param path tree path to a row on which user is initiating a drag
 @return TRUE if the row can be dragged
 */
-static gboolean _e2_bookmark_tree_draggable_check_cb
-	(GtkTreeDragSource *drag_source, GtkTreePath *path)
+static gboolean _e2_bookmark_tree_draggable_check_cb (
+	GtkTreeDragSource *drag_source, GtkTreePath *path)
 {
+//	NEEDCLOSEBGL
+//	NEEDOPENBGL
 	if (!GTK_IS_TREE_MODEL (drag_source))
 		return TRUE;
 	GtkTreeModel *model = GTK_TREE_MODEL (drag_source);
@@ -163,6 +169,7 @@ static gboolean _e2_bookmark_tree_draggable_check_cb
 static void _e2_bookmark_confirm_response_cb (GtkDialog *dialog,
 	gint response, GtkTreeRowReference *ref)
 {
+	NEEDCLOSEBGL
 	if (response == GTK_RESPONSE_YES)
 	{
 		if (ref != NULL)
@@ -178,6 +185,7 @@ static void _e2_bookmark_confirm_response_cb (GtkDialog *dialog,
 		}
 	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
+	NEEDOPENBGL
 }
 /**
 @brief callback to add an item (directory) to the bookmarks list
@@ -231,9 +239,9 @@ static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 	if (arg != NULL && *arg != '\0')
 	{
 		gchar *n;
-		if (strstr (arg, _A(119)) != NULL)  //_(top
+		if (strstr (arg, _A(122)) != NULL)  //_(top
 			before = TRUE;
-		if (strstr (arg, _A(109)) != NULL) //_(child
+		if (strstr (arg, _A(112)) != NULL) //_(child
 		{
 			before = TRUE;
 			parent = TRUE;
@@ -304,7 +312,7 @@ static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 	} */
 	void *options[4];
 	options[0] = basename;	//menu label
-	options[1] = GTK_STOCK_DIRECTORY;  //generic icon in case it's in taskbar
+	options[1] = STOCK_NAME_DIRECTORY;  //generic icon in case it's in taskbar
 	//strip the trailing / for paths other than root
 	gint len = strlen (dirname);
 	if (len > 1)
@@ -335,11 +343,16 @@ static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 		e2_option_tree_add (bookmarks_set, &iter, NULL, FALSE, !before, 4, options);
 
 	ViewInfo *view = curr_view;
+	
+	NEEDCLOSEBGL
+	
 	_e2_bookmark_recreate_toolbars (); //kludgy way to update bookmarks data in bars
 	if (view != curr_view)
 		e2_pane_activate_other ();
 	gtk_widget_grab_focus (view->treeview);
 
+	NEEDOPENBGL
+
 	if (s != NULL)
 		g_free (s);
 	g_free (basename);
@@ -356,6 +369,7 @@ static void _e2_bookmark_delete_cb (GtkMenuItem *widget, GtkTreePath *path)
 	GtkTreeIter iter;
 	if (gtk_tree_model_get_iter (bookmarks_set->ex.tree.model, &iter, path))
 	{
+		NEEDCLOSEBGL
 		gint children = gtk_tree_model_iter_n_children (bookmarks_set->ex.tree.model, &iter);
 		if (e2_option_bool_get ("bookmarks-confirm-delete")
 			|| ((children > 0) && (e2_option_bool_get ("bookmarks-confirm-multi-delete")))
@@ -373,7 +387,7 @@ static void _e2_bookmark_delete_cb (GtkMenuItem *widget, GtkTreePath *path)
 			else
 				question = g_strdup_printf ("%s '<b>%s</b>' ?", prompt, name);
 
-			GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION, question,
+			GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_QUESTION, question,
 				_("confirm bookmark delete"),
 				(ResponseFunc)_e2_bookmark_confirm_response_cb,
 				gtk_tree_row_reference_new (bookmarks_set->ex.tree.model, path));
@@ -391,6 +405,7 @@ static void _e2_bookmark_delete_cb (GtkMenuItem *widget, GtkTreePath *path)
 		}
 		else	//no confirmation
 			_e2_bookmark_delete_mark (&iter); //changes active pane and focus-widget
+		NEEDOPENBGL
 	}
 }
 /**
@@ -404,7 +419,9 @@ static void _e2_bookmark_change_selected_cb (GtkWidget *menu, E2_MarkData *mdata
 	e2_menu_selection_done_cb (menu, NULL); //clean up actions-menu
 	if (mdata->top_menu != NULL)
 	{
+		NEEDCLOSEBGL
 		gtk_menu_popdown (GTK_MENU (mdata->top_menu)); //hide marks-menu and all subs
+		NEEDOPENBGL
 		if (mdata->mark_flags & E2_MARKFLAG_ONETIME)
 			e2_menu_selection_done_cb (mdata->top_menu, NULL); //clean up marks-menu
 	}
@@ -446,7 +463,11 @@ gboolean e2_bookmark_button_press_cb (GtkWidget *widget, GdkEventButton *event,
 #endif
 					e2_pane_change_dir (other_pane, mdata->mark_path);	//mark string is UTF-8
 					if (e2_option_bool_get ("bookmarks-focus-after-open"))
+					{
+						NEEDCLOSEBGL
 						e2_pane_activate_other ();
+						NEEDOPENBGL
+					}
 					return TRUE;
 				}
 			}
@@ -467,30 +488,32 @@ gboolean e2_bookmark_button_press_cb (GtkWidget *widget, GdkEventButton *event,
 				else
 					parent = FALSE;
 
-				menu = gtk_menu_new ();
+				menu = e2_menu_get ();
 				//adding a child to an existing mark prevents its operation in the menu
 				if (parent)
 				{
 					//must preface callback arg with ',' to ensure correct parsing
-					gchar *arg = e2_utils_strcat (",", _A(109)); //_(child
-					menuitem = e2_menu_add (menu, _("_Add child"), GTK_STOCK_INDENT,
+					gchar *arg = e2_utils_strcat (",", _A(112)); //_(child
+					menuitem = e2_menu_add (menu, _("_Add child"), STOCK_NAME_INDENT,
 						_("Bookmark the current directory a child of the selected item"),
 						_e2_bookmark_add_cb, arg);
 					g_object_set_data_full (G_OBJECT (menuitem), "__freeme", arg, g_free);
 				}
 				else
-					menuitem = e2_menu_add (menu, _("_Add after"), GTK_STOCK_ADD,
+					menuitem = e2_menu_add (menu, _("_Add after"), STOCK_NAME_ADD,
 						_("Bookmark the current directory after the selected item"),
 						_e2_bookmark_add_cb, NULL);
 				g_object_set_data (G_OBJECT (menuitem), "mark-data", mdata);
 
-				e2_menu_add (menu, _("_Delete"), GTK_STOCK_DELETE,
+				e2_menu_add (menu, _("_Delete"), STOCK_NAME_DELETE,
 					_("Delete the selected bookmark, and its children if any"),
 					_e2_bookmark_delete_cb, mdata->tpath);
 
 				g_signal_connect (G_OBJECT (menu), "selection-done",
 					G_CALLBACK (_e2_bookmark_change_selected_cb), mdata);
+				NEEDCLOSEBGL
 				e2_menu_popup (menu, event->button, event->time);
+				NEEDOPENBGL
 				return TRUE;
 			}
 		}
@@ -518,7 +541,9 @@ static gboolean _e2_bookmark_add (gpointer from, E2_ActionRuntime *art)
 	if (rt == NULL)
 		return FALSE;
 */
+	NEEDOPENBGL
 	_e2_bookmark_add_cb (NULL, (const gchar *) art->data);
+	NEEDCLOSEBGL
 	return TRUE;
 }
 /**
@@ -544,7 +569,9 @@ static gboolean _e2_bookmark_delete (gpointer from, E2_ActionRuntime *art)
 		{
 			GtkTreePath *tpath;
 			tpath = gtk_tree_model_get_path (bookmarks_set->ex.tree.model, &iter);
+			NEEDOPENBGL
 			_e2_bookmark_delete_cb (NULL, tpath);
+			NEEDCLOSEBGL
 			gtk_tree_path_free (tpath);
 			retval = TRUE;
 		}
@@ -578,10 +605,10 @@ static gboolean _e2_bookmark_show (gpointer from, E2_ActionRuntime *art)
 			pane = E2PANECUR;
 		else
 			pane = (rt == &app.pane1) ? E2PANE1 : E2PANE2;
-		gchar *action_name = g_strconcat (_A(0),".",_A(27),NULL);
+		gchar *action_name = g_strconcat (_A(0),".",_A(28),NULL);
 		E2_Action *action = e2_action_get (action_name);
 		g_free (action_name);
-		GtkWidget *menu = gtk_menu_new ();
+		GtkWidget *menu = e2_menu_get ();
 		e2_menu_add_bookmark_items (menu, menu, action,
 			E2_MARKFLAG_ACTIONS | E2_MARKFLAG_ONETIME, pane, set->ex.tree.model, &iter);
 		g_signal_connect (G_OBJECT (menu), "selection-done",
@@ -655,9 +682,9 @@ void e2_bookmark_actions_register (void)
 {
 	E2_Action actions[] =
 	{
-	{g_strconcat(_A(0),".",_A(27),NULL), _e2_bookmark_open,      TRUE, E2_ACTION_TYPE_BOOKMARKS, E2_ACTION_EXCLUDE_ACCEL, NULL, NULL},
-	{g_strconcat(_A(0),".",_A(30),NULL), _e2_bookmark_add,       TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
-	{g_strconcat(_A(0),".",_A(42),NULL), _e2_bookmark_delete,    TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
+	{g_strconcat(_A(0),".",_A(28),NULL), _e2_bookmark_open,      TRUE, E2_ACTION_TYPE_BOOKMARKS, E2_ACTION_EXCLUDE_ACCEL, NULL, NULL},
+	{g_strconcat(_A(0),".",_A(31),NULL), _e2_bookmark_add,       TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
+	{g_strconcat(_A(0),".",_A(45),NULL), _e2_bookmark_delete,    TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
 	{g_strconcat(_A(13),".",_A(24),NULL),_e2_bookmark_show,      TRUE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
 	{g_strconcat(_A(3),".",_C(1),NULL),  _e2_bookmarks_configure,FALSE,E2_ACTION_TYPE_ITEM, 0, NULL, NULL},
 	};
