@@ -1,6 +1,6 @@
-/* $Id: e2_context_menu.h 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_context_menu.h 2534 2012-05-27 08:07:07Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 Portions copyright (C) 2004 Florian Zaehringer <flo.zaehringer@web.de>
 
 This file is part of emelFM2.
@@ -23,12 +23,7 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 
 #include "emelfm2.h"
 
-void e2_context_menu_add_filetype_item (GtkWidget *menu);
-void e2_context_menu_create_desktop_actions_menu (GtkWidget *menu,
-	gchar *desktopfilepath);
-void e2_context_menu_create_filetype_actions_menu (GtkWidget *menu,
-	const gchar **actions);
-void e2_context_menu_show (guint button, guint32 time, gint type);
+void e2_context_menu_show (guint button, guint32 time, gint type, ViewInfo *view);
 gboolean e2_context_menu_show_menu_action (gpointer from, E2_ActionRuntime *art);
 void e2_context_menu_options_register (void);
 
