@@ -1,6 +1,6 @@
-/* $Id: e2_dnd.c 2377 2011-06-11 00:16:34Z tpgww $
+/* $Id: e2_dnd.c 3001 2014-01-17 23:05:07Z tpgww $
 
-Copyright (C) 2006-2011 tooar <tooar@emelfm2.net>.
+Copyright (C) 2006-2014 tooar <tooar@emelfm2.net>.
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -30,6 +30,8 @@ Relevant button-events are handled in e2_fileview.c
 #include "e2_fileview.h"
 #include "e2_filelist.h"
 #include "e2_task.h"
+#include "e2_icons.h"
+
 //order of table entries matters !
 GtkTargetEntry target_table[] = {
 	{ "text/uri-list", 0, TARGET_URI },
@@ -88,10 +90,16 @@ void e2_dnd_drag_begin_cb (GtkWidget *treeview, GdkDragContext *context,
 	ViewInfo *view)
 {
 	printd (DEBUG, "drag begin");
+#ifndef USE_GTK3_10
 	GtkSettings *s = gtk_settings_get_default ();
+#endif
 	gint pointer_width;
+#ifdef USE_GTK3_10
+	if (!gtk_icon_size_lookup (GTK_ICON_SIZE_DND, &pointer_width, &pointer_height))
+#else
 	if (!gtk_icon_size_lookup_for_settings (s, GTK_ICON_SIZE_DND, //? is too big !
 		&pointer_width, &pointer_height))
+#endif
 			pointer_height = POINTER_HEIGHT;	//can't find useful size, use the default
 	pointer_height--;	//effectively make the 'y hotspot' 1-down from cursor top
 
@@ -105,15 +113,18 @@ void e2_dnd_drag_begin_cb (GtkWidget *treeview, GdkDragContext *context,
 		e2_utils_get_modifiers ()
 # endif
 		& gtk_accelerator_get_default_mod_mask ();
+
+	NEEDCLOSEBGL
 	if (left_pressed && (mask & GDK_CONTROL_MASK) && !(mask & GDK_SHIFT_MASK))
 	{
 		drag_sel_now = TRUE;	//signal that a drag-sel has started
 		//custom (blank) icon for drag-sel, if the relevant mod keys are ok
-		gchar *iconsdir = e2_utils_get_icons_path (TRUE);
+		gchar *iconsdir = e2_icons_get_custom_path (TRUE);
 		gchar *localpath = e2_utils_strcat (iconsdir, "dragsel.png");
 
-		E2_Image *cached = e2_cache_image_get (localpath, GTK_ICON_SIZE_LARGE_TOOLBAR);
-		gtk_drag_set_icon_pixbuf (context, cached->pixbuf, 10, 0);	//x offset is ineffective ...
+		GdkPixbuf *pxb = e2_icons_get_puxbuf (localpath, GTK_ICON_SIZE_LARGE_TOOLBAR, FALSE);
+		if (pxb != NULL)
+			gtk_drag_set_icon_pixbuf (context, pxb, 10, 0);	//x offset is ineffective ...
 
 		g_free (iconsdir);
 		g_free (localpath);
@@ -124,13 +135,17 @@ void e2_dnd_drag_begin_cb (GtkWidget *treeview, GdkDragContext *context,
 		//setup dnd cursor icon
 		guint count = gtk_tree_selection_count_selected_rows (view->selection);
 		if (count > 1)
-			gtk_drag_set_icon_stock (context, GTK_STOCK_DND_MULTIPLE, 10, 0);	//y was 1, x makes no diff.
+#ifdef USE_GTK3_10
+			gtk_drag_set_icon_name (context, STOCK_NAME_DND_MULTIPLE, 10, 0);
+#else
+			gtk_drag_set_icon_stock (context, STOCK_NAME_DND_MULTIPLE, 10, 0);	//y was 1, x makes no diff.
+#endif
 //		else //no point in checking for > 0  as default will be used, anyway
-//			gtk_drag_set_icon_stock (dc, GTK_STOCK_DND, 10, 0);	//y was 1
+//			gtk_drag_set_icon_stock (dc, STOCK_NAME_DND, 10, 0);	//y was 1
 #ifdef E2_ALTLEFTMOUSE
 	}
 #endif
-	return;
+	NEEDOPENBGL
 }
 /**
 @brief 'chained' timer callback function which scrolls the filelist treeview for @a view
@@ -153,9 +168,14 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 #ifdef USE_GTK3_0 //CHECKME pointer position
 	GdkDevice *device = (GdkDevice *)g_object_get_data (G_OBJECT (view->treeview),
 		"e2-drag-device");
-	if (device == NULL || gdk_window_get_device_position (
-			gtk_widget_get_window (view->treeview), device,
-			NULL, &my, NULL) == NULL)
+	if (device != NULL)
+	{
+		GdkWindow *win = gtk_widget_get_window (view->treeview);
+		//this func can return NULL, but that's irrelevant, contrary to implication of API doc
+		gdk_window_get_device_position (win, device, &my, NULL, NULL);
+	}
+	else
+		return FALSE;
 #else //gtk 2
 	if (gdk_window_get_pointer (
 # ifdef USE_GTK2_14
@@ -164,8 +184,8 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 			view->treeview->window,
 # endif
 			NULL, &my, NULL) == NULL)
-#endif
 		return FALSE;
+#endif
 
 #ifdef USE_GTK2_18
 	GtkAllocation alloc;
@@ -234,9 +254,9 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 
 	if (scrollnow >= 0)
 	{
-		gdk_threads_enter ();
+		CLOSEBGL
 		gtk_adjustment_set_value (vadj, scrollnow);
-		gdk_threads_leave ();
+		OPENBGL
 		prior_scroll = scrollnow;
 	}
 	return TRUE;
@@ -286,6 +306,9 @@ gboolean e2_dnd_drag_motion_cb (GtkWidget *treeview, GdkDragContext *context,
 //	GtkTreeModel *model = gtk_tree_model_filter_get_model (fmodel);
 //same as model - GTK_TREE_MODEL (view->store);
 	GtkTreeModel *model = GTK_TREE_MODEL (view->store);
+	
+	NEEDCLOSEBGL
+
 	//get the path of the item under the cursor TOP ie y-pointer_height
 	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
 		x, y-pointer_height, &path, NULL, NULL, NULL))
@@ -302,6 +325,7 @@ gboolean e2_dnd_drag_motion_cb (GtkWidget *treeview, GdkDragContext *context,
 				//select it
 				gtk_tree_selection_select_path (sel, path);
 				gtk_tree_path_free (path);
+				NEEDOPENBGL
 				return FALSE;
 			}
 		}
@@ -332,11 +356,20 @@ gboolean e2_dnd_drag_motion_cb (GtkWidget *treeview, GdkDragContext *context,
 			&& gtk_tree_model_get_iter (model, &iter, childpath))
 		{
 			view->drop_row = row;
+
 			FileInfo *info;
 			gtk_tree_model_get (model, &iter, FINFO, &info, -1);
 			//only highlight dirs (either pane)
 			//FIXME also highlight executable items
-			if (e2_fs_is_dir (info, view))
+			gchar *infopath = F_FILENAME_TO_LOCALE (view->dir);
+#ifdef E2_VFS
+			VPATH sdata = { infopath, view->spacedata };
+			gboolean dir = e2_fs_is_dir (&sdata, info);
+#else
+			gboolean dir = e2_fs_is_dir (infopath, info);
+#endif
+			F_FREE (infopath, view->dir);
+			if (dir)
 			{
 //				printd (DEBUG, "CHANGED color of row %d in same pane", row);
 				e2_fileview_set_row_background (view, &iter,
@@ -373,6 +406,9 @@ gboolean e2_dnd_drag_motion_cb (GtkWidget *treeview, GdkDragContext *context,
 	GtkAllocation alloc;
 	gtk_widget_get_allocation (view->treeview, &alloc);
 #endif
+
+	NEEDOPENBGL
+
 	if (y < (pointer_height + AUTO_SCROLL_MARGIN)
 	 || y > (
 #ifdef USE_GTK2_18
@@ -437,7 +473,9 @@ gboolean e2_dnd_drag_leave_cb (GtkWidget *treeview, GdkDragContext *context,
 	}
 	else
 	{
+		NEEDCLOSEBGL
 		e2_fileview_clear_row_background (view, &iter);
+		NEEDOPENBGL
 		retval = TRUE;
 	}
 	view->drop_row = -1;	//make sure any re-entry will trigger a highlight
@@ -459,7 +497,9 @@ was successfully completed.
 	ViewInfo *view)
 {
 	printd (DEBUG, "drag data delete");
+	NEEDCLOSEBGL
 	e2_task_delete (NULL, NULL, NULL);	//delete active-pane selected items
+	NEEDOPENBGL
 	return;
 } */
 /**
@@ -515,7 +555,9 @@ OR
 NOTE some apps omit the leading "//" from the first example, resulting in file:/foo/bar
 */
 	GList *base, *tmp;
+//	NEEDCLOSEBGL
 	base = e2_fileview_get_selected_local (view);
+//	NEEDOPENBGL
 	if (base != NULL)
 	{
 		//there is actually a selection to process ...
@@ -577,7 +619,7 @@ This is called for only XDS protocol ??
 
 @return TRUE always (the cursor is always in a drop zone)
 */
-/* THIS DOES NOT WORK
+/*THIS DOES NOT WORK
 gboolean e2_dnd_drag_drop_cb (GtkWidget *treeview, GdkDragContext *context,
 	gint x, gint y, guint time, ViewInfo *view)
 {
@@ -587,6 +629,8 @@ gboolean e2_dnd_drag_drop_cb (GtkWidget *treeview, GdkDragContext *context,
 		gint srcformat, srclen;
 		guchar *srcitem;
 		gchar *dest_dir, *destpath = NULL;
+
+		NEEDCLOSEBGL
 		if (gdk_property_get (context->source_window,
 			atom_XdndDirectSave0, atom_text_plain,
 			0, PATH_MAX, FALSE, NULL, &srcformat, &srclen, &srcitem))
@@ -620,12 +664,21 @@ gboolean e2_dnd_drag_drop_cb (GtkWidget *treeview, GdkDragContext *context,
 						gtk_tree_model_get (model, &iter, FINFO, &info, -1);
 						//NICE to add check for executable, here, and do a different drop menu
 						//check if dropping onto a dir
-						if (e2_fs_is_dir (info, view))
+						gchar *infopath = F_FILENAME_TO_LOCALE (view->dir);
+#ifdef E2_VFS
+						VPATH sdata = { infopath, view->spacedata };
+						gboolean dir = e2_fs_is_dir (&sdata, info);
+#else
+						gboolean dir = e2_fs_is_dir (infopath, info);
+#endif
+						F_FREE (infopath, view->dir);
+						if (dir)
 						{
 							//prevent dropping into a dir that's part of the selection
 							if ((treeview == gtk_drag_get_source_widget (context))	//we're in the same selection as we started with
 								&& gtk_tree_selection_path_is_selected (view->selection, path))
 							{
+								NEEDOPENBGL
 								gtk_tree_path_free (path);
 								gtk_tree_path_free (childpath);
 								gtk_drag_finish (context, FALSE, FALSE, time);
@@ -653,6 +706,8 @@ gboolean e2_dnd_drag_drop_cb (GtkWidget *treeview, GdkDragContext *context,
 					atom_XdndDirectSave0, atom_text_plain, srcformat,
 					GDK_PROP_MODE_REPLACE, (guchar *)desturi, strlen (desturi));
 
+				NEEDOPENBGL
+
 				F_FREE (dest_dir);
 				g_free (destpath);
 				g_free (desturi);
@@ -661,6 +716,7 @@ gboolean e2_dnd_drag_drop_cb (GtkWidget *treeview, GdkDragContext *context,
 				return TRUE;
 			}
 		}
+		NEEDOPENBGL
 	}
 / *	else
 	{
@@ -697,7 +753,9 @@ typedef struct _E2_MenuWait
 static void _e2_dnd_drag_op_selected_cb (GtkMenuItem *menuitem, E2_MenuWait *w)
 {
 	w->chosen = GTK_WIDGET(menuitem);
+	NEEDCLOSEBGL
 	e2_main_loop_quit (w->loopdata);
+	NEEDOPENBGL
 }
 /**
 @brief execute drop
@@ -748,7 +806,9 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 #ifdef USE_GTK3_0
 			ask = (e2_utils_get_savedstate (app.main_window) & GDK_MOD1_MASK); //alt key pressed in main window
 #else
+			NEEDCLOSEBGL
 			ask = (e2_utils_get_modifiers () & GDK_MOD1_MASK); //alt key pressed
+			NEEDOPENBGL
 #endif
 		}
 	}
@@ -760,7 +820,7 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 		wdata.loopdata = e2_main_loop_new (TRUE);
 		if (wdata.loopdata != NULL)
 		{
-			GtkWidget *drag_op_menu = gtk_menu_new();
+			GtkWidget *drag_op_menu = e2_menu_get();
 			GtkWidget *item1, *item2, *item3, *item4;
 
 			if (action & GDK_ACTION_COPY)
@@ -791,11 +851,15 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 				_e2_dnd_drag_op_selected_cb, &wdata);
 			wdata.chosen = item4; //default in case of abort == cancel
 
+			NEEDCLOSEBGL
+
 			gtk_menu_popup (GTK_MENU (drag_op_menu), NULL, NULL, NULL, NULL, 0,
 				(guint32) time);
 
 			e2_main_loop_run (wdata.loopdata);
 
+			NEEDOPENBGL
+
 			e2_menu_selection_done_cb (drag_op_menu, NULL);
 
 			if (wdata.chosen == item1)
@@ -833,6 +897,7 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 	else
 		goto badexit;
 
+	NEEDCLOSEBGL
 	//determine whether to drop the selection other than in view->dir
 	GtkTreePath *path;
 	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
@@ -847,12 +912,20 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 			(fmodel, path);
 		if (gtk_tree_model_get_iter (model, &iter, childpath))
 		{
-			gchar *filename;
+			gchar *infopath, *filename;
 			FileInfo *info;
 			gtk_tree_model_get (model, &iter, FILENAME, &filename, FINFO, &info, -1);
 			//NICE to add check for executable, here, and do a different drop menu
 			//check if dropping onto a dir
-			if (e2_fs_is_dir (info, view))
+			infopath = F_FILENAME_TO_LOCALE (view->dir);
+#ifdef E2_VFS
+			VPATH sdata = { infopath, view->spacedata };
+			gboolean dir = e2_fs_is_dir (&sdata, info);
+#else
+			gboolean dir = e2_fs_is_dir (infopath, info);
+#endif
+			F_FREE (infopath, view->dir);
+			if (dir)
 			{
 				//now that we're finishing, un-light the row
 				e2_fileview_clear_row_background (view, &iter);
@@ -894,6 +967,8 @@ badexit:
 		move = FALSE;
 		action = 0;
 	}
+
+	NEEDOPENBGL
 #ifdef USE_GTK2_22
 	gdk_drag_status (context, action, (guint32) time);
 #else
