@@ -1,6 +1,6 @@
-/* $Id: e2_filetype.h 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_filetype.h 2743 2013-09-19 22:29:00Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -26,12 +26,12 @@ void e2_filetype_config_show (gchar *extension);
 void e2_filetype_apply_allnew (void);
 const gchar **e2_filetype_get_actions (gchar *ext);
 //gchar **e2_filetype_get_extensions (gchar *category);
-gchar *e2_filetype_get_default_action (gchar *ext);
+gchar *e2_filetype_get_default_action (gchar *ext) G_GNUC_MALLOC;
 // UNUSED gchar **e2_filetype_get (gchar *ext);
 // OLD void add_filetype(gchar *ext, gchar *prog, gchar *desc);
 void e2_filetype_add_all (void);
-void e2_filetype_exec_action (gchar *action);
-void	e2_filetype_actions_register (void);
+void e2_filetype_exec_action (const gchar *action);
+void e2_filetype_actions_register (void);
 void e2_filetype_options_register (void);
 
 #endif //ndef __E2_FILETYPE_H__
