@@ -1,12 +1,11 @@
-/* $Id: e2_fileview.h 2316 2011-01-27 03:24:35Z tpgww $
+/* $Id: e2_fileview.h 3018 2014-01-21 21:50:09Z tpgww $
 
-Copyright (C) 2004-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
-This file is part of emelFM2.
-emelFM2 is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+This file is part of emelFM2, which is free software. You can redistribute it
+and/or modify it under the terms of the GNU General Public License as published
+by the Free Software Foundation - either version 3, or (at your option) any
+later version.
 
 emelFM2 is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +26,11 @@ typedef enum { LT, EQ, GT } Operator;
 typedef enum { E2_REFRESH, E2_RECREATE, E2_CHANGE } E2_RefreshType;
 typedef enum { CD_NOTFINISHED, CD_TIMEOUT, CD_SUCCESS } E2_CDType;
 //filelist treestore column-number enumerator
-enum { FILENAME = 0, SIZE, PERM, OWNER, GROUP, MODIFIED, ACCESSED, CHANGED, MAX_COLUMNS };
+enum { FILENAME = 0,
+#ifdef E2_EXTCOL
+	EXTN,
+#endif
+	SIZE, PERM, OWNER, GROUP, MODIFIED, ACCESSED, CHANGED, MAX_COLUMNS };
 #define EXTENSION -1 //enum for sort action
 
 //directory-read failure codes
@@ -44,6 +47,7 @@ enum
 };
 #define E2DREAD_FAILED(ptr) (!(ptr == NULL || (gpointer)ptr > GUINT_TO_POINTER (E2DREAD_ENOMEM)))
 
+//this is here to workaround build circularity
 typedef enum
 {
 	E2FSCASE_UNKNOWN,
@@ -80,7 +84,6 @@ typedef struct _E2_ViewInfo
 	GtkTreeModel *model;	//may be a filter model or standard
 	GtkWidget *treeview;
 	GtkTreeSelection *selection;	//pointer to selection for this view's treeview
-	GHashTable *selected_names;  //names of selected items (for reselection), or NULL
 	gint sort_column;	//copy of current sort column, can be <0
 	gboolean extsort;	//TRUE when name-sorting is by extension
 	GtkSortType sort_order; //copy of current sort order
@@ -231,7 +234,9 @@ void e2_fileview_focus_row (ViewInfo *view, gint row,
 	gboolean center, gboolean grab_focus);
 //void e2_fileview_select_row_by_filename (ViewInfo *view, gchar *filename); unused
 GHashTable *e2_fileview_log_selected_names (ViewInfo *view);
-void e2_fileview_reselect_names (ViewInfo *view);
+void e2_fileview_reselect_names (ViewInfo *view, GHashTable *selnames, gboolean clean);
+void e2_fileview_adjust_name (ViewInfo *view, const gchar *oldname,
+	const gchar *newname, const gchar *oldutf, const gchar *newutf);
 void e2_fileview_translate_cols_array (gint *src_array,
 	gint *dest_array, gint size);
 void e2_fileview_switch_views (void);
@@ -242,25 +247,26 @@ void e2_fileview_set_row_background (ViewInfo *view,
 void e2_fileview_clear_row_background (ViewInfo *view,
 	GtkTreeIter *iter);
 void e2_fileview_update_col_cachedata (void);
+gboolean e2_fileview_is_cased (ViewInfo *view);
 void e2_fileview_get_scroll_data (ViewInfo *view, gint *leftcol, gint *toprow);
 gboolean e2_fileview_scroll_to_position (ViewInfo *view, gint leftcol, gint toprow);
 //gint e2_fileview_find_name_col (ViewInfo *view); unused
 gboolean e2_fileview_prepare_list (ViewInfo *view);
 void e2_fileview_scroll_list (GThread *viewthread);
-gpointer e2_fileview_refresh_list (ViewInfo *view);
-gboolean e2_fileview_treehash_free (GHashTable *data);
 gboolean e2_fileview_cd_watch (E2_CDwatch *data);
 gboolean e2_fileview_cd_manage (E2_Listman *data);
-GtkWidget *e2_fileview_create_list (ViewInfo *view);
+GtkWidget *e2_fileview_create_list (ViewInfo *view) G_GNUC_MALLOC;
 void e2_fileview_select_all (GtkWidget *widget, ViewInfo *view);
 void e2_fileview_clean_selected (GPtrArray *selected);
 void e2_fileview_clean1_history (gpointer data);
-GPtrArray *e2_fileview_get_selected (ViewInfo *view);
-GList *e2_fileview_get_selected_local (ViewInfo *view);
+void e2_fileview_clear_filter_patterns (ViewInfo *view);
+GPtrArray *e2_fileview_get_selected (ViewInfo *view) G_GNUC_MALLOC;
+GList *e2_fileview_get_selected_local (ViewInfo *view) G_GNUC_MALLOC;
 FileInfo *e2_fileview_get_selected_first_local (ViewInfo *view, gboolean updir);
-gchar *e2_fileview_get_row_name (ViewInfo *view, gint row);
+gchar *e2_fileview_get_row_name (ViewInfo *view, gint row) G_GNUC_MALLOC;
 void e2_fileview_register_keybindings (GtkWidget *treeview, ViewInfo *view);
 #ifdef E2_MOUSECUSTOM
-void e2_fileview_register_pointerbindings (GtkWidget *treeview, ViewInfo *view);
+void e2_fileview_register_pointerbindings (GtkWidget *treeview);
 #endif
+
 #endif //ndef __E2_FILEVIEW_H__
