@@ -1,6 +1,6 @@
-/* $Id: e2_keybinding.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_keybinding.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -525,7 +525,7 @@ static gboolean _e2_keybinding_walk_sync (GNode *node, gpointer data)
 This func is mainloop idle callback CHECKME why sync that way?
 @param data pointer to binding to be processed, or NULL to do whole list
 
-@return FALSE (so the idle is cancelled)
+@return FALSE (so the idle-source is cancelled)
 */
 static gboolean _e2_keybinding_idle_sync_cb (E2_KeyCatRuntime *data)
 {
@@ -594,8 +594,8 @@ Config dialog page buttons are de-sensitized if the row is a category.
 
 @return TRUE always (the row is always selectable)
 */
-static gboolean _e2_keybinding_tree_selection_check_cb
-	(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path,
+static gboolean _e2_keybinding_tree_selection_check_cb (
+	GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path,
 	gboolean path_currently_selected, E2_OptionSet *set)
 {
 	GtkTreeIter iter;
@@ -606,7 +606,9 @@ static gboolean _e2_keybinding_tree_selection_check_cb
 		gboolean retval = (cat[0] == '\0');
 		g_free (cat);
 		GtkTreeView *view = gtk_tree_selection_get_tree_view (selection);
+		NEEDCLOSEBGL
 		e2_option_tree_adjust_buttons (view, retval);
+		NEEDOPENBGL
 	}
 	return TRUE;
 }
@@ -641,8 +643,8 @@ ie the row is not a category. If so, the row is draggable
 
 @return TRUE if the row can be dragged
 */
-static gboolean _e2_keybinding_tree_draggable_check_cb
-	(GtkTreeDragSource *drag_source, GtkTreePath *path)
+static gboolean _e2_keybinding_tree_draggable_check_cb (
+	GtkTreeDragSource *drag_source, GtkTreePath *path)
 {
 	if (!GTK_IS_TREE_MODEL (drag_source))
 		return TRUE;
@@ -685,7 +687,7 @@ have been de-localised by e2_utils_translate_key_event() if that's relevant.
 @param event data struct for the event
 @param user_data UNUSED data specified when cb was connected
 
-@return TRUE (i.e. prevent other handlers) if any key has been acted upon
+@return TRUE (i.e. prevent other handlers) if any key has been acted upon (even if the action failed)
 */
 static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 	GdkEventKey *event, gpointer user_data)
@@ -744,7 +746,10 @@ static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 					}
 					if (*k->action != '\0')
 					{	//there is an action to run
+//						NEEDCLOSEBGLX
+						OPENBGL
 						e2_action_run_simple_from (k->action, k->action_data, widget);
+						CLOSEBGL
 						if (!k->cont)
 						{
 //							printd (DEBUG,"key binding cb ends, returned value = TRUE");
@@ -792,7 +797,10 @@ static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 					}
 					if (*k->action != '\0')
 					{
+//						NEEDCLOSEBGLX
+						OPENBGL
 						e2_action_run_simple_from (k->action, k->action_data, widget);
+						CLOSEBGL
 						//remember how far up this branch we've gone, to prevent repeats for other categories
 //CHECKME				if (topnode == NULL || g_node_depth (node) < g_node_depth (topnode))
 							topnode = node;
@@ -1472,7 +1480,7 @@ static gboolean _e2_keybinding_issue (gpointer from, E2_ActionRuntime *art)
 void e2_keybinding_actions_register (void)
 {
 	E2_Action action =
-	{g_strconcat(_A(124),".",_A(125),NULL),_e2_keybinding_issue,TRUE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	{g_strconcat(_A(127),".",_A(128),NULL),_e2_keybinding_issue,TRUE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
 	e2_action_register (&action);
 }
 #endif	//def WITH_KEYFAKE
@@ -1491,112 +1499,114 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	e2_option_tree_setup_defaults (set,
 	g_strdup("keybindings=<"),  //internal name
 	g_strconcat(_C(17),"||||",NULL),  //_("general"
-	g_strconcat("\t|F10||",_A(1),".",_A(72),"|",NULL), //"quit"
-	g_strconcat("\t|F11||",_A(17),".",_A(48),"|",NULL), //"toggle.fullscreen"
-	g_strconcat("\t|F12||",_A(3),".",_A(32),"|",NULL),  //"configure"
+	g_strconcat("\t|F10||",_A(1),".",_A(75),"|",NULL), //"quit"
+	g_strconcat("\t|F11||",_A(17),".",_A(51),"|",NULL), //"toggle.fullscreen"
+	g_strconcat("\t|F12||",_A(3),".",_A(34),"|",NULL),  //"configure"
 
 	g_strconcat("\t",_C(23),"||||",NULL),  //_("main"
-	g_strconcat("\t\t|F1||",_A(14),".",_A(73),"|",NULL), //"refresh"
-	g_strconcat("\t\t|<Shift>F1||",_A(8),".",_A(58),"|",NULL), //"history.list
-	g_strconcat("\t\t|<Control>F1||",_A(15),".",_A(58),"|",NULL), //"pending.list
-	g_strconcat("\t\t|<Alt>F1||",_A(2),".",_A(58),"|",NULL), //"children.list
-	g_strconcat("\t\t|F2||",_A(6),".",_A(76),"|",NULL), //"rename"
-	g_strconcat("\t\t|F3||",_A(6),".",_A(106),"|",NULL), //"view"
-	g_strconcat("\t\t|<Shift>F3||",_A(6),".",_A(107),"|",NULL), //"view_again"
-	g_strconcat("\t\t|<Control>F3||",_A(6),".",_A(46),"|",NULL), //"find" (was F4)
-	g_strconcat("\t\t|F4||",_A(6),".",_A(43),"|",NULL), //"edit"
-	g_strconcat("\t\t|<Shift>F4||",_A(6),".",_A(44),"|",NULL), //"edit_again"
-	g_strconcat("\t\t|F5||",_A(6),".",_A(36),"|",NULL),  //"copy"
-	g_strconcat("\t\t|<Shift>F5||",_A(6),".",_A(37),"|",NULL),  //"copy_as"
-	g_strconcat("\t\t|<Control>F5||",_A(6),".",_A(39),"|",NULL),  //"copy_with_time"
-	g_strconcat("\t\t|<Alt>F5||",_A(6),".",_A(38),"|",NULL),  //"copy_merge"
-	g_strconcat("\t\t|F6||",_A(6),".",_A(62),"|",NULL), //"move"
-	g_strconcat("\t\t|<Shift>F6||",_A(6),".",_A(63),"|",NULL), //"move_as"
-	g_strconcat("\t\t|F7||",_A(1),".",_A(60),"|",NULL), //"mkdir"
+	g_strconcat("\t\t|F1||",_A(14),".",_A(76),"|",NULL), //"refresh"
+	g_strconcat("\t\t|<Shift>F1||",_A(8),".",_A(61),"|",NULL), //"history.list
+	g_strconcat("\t\t|<Control>F1||",_A(15),".",_A(61),"|",NULL), //"pending.list
+	g_strconcat("\t\t|<Alt>F1||",_A(2),".",_A(61),"|",NULL), //"children.list
+	g_strconcat("\t\t|F2||",_A(6),".",_A(79),"|",NULL), //"rename"
+	g_strconcat("\t\t|F3||",_A(6),".",_A(109),"|",NULL), //"view"
+	g_strconcat("\t\t|<Shift>F3||",_A(6),".",_A(110),"|",NULL), //"view_again"
+	g_strconcat("\t\t|<Control>F3||",_A(6),".",_A(49),"|",NULL), //"find" (was F4)
+	g_strconcat("\t\t|F4||",_A(6),".",_A(46),"|",NULL), //"edit"
+	g_strconcat("\t\t|<Shift>F4||",_A(6),".",_A(47),"|",NULL), //"edit_again"
+	g_strconcat("\t\t|F5||",_A(6),".",_A(39),"|",NULL),  //"copy"
+	g_strconcat("\t\t|<Shift>F5||",_A(6),".",_A(40),"|",NULL),  //"copy_as"
+	g_strconcat("\t\t|<Control>F5||",_A(6),".",_A(42),"|",NULL),  //"copy_with_time"
+	g_strconcat("\t\t|<Alt>F5||",_A(6),".",_A(41),"|",NULL),  //"copy_merge"
+	g_strconcat("\t\t|F6||",_A(6),".",_A(65),"|",NULL), //"move"
+	g_strconcat("\t\t|<Shift>F6||",_A(6),".",_A(66),"|",NULL), //"move_as"
+	g_strconcat("\t\t|F7||",_A(1),".",_A(63),"|",NULL), //"mkdir"
 	g_strconcat("\t\t|F8||",_A(6),".",_A(18),"|",NULL), //"trash"
-	g_strconcat("\t\t|<Control>F8||",_A(18),".",_A(64),"|",NULL), //"trash.open"  modifiers not translated, gtk needs english to parse
-	g_strconcat("\t\t|<Shift><Control>F8||",_A(6),".",_A(42),"|",NULL), //"delete"
-	g_strconcat("\t\t|F9||",_A(6),".",_A(55),"|",NULL), //"file.info"
+	g_strconcat("\t\t|<Control>F8||",_A(18),".",_A(67),"|",NULL), //"trash.open"  modifiers not translated, gtk needs english to parse
+	g_strconcat("\t\t|<Shift><Control>F8||",_A(6),".",_A(45),"|",NULL), //"delete"
+	g_strconcat("\t\t|F9||",_A(6),".",_A(58),"|",NULL), //"file.info"
 #ifdef E2_TREEDIALOG
-	g_strconcat("\t\t|<Shift>F9||",_A(13),".",_A(103),"|",NULL), //"pane_active.tree" popup directories tree dialog
+	g_strconcat("\t\t|<Shift>F9||",_A(13),".",_A(106),"|",NULL), //"pane_active.tree" popup directories tree dialog
 #endif
 #ifdef E2_VFS
-	g_strconcat("\t\t|<Control>F9||",_A(4),".",_A(122),"|",NULL), //"dialog.namespace"
-	g_strconcat("\t\t|<Alt>F9||",_A(13),".",_A(120),"|",NULL), //"pane.<spacemenu>"
+	g_strconcat("\t\t|<Control>F9||",_A(4),".",_A(125),"|",NULL), //"dialog.namespace"
+	g_strconcat("\t\t|<Alt>F9||",_A(13),".",_A(123),"|",NULL), //"pane.<spacemenu>"
 #endif
-	g_strconcat("\t\t|<Control>F11||",_A(10),".",_A(30),"|",NULL), //"output.add
-	g_strconcat("\t\t|<Shift><Control>F11||",_A(10),".",_A(42),"|",NULL), //"output.delete
-	g_strconcat("\t\t|<Shift>F12||",_A(14),".",_A(99),"|",NULL), //"panes.toggle direction"
+	g_strconcat("\t\t|<Control>F11||",_A(10),".",_A(31),"|",NULL), //"output.add
+	g_strconcat("\t\t|<Shift><Control>F11||",_A(10),".",_A(45),"|",NULL), //"output.delete
+	g_strconcat("\t\t|<Shift>F12||",_A(14),".",_A(102),"|",NULL), //"panes.toggle direction"
 //	_I( single letters in the following not worth translating ?
 	//conform this help string to the following 'jump-keys'
-	g_strconcat("\t\t|<Control>g||",_A(10),".",_A(71),"|",_("Now press one of h,m,d\\n"),NULL), //"print"
+	g_strconcat("\t\t|<Control>g||",_A(10),".",_A(74),"|",_("Now press one of h,m,d\\n"),NULL), //"print"
 	//these 3 children are the so-called "directory jump keys"
 	g_strconcat("\t\t\t|h||cd|$HOME",NULL), //_A(20)
 	g_strconcat("\t\t\t|m||cd|/mnt",NULL),  //_A(20)
 	g_strconcat("\t\t\t|d||cd|$HOME/downloads",NULL),  //_A(20)
-	g_strconcat("\t\t|<Control>w||!",_A(10),".",_A(31),"|1,*",NULL),  //"adjust_ratio" no arg translation NOTE blocked * expansion
-	g_strconcat("\t\t|<Control>e||!",_A(10),".",_A(31),"|0,*",NULL),  //"adjust_ratio" no arg translation ditto
-	g_strconcat("\t\t|<Control>z||",_A(1),".",_A(100),"|",NULL), //"focus toggle"
-	g_strconcat("\t\t|<Control>r||",_A(14),".",_A(73),"|",NULL), //"refresh"
-	g_strconcat("\t\t|<Control>1||",_A(5),".",_A(100),"|1",NULL), //"focus toggle" no arg translation
-	g_strconcat("\t\t|<Control>2||",_A(5),".",_A(100),"|2",NULL), //"focus toggle"no arg translation
-	g_strconcat("\t\t|<Control>Insert||",_A(1),".",_A(56),"|",NULL), //"insert selection"
-	g_strconcat("\t\t|<Alt>Insert||",_A(1),".",_A(56),"|",_A(117),NULL), //"insert selection, quote"
+	g_strconcat("\t\t|<Control>w||!",_A(10),".",_A(33),"|1,*",NULL),  //"adjust_ratio" no arg translation NOTE blocked * expansion
+	g_strconcat("\t\t|<Control>e||!",_A(10),".",_A(33),"|0,*",NULL),  //"adjust_ratio" no arg translation ditto
+	g_strconcat("\t\t|<Control>z||",_A(1),".",_A(103),"|",NULL), //"focus toggle"
+	g_strconcat("\t\t|<Control>r||",_A(14),".",_A(76),"|",NULL), //"refresh"
+	g_strconcat("\t\t|<Control>1||",_A(5),".",_A(103),"|1",NULL), //"focus toggle" no arg translation
+	g_strconcat("\t\t|<Control>2||",_A(5),".",_A(103),"|2",NULL), //"focus toggle"no arg translation
+	g_strconcat("\t\t|<Control>Insert||",_A(1),".",_A(59),"|",NULL), //"insert selection"
+	g_strconcat("\t\t|<Alt>Insert||",_A(1),".",_A(59),"|",_A(120),NULL), //"insert selection, quote"
 	g_strconcat("\t\t",_C(33),"||||",NULL),  //_("panes"
 	//note "Space" not recognised, must be "space"
-//	g_strconcat("\t\t\t|space||",_A(14),".",_A(95),"|",NULL), //"switch"  something else makes this 'open' the selected item
-	g_strconcat("\t\t\t|space||",_A(7),".",_A(102),"|",NULL), //"toggle selected"
-	g_strconcat("\t\t\t|Tab||",_A(14),".",_A(95),"|",NULL), //"switch"
-	g_strconcat("\t\t\t|Return||",_A(6),".",_A(64),"|",NULL), //"open"
-	g_strconcat("\t\t\t|<Shift>Return||",_A(6),".",_A(65),"|",NULL), //"open in other"
+//	g_strconcat("\t\t\t|space||",_A(14),".",_A(98),"|",NULL), //"switch"  something else makes this 'open' the selected item
+	g_strconcat("\t\t\t|space||",_A(7),".",_A(105),"|",NULL), //"toggle selected"
+	g_strconcat("\t\t\t|Tab||",_A(14),".",_A(98),"|",NULL), //"switch"
+	g_strconcat("\t\t\t|Return||",_A(6),".",_A(67),"|",NULL), //"open"
+	g_strconcat("\t\t\t|<Shift>Return||",_A(6),".",_A(68),"|",NULL), //"open in other"
 	g_strconcat("\t\t\t|<Control>Return|||",NULL), //nothing
 	//these 2 children are examples, 1 to 6 are allowed
-	g_strconcat("\t\t\t\t|","2","||",_A(6),".",_A(66),"2|",NULL),//"open with2"
-	g_strconcat("\t\t\t\t|","3","||",_A(6),".",_A(66),"3|",NULL),//"open with3"
-	g_strconcat("\t\t\t|Left||",_A(13),".",_A(49),"|",NULL), //"go back"
-	g_strconcat("\t\t\t|Right||",_A(13),".",_A(50),"|",NULL), //"go forward"
+	g_strconcat("\t\t\t\t|","2","||",_A(6),".",_A(69),"2|",NULL),//"open with2"
+	g_strconcat("\t\t\t\t|","3","||",_A(6),".",_A(69),"3|",NULL),//"open with3"
+	g_strconcat("\t\t\t|Left||",_A(13),".",_A(52),"|",NULL), //"go back"
+	g_strconcat("\t\t\t|Right||",_A(13),".",_A(53),"|",NULL), //"go forward"
 	g_strconcat("\t\t\t|<Alt>Up||cd|..",NULL),  //_A(20)
 	g_strconcat("\t\t\t|BackSpace||cd|..",NULL),  //_A(20)
-	g_strconcat("\t\t\t|Delete||",_A(6),".",_A(42),"|",NULL), //"delete"
-	g_strconcat("\t\t\t|<Shift>Delete||",_A(18),".",_A(42),"|",NULL), //"trash.delete"
+	g_strconcat("\t\t\t|Delete||",_A(6),".",_A(45),"|",NULL), //"delete"
+	g_strconcat("\t\t\t|<Shift>Delete||",_A(18),".",_A(45),"|",NULL), //"trash.delete"
 //	g_strconcat("\t\t\t|Home||cd|$HOME",NULL),   //_A(20)
-	g_strconcat("\t\t\t|<Control>a||",_A(7),".",_A(101),"|",NULL), //"toggle select all"
-	g_strconcat("\t\t\t|<Control>i||",_A(7),".",_A(57),"|",NULL), //"invert selection"
-	g_strconcat("\t\t\t|<Control>d||",_A(7),".",_A(80),"|",NULL), //"select_type"
-	g_strconcat("\t\t\t|<Control>f||",_A(6),".",_A(46),"|",NULL), //"find"
-	g_strconcat("\t\t\t|<Control>x||",_A(14),".",_A(98),"|",NULL), //"sync"
-	g_strconcat("\t\t\t|<Control>h||",_A(13),".",_A(84),"|",NULL),  //"toggle hidden"
+	g_strconcat("\t\t\t|<Control>a||",_A(7),".",_A(104),"|",NULL), //"toggle select all"
+	g_strconcat("\t\t\t|<Control>i||",_A(7),".",_A(60),"|",NULL), //"invert selection"
+	g_strconcat("\t\t\t|<Control>d||",_A(7),".",_A(83),"|",NULL), //"select_type"
+	g_strconcat("\t\t\t|<Control>f||",_A(6),".",_A(49),"|",NULL), //"find"
+	g_strconcat("\t\t\t|<Control>x||",_A(14),".",_A(101),"|",NULL), //"sync"
+	g_strconcat("\t\t\t|<Control>h||",_A(13),".",_A(87),"|",NULL),  //"toggle hidden"
 	g_strconcat("\t\t\t|<Control>m||",_A(13),".",_A(24),"|",NULL), //"<bookmarks>"
 	g_strconcat("\t\t\t|<Control>t||",_A(13),".",_A(25),"|",NULL), //"<filters>"
-//	g_strconcat("\t\t\t|/||",_A(6),".",_A(46),"|",NULL),  //"find"
-	g_strconcat("\t\t\t|<Control>Left||!",_A(14),".",_A(31),"|0,*",NULL), //"adjust ratio" no arg translation NOTE blocked * expansion
-	g_strconcat("\t\t\t|<Control>Right||!",_A(14),".",_A(31),"|1,*",NULL), //"adjust ratio" no arg translation ditto
-	g_strconcat("\t\t\t|<Control>p||",_A(13),".",_A(85),"|",NULL),  //"show menu"
-	g_strconcat("\t\t\t|<Shift><Control>p||",_A(13),".",_A(85),"|",_A(118),NULL), //"show menu, shift"
-	g_strconcat("\t\t\t|<Control><Alt>p||",_A(13),".",_A(85),"|",_A(110),NULL),  //"show menu, ctrl"
-	g_strconcat("\t\t\t|<Control><Alt>n||",_A(7),".",_A(91),"|",NULL),  //"filelist.sortname"
-	g_strconcat("\t\t\t|<Control><Alt>m||",_A(7),".",_A(90),"|",NULL),  //"filelist.sortmodified"
-	g_strconcat("\t\t\t|<Control><Alt>s||",_A(7),".",_A(93),"|",NULL),  //"filelist.sortsize"
+//	g_strconcat("\t\t\t|/||",_A(6),".",_A(49),"|",NULL),  //"find"
+	g_strconcat("\t\t\t|<Control>Left||!",_A(14),".",_A(33),"|0,*",NULL), //"adjust ratio" no arg translation NOTE blocked * expansion
+	g_strconcat("\t\t\t|<Control>Right||!",_A(14),".",_A(33),"|1,*",NULL), //"adjust ratio" no arg translation ditto
+	g_strconcat("\t\t\t|<Control>p||",_A(13),".",_A(88),"|",NULL),  //"show menu"
+	g_strconcat("\t\t\t|<Shift><Control>p||",_A(13),".",_A(88),"|",_A(121),NULL), //"show menu, shift"
+	g_strconcat("\t\t\t|<Control><Alt>p||",_A(13),".",_A(88),"|",_A(113),NULL),  //"show menu, ctrl"
+	g_strconcat("\t\t\t|<Control><Alt>n||",_A(7),".",_A(94),"|",NULL),  //"filelist.sortname"
+	g_strconcat("\t\t\t|<Control><Alt>m||",_A(7),".",_A(93),"|",NULL),  //"filelist.sortmodified"
+	g_strconcat("\t\t\t|<Control><Alt>s||",_A(7),".",_A(96),"|",NULL),  //"filelist.sortsize"
 	g_strconcat("\t\t",_C(12),"||||",NULL),  //_"directory line"
-	g_strconcat("\t\t\t|Tab||",_A(1),".",_A(35),"|",_A(111),NULL), //"complete dirs"
-	g_strconcat("\t\t\t|<Control>Tab||",_A(13),".",_A(64),"|",NULL), //"open"
-	g_strconcat("\t\t\t|<Control>Delete||",_A(1),".",_A(33),"|",NULL),  //"clear"
-	g_strconcat("\t\t\t|<Control>k||",_A(5),".",_A(34),"|",NULL),  //"unlog_current"
-	g_strconcat("\t\t\t|<Alt>Delete||",_A(1),".",_A(34),"|",NULL),  //"clear_history"
+	g_strconcat("\t\t\t|Tab||",_A(1),".",_A(38),"|",_A(114),NULL), //"complete dirs"
+	g_strconcat("\t\t\t|<Control>Tab||",_A(13),".",_A(67),"|",NULL), //"open"
+	g_strconcat("\t\t\t|<Control>Delete||",_A(5),".",_A(36),"|",NULL),  //"clear"
+	g_strconcat("\t\t\t|<Shift>Insert||",_A(5),".",_A(32),"|",NULL),  //"add_history"
+	g_strconcat("\t\t\t|<Alt>Delete||",_A(5),".",_A(44),"|",NULL),  //"del_history"
+	g_strconcat("\t\t\t|<Shift><Alt>Delete||",_A(5),".",_A(37),"|",NULL),  //"clear_history"
 	g_strconcat("\t\t",_C(5),"||||",NULL),  //_("command line"
-	g_strconcat("\t\t\t|Tab||",_A(1),".",_A(35),"|",NULL), //"complete"
-	g_strconcat("\t\t\t|<Shift>Return||",_A(1),".",_A(81),"|",NULL), //"command.send"
-	g_strconcat("\t\t\t|<Control>Delete||",_A(1),".",_A(33),"|",NULL), //"clear"
-	g_strconcat("\t\t\t|<Alt>Delete||",_A(1),".",_A(34),"|",NULL),  //"clear_history"
-	g_strconcat("\t\t\t|<Alt>Insert||",_A(2),".",_A(27),"|",NULL), //"children<menu>"
+	g_strconcat("\t\t\t|Tab||",_A(1),".",_A(38),"|",NULL), //"complete"
+	g_strconcat("\t\t\t|<Shift>Return||",_A(1),".",_A(84),"|",NULL), //"command.send"
+	g_strconcat("\t\t\t|<Control>Delete||",_A(1),".",_A(36),"|",NULL), //"clear"
+	g_strconcat("\t\t\t|<Alt>Delete||",_A(1),".",_A(44),"|",NULL),  //"del_history"
+	g_strconcat("\t\t\t|<Shift><Alt>Delete||",_A(1),".",_A(37),"|",NULL),  //"clear_history"
+	g_strconcat("\t\t\t|<Alt>Insert||",_A(2),".",_A(28),"|",NULL), //"children<menu>"
 //	g_strconcat("\t\t",_C(28),"||||",NULL),  //_("output"
 //CHECKME these stay as command-line bindings ??
-	g_strconcat("\t\t\t|Page_Up||",_A(10),".",_A(69),"|",NULL), //"page up"
-	g_strconcat("\t\t\t|Page_Down||",_A(10),".",_A(68),"|",NULL), //"page down"
-	g_strconcat("\t\t\t|<Shift>Page_Up||",_A(10),".",_A(53),"|",NULL), //"goto top"
-	g_strconcat("\t\t\t|<Shift>Page_Down||",_A(10),".",_A(52),"|",NULL), //"goto bottom"
-	g_strconcat("\t\t\t|<Shift>Up||",_A(10),".",_A(78),"|",NULL),  //"scroll up"
-	g_strconcat("\t\t\t|<Shift>Down||",_A(10),".",_A(77),"|",NULL),  //"scroll down"
+	g_strconcat("\t\t\t|Page_Up||",_A(10),".",_A(72),"|",NULL), //"page up"
+	g_strconcat("\t\t\t|Page_Down||",_A(10),".",_A(71),"|",NULL), //"page down"
+	g_strconcat("\t\t\t|<Shift>Page_Up||",_A(10),".",_A(56),"|",NULL), //"goto top"
+	g_strconcat("\t\t\t|<Shift>Page_Down||",_A(10),".",_A(55),"|",NULL), //"goto bottom"
+	g_strconcat("\t\t\t|<Shift>Up||",_A(10),".",_A(81),"|",NULL),  //"scroll up"
+	g_strconcat("\t\t\t|<Shift>Down||",_A(10),".",_A(80),"|",NULL),  //"scroll down"
 	g_strconcat("\t\t",_C(28),"||||",NULL),  //_("output"
 
 	g_strconcat("\t",_C(11),"||||",NULL),  //_("dialogs"
