@@ -1,6 +1,6 @@
-/* $Id: e2_main.c 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_main.c 3059 2014-02-14 23:38:41Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2014 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -123,8 +123,8 @@ The following items are covered:
 #ifdef E2_MOUSECUSTOM
 # include "e2_mousebinding.h"
 #endif
-
-#define UPGRADE_PNAME "e2p_upgrade.so"
+#include "e2_icons.h"
+#include "../plugins/e2p_upgrade.h"
 
 extern volatile gint cfg_refresh_refcount;
 extern GList *open_history;
@@ -141,74 +141,69 @@ pthread_mutex_t wait_mutex; // = PTHREAD_MUTEX_INITIALIZER;	if no recursion
 
 static void _e2_main_system_shutdown (gint num); //__attribute__ ((noreturn));
 
-#ifdef NATIVE_BGL
-# warning "using native gdk mutex"
-#else
 /**
-@brief local management of gdk threads mutex
+@brief local management of UI-backend-access mutex
 These allow a conservative approach to locking without risk of deadlock
 @return
 */
-void e2_main_close_gdklock (void)
+void e2_main_close_uilock (void)
 {
 	//a single thread can't write to screen more than once at a time,
 	//so deadlocks just reflect "unnecessary" re-locks, and can be ignored
 #ifdef DEBUG_MESSAGES
-	if (pthread_mutex_lock (&gdklock) == EDEADLK)
+	if (pthread_mutex_lock (&display_mutex) == EDEADLK)
 		printd (DEBUG, "attempted BGL re-lock");
 #else
-	pthread_mutex_lock (&gdklock);
+	pthread_mutex_lock (&display_mutex);
 #endif
 }
-void e2_main_open_gdklock (void)
+
+void e2_main_open_uilock (void)
 {
 #ifdef DEBUG_MESSAGES
-	if (pthread_mutex_unlock (&gdklock) == EPERM)
+	if (pthread_mutex_unlock (&display_mutex) == EPERM)
 		printd (DEBUG, "bad attempt to open BGL");
 #else
-	pthread_mutex_unlock (&gdklock);
+	pthread_mutex_unlock (&display_mutex);
 #endif
 }
-#endif
 /* *
 @brief aborted-thread cleanup function
 @param data pointerised boolean, non-NULL to re-close BGL
 */
-/*void e2_main_cleanup_gdklock (gpointer data)
+/*void e2_main_cleanup_uilock (gpointer data)
 {
-	printd (DEBUG, "e2_main_cleanup_gdklock");
+	printd (DEBUG, "e2_main_cleanup_uilock");
 	gboolean FIXME;
 	if (data != NULL)
 	{
-		//e2_main_close_gdklock ();
+		//e2_main_close_uilock ();
 		CLOSEBGL_IF_OPEN
 	}
-	printd (DEBUG, "e2_main_cleanup_gdklock FINISH");
+	printd (DEBUG, "e2_main_cleanup_uilock FINISH");
 }
 */
-#ifndef NATIVE_BGL
 /**
 @brief re-initialize BGL
 @param free TRUE to kill the current mutex first
 
 @return
 */
-void e2_main_replace_gdklock (gboolean free)
+void e2_main_init_uilock (gboolean free)
 {
 //PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP isn't always available to init mutex
 	pthread_mutexattr_t attr;
 	if (free)
-		pthread_mutex_destroy (&gdklock);
+		pthread_mutex_destroy (&display_mutex);
 	pthread_mutexattr_init (&attr);
 #ifdef DEBUG_MESSAGES
 	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_ERRORCHECK);
 #else
 	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
 #endif
-	pthread_mutex_init (&gdklock, &attr);
+	pthread_mutex_init (&display_mutex, &attr);
 	pthread_mutexattr_destroy (&attr);
 }
-#endif
 /**
 @brief idle and then timer callback to do non-time-critical things at session start
 This waits until after both filelists are initialised, as determined by both
@@ -233,13 +228,33 @@ static gboolean _e2_main_defer_init (gpointer data)
 			GINT_TO_POINTER (100));
 			return FALSE;
 	}
-	//in case async setup of initial dirs results in pane2 completion after pane 1
-	if (curr_pane == &app.pane2)
+
+	if (curr_pane == &app.pane1)
 	{
-		gdk_threads_enter ();
-		e2_pane_activate_other ();
-		gdk_threads_leave ();
+		if (e2_cl_options.force_path)
+		{
+			CLOSEBGL
+			e2_pane_activate_other (); //focus the specified dir in pane 2
+			OPENBGL
+		}
 	}
+	else //async setup of initial dirs results in pane2 completion after pane 1
+	{
+		if (!e2_cl_options.force_path)
+		{
+			CLOSEBGL
+			e2_pane_activate_other (); //focus pane 1
+			OPENBGL
+		}
+	}
+
+	//prevent further pane-divider-position overrides in the window_show callback
+	e2_cl_options.force_path = FALSE;
+	g_free (e2_cl_options.pane2_path);
+	e2_cl_options.pane2_path = NULL;
+	g_free (e2_cl_options.pane1_path);
+	e2_cl_options.pane1_path = NULL;
+
 #ifdef E2_FAM
 	//hookup to a file monitor if possible
 	e2_fs_FAM_connect ();
@@ -273,38 +288,6 @@ static gboolean _e2_main_defer_init (gpointer data)
 #ifdef E2_DEVKIT
 	e2_devkit_init ();
 #endif
-
-#ifndef USE_GTK3_99
-	//add a discard if gtk does not provide a stock discard icon as it should (still none, in 3.0)
-# ifdef USE_GTK3_0
-	GtkStyleContext *sc = gtk_widget_get_style_context (app.main_window);
-	GtkIconSet *iset = gtk_style_context_lookup_icon_set (sc, GTK_STOCK_DISCARD);
-# else //gtk 2
-	GtkIconSet *iset = gtk_style_lookup_icon_set (
-#  ifdef USE_GTK2_14
-		gtk_widget_get_style (app.main_window),
-#  else
-		app.main_window->style,
-#  endif
-		GTK_STOCK_DISCARD);
-# endif
-	if (iset == NULL)
-	{
-		GtkIconSource *isrc = gtk_icon_source_new ();
-		gtk_icon_source_set_filename (isrc, ICON_DIR G_DIR_SEPARATOR_S GTK_STOCK_DISCARD "_20.png"); //or ".svg");
-		gtk_icon_source_set_direction_wildcarded (isrc, TRUE);
-		gtk_icon_source_set_size_wildcarded (isrc, TRUE);
-		iset = gtk_icon_set_new ();
-		gtk_icon_set_add_source (iset, isrc);
-		GtkIconFactory *extra_gtk_icons = gtk_icon_factory_new ();
-		gtk_icon_factory_add (extra_gtk_icons, GTK_STOCK_DISCARD, iset);
-
-		gtk_icon_factory_add_default (extra_gtk_icons);
-		g_object_unref (G_OBJECT (extra_gtk_icons));	//ensure cleanup on gtk close
-	}
-	//else ??
-#endif
-
 	return FALSE;
 }
 /**
@@ -414,37 +397,35 @@ static void _e2_main_message_handler (const gchar *log_domain,
 
 /**
 @brief create new local wait loop
-@param mainctx TRUE if wait is initiated from main-context
+@param mainwait TRUE if wait is initiated from default GMainContext
 @return the loop data struct or NULL in case of error
 */
-E2_MainLoop *e2_main_loop_new (gboolean mainctx)
+E2_MainLoop *e2_main_loop_new (gboolean mainwait)
 {
 #ifdef WAIT_POLL
 	E2_MainLoop *loopdata = ALLOCATE0 (E2_MainLoop);
 	CHECKALLOCATEDWARN (loopdata, return NULL;)
 	//without a threadID yet, an (unlikely) immediate abort will not find this
 	//loopdata in the list, hence leak
-	loopdata->maincontext = mainctx;
+	loopdata->maincontext = mainwait;
 #else
-/*	GMainContext *ctx;
-	if (mainctx)
-		ctx = NULL;
-	else
-	{
-		if (localctx == NULL)
-			localctx = g_main_context_new ();
-		ctx = localctx;
-	}
-	GMainLoop *loop = g_main_loop_new (ctx, FALSE);
-*/
-	GMainLoop *loop = g_main_loop_new (NULL, FALSE);
-	if (loop == NULL)
-		return NULL;
-
 	E2_MainLoop *loopdata = MALLOCATE (E2_MainLoop);
 	CHECKALLOCATEDWARN (loopdata, return NULL;)
-	loopdata->threadID = 0;
+/*	GMainContext *ctx;
+	if (mainwait)
+		ctx = DEFAULT_CONTEXT;
+	else
+		ctx = g_main_context_new (); FIXME leak
+	GMainLoop *loop = g_main_loop_new (ctx, FALSE);
+*/
+	GMainLoop *loop = g_main_loop_new (DEFAULT_CONTEXT, FALSE);
+	if (loop == NULL)
+	{
+		DEMALLOCATE (E2_MainLoop, loopdata);
+		return NULL;
+	}
 	loopdata->loop = loop;
+	loopdata->threadID = 0;
 #endif
 	app.mainloops = g_slist_prepend (app.mainloops, loopdata);
 	return loopdata;
@@ -474,8 +455,8 @@ Expects BGL open/off.
 static void _e2_main_loop_clean_BGL (E2_MainLoop *loopdata)
 {
 	printd (DEBUG, "Start %s, loopdata: %x thread %u", __PRETTY_FUNCTION__, loopdata, pthread_self());
-	while (g_main_context_iteration (NULL, FALSE)); //CHECKME useful ?
-	gdk_threads_enter ();
+	while (g_main_context_iteration (DEFAULT_CONTEXT, FALSE)); //CHECKME useful ?
+	CLOSEBGL
 //	printd (DEBUG, "_e2_main_loop_clean_BGL, thread %u, BGL back on", pthread_self());
 	gdk_flush (); //stops UI freezes ? (gtk does this)
 //	printd (DEBUG, "End %s, loopdata: %x", __PRETTY_FUNCTION__, loopdata);
@@ -517,7 +498,7 @@ void e2_main_loop_run (E2_MainLoop *loopdata)
 		pthread_cleanup_push ((gpointer)_e2_main_loop_clean_loop, loopdata);
 #endif
 			pthread_cleanup_push ((gpointer)_e2_main_loop_clean_BGL, loopdata);
-				gdk_threads_leave ();
+				OPENBGL
 //				printd (DEBUG, "e2_main_loop_run, BGL off");
 #ifdef WAIT_POLL
 //				pthread_cleanup_push ((gpointer)_e2_main_loop_clean, loopdata);
@@ -533,14 +514,14 @@ void e2_main_loop_run (E2_MainLoop *loopdata)
 
 					//"un-related" events may need to be processed e.g. if an abort-confirmation dialog is popped up
 //					WAIT_FOR_EVENTS_UNLOCKED
-#ifdef DEBUG_MESSAGES
-					if (g_main_context_iteration (NULL, FALSE))
+#if 0 //def DEBUG_MESSAGES
+					if (g_main_context_iteration (DEFAULT_CONTEXT, FALSE))
 					{
-						while (g_main_context_iteration (NULL, FALSE));
+						while (g_main_context_iteration (DEFAULT_CONTEXT, FALSE));
 						printd (DEBUG, "mainloop source(s) were dispatched");
 					}
 #else
-					while (g_main_context_iteration (NULL, FALSE));
+					while (g_main_context_iteration (DEFAULT_CONTEXT, FALSE));
 #endif
 
 					WAIT_LOCK
@@ -554,7 +535,7 @@ void e2_main_loop_run (E2_MainLoop *loopdata)
 				pthread_cleanup_pop (0);	//don't cleanup loop data this way (that re-enters the quitter!)
 #endif
 //				_e2_main_loop_clean_BGL (loopdata);	//pop(1) BAD when aborting ?
-				gdk_threads_enter ();
+				CLOSEBGL
 				printd (DEBUG, "BGL back on");
 			pthread_cleanup_pop (0);	//BGL back on
 
@@ -607,10 +588,10 @@ void e2_main_loop_quit (E2_MainLoop *loopdata)
 		{
 			if (mainctx)
 			{
-				gdk_threads_leave ();
+				OPENBGL
 				while (g_main_context_pending (ctx)) //INTERFERES with other mainloops
 					g_main_context_iteration (ctx, FALSE);
-				gdk_threads_enter ();
+				CLOSEBGL
 			}
 			g_main_loop_quit (tmp);
 			usleep (5000);	//allow some post-block activity
@@ -686,17 +667,19 @@ gint main (gint argc, gchar *argv[])
 	pthread_mutex_init (&wait_mutex, &attr);
 #endif
 	pthread_mutexattr_destroy (&attr);
-#ifndef NATIVE_BGL
-	e2_main_replace_gdklock (FALSE);
-#endif
 
 #ifndef USE_GLIB2_32
 	g_thread_init (NULL);
 #endif
-#ifndef NATIVE_BGL
-	gdk_threads_set_lock_functions (e2_main_close_gdklock, e2_main_open_gdklock);
+	e2_main_init_uilock (FALSE);
+# ifndef LOCAL_BGL
+#ifdef USE_GTK3_6
+#warning GTK 3.6 deprecates use of an application-specific display mutex. No reasonable workaround is available.
 #endif
+	//these are deprecated for gtk 3.6+
+	gdk_threads_set_lock_functions (e2_main_close_uilock, e2_main_open_uilock);
 	gdk_threads_init ();	//setup gdk mutex
+# endif
 #ifdef ENABLE_NLS
 	//before gtk_init(), ensure that _() in recent gtk's works cleanly
 	textdomain (BINNAME);
@@ -713,7 +696,12 @@ gint main (gint argc, gchar *argv[])
 #endif
 	g_set_application_name (_("emelFM2"));
 
+#if 0
+	gchar *parms[4] = { argv[0], "-c", "/home/maker/tmp", NULL };
+	e2_cl_option_process (3, parms);
+#else
 	e2_cl_option_process (argc, argv);
+#endif
 	_e2_main_check_locale ();
 //#ifndef E2_FILES_UTF8ONLY
 //	e2_fs_check_coding ();	//setup for path/filename conversion if needed
@@ -735,42 +723,50 @@ gint main (gint argc, gchar *argv[])
 
 	e2_option_init ();  //setup default options (before cache or config read)
 
-	gboolean confdir_ok = e2_option_set_config_dir ();  //set name for config dir, make it if not there already
-	e2_cache_init (confdir_ok);  //read and process cache file (if any) (before options init)
-	gboolean confile_ok = (confdir_ok) ?
-		e2_option_file_read (): //read & process saved options (if any)
-		FALSE;
-
-	if (confile_ok)
+	if (!e2_cl_options.original)
 	{
-		e2_option_date_style (); //customise date-format option
-		//ASAP after config settled
-		if (strcmp (app.cfgfile_version, VERSION RELEASE) < 0)
-		{
-			Plugin *p = e2_plugins_open1 (PLUGINS_DIR G_DIR_SEPARATOR_S UPGRADE_PNAME);	//localised path
-			if (p != NULL)
-			{
-				//CHECKME downstream custom dialog ok without explicit BGL close ?
-				if (!p->plugin_init (p))	//do any upgrades
-					printd (ERROR, "Can't initialize upgrade plugin: "UPGRADE_PNAME);
-				gdk_threads_enter ();
-				e2_plugins_unload1 (p, FALSE);
-				gdk_threads_leave ();
-			}
-			else
-				printd (ERROR, "Can't find upgrade plugin "UPGRADE_PNAME", so can't upgrade the config file");
-		}
-		gchar **values = e2_list_to_strv (e2_cl_options.option_overrides);
-		e2_option_read_array (values);
-		g_strfreev (values);
-	}
-//	else
-//		app.keytrans = TRUE; //translate the newly-created keybindings, later on
+		gboolean confdir_ok = e2_option_set_config_dir ();  //set name for config dir, make it if not there already
+		e2_cache_init (confdir_ok);  //read and process cache file (if any) (before options init)
+		gboolean confile_ok = (confdir_ok) ?
+			e2_option_file_read (e2_cl_options.config_dir): //read & process saved options (if any)
+			FALSE;
+		//read & process shared options-data (if wanted)
+		if (e2_cl_options.sharedconfig_dir != NULL)
+			if (e2_option_file_read (e2_cl_options.sharedconfig_dir))
+				confile_ok = TRUE;
 
+		if (confile_ok)
+		{
+			e2_option_date_style (); //customise date-format option
+			//ASAP after config settled
+			if (strcmp (app.cfgfile_version, VERSION RELEASE) < 0)
+			{
+				E2P_InitData data;
+				if (e2_plugins_open_module (PLUGINS_DIR G_DIR_SEPARATOR_S UPGRADE_PNAME, &data))	//localised path
+				{
+					//do any upgrades
+					UpgradeIface *u = UPGRADE_IFACE ((*data.init) (E2P_SETUP));
+					if (!(*u->update_config)())
+						printd (ERROR, "Upgrade failed");
+					g_module_close (data.module); //no cleanup needed, just dump it
+				}
+				else
+					printd (ERROR, "Can't find upgrade plugin "UPGRADE_PNAME", so can't upgrade the config file");
+			}
+			gchar **values = e2_list_to_strv (e2_cl_options.option_overrides);
+			e2_option_read_array (values);
+			g_strfreev (values);
+		}
+	//	else
+	//		app.keytrans = TRUE; //translate the newly-created keybindings, later on
+	}
 	//after config & updates, install default tree options where needed
 	e2_option_tree_install_defaults ();
 
 	//before plugins loaded, before bars/menus created
+#ifdef E2_ICONCACHE
+	e2_icons_cache_init ();
+#endif
 	e2_actions_init (); //setup action data stores & register most actions
 	e2_complete_init (); //"like" actions
 
@@ -784,12 +780,8 @@ gint main (gint argc, gchar *argv[])
 //	if (!defer_plugs)
 	//if this is a fresh start,
 	//do this before bars/menus created
-	e2_plugins_load_all (); // load plugins specified in the config data
+	e2_plugins_load_configured (); // load plugins specified in the config data
 	//some of these are used during pane creation
-#ifdef E2_VFS
-	app.vdir_cache = g_hash_table_new_full (e2_fileview_vdir_hash,
-		e2_fileview_vdir_equal, e2_fieview_vdir_free, NULL); //uses same data for key and value
-#endif
 	app.dir_history = g_hash_table_new_full (
 #ifdef E2_VFS
 		g_direct_hash, g_direc_equal, NULL,
@@ -831,11 +823,14 @@ gint main (gint argc, gchar *argv[])
 	//do pane 1 last, possibly making it the default at session start
 	//(active pane is changed if needed in _e2_main_defer_init())
 	e2_pane_goto_accessible_path (&app.pane1);
+	//update window title to initial directory if wanted
+	e2_window_set_title_path (app.main_window, &app.pane1.view);
+	e2_window_set_title_path (app.main_window, &app.pane2.view);
 
 	gint pos = gtk_paned_get_position (GTK_PANED (app.window.output_paned));
 	if (pos == 0)
 	{
-		gchar *command = g_strconcat(_A(1),".",_A(47),NULL);  //_("command.focus")
+		gchar *command = g_strconcat(_A(1),".",_A(50),NULL);  //_("command.focus")
 		e2_action_run_simple_from (command, NULL, app.main_window);
 		g_free (command);
 	}
@@ -850,8 +845,8 @@ gint main (gint argc, gchar *argv[])
 	e2_cache_int_register ("config-height", &app.cfg_alloc.height, 400);
 	e2_option_set_trash_dir ();
 
-	GList *tmp;
-	for (tmp = e2_cl_options.startup_commands; tmp != NULL; tmp = g_list_next (tmp))
+	GSList *tmp;
+	for (tmp = e2_cl_options.startup_commands; tmp != NULL; tmp = g_slist_next (tmp))
 		e2_command_run (tmp->data, E2_COMMAND_RANGE_DEFAULT, app.main_window
 #ifdef E2_COMMANDQ
 		, FALSE
@@ -864,13 +859,13 @@ gint main (gint argc, gchar *argv[])
 #endif
 	)
 	{
-#if 0
-		//FIXME manual daemon setup, as per <URL>linux-daemon-howto.html
-#else
+#if 1
 		if (daemon (1, 1) != 0)	//this func not available for System V-based OS
 		{
 			printd (WARN, "daemon creation failed, running normally instead");
 		}
+#else
+		//FIXME manual daemon setup, as per www.netzmafia.de/skripten/unix/linux-daemon-howto.html
 #endif
 	}
 
@@ -910,12 +905,12 @@ gint main (gint argc, gchar *argv[])
 	//get all actions, for help document
 //	e2_action_list_all ();
 
-	printd (DEBUG, "enter top-level main loop");
-	gdk_threads_enter ();
+	CLOSEBGL //gtk_main() opens lock before running loop
 	gtk_main ();
-	printd (DEBUG, "leave top-level main loop");
-	gdk_threads_leave ();
-	exit (0);
+	OPENBGL
+
+	e2_main_closedown (TRUE, TRUE, TRUE); //should never happen
+	exit (0); //ditto
 }
 
   /************************/
@@ -944,14 +939,14 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 			{
 				//can't use standard warning dialog, both its labels are unsuitable
 				gchar *prompt = g_strdup_printf (_("%u process(es) are running"), running);
-				GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_WARNING,
+				GtkWidget *dialog = e2_dialog_create (STOCK_NAME_DIALOG_WARNING,
 					prompt, _("confirm"), DUMMY_RESPONSE_CB, NULL);
 				g_free (prompt);
 
-				E2_Button yes_btn = { _("_Continue"), GTK_STOCK_EXECUTE,
+				E2_Button yes_btn = { _("_Continue"), STOCK_NAME_EXECUTE,
 					_("Continue working"), E2_BTN_DEFAULT | E2_BTN_TIPPED, 0,
 					E2_BUTTON_YES.response };
-				E2_Button no_btn = { _("_Quit"), GTK_STOCK_QUIT,
+				E2_Button no_btn = { _("_Quit"), STOCK_NAME_QUIT,
 					NULL, 0, 0, E2_BUTTON_NO.response };
 
 				DialogButtons choice = e2_dialog_show (dialog, app.main_window,
@@ -983,11 +978,14 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 #ifdef E2_DEVKIT
 	e2_devkit_disconnect ();
 #endif
-#ifdef E2_IMAGECACHE
-	e2_cache_image_clearall ();
+#ifdef E2_ADD_STOCKS
+	e2_icons_clear_stocks ();
+#endif
+#ifdef E2_ICONCACHE
+	e2_icons_cache_clear ();
 #endif
 	//before unloading plugins, in case they have config options
-	if (saveconfig)
+	if (saveconfig && !e2_cl_options.original)
 	{
 		e2_option_file_write (NULL);
 #ifdef USE_GTK2_18
@@ -1008,7 +1006,7 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 /*#ifdef E2_VFS
 	if (vfs.loaded)
 	{	//unloading does cacheing and proper cleanups
-		Plugin *p = e2_plugins_check_installed ("vfs"VERSION);
+		Plugin *p = e2_plugins_get_installed ("vfs"VERSION);
 		e2_plugins_unload1 (p, TRUE);
 	}
 #endif */
@@ -1021,20 +1019,14 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 	if (app.pane2.view.hook_refresh.is_setup)
 		g_hook_list_clear (&app.pane2.view.hook_refresh);
 #ifdef USE_GLIB2_10
-//	GList *member;
+	e2_plugins_clean ();	//needs actions_hash
+	GList *member;
 	//need to deallocate the glib slices, at least
 	//NOTE this must be after all cleanups involving actions
 	//printd (DEBUG, "clean actions");
 	//e2_actions_clean ()
 	g_hash_table_destroy (actions_hash);
-	//NOTE this must be after all cleanups involving options
-/*	printd (DEBUG, "clean options");
-	e2_option_destroy_config ();//why bother, if the kernel does it all anyway?
-								//cache(s) @ cache or named (lists ??)
-								//FIXME?? = all those non-constant config strings
-*/
-	g_hash_table_destroy (options_hash);
-/*	e2_cache_clean ();
+	e2_cache_clean ();
 	e2_complete_clear ();
 	e2_alias_clean ();
 	e2_command_line_clean_all ();
@@ -1047,17 +1039,17 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 	//output tabs
 	for (member = app.tabslist; member != NULL; member = member->next)
 		DEALLOCATE (E2_OutputTabRuntime, (E2_OutputTabRuntime *) member->data);
-*/
-	e2_toolbar_data_clean ();	//toolbar data
-/*	e2_keybinding_clean (); //keybinding runtimes
-	//FIXME deallocate other slices:
-	//EncArray
-	//E2_ToggleBox's ?
-	//E2_ToggleData's ?
+
+	e2_toolbar_data_clean ();	//toolbar & related data
+	e2_keybinding_clean (); //keybinding runtimes
+	if (app.keysnative != NULL)
+		g_hash_table_destroy (app.keysnative);
+	if (app.keyslocal != NULL)
+		g_hash_table_destroy (app.keyslocal);
+/* FIXME deallocate other slices:
 	//anything from e2_fileview_get_selected()
 	//PlaceInfo's handled by unloading the vfs plugin
 	//FileInfo's tied to treeview lines are cleaned already ?? HOW?
-	//all E2_OptionTreeColumns
 //unref all optiontree stores ?
 	//view & edit dialog history items (if sliced!)
 	//edit dialog undo items
@@ -1067,33 +1059,32 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 	//these might be in some backup place, for VFS
 	//dirline histories ?
 	//directory history list items
-	g_hash_table_destroy (app.keysnative);
-	if (app.keyslocal != NULL)
-		g_hash_table_destroy (app.keyslocal);
-#ifdef E2_VFS
-	g_hash_table_destroy (app.vdir_cache);
-#endif
 //	printd (DEBUG, "clear dirs hash");
 	g_hash_table_destroy (app.dir_history);
 	g_list_free (app.pane1.opendirs);	//list data are owned by app.dir_history
 	g_list_free (app.pane2.opendirs);	//ditto
+	e2_fileview_clear_filter_patterns (&app.pane1.view);
+	e2_fileview_clear_filter_patterns (&app.pane2.view);
 /*	printd (DEBUG, "clean bookmarks");
 	e2_bookmark_clean ();
 */
+	//NOTE this must be after all cleanups involving options
+	g_hash_table_destroy (options_hash);
+	e2_cl_option_clear ();
+
 	if (doexit)
 	{
-		gtk_main_quit ();
+		guint lvl = gtk_main_level ();
+		while (lvl-- > 0)
+			gtk_main_quit ();
 
 		pthread_mutex_destroy (&list_mutex);
 		pthread_mutex_destroy (&history_mutex);
 #ifdef WAIT_POLL
 		pthread_mutex_destroy (&wait_mutex);
 #endif
-#ifdef NATIVE_BGL
-		gdk_threads_leave ();
-#else
-		pthread_mutex_destroy (&gdklock);
-#endif
+		OPENBGL //open lock closed around gtk_main()
+		pthread_mutex_destroy (&display_mutex);
 		printd (DEBUG, "exit(0) from lounge");
 		exit (0);
 	}
@@ -1123,9 +1114,9 @@ processes
 static void _e2_main_system_shutdown (gint num)
 {
 	printd (DEBUG, "system shutdown initiated by signal %d", num);
-	gdk_threads_enter ();	//downstream errors expect BGL closed
+	CLOSEBGL	//downstream errors expect BGL closed
 	e2_main_closedown (TRUE, (num != SIGINT && num != SIGKILL), FALSE);
-	gdk_threads_leave ();
+	OPENBGL
 	//in this handler, current signal is blocked, unblock it for this use
 	struct sigaction sigdata;
 	sigdata.sa_handler = SIG_DFL;
