@@ -1,12 +1,11 @@
-/* $Id: e2_mousegesture.c 2319 2011-01-27 03:52:56Z tpgww $
+/* $Id: e2_mousegesture.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2008-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2008-2013 tooar <tooar@emelfm2.net>
 
-This file is part of emelfm2.
-emelfm2 is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+This file is part of emelFM2, which is free software. You can redistribute it
+and/or modify it under the terms of the GNU General Public License as published
+by the Free Software Foundation - either version 3, or (at your option) any
+later version.
 
 emelfm2 is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -125,8 +124,6 @@ typedef struct _E2_GestureCatRuntime
 
 static void _e2_mousegesture_run_task (GtkWidget *from,
 	gpointer action_string);
-static gboolean _stroke_watcher_button_event_cb (GtkWidget *widget,
-	GdkEventButton *event, gpointer user_data);
 
 //tree of E2_GestureCatRuntime's
 static GNode *movebindings = NULL;
@@ -330,6 +327,7 @@ static gboolean _stroke_watcher_key_event_cb (GtkWidget *widget,
 #endif
 	{
 		gint x, y;
+		NEEDCLOSEBGL
 #ifdef USE_GTK3_0 //CHECKME pointer position
 		if (gdk_window_get_device_position (event->window, gw->device, &x, &y, NULL)
 			== event->window)
@@ -353,6 +351,7 @@ static gboolean _stroke_watcher_key_event_cb (GtkWidget *widget,
 			}
 */
 		}
+		NEEDOPENBGL
 	}
 	return FALSE;
 }
@@ -376,7 +375,8 @@ static gboolean _stroke_watcher_button_event_cb (GtkWidget *widget,
 	printd (DEBUG,"_stroke_watcher_button_event_cb begins: event-type: %d", event->type);
 	new = FALSE;
 	bmask = 1 << (event->button - 1);
-	last_buttons = (guint32) g_object_get_data (G_OBJECT (widget), BUTTONS_DATA_KEY);
+	//double-cast to prevent compiler warning
+	last_buttons = (guint32)(gulong)g_object_get_data (G_OBJECT (widget), BUTTONS_DATA_KEY);
 	watches = (GPtrArray *) g_object_get_data (G_OBJECT (widget), WATCHES_DATA_KEY);
 	if (watches != NULL)
 	{
@@ -432,6 +432,7 @@ static gboolean _stroke_watcher_button_event_cb (GtkWidget *widget,
 		}
 		g_object_set_data (G_OBJECT (widget), BUTTONS_DATA_KEY,
 				GUINT_TO_POINTER (last_buttons));
+//		printd (DEBUG,"_stroke_watcher_button_event_cb returns FALSE");
 		return FALSE;
 	}
 
@@ -906,7 +907,9 @@ static void _e2_mousegesture_run_task (GtkWidget *from, gpointer action_string)
 	E2_Action *action = e2_action_check (command);
 	if (action != NULL)
 	{
+		OPENBGL
 		e2_action_run_simple_from (command, arg, from);
+		CLOSEBGL
 	}
 	else
 	{
@@ -1287,14 +1290,14 @@ static void _e2_mousegesture_tree_defaults (E2_OptionSet *set)
 	g_strconcat(_C(17),"|||||",NULL),  //_("general"
 	g_strconcat("\t",_C(23),"|||||",NULL),  //_("main"
 	g_strconcat("\t\t",_C(33),"|||||",NULL),  //_("panes"
-	g_strconcat("\t\t\t|<Alt>3|^|8,2|",_A(13),".",_A(51),"|",NULL), //"go up
-	g_strconcat("\t\t\t|<Alt>3|<-|6,4|",_A(13),".",_A(49),"|",NULL), //"go back
-	g_strconcat("\t\t\t|<Alt>3|->|4,6|",_A(13),".",_A(50),"|",NULL), //"go forward
+	g_strconcat("\t\t\t|<Alt>3|^|8,2|",_A(13),".",_A(54),"|",NULL), //"go up
+	g_strconcat("\t\t\t|<Alt>3|<-|6,4|",_A(13),".",_A(52),"|",NULL), //"go back
+	g_strconcat("\t\t\t|<Alt>3|->|4,6|",_A(13),".",_A(53),"|",NULL), //"go forward
 	g_strconcat("\t\t",_C(12),"|||||",NULL),  //_"directory line"
 	g_strconcat("\t\t",_C(5),"|||||",NULL),  //_("command line"
 	g_strconcat("\t\t",_C(28),"|||||",NULL),  //_("output"
-	g_strconcat("\t\t\t|<Alt>3||2,8|",_A(10),".",_A(31),"|1",NULL), //"adjust ratio
-	g_strconcat("\t\t\t|<Alt>3|^|8,2|",_A(10),".",_A(69),"|",NULL), //"page up"
+	g_strconcat("\t\t\t|<Alt>3||2,8|",_A(10),".",_A(33),"|1",NULL), //"adjust ratio
+	g_strconcat("\t\t\t|<Alt>3|^|8,2|",_A(10),".",_A(72),"|",NULL), //"page up"
 	g_strconcat("\t",_C(11),"|||||",NULL),  //_("dialogs"
 	g_strdup(">"),
 	NULL);
