@@ -1,6 +1,6 @@
-/* $Id: e2_pane.h 2202 2010-11-12 22:23:28Z tpgww $
+/* $Id: e2_pane.h 2742 2013-09-11 00:37:42Z tpgww $
 
-Copyright (C) 2004-2010 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -32,9 +32,9 @@ typedef struct _E2_PaneRuntime
 	ViewInfo view;	//at start of struct to support casting between pane and view
 	gchar *path;	//UTF-8 string, generally = view->dir, for use when allocated version is needed
 	gchar *name;	//"pane1" or "pane2", not translated, for constructing set names
-	GtkWidget *outer_box;
-	GtkWidget *inner_box;
-	GtkWidget *pane_sw;
+	GtkWidget *outer_box;	//hbox, parent of inner_box
+	GtkWidget *inner_box; //vbox, parent of pane_sw (& often, the toolbar)
+	GtkWidget *pane_sw; //scroller, parent of filelist treeview
 //	GtkWidget *focus_widget;
 	E2_ToolbarRuntime toolbar;
 #ifdef E2_FAM
@@ -55,7 +55,6 @@ typedef struct _E2_PaneRuntime
 
 gboolean e2_pane_choose_new_dir (E2_PaneRuntime *rt, GtkWidget *entry);
 gboolean e2_pane_cd_checks (gchar *path);
-gboolean e2_pane_change_dir_action (gpointer from, E2_ActionRuntime *art);
 #ifdef E2_VFS
 gboolean e2_pane_change_space (E2_PaneRuntime *rt, VPATH *utfpath);
 gboolean e2_pane_change_space_byuri (E2_PaneRuntime *rt, const gchar *spacedescriptor);
