@@ -1,6 +1,6 @@
-/* $Id: e2_print.c 2362 2011-06-02 00:24:30Z tpgww $
+/* $Id: e2_print.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2010-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2010-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -38,12 +38,14 @@ static void _e2_view_dialog_begin_print_cb (GtkPrintOperation *op,
 		GtkPrintContext *context, E2_ViewDialogRuntime *rt)
 {
 	GtkTextIter start, end;
+	NEEDCLOSEBGL
 	if (gtk_text_buffer_get_has_selection (rt->textbuffer))
 		gtk_text_buffer_get_selection_bounds (rt->textbuffer, &start, &end);
 	else
 		gtk_text_buffer_get_bounds (rt->textbuffer, &start, &end);
 
 	gchar *seltext = gtk_text_buffer_get_text (rt->textbuffer, &start, &end, FALSE);
+	NEEDOPENBGL
 	g_strstrip (seltext);
 	if (*seltext != '\0')
 	{
@@ -113,6 +115,8 @@ static void _e2_view_dialog_begin_print_cb (GtkPrintOperation *op,
 /*static void _e2_view_dialog_end_print_cb (GtkPrintOperation *op,
 	GtkPrintContext *context, gpointer data)
 {
+	NEEDCLOSEBGL
+	NEEDOPENBGL
 } */
 /**
 @brief callback for "draw-page" signal on print-operation for selected or all text
@@ -127,6 +131,7 @@ static void _e2_view_dialog_begin_print_cb (GtkPrintOperation *op,
 static void _e2_view_dialog_draw_page_cb (GtkPrintOperation *op,
 	GtkPrintContext *context, gint page_num, E2_ViewDialogRuntime *rt)
 {
+//	NEEDCLOSEBGL
 	cairo_t *cr = gtk_print_context_get_cairo_context (context);
 	if (cr != NULL)
 	{
@@ -186,6 +191,7 @@ static void _e2_view_dialog_draw_page_cb (GtkPrintOperation *op,
 			pango_cairo_show_layout_line (cr, linelay);
 		}
 	}
+//	NEEDOPENBGL
 }
 /**
 @brief callback for "status-changed" signal on print-operation for selected or all text
@@ -198,7 +204,9 @@ static void _e2_view_dialog_draw_page_cb (GtkPrintOperation *op,
 static void _e2_view_dialog_print_status_cb (GtkPrintOperation *op,
 	gpointer data)
 {
+	NEEDCLOSEBGL
 	GtkPrintStatus pstat = gtk_print_operation_get_status (op);
+	NEEDOPENBGL
 	switch (pstat)
 	{
 		case GTK_PRINT_STATUS_PENDING_ISSUE:
@@ -224,6 +232,7 @@ void e2_dialog_print_cb (GtkMenuItem *menuitem, E2_ViewDialogRuntime *rt)
 #endif */
 	static GtkPageSetup *page_setup = NULL;
 
+	NEEDCLOSEBGL
 	GtkPrintOperation *op = gtk_print_operation_new ();
 /*#ifdef USE_GTK2_18
 	if (settings == NULL)
@@ -284,6 +293,7 @@ void e2_dialog_print_cb (GtkMenuItem *menuitem, E2_ViewDialogRuntime *rt)
 				g_object_unref (G_OBJECT (op));
 			break;
 	}
+	NEEDOPENBGL
 }
 #endif //def SIMPLEPRINT
 
@@ -553,7 +563,12 @@ static gint _e2_print_range_pages_compare (gconstpointer a, gconstpointer b)
 */
 static GObject *_e2_print_get_custom_tab_cb (GtkPrintOperation *po, PrintData *pd)
 {
+	NEEDCLOSEBGL
+#ifdef USE_GTK3_0
+	GtkWidget *vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, E2_PADDING);
+#else
 	GtkWidget *vbox = gtk_vbox_new (FALSE, E2_PADDING);
+#endif
 
 	GtkWidget *btn = gtk_check_button_new_with_mnemonic (_("Add page _header"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (btn), printheader);
@@ -575,7 +590,11 @@ static GObject *_e2_print_get_custom_tab_cb (GtkPrintOperation *po, PrintData *p
 	gtk_box_pack_start (GTK_BOX (vbox), btn, FALSE, FALSE, 0);
 	pd->invertbtn = btn;
 
+#ifdef USE_GTK3_0
+	GtkWidget *hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, E2_PADDING_SMALL);
+#else
 	GtkWidget *hbox = gtk_hbox_new (FALSE, E2_PADDING_SMALL);
+#endif
 	GtkWidget *lbl = gtk_label_new (_("Tab size"));
 	gtk_box_pack_start (GTK_BOX (hbox), lbl, FALSE, FALSE, 0);
 	GtkWidget *spin = gtk_spin_button_new_with_range (1.0, 20.0, 1.0);
@@ -589,10 +608,15 @@ static GObject *_e2_print_get_custom_tab_cb (GtkPrintOperation *po, PrintData *p
 
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
+#ifdef USE_GTK3_0
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
 	hbox = gtk_hbox_new (FALSE, 0);
+#endif
 	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, E2_PADDING);
 	gtk_widget_show_all (hbox);
 
+	NEEDOPENBGL
 	return G_OBJECT(hbox);
 }
 /**
@@ -607,11 +631,13 @@ static GObject *_e2_print_get_custom_tab_cb (GtkPrintOperation *po, PrintData *p
 static void _e2_print_apply_extras_cb (GtkPrintOperation *po,
 	GtkWidget *tabchild, PrintData *pd)
 {
+//	NEEDCLOSEBGL
 	printinvert = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pd->invertbtn));
 	printheader = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pd->headerbtn));
 	wraplines = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pd->wrapbtn));
 	doublespace = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pd->doublebtn));
 	tabsize = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pd->tabspin));
+//	NEEDOPENBGL
 }
 /**
 @brief callback for "begin-print" signal on @a po
@@ -636,11 +662,18 @@ static void _e2_print_begin_cb (GtkPrintOperation *po, GtkPrintContext *context,
 		return;
 	}
 
+	NEEDCLOSEBGL
 	PangoAttrList *attrs = pango_attr_list_new ();
 #ifdef USE_GTK3_0
 	GtkStyleContext *sc = gtk_widget_get_style_context (pd->textview);
+# ifdef USE_GTK3_8
+	const PangoFontDescription *font_desc;
+	gtk_style_context_get (sc, GTK_STATE_FLAG_NORMAL,
+		GTK_STYLE_PROPERTY_FONT, &font_desc, NULL);
+# else
 	const PangoFontDescription *font_desc = gtk_style_context_get_font (sc,
 		GTK_STATE_NORMAL);
+# endif
 #else
 	PangoFontDescription *font_desc = gtk_widget_get_style (pd->textview)->font_desc;
 #endif
@@ -802,12 +835,14 @@ static void _e2_print_begin_cb (GtkPrintOperation *po, GtkPrintContext *context,
 			else
 			{
 				gtk_print_operation_cancel (po);
+				NEEDOPENBGL
 				return;
 			}
 		}
 		else /* some problem with the page-range(s) entered */
 		{
 			gtk_print_operation_cancel (po);
+			NEEDOPENBGL
 			return;
 		}
 	}
@@ -827,6 +862,7 @@ static void _e2_print_begin_cb (GtkPrintOperation *po, GtkPrintContext *context,
 		else
 		{
 			gtk_print_operation_cancel (po);
+			NEEDOPENBGL
 			return;
 		}
 	}
@@ -881,6 +917,7 @@ static void _e2_print_begin_cb (GtkPrintOperation *po, GtkPrintContext *context,
 		pd->lo_name = NULL;
 		pd->lo_pagenum = NULL;
 	}
+	NEEDOPENBGL
 }
 /**
 @brief callback for "draw-page" signal on @a po
@@ -906,6 +943,7 @@ static void _e2_print_draw_page_cb (GtkPrintOperation *po,
 	guint16 fullpages;
 	gint bufferlineindex;
 
+	NEEDCLOSEBGL
 	cr = gtk_print_context_get_cairo_context (context);
 next_page:
 	if (!GTK_IS_TEXT_BUFFER (pd->buf)) //TODO (if relevant) check its selection unchanged
@@ -1128,6 +1166,7 @@ checkspace2:
 				goto next_page;
 		}
 	}
+	NEEDOPENBGL
 }
 /**
 @brief callback for "done" signal on @a po
@@ -1151,7 +1190,9 @@ static void _e2_print_done_cb (GtkPrintOperation *po,
 	{
 		GError *error;
 		gtk_print_operation_get_error (po, &error);
+		NEEDCLOSEBGL
 		e2_output_print_error (error->message, FALSE);
+		NEEDOPENBGL
 		g_error_free (error);
 	}
 	g_object_unref (G_OBJECT (po));
@@ -1185,7 +1226,7 @@ void e2_dialog_print_cb (GtkMenuItem *menuitem, E2_ViewDialogRuntime *rt)
 	pd->localpath = rt->localpath;
 
 	if (print_settings == NULL)
-		print_settings = gtk_print_settings_new (); /* never cleaned */
+		print_settings = gtk_print_settings_new (); //cleaned in e2_print_clear()
 	if (page_setup == NULL)
 	{
 		page_setup = gtk_page_setup_new ();	/* never cleaned */
@@ -1201,13 +1242,19 @@ void e2_dialog_print_cb (GtkMenuItem *menuitem, E2_ViewDialogRuntime *rt)
 	gtk_print_operation_set_allow_async (po, TRUE);
 //	gtk_print_operation_set_use_full_page (po, gboolean use_full); default FALSE
 	gtk_print_operation_set_current_page (po, 0); /* enable current-page selection */
+
 #ifdef USE_GTK2_18
 	gtk_print_operation_set_embed_page_setup (po, TRUE);
 	gtk_print_operation_set_support_selection (po, TRUE);
 	gtk_print_operation_set_has_selection (po, TRUE); /* always selectable */
+
+	NEEDCLOSEBGL
+
 	if (gtk_text_buffer_get_has_selection (rt->textbuffer))
 		gtk_print_settings_set_print_pages (print_settings, GTK_PRINT_PAGES_SELECTION);
 #else
+	NEEDCLOSEBGL
+
 	if (gtk_text_buffer_get_has_selection (rt->textbuffer))
 		pd->printpages = GTK_PRINT_PAGES_SELECTION;
 #endif
@@ -1257,6 +1304,7 @@ void e2_dialog_print_cb (GtkMenuItem *menuitem, E2_ViewDialogRuntime *rt)
 		break;
 	}
 
+	NEEDOPENBGL
 	g_object_unref (G_OBJECT (po));
 	_e2_print_data_free (pd);
 }
