@@ -1,6 +1,6 @@
-/* $Id: emelfm2.h 2484 2012-04-06 10:07:40Z tpgww $
+/* $Id: emelfm2.h 2999 2014-01-17 09:29:12Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -86,7 +86,15 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 # if GLIB_CHECK_VERSION (2,32,0)
 #  define USE_GLIB2_32
 # endif
-
+# if GLIB_CHECK_VERSION (2,34,0)
+#  define USE_GLIB2_34
+# endif
+# if GLIB_CHECK_VERSION (2,36,0)
+#  define USE_GLIB2_36
+# endif
+# if GLIB_CHECK_VERSION (2,38,0)
+#  define USE_GLIB2_38
+# endif
 # if GTK_CHECK_VERSION (2,8,0)
 #  define USE_GTK2_8
 # endif
@@ -126,6 +134,21 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 # if GTK_CHECK_VERSION (3,6,0)
 #  define USE_GTK3_6
 # endif
+# if GTK_CHECK_VERSION (3,8,0)
+#  define USE_GTK3_8
+# endif
+# if GTK_CHECK_VERSION (3,10,0)
+#  define USE_GTK3_10
+# endif
+# if GTK_CHECK_VERSION (3,12,0)
+#  define USE_GTK3_12
+# endif
+# if GTK_CHECK_VERSION (3,14,0)
+#  define USE_GTK3_14
+# endif
+# if GTK_CHECK_VERSION (3,16,0)
+#  define USE_GTK3_16
+# endif
 
 #elif defined(E2_MIN_GTK3)
 //GTK3 is available and either: explicitly requested or GTK2 is not available
@@ -176,6 +199,13 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 # include <gdk/gdkkeysyms.h>
 #endif
 
+//install stock icons only for gtk 3.10+
+#ifdef E2_ADD_STOCKS
+# ifndef USE_GTK3_10
+#  undef E2_ADD_STOCKS
+# endif
+#endif
+
 //longest item name (bytes, NOT chars) is a *NIX define
 //#define NAME_MAX 255
 //length of various buffers
@@ -296,6 +326,9 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #define E2_PTRGESTURES
 #endif
 
+//enable workaround for bad behaviour of [g]libc symlink() with relative links
+#define RELLINK_FIX
+
 //enable key.fake action LITTLE TESTED
 //#define WITH_KEYFAKE
 
@@ -309,9 +342,6 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 //enable file.untrash action LITTLE TESTED
 //#define WITH_UNTRASH
 
-//enable config/button/menu image-cacheing
-#define E2_IMAGECACHE
-
 //multi-colored filelists
 #define E2_RAINBOW
 
@@ -327,6 +357,9 @@ typedef struct _E2_ColorData
 # endif
 #endif
 
+//basename of upgrade plugin
+#define UPGRADE_PNAME "e2p_upgrade.so"
+
 //maybe useful for multi-processor optimisation ?
 //#define VOLATILE volatile
 #define VOLATILE
@@ -338,6 +371,9 @@ typedef struct _E2_ColorData
 //include code for handling [%]%e [%]%E macros
 #define E2_BADQUOTES
 
+//include code for providing an additional filelist-column showing extension
+//#define E2_EXTCOL
+
 //support mountpoint-related capabilities
 #define E2_FS_MOUNTABLE
 
@@ -558,20 +594,7 @@ typedef struct _E2_Vpath
 #include "e2_widget.h"
 #include "e2_window.h"
 
-// some nice helper/utility macros
-//these 2 must both be called with BGL closed
-//the gtk-versions block until at least one event is processed
-//#define WAIT_FOR_EVENTS while (gtk_events_pending ()) { gtk_main_iteration (); }
-//#define WAIT_FOR_EVENTS_SLOWLY while (gtk_events_pending ()) { gtk_main_iteration ();usleep(5000); }
-#define WAIT_FOR_EVENTS gdk_threads_leave (); \
-	while (g_main_context_pending (NULL)) { g_main_context_iteration (NULL, TRUE); } \
-	gdk_threads_enter ();
-#define WAIT_FOR_EVENTS_SLOWLY gdk_threads_leave (); \
-	while (g_main_context_pending (NULL)) { g_main_context_iteration (NULL, TRUE); usleep(5000); } \
-	gdk_threads_enter ();
-//and these 2 are the corresponding ones with BGL open
-#define WAIT_FOR_EVENTS_UNLOCKED while (g_main_context_pending (NULL)) { g_main_context_iteration (NULL, TRUE); }
-#define WAIT_FOR_EVENTS_UNLOCKED_SLOWLY while (g_main_context_pending (NULL)) { g_main_context_iteration (NULL, TRUE); usleep(5000); }
+//helper/utility macros
 
 #define ITEM_ISHIDDEN(expr) (expr[0]=='.')
 //#define ITEM_ISHIDDEN(expr) (*expr=='.')
@@ -597,7 +620,7 @@ typedef struct _E2_Vpath
 #endif
 
 //do not implement slices everywhere until comprehensive session-end cleanups are done
-#if 0 //def USE_GLIB2_10
+#ifdef USE_GLIB2_10
 # define ALLOCATE(type) (type*)g_slice_alloc(sizeof(type));
 # define ALLOCATE0(type) (type*) g_slice_alloc0(sizeof(type));
 # define DEALLOCATE(type,ptr) g_slice_free1(sizeof(type),ptr);
@@ -636,15 +659,18 @@ typedef struct _E2_Vpath
 #endif
 
 //survive or exit gracefully when allocation fails
-//can implement this now, if so desired
-//#define CHECKALLOCATEDFATAL(p) if (p==NULL) e2_utils_memory_error ();
-//#define CHECKALLOCATEDWARN(p,more) if (p==NULL) {e2_utils_show_memory_message (); more;}
-//#define CHECKALLOCATEDFATALT(p) if (p==NULL) {gdk_threads_enter();e2_utils_memory_error();gdk_threads_leave();}
-//#define CHECKALLOCATEDWARNT(p,more) if (p==NULL) {gdk_threads_enter();e2_utils_show_memory_message();gdk_threads_leave(); more;}
-#define CHECKALLOCATEDFATAL(p)
-#define CHECKALLOCATEDWARN(p,more)
-#define CHECKALLOCATEDFATALT(p)
-#define CHECKALLOCATEDWARNT(p,more)
+//can't implement this now, some contexts wrongly assume BGL closed - check g_try_malloc() uses
+#if 0
+# define CHECKALLOCATEDFATAL(p) if (p==NULL) e2_utils_memory_error ();
+# define CHECKALLOCATEDWARN(p,more) if (p==NULL) {e2_utils_show_memory_message (); more;}
+# define CHECKALLOCATEDFATALT(p) if (p==NULL) {CLOSEBGL e2_utils_memory_error(); OPENBGL}
+# define CHECKALLOCATEDWARNT(p,more) if (p==NULL) {CLOSEBGL e2_utils_show_memory_message();OPENBGL more;}
+#else
+# define CHECKALLOCATEDFATAL(p)
+# define CHECKALLOCATEDWARN(p,more)
+# define CHECKALLOCATEDFATALT(p)
+# define CHECKALLOCATEDWARNT(p,more)
+#endif
 
 //index enumerator for cancellable timers
 //CHECKME some other repeating, non-trivial-delay, timers are non-static
@@ -672,6 +698,7 @@ typedef enum
 
 //define to use local polling instead of full-blown main loop for blocking dialogs
 #define WAIT_POLL
+
 typedef struct _E2_MainLoop
 {
 	pthread_t threadID;	//thread which started this loop
@@ -691,9 +718,6 @@ typedef struct _E2_MainData
 	GtkWidget *hbox_main;
 	GtkWidget *vbox_main;
 	GtkWidget *outbook; //notebook container for tabbed output-panes
-//#ifdef E2_POLKIT
-//	GtkWidget *status_bar_label1;
-//#endif
 	GtkWidget *status_bar_label2;
 	//this is a hbox, at the right of the status bar, for general usage
 	// needs to be shown when used
@@ -724,9 +748,6 @@ typedef struct _E2_MainData
 #ifndef USE_GTK2_12TIPS
 	GtkTooltips *tooltips;
 	gint gtkversion;		//runtime version, = major * 10000 + minor * 100 + micro
-#endif
-#ifdef E2_VFS
-	GHashTable *vdir_cache; //table of vath's for places used in the session, each with 0x1 data
 #endif
 	GHashTable *dir_history;//table of E2_DirHistoryEntry's for dirs opened in session
 	GHashTable *keysnative;	//table of keycodes for asciifying localised alphabetic
@@ -738,14 +759,10 @@ typedef struct _E2_MainData
 							//each corresponding value is a UTF-8 string
 							//representing the corresponding localised letter
 							//This table is not often used
-	GList *plugins;	//list of data structs for all loaded plugins, "non-children"
-					//in menu-order (children-submenu-order depends on a list in
-					//corresponding parent)
+	GHashTable *plugins;	//table of PluginIface's of loaded plugins
+	GPtrArray *plugacts;	//array of PluginAction's for all loaded plugins, in menu-order
 	GHashTable *filetypes;
 	GSList *typelist;		//list of string arrays, needed tor cleanup
-#ifdef E2_IMAGECACHE
-	GHashTable *icons;		//table of E2_Images, for cached icons
-#endif
 #ifdef E2_RAINBOW
 	GHashTable *colors;
 	GList *colorchunks;
@@ -786,33 +803,43 @@ pthread_mutex_t history_mutex;
 #define HISTORY_LOCK pthread_mutex_lock (&history_mutex);
 #define HISTORY_UNLOCK pthread_mutex_unlock (&history_mutex);
 
-//for testing - define to use gtk's own mutex for BGL
-//#define NATIVE_BGL
-#ifdef NATIVE_BGL
-	//WAIT_FOR_EVENTS expects BGL closed
-# define CLOSEBGL_IF_OPEN \
-	WAIT_FOR_EVENTS_UNLOCKED
-# define OPENBGL_IF_CLOSED
-#else //ndef NATIVE_BGL
+//macros related to blocking simultaneous access to thread-unsafe UI backend (notably xlib)
+
 //local management of gdk mutex, to enable tolerant re-locking by the same thread
 //direct mutex-manipulation
-pthread_mutex_t gdklock;	//BGL replacement
-void e2_main_close_gdklock (void);
-void e2_main_open_gdklock (void);
-# ifdef DEBUG_MESSAGES
-#  define gdk_threads_enter e2_main_close_gdklock
-#  define gdk_threads_leave e2_main_open_gdklock
-#else
-#  define gdk_threads_enter() pthread_mutex_lock (&gdklock)
-#  define gdk_threads_leave() pthread_mutex_unlock (&gdklock)
-#endif
-void e2_main_replace_gdklock (gboolean free);
+pthread_mutex_t display_mutex;	//BGL replacement
 
-	//not locked now and not already locked - this should never happen
-	//prevent unlock
-# define CLOSEBGL_IF_OPEN \
+#ifdef DEBUG_MESSAGES
+# define CLOSEBGL e2_main_close_uilock ();
+# define OPENBGL e2_main_open_uilock ();
+# define OPENBGL_NAME e2_main_open_uilock
+#else
+# define CLOSEBGL pthread_mutex_lock (&display_mutex);
+# define OPENBGL pthread_mutex_unlock (&display_mutex);
+# define OPENBGL_NAME pthread_mutex_unlock
+#endif
+
+#ifdef USE_GTK3_6
+//workaround for deprecated/lack-of internal X11 lock in gtk3.6+
+//# define LOCAL_BGL
+# ifdef LOCAL_BGL
+#  define NEEDCLOSEBGL CLOSEBGL
+#  define NEEDCLOSEBGLX CLOSEBGL //locks needing further attention
+#  define NEEDOPENBGL OPENBGL
+# endif
+#endif
+
+#ifndef NEEDCLOSEBGL
+# define NEEDCLOSEBGL
+# define NEEDCLOSEBGLX
+# define NEEDOPENBGL
+#endif
+
+//not locked now and not already locked - this should never happen
+//prevent unlock
+#define CLOSEBGL_IF_OPEN \
 	gint _lockres; \
-	_lockres = pthread_mutex_trylock (&gdklock); \
+	_lockres = pthread_mutex_trylock (&display_mutex); \
 	printd (DEBUG, "%s BGL trylock result %d", __PRETTY_FUNCTION__, _lockres); \
 	if (!(_lockres == 0 || _lockres == EBUSY)) \
 	{ \
@@ -821,11 +848,86 @@ void e2_main_replace_gdklock (gboolean free);
 		_lockres = 1; \
 	}
 
-# define	OPENBGL_IF_CLOSED \
+#define OPENBGL_IF_CLOSED \
 	if (_lockres == 0) \
-		pthread_mutex_unlock (&gdklock);
+		pthread_mutex_unlock (&display_mutex);
+
+//#define NEWLOOP
+#ifdef NEWLOOP
+extern GMainContext *localctx;
+# define DEFAULT_CONTEXT localctx
+#else
+# define DEFAULT_CONTEXT NULL
+#endif
+	
+//gtk-versions of these macros block until at least one event is processed
+
+#if 1
+//this version of the macros seem be faster at session-start
+
+//these 2 must be called with BGL closed
+#define WAIT_FOR_EVENTS \
+GMainContext *__ctx = g_main_context_default (); \
+OPENBGL \
+while (g_main_context_pending (__ctx)) { g_main_context_iteration (__ctx, TRUE); } \
+CLOSEBGL
+#define WAIT_FOR_EVENTS_SLOWLY \
+GMainContext *__ctx = g_main_context_default (); \
+OPENBGL \
+while (g_main_context_pending (__ctx)) { g_main_context_iteration (__ctx, TRUE); usleep(5000); } \
+CLOSEBGL
+#define EXTRA_WAIT_FOR_EVENTS \
+OPENBGL \
+while (g_main_context_pending (__ctx)) { g_main_context_iteration (__ctx, TRUE); usleep(5000); } \
+CLOSEBGL
+
+//and these 2 are the corresponding ones with BGL open
+#define WAIT_FOR_EVENTS_UNLOCKED \
+GMainContext *__ctx = g_main_context_default (); \
+while (g_main_context_pending (__ctx)) { g_main_context_iteration (__ctx, TRUE); }
+#define WAIT_FOR_EVENTS_UNLOCKED_SLOWLY \
+GMainContext *__ctx = g_main_context_default (); \
+while (g_main_context_pending (__ctx)) { g_main_context_iteration (__ctx, TRUE); usleep(5000); }
+#define GET_EVENTS_CONTEXT GMainContext *__ctx = g_main_context_default ();
+#define EXTRA_WAIT_FOR_EVENTS_UNLOCKED \
+while (g_main_context_pending (__ctx)) { g_main_context_iteration (__ctx, TRUE); usleep(5000); }
+
+
+
+#else //other version
+
+#define WAIT_FOR_EVENTS \
+GMainContext *__ctx = g_main_context_default (); \
+OPENBGL \
+while (g_main_context_iteration (__ctx, FALSE)){} \
+CLOSEBGL
+#define WAIT_FOR_EVENTS_SLOWLY \
+GMainContext *__ctx = g_main_context_default (); \
+OPENBGL \
+while (g_main_context_iteration (__ctx, FALSE)){usleep(5000);} \
+CLOSEBGL
+
+//and these 2 are the corresponding ones with BGL open
+#define WAIT_FOR_EVENTS_UNLOCKED GMainContext *__ctx = g_main_context_default (); while (g_main_context_iteration (__ctx, FALSE)){}
+#define WAIT_FOR_EVENTS_UNLOCKED_SLOWLY GMainContext *__ctx = g_main_context_default (); while (g_main_context_iteration (__ctx, FALSE)){usleep(5000);}
+#define GET_EVENTS_CONTEXT GMainContext *__ctx = g_main_context_default ();
+#define EXTRA_WAIT_FOR_EVENTS_UNLOCKED while (g_main_context_iteration (__ctx, FALSE)){}
+
+#endif
+
+void e2_main_init_uilock (gboolean free);
+//void e2_main_cleanup_uilock (gpointer data);
+#ifdef DEBUG_MESSAGES
+void e2_main_close_uilock (void);
+void e2_main_open_uilock (void);
+#endif
+
+#ifdef NEWLOOP
+# define DISPLAYLOCK_PARENT GUINT_TO_POINTER(1)
+# define DISPLAYLOCK_DEPREC GUINT_TO_POINTER(2)
+void e2_main_level_run (GSourceLock *data);
+void e2_main_level_quit (void);
 #endif
-//void e2_main_cleanup_gdklock (gpointer data);
 
 E2_MainLoop *e2_main_loop_new (gboolean mainctx);
 void e2_main_loop_run (E2_MainLoop *loopdata);
@@ -833,6 +935,5 @@ void e2_main_loop_quit (E2_MainLoop *loopdata);
 gboolean e2_main_loop_abort (pthread_t ID);
 gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean doexit);
 gboolean e2_main_user_shutdown (gpointer from, E2_ActionRuntime *art);
-//	__attribute__ ((noreturn));
 
 #endif //ndef __EMELFM2_H__
