@@ -1,6 +1,6 @@
-/* $Id: e2_devkit.c 2398 2011-10-12 23:02:15Z tpgww $
+/* $Id: e2_devkit.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2009-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2009-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -1087,6 +1087,7 @@ static gboolean _e2_devkit_unmount_volume (DevicesSession *sessionptr, DeviceDat
 
 /**
 @brief callback for completion of async mount operation
+Unlike gtk callbacks, assumes BGL open/off when using libdbus-glib
 @param proxy proxy for the remote interface
 @param call the pending call ID from dbus_g_proxy_begin_call()
 @param user_data UNUSED pointer specified when cb was connected
@@ -1135,7 +1136,17 @@ DBusGProxy *proxy, DBusGProxyCall *call,
 	else
 	{
 		gchar *msg = g_strdup_printf (_("Error mounting device: %s"), error->message);
+#ifdef USE_GLIB2_26
+		NEEDCLOSEBGL
+#else
+		CLOSEBGL
+#endif
 		e2_output_print_error (msg, TRUE);
+#ifdef USE_GLIB2_26
+		NEEDOPENBGL
+#else
+		OPENBGL
+#endif
 		g_error_free (error);
 	}
 #ifdef USE_GLIB2_26
@@ -1176,7 +1187,17 @@ gpointer user_data)
 	else
 	{
 		gchar *msg = g_strdup_printf (_("Error unmounting device: %s"), error->message);
+#ifdef USE_GLIB2_26
+		NEEDCLOSEBGL
+#else
+		CLOSEBGL
+#endif
 		e2_output_print_error (msg, TRUE);
+#ifdef USE_GLIB2_26
+		NEEDOPENBGL
+#else
+		OPENBGL
+#endif
 		g_error_free (error);
 	}
 #ifdef USE_GLIB2_26
@@ -1267,7 +1288,13 @@ DBusGProxy *proxy,
 		}
 		printd (WARN, "Devices daemon add-event for previously added device %s, treat as change",
 			object_path);
+#ifdef USE_GLIB2_26
+		NEEDOPENBGL
+#endif
 		_e2_devkit_device_changed_cb (proxy, object_path, sessionptr);
+#ifdef USE_GLIB2_26
+		NEEDCLOSEBGL
+#endif
 	}
 }
 /**
@@ -1473,6 +1500,7 @@ static void _e2_devkit_device_signal_cb (GDBusProxy *proxy,
 	g_variant_get_child (parameters, 0, "&o", &object_path);
 	if (G_LIKELY (object_path != NULL))
 	{
+		NEEDOPENBGL
 		switch (mode)
 		{
 		case 1:
@@ -1485,6 +1513,7 @@ static void _e2_devkit_device_signal_cb (GDBusProxy *proxy,
 			_e2_devkit_device_changed_cb (proxy, object_path, sessionptr);
 			break;
 		}
+		NEEDCLOSEBGL
 	}
 }
 #endif
@@ -1565,6 +1594,7 @@ gboolean e2_devkit_device_is_ejectable (const gchar *devpath)
 }
 /**
 @brief initialise connection to devicekit/udisks daemon, and log current removables
+Called with BGL open/off
 @return
 */
 void e2_devkit_init (void)
