@@ -1,6 +1,6 @@
-/* $Id: e2_fs.c 2454 2012-03-09 11:53:45Z tpgww $
+/* $Id: e2_fs.c 3062 2014-02-15 00:47:11Z tpgww $
 
-Copyright (C) 2005-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2005-2014 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -61,6 +61,17 @@ Not implemented yet.
 #include "e2_complete.h"
 #endif
 
+//some file-system identifiers (from <linux/magic.h>, which isn't generally available)
+#ifndef MSDOS_SUPER_MAGIC
+# define MSDOS_SUPER_MAGIC 0x4d44
+#endif
+#ifndef SMB_SUPER_MAGIC
+# define SMB_SUPER_MAGIC 0x517B
+#endif
+#ifndef USBDEVICE_SUPER_MAGIC
+# define USBDEVICE_SUPER_MAGIC 0x9fa2
+#endif
+
 typedef struct _E2_DRFuncArgs
 {
 	const gchar *localpath;	//absolute path of dir to process, localised string FIXME
@@ -179,7 +190,7 @@ NOTE probably this does not work properly on FAT*, which are case-insensitive
 gint e2_fs_access2 (VPATH *localpath E2_ERR_ARG())
 {
 	struct stat statbuf;
-//	if (e2_fs_mount_is_cased (view))
+//	if (e2_fs_path_is_cased (localpath, NULL))
 		return (e2_fs_lstat (localpath, &statbuf E2_ERR_SAMEARG()));
 /*	else
 	{
@@ -331,11 +342,12 @@ gboolean e2_fs_ingroup (gid_t gid)
 @brief execute the file(1) command on @a localpath, and search each line of its output for @a string
 Used in e2_fs_is_text, e2_fs_is_exec2, and e2_fs_is_executable
 If @a localpath is a symlink, it will be traversed.
+TODO error message assumes BGL closed.
 
 @param localpath localised name of file to test
 @param string ascii string for which to search, in the output from file command
 
-@return TRUE if @a string is found
+@return TRUE if @a string is found, FALSE if not found, or some sort of error happened
 */
 static gboolean _e2_fs_grep_file_output (VPATH *localpath, gchar *string E2_ERR_ARG())
 {
@@ -411,7 +423,12 @@ static gboolean _e2_fs_grep_file_output (VPATH *localpath, gchar *string E2_ERR_
 # else
 		gchar *buf = (gchar *) g_try_malloc ((gulong) bsize);
 # endif
-		CHECKALLOCATEDWARN (buf, return FALSE;);
+#if (CHECKALLOCATEDWARN)
+		CHECKALLOCATEDWARN (buf, return FALSE;); //TODO assumes BGL closed
+#else
+		if (buf == NULL)
+			return FALSE;
+#endif
 
 //tag E2_BADQUOTES
 		gchar *qp = e2_utils_quote_string (VPCSTR (localpath));
@@ -515,16 +532,16 @@ gboolean e2_fs_is_exec2 (VPATH *localpath E2_ERR_ARG())
 @brief test whether @a info belongs to an executable item
 If the item is a symlink, its target will be tested
 @param info pointer to a FileInfo data struct
-@param view pointer to view data struct from which to get path info for @a info
+@param infopath pointer to path data for @a info, with localised string
 
 @return TRUE if system reports x permission, and the file(1) command says it's an application file
 */
-gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view)
+gboolean e2_fs_is_executable (VPATH *infopath, FileInfo *info)
 {
 	gboolean retval = FALSE;
-	gchar *local = e2_utils_dircat (view, info->filename, TRUE);
+	gchar *local = g_build_filename (VPCSTR(infopath), info->filename, NULL);
 #ifdef E2_VFS
-	VPATH ddata =  { local, view->spacedata };
+	VPATH ddata =  { local, infopath->spacedata };
 	VPATH *localpath = &ddata;
 #else
 	VPATH *localpath = local;
@@ -576,11 +593,11 @@ gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view)
 This assumes that the item's mode flags have been gathered using lstat(),
 not stat() (i.e. no link look-through)
 @param info ptr to FileInfo struct for the item
-@param view pointer to view data struct from which to get path info for @a info
+@param infopath pointer to path info for @a info, with localised string
 
 @return TRUE if @a info belongs to a directory or a link to a directory
 */
-gboolean e2_fs_is_dir (FileInfo *info, ViewInfo *view)
+gboolean e2_fs_is_dir (VPATH *infopath, FileInfo *info)
 {
 	//CHECKME want to return any vfs error ?
 	if ( S_ISDIR (info->statbuf.st_mode) )
@@ -588,16 +605,17 @@ gboolean e2_fs_is_dir (FileInfo *info, ViewInfo *view)
 	if ( S_ISLNK(info->statbuf.st_mode) )
 	{
 		struct stat statbuf;
-		gchar *localpath = e2_utils_dircat (view, info->filename, TRUE);
+
+		gchar *localpath = g_build_filename (VPCSTR(infopath), info->filename, NULL);
 #ifdef E2_VFS
-		VPATH data = { localpath, view->spacedata };
-		gboolean ok = !e2_fs_stat (&data, &statbuf E2_ERR_NONE());
+		VPATH data = { localpath, infopath->spacedata };
+		gboolean found = !e2_fs_stat (&data, &statbuf E2_ERR_NONE());
 #else
-		gboolean ok = !e2_fs_stat (localpath, &statbuf E2_ERR_NONE());
+		gboolean found = !e2_fs_stat (localpath, &statbuf E2_ERR_NONE());
 #endif
 		g_free (localpath);
-		if (ok && S_ISDIR (statbuf.st_mode))
-		  return TRUE;
+		if (found && S_ISDIR (statbuf.st_mode))
+			return TRUE;
 	}
 	return FALSE;
 }
@@ -811,6 +829,7 @@ If a different path string is returned, the old one is freed, of course
 */
 gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG())
 {
+	gboolean sub = FALSE;
 #ifdef E2_VFSTMP
 	//FIXME relevant path to open ? walkup *utfpath, past its root if vdir,
 	//revert to local space if vfs fails
@@ -835,6 +854,7 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 			{
 				g_free (local);
 				local = g_strdup (g_get_home_dir ());
+				sub = TRUE;
 			}
 			else
 				if (local[1] == G_DIR_SEPARATOR)
@@ -842,6 +862,7 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 				freeme = local;
 				local = g_strconcat (g_get_home_dir (), local + sizeof(gchar), NULL);
 				g_free (freeme);
+				sub = TRUE;
 			}
 			else
 			{	//try to get some user's home
@@ -851,9 +872,18 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 					g_free (local);
 					local = D_FILENAME_TO_LOCALE (freeme);
 					g_free (freeme);
+					sub = TRUE;
 				}
 			}
-
+		}
+		else if (local[0] != G_DIR_SEPARATOR) //relative path check
+		{
+			p = g_get_current_dir ();
+			freeme = local;
+			local = e2_utils_translate_relative_path (p, freeme);
+			g_free (freeme);
+			g_free (p);
+			sub = TRUE;
 		}
 
 #ifdef E2_VFS
@@ -866,7 +896,13 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 			&& (!accessible || !e2_fs_access (local, R_OK | X_OK E2_ERR_NONE())))
 #endif
 		{
-			g_free (local);
+			if (sub)
+			{
+				g_free (*utfpath);
+				*utfpath = local;
+			}
+			else
+				g_free (local);
 			return TRUE;
 		}
 	}
@@ -1117,6 +1153,147 @@ retry:
 	g_free (localpath);
 	return TRUE;
 }
+/**
+@brief get enumerator describing whether @a localpath's file-system is case-sensitive
+
+MS-DOS/Windows FAT* are case-insensitive, but so-called "long" names are
+case-preserving.
+NTFS can be fully case sensitive or just case preserving.
+Classic Mac OS was case-insensitive but case-preserving. HFS+ on Mac OS X is
+probably the same. (its tag is "hfsplus") Mac HFSX is fully case senstive.
+Samba servers usually exhibit case-insensitive behaviour, regardless of the
+actual fs properties.
+For our purposes here, probably best to treat treat -preserving as -sensitive.
+
+@param localpath localised name of file/dir to test
+
+@return the value
+*/
+E2_FSSensitive e2_fs_path_case (VPATH *localpath)
+{
+	E2_FSSensitive case_sensitive_names;
+
+#ifdef E2_VFSTMP
+	if (localpath->spacedata == NULL)
+	{
+#endif
+		gchar *local = VPSTR(localpath);
+//=========== OS-specific stuff =============
+#if defined(__linux__)
+#include <sys/vfs.h>
+//#include <linux/magic.h> only in glibc-devel
+		struct statfs buf;
+	retry:
+		if (statfs (local, &buf))
+		{
+			if (errno == EINTR)
+				goto retry;
+			case_sensitive_names = E2FSCASE_NEVER;
+		}
+		else
+		{
+			switch (buf.f_type)
+			{
+				case MSDOS_SUPER_MAGIC:
+				case SMB_SUPER_MAGIC:
+					case_sensitive_names = E2FSCASE_ANY;
+					break;
+				default:
+					case_sensitive_names = E2FSCASE_SENSITIVE;
+					break;
+			}
+		}
+#elif defined(__E2BSD__)
+#include <sys/param.h>
+#include <sys/mount.h>
+		struct statfs buf;
+
+		if (statfs (local, &buf))
+		{
+			case_sensitive_names = E2FSCASE_NEVER;
+		}
+		else
+		{
+			gchar *type = buf.f_fstypename;
+			if (strcmp (type, "vfat") == 0)
+				case_sensitive_names = E2FSCASE_ANY;
+			else
+				case_sensitive_names = E2FSCASE_SENSITIVE;
+		}
+#elif defined(__solaris__) || defined(sco) || defined(hpux) || defined(__svr4__)
+#include <sys/types.h>
+#include <sys/statvfs.h>
+		struct statvfs buf;
+
+		if (statvfs (local, &buf))
+		{
+			case_sensitive_names = E2FSCASE_NEVER;
+		}
+		else
+		{
+			gchar *type = buf.f_basetype;
+			if (strcmp (type, "vfat") == 0)
+				case_sensitive_names = E2FSCASE_ANY;
+			else
+				case_sensitive_names = E2FSCASE_SENSITIVE;
+		}
+#else
+#include <sys/param.h>
+#include <sys/mount.h>
+		struct statfs buf;
+
+		if (statfs (local, &buf))
+		{
+			case_sensitive_names = E2FSCASE_NEVER;
+		}
+		else
+		{
+			gchar *type = buf.f_fstypename;
+			if (strcmp (type, "vfat") == 0)
+				case_sensitive_names = E2FSCASE_ANY;
+			else
+				case_sensitive_names = E2FSCASE_SENSITIVE;
+		}
+#endif	//which OS
+//=========================
+#ifdef E2_VFSTMP
+	}
+	else
+	{
+		figure it out for virtual dir
+	}
+#endif
+	return case_sensitive_names;
+}
+/**
+@brief test whether @a localpath's file-system is case-sensitive
+
+@param localpath localised name of file/dir to test
+
+@return TRUE if path/name is case-sensitive
+*/
+gboolean e2_fs_path_is_cased (VPATH *localpath)
+{
+	E2_FSSensitive cased = e2_fs_path_case (localpath);
+	return (cased == E2FSCASE_SENSITIVE ||
+			cased == E2FSCASE_SENSITIVENOW);
+}
+/**
+@brief check whether @a localpath exists
+
+@brief localpath localised path string
+
+@return 2 if it exists, 1 if cannot be sure (due to possible name-case-difference), 0 if not exists
+*/
+gint e2_fs_path_exists (VPATH *localpath)
+{
+	if (e2_fs_access2 (localpath E2_ERR_NONE()) == 0)
+	{
+		//BUT maybe we've only found a case-difference that looks the same to the kernel
+		return (e2_fs_path_is_cased (localpath)) ? 2 : 1;
+	}
+	return 0;
+}
 #ifdef E2_VFS
 /* *
 @brief thread-function to read all of a virtual directory
@@ -1180,32 +1357,33 @@ To eliminate BGL-racing, no UI-change from here
 } */
 #endif
 /**
-@brief synchronous or thread-function to read all of a mounted directory
-This is called synchronously when @a view ->fstype is FS_LOCAL, or as a thread
-when @a view ->fstype is FS_FUSE
-To eliminate BGL-racing, no BGL- or UI-change here
-@a view ->dir has the target path
-@param view ptr to data struct for the view to be updated
+@brief synchronous or thread-function to record some or all the items in a mounted directory
 
-@return list of FileInfo's for entries (maybe NULL), or error code if a problem occurred
+This is called synchronously when the fstype of the directory being processed is
+FS_LOCAL, or as a thread if the fstype is FS_FUSE, or the parent function was
+called with argument E2_DIRWATCH_YES.
+To eliminate BGL-racing, no BGL- or UI-change here
+
+@param data pointer to operation data detailing directory path, filterer etc
+
+@return GList of allocated names of items (maybe NULL) in the directory,
+ or pointerised error code if a problem occurred
 */
 static gpointer _e2_fs_read_mounted_dir (E2_DRFuncArgs *data)
 {
-/*	would be nice to prevent access-time updates purely due to
-	refreshing, but the atime really does change as a result
-	of the refresh process, and if we were to revert it, the
-	ctime would be changed instead
-	times are immediately shown in the other pane if it's the
-	parent of this one
-	if we could suspend monitoring of such parent dir, we
-	must not lose any prior reports for there
-	QUERY can the changed atime itself trigger a report which
-	causes a subsequent refresh, creating an endless cycle?
-	especially with multiple threads and/or processors ?
-	Assuming it can do so, we clear the reports 'queue'
-	after the dir is opened
-	Would be nice if we could just clear any report(s) due to
-	the refresh per se - but the reporting is not that detailed */
+/*	Would be nice to prevent access-time updates purely due to refreshing, but
+	the atime really does change as a result of the refresh process, and if we
+	were to revert it, the ctime would be changed instead
+	Times are immediately shown in the other pane if it's the parent of this one
+	If we could suspend monitoring of such parent dir, we must not lose any
+	prior reports for there
+	QUERY can the changed atime itself trigger a report which causes a subsequent
+	refresh, creating an endless cycle?
+	Especially with multiple threads and/or processors ?
+	Assuming it can do so, we clear the reports 'queue' after the dir is opened
+	Would be nice if we could just clear any report(s) due to the refresh per se
+	- but the reporting is not that detailed
+*/
 
 	/*#ifdef E2_FAM_KERNEL
 	if (view->refresh)
@@ -1314,6 +1492,7 @@ static void _e2_fs_slowread_response_cb (GtkDialog *dialog, gint response,
 {
 	pthread_t ID;
 	GtkWidget *wid;
+	CLOSEBGL
 	switch (response)
 	{
 		case GTK_RESPONSE_NO:	//abort the operation
@@ -1354,6 +1533,7 @@ static void _e2_fs_slowread_response_cb (GtkDialog *dialog, gint response,
 //			pthread_mutex_unlock (&?_mutex);
 			break;
 	}
+	OPENBGL
 }
 /**
 @brief thread function to manage timeout when reading a directory
@@ -1392,11 +1572,11 @@ static gpointer _e2_fs_progress_monitor (E2_DRead *data)
 #endif
 		if (data->dialog == NULL) //once-only, create the dialog
 		{
-			gdk_threads_enter ();
+			CLOSEBGL
 			data->dialog = e2_dialog_slow (_("Reading directory data"),
 				_("directory read"),
-			(ResponseFunc)_e2_fs_slowread_response_cb, data);
-			gdk_threads_leave ();
+				(ResponseFunc)_e2_fs_slowread_response_cb, data);
+			OPENBGL
 		}
 
 		if (data->dialog != NULL &&
@@ -1406,11 +1586,11 @@ static gpointer _e2_fs_progress_monitor (E2_DRead *data)
 			!GTK_WIDGET_VISIBLE (data->dialog))
 #endif
 		{
-			gdk_threads_enter ();
+			CLOSEBGL
 			gtk_widget_show (data->dialog);
 			gtk_window_present (GTK_WINDOW (data->dialog));
 //			WAIT_FOR_EVENTS;
-			gdk_threads_leave ();
+			OPENBGL
 			WAIT_FOR_EVENTS_UNLOCKED_SLOWLY;
 			printd (DEBUG,"dir-monitor-thread dialog presented");
 		}
@@ -1425,19 +1605,22 @@ static gpointer _e2_fs_progress_monitor (E2_DRead *data)
 }
 /**
 @brief create a list of some or almost all items in the directory @a localpath
+
 Items in the directory can be passed to @a filterfunc to determine whether the
 respective item should be listed. Otherwise, all items other than "." are listed.
 CWD is temporarily changed to the directory, so threads cannot assume that CWD
 always matches curr_view->dir or whatever for non-local dirs.
 If the read is lengthy, a downstream dialog will be shown. That locks gdk threads mutex ?
 This assumes BGL is open. Any @a filterfunc is called with BGL open
+
 @param localpath absolute path of dir to process, localised string
 @param monitor enumerator signalling how to handle threaded dir reading
 @param filterfunc function to call for each processed item, or NULL to append each item's name to returned list
 @param cb_data pointer to data to send to @a filterfunc
 @param free_data_func function to call at end of scan, to clean @a cb_data, or NULL
 
-@return list of FileInfo's for entries (maybe NULL), or error code if a problem occurred
+@return GList of allocated names (maybe NULL) for matching items in @a localpath,
+ or pointerised error code if a problem occurred
 */
 gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 	gpointer filterfunc, gpointer cb_data, GDestroyNotify free_data_func E2_ERR_ARG())
@@ -1519,15 +1702,15 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 		pthread_attr_destroy (&attr);
 
 		e2_window_clear_status_message ();	//no BGL management neeeded
-		WAIT_FOR_EVENTS_UNLOCKED;
+		EXTRA_WAIT_FOR_EVENTS_UNLOCKED;
 		//some race-minimisation here
 		GtkWidget *wid = data.dialog;
 		data.dialog = NULL;
 		if (GTK_IS_WIDGET (wid))
 		{
-			gdk_threads_enter ();
+			CLOSEBGL
 			gtk_widget_destroy (wid);
-			gdk_threads_leave ();
+			OPENBGL
 		}
 		ID = data.mid;
 		data.mid = 0;
@@ -1802,7 +1985,7 @@ E2_FILE *e2_fs_file_open (VPATH *localpath, const gchar *how, GError **E2_ERR_NA
 	E2_FILE *result;
 	if (e2_fs_item_is_mounted (localpath))
 	{
-		result = fopen (VPCSTR(localpath), how);
+		result = e2_fs_open_stream (VPCSTR(localpath), how);
 		if (result != NULL)
 			e2_fs_set_error_from_errno (E2_ERR_NAME);
 		return result;
@@ -2352,7 +2535,7 @@ ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize
 Note the file needs to be size-limited, for local files at least, to what can be
 represented by a gulong, i.e. to ULONG_MAX. This is to allow allocation by g_...,
 to simplify later cleanups (not supposed to mix glib and glibc mamaory-management)
-Memory is allocated by g_malloc, not malloc, and needs to be cleared by g_free, not free
+Memory is allocated by g_try_malloc, not malloc, and needs to be cleared by g_free, not free
 Error messages expect BGL open/off
 @param localpath path of file to be read, localised string
 @param contents store for ptr start of the allocated contents
@@ -2362,7 +2545,7 @@ Error messages expect BGL open/off
 @return TRUE if @a localpath was read successfully
 */
 gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
-	/*ssize_t*/ gulong *contlength, gboolean terminate E2_ERR_ARG())
+	/*size_t*/gulong *contlength, gboolean terminate E2_ERR_ARG())
 {
 	gboolean retval;
 	const gchar *fmt;
@@ -2382,16 +2565,22 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 
 //				*contents = malloc (len);	//not size-limited, but imposes cleanup hassles
 				*contents = g_try_malloc (len); //allows up to ULONG_MAX
+#if (CHECKALLOCATEDWARNT)
+				CHECKALLOCATEDWARNT (*contents, e2_fs_safeclose (fdesc); return FALSE;);
+#else
 				if (*contents == NULL)
 				{
+					CLOSEBGL
 					e2_utils_show_memory_message ();
-#ifdef E2_VFSTMP
+					OPENBGL
+# ifdef E2_VFSTMP
 //					*E2_ERR_NAME = g_error_new_literal (G_FILE_ERROR, errno, g_strerror (errno));
 //					E2_ERR_CLEARBACKUP (localerr);
-#endif
+# endif
 					e2_fs_safeclose (fdesc);	//bypass the normal cleanup process
 					return FALSE;
 				}
+#endif
 
 				retval = TRUE;
 				fmt = NULL;
@@ -2473,6 +2662,63 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 	}
 	return retval;
 }
+
+/* *
+@brief read the start of file @a localpath into memory
+Error messages expect BGL open/off
+@param localpath path of file to be read, localised string
+@param contents store for ptr to allocated buffer
+@param bufsize length of buffer, max. amount of the file to be read
+@param terminate TRUE to add a trailing 0 if that's not present in file as read
+
+@return TRUE if @a localpath was read successfully
+*/
+/*gboolean e2_fs_get_file_start (VPATH *localpath, gpointer *contents,
+	/ *size_t* /gulong bufsize, gboolean terminate E2_ERR_ARG())
+{
+#ifdef USE_GLIB2_10
+	*contents = g_slice_alloc (bufsize);
+#else
+	*contents = g_try_malloc (bufsize);
+#endif
+#if (CHECKALLOCATEDWARNT)
+	CHECKALLOCATEDWARNT (*contents, return FALSE;);
+#else
+	if (*contents == NULL)
+		return FALSE;
+#endif
+
+#ifdef VFSTMP
+	TODO read start of non-local file
+#else
+	gint fdesc = e2_fs_safeopen (VPCSTR(localpath), O_RDONLY, 0);
+	if (fdesc >= 0)
+	{
+		ssize_t length = e2_fs_read (fdesc, *contents, bufsize E2_ERR_PTR());
+		e2_fs_safeclose (fdesc);
+		if (length >= 0)
+		{
+			if (terminate)
+			{
+				if (length < bufsize)
+					*((gchar*)*contents + length) = '\0';
+				else
+					*((gchar *)*contents + bufsize - 1) = '\0';
+			}
+			return TRUE;
+		}
+	}
+#endif
+
+#ifdef USE_GLIB2_10
+	g_slice_free1 (bufsize, *contents);
+#else
+	g_free (*contents);
+#endif
+	*contents = NULL;
+	return FALSE;
+}
+*/
 /**
 @brief write from buffer to file
 
@@ -2684,9 +2930,9 @@ gboolean e2_fs_copy_file (VPATH *src, const struct stat *src_sb,
 		{
 			if (buf_size < dest_sb.st_blksize)
 			{
-				gdk_threads_enter ();
+				CLOSEBGL
 				e2_utils_show_memory_message ();
-				gdk_threads_leave ();
+				OPENBGL
 				TEMP_FAILURE_RETRY (close (src_desc));
 				TEMP_FAILURE_RETRY (close (dest_desc));
 				return FALSE;
@@ -2857,62 +3103,6 @@ gboolean e2_fs_get_command_output (gchar *command, gpointer *output)
 	}
 	return (total > 0);
 }
-/**
-@brief read up to @a count -1 bytes of piped command output into newly-allocated memory
-A terminating 0 is appended after a successful read
-@param command localised command string to be executed
-@param contents store for address of read contents
-@param count size of block to read + 1 (for added 0)
-
-@return TRUE if @a command result was read successfully
-*/
-gboolean e2_fs_sniff_command_output (gchar *command, gpointer *contents, gulong count)
-{
-	gboolean retval = FALSE;
-	E2_FILE *pipe = e2_fs_open_pipe (command);
-	if (pipe != NULL)
-	{
-#ifdef USE_GLIB2_10
-		*contents = g_slice_alloc (count);
-#else
-		*contents = g_try_malloc (count);
-#endif
-		if (*contents != NULL)
-		{
-			ssize_t bytes_read, total = 0;
-reread:
-			errno = 0;
-			bytes_read = fread ((*contents + total), 1, (size_t)(count - total - 1), pipe);
-			if (bytes_read >= 0)
-			{
-				total += bytes_read;
-				if (total == count - 1) //got as much as we want
-					retval = TRUE;
-				else
-					goto reread;
-			}
-			else if (feof (pipe))
-			{
-				if (total > 0)
-					retval = TRUE;
-			}
-			else if (!ferror (pipe) || errno == EAGAIN || errno == EINTR)
-				goto reread;
-
-			if (retval)
-				*(*((gchar**)contents) + total) = '\0';
-			else
-				//FIXME warn user upon error
-#ifdef USE_GLIB2_10
-				g_slice_free1 (count, *contents);
-#else
-				g_free (*contents);
-#endif
-		}
-		e2_fs_pipe_close (pipe);
-	}
-	return retval;
-}
 /* *
 @brief blockwize read from stdin into memory
 A terminating 0 is always added
@@ -2938,8 +3128,12 @@ gboolean e2_fs_read_stdin (gchar **contents)
 	size_t total = 0;
 	size_t bsize = 1024;	//block size
 	gchar *buf = (gchar *) g_try_malloc ((gulong) bsize);
+#if (CHECKALLOCATEDWARN)
+	CHECKALLOCATEDWARN (buf, return FALSE;);
+#else
 	if (buf == NULL)
 		return FALSE;
+#endif
 	gchar *store = buf;
 	ssize_t bytes_read;
 	while (feof (stdin) == 0)
@@ -3117,16 +3311,17 @@ gboolean e2_fs_error (gchar *msg
 #endif
 	)
 {
-	gchar *long_msg = g_strconcat (msg, " - ",
+	gchar *long_msg;
 #ifdef E2_VFS
-		reason
+	long_msg = (reason != NULL) ?
+		g_strconcat (msg, " - ", reason, NULL) : g_strdup (msg);
 #else
-		(gchar *) g_strerror (errno)
+	long_msg = (errno != 0) ?
+		g_strconcat (msg, " - ", (gchar *) g_strerror (errno), NULL) : g_strdup (msg);
 #endif
-	, NULL);
-	gdk_threads_enter ();
+	CLOSEBGL
 	e2_output_print_error (long_msg, TRUE);
-	gdk_threads_leave ();
+	OPENBGL
 	return FALSE;
 }
 /**
@@ -3169,9 +3364,9 @@ gboolean e2_fs_error_simple (const gchar *format, VPATH *local)
 	gchar *utf = F_DISPLAYNAME_FROM_LOCALE (VPSTR(local));
 	//no need to escape any pango-annoying component of the itemname
 	gchar *msg = g_strdup_printf (format, utf);
-	gdk_threads_enter ();
+	CLOSEBGL
 	e2_output_print_error (msg, TRUE);
-	gdk_threads_leave ();
+	OPENBGL
 	F_FREE (utf, VPSTR(local));
 	return FALSE;
 }
@@ -3190,7 +3385,7 @@ void e2_fs_set_error_from_errno (GError **error)
 	{
 		g_clear_error (error);
 //		GFileError err_no = g_file_error_from_errno (errno);
-//		*error = g_error_new_literal (G_FILE_ERROR, err_no, g_str (errno));
+//		*error = g_error_new_literal (G_FILE_ERROR, err_no, g_strerror (errno));
 		*error = g_error_new_literal (G_FILE_ERROR, errno, g_strerror (errno));
 	}
 }
@@ -3248,7 +3443,7 @@ gboolean e2_fs_vfsfunc_ready (gpointer *func)
 	if (!vfs.loaded)
 	{
 		//e2_window_set_cursor (GDK_WATCH); may freeze, probably unnecessary
-		if (!e2_plugins_load_plugin ("vfs"VERSION))	//this will set the function ptr if it succeeds
+		if (!e2_plugins_get_plugin ("vfs"VERSION))	//this will set the function ptr if it succeeds
 			return FALSE;
 		//e2_window_set_cursor (GDK_LEFT_PTR);
 	}
