@@ -1,6 +1,6 @@
-/* $Id: e2_fs.h 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_fs.h 2902 2013-10-31 22:31:52Z tpgww $
 
-Copyright (C) 2005-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2005-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -51,11 +51,17 @@ gboolean e2_fs_recurse_mkdir (VPATH *localpath, gint mode, GError **error);
 gboolean e2_fs_chdir (gchar *path, GError **error);
 gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible, GError **error);
 gboolean e2_fs_cd_isok (gchar *path, GError **error);
+gboolean e2_fs_path_is_cased (VPATH *localpath);
+E2_FSSensitive uint e2_fs_path_case (VPATH *localpath);
+gint e2_fs_path_exists (VPATH *localpath);
+
 //FIXME make this work for vfs
 ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize,
 	GError **error);
 gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
-	/*ssize_t*/ gulong *contlength,	gboolean string, GError **error);
+	/*size_t*/gulong *contlength, gboolean terminate, GError **error);
+//gboolean e2_fs_get_file_start (VPATH *localpath, gpointer *contents,
+//	/*size_t*/gulong bufsize, gboolean terminate, GError **error);
 ssize_t e2_fs_write (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize,
 	GError **error);
 gboolean e2_fs_set_file_contents (VPATH *localpath, gpointer contents,
@@ -103,10 +109,15 @@ gboolean e2_fs_recurse_mkdir (VPATH *localpath, gint mode);
 gboolean e2_fs_chdir (gchar *path);
 gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible);
 gboolean e2_fs_cd_isok (gchar *path);
+gboolean e2_fs_path_is_cased (VPATH *localpath);
+E2_FSSensitive e2_fs_path_case (VPATH *localpath);
+gint e2_fs_path_exists (VPATH *localpath);
 
 ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize);
 gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
-	/*ssize_t*/ gulong *contlength, gboolean string);
+	/*size_t*/gulong *contlength, gboolean terminate);
+//gboolean e2_fs_get_file_start (VPATH *localpath, gpointer *contents,
+//	/*size_t*/gulong bufsize, gboolean terminate);
 ssize_t e2_fs_write (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize);
 gboolean e2_fs_set_file_contents (VPATH *localpath, gpointer contents,
 	size_t contlength, mode_t mode);
@@ -155,7 +166,6 @@ gboolean e2_fs_error_local (const gchar *format, VPATH *local
 gboolean e2_fs_error_simple (const gchar *format, VPATH *local);
 
 void e2_fs_check_coding (void);
-gchar *e2_fs_get_localdir (ViewInfo *view);
 gboolean e2_fs_ingroup (gid_t gid);
 //formatted file write, used locally for config and cache writing
 #define e2_fs_file_write fprintf
@@ -163,12 +173,12 @@ gboolean e2_fs_ingroup (gid_t gid);
 #define e2_fs_pipe_close pclose
 E2_FILE *e2_fs_open_pipe (gchar *command);	//always native
 
-gchar *e2_fs_get_perm_string (mode_t mode);	//string utility only
+gchar *e2_fs_get_perm_string (mode_t mode) G_GNUC_MALLOC;	//string utility only
 gboolean e2_fs_dir_is_native (const gchar *utfpath, gboolean descend);	//native comparisons only
 gboolean e2_fs_complete_dir (GtkWidget *entry, guint keyval, guint pane);
 
-gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view);
-gboolean e2_fs_is_dir (FileInfo *info, ViewInfo *view);
+gboolean e2_fs_is_executable (VPATH *infopath, FileInfo *info);
+gboolean e2_fs_is_dir (VPATH *infopath, FileInfo *info);
 
 //these flags are not standard, their position is > those used in S_IFMT (0170000)
 //FIXME generalise this
@@ -199,8 +209,6 @@ items of interest ...
 #define e2_fs_is_dir_fast(a) *(a+strlen(a)-sizeof(gchar))==G_DIR_SEPARATOR
 
 gboolean e2_fs_get_command_output (gchar *command, gpointer *contents);	//always native
-gboolean e2_fs_sniff_command_output (gchar *command, gpointer *contents,
-	gulong count);	//always native
 
 gint e2_fs_safeopen (const gchar *localpath, gint openflags, mode_t mode);
 gint e2_fs_safeclose (gint file_desc);
@@ -302,17 +310,16 @@ gboolean e2_fs_tw (VPATH *start_item, E2_TwResult (*callback) (),
 //a limit on level of nested dirs that will be processed in a treewalk
 #define E2_DIRNEST_LIMIT 32
 
-gchar *e2_fs_mount_get_enclosing_point (const gchar *localpath);
+gchar *e2_fs_mount_get_enclosing_point (const gchar *localpath) G_GNUC_MALLOC;
 #ifdef E2_HAL
-gchar *e2_fs_mount_get_mountpoint_for_device (const gchar *device);
+gchar *e2_fs_mount_get_mountpoint_for_device (const gchar *device) G_GNUC_MALLOC;
 #endif
 gboolean e2_fs_mount_is_mountpoint (VPATH *localpath);
 gboolean e2_fs_mount_is_removable (const gchar *utfpath);
 gboolean e2_fs_mount_is_ejectable (const gchar *utfpath);
-gboolean e2_fs_mount_is_cased (ViewInfo *view);
-GList *e2_fs_mount_get_mounts_list (void);	//gboolean permcheck)
-GList *e2_fs_mount_get_mountable_list (void);	//gboolean permcheck)
-GList *e2_fs_mount_get_fusemounts_list (void);
+GList *e2_fs_mount_get_mounts_list (void) G_GNUC_MALLOC;	//gboolean permcheck)
+GList *e2_fs_mount_get_mountable_list (void) G_GNUC_MALLOC;	//gboolean permcheck)
+GList *e2_fs_mount_get_fusemounts_list (void) G_GNUC_MALLOC;
 gboolean e2_fs_mount_is_fusepoint (VPATH *localpath);
 #ifdef E2_FS_MOUNTABLE
 void e2_fs_mount_actions_register (void);
