@@ -1,6 +1,6 @@
-/* $Id: e2_fs_mount.c 2309 2011-01-17 00:12:22Z tpgww $
+/* $Id: e2_fs_mount.c 2693 2013-08-25 04:38:17Z tpgww $
 
-Copyright (C) 2007-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2007-2013 tooar <tooar@emelfm2.net>
 Copyright (C) 2004-2007 Florian Zaehringer <flo.zaehringer@web.de>
 Copyright (C) 1999-2004 Bill Wilson <bill@gkrellm.net>
 
@@ -34,17 +34,6 @@ Functions dealing with fstab / mtab have in part been sourced from gkrellm
 #include <string.h>
 #include <unistd.h>
 
-//some file-system identifiers (from <linux/magic.h>, which isn't generally available)
-#ifndef MSDOS_SUPER_MAGIC
-# define MSDOS_SUPER_MAGIC 0x4d44
-#endif
-#ifndef SMB_SUPER_MAGIC
-# define SMB_SUPER_MAGIC 0x517B
-#endif
-#ifndef USBDEVICE_SUPER_MAGIC
-# define USBDEVICE_SUPER_MAGIC 0x9fa2
-#endif
-
 //in general, we support [u]mount completion
 #define E2_FS_MOUNTABLE
 // OSX 10.4 ("Tiger") automounts all devices. Unlike its predecessors, it uses statvfs()
@@ -105,34 +94,85 @@ Solaris  statvfs() with sys/statvfs.h */
 */
 # endif	//which os-specifis includes
 
-//test which mounted devices should be included in the list
-//of unmountable ones
-//CHECKME does this need to be os-specific ? if so, how ?
-# if defined(__linux__) || defined(__E2BSD__)
-#  define E2_TEST_MOUNTED_DEVICE \
-		if (   strcmp (dev, "none") \
-			&& strcmp (dev, "/dev") \
-			&& strcmp (type, "proc") \
-			&& strcmp (type, "sysfs") \
-			&& strcmp (type, "fuse") \
-			&& strcmp (dir, G_DIR_SEPARATOR_S)) /*no need to support umounting '/' */ \
+//finesse mounted device name before inclusion in the list of unmountable devices
+# define E2_PROCESS_MOUNTED_DEVICE
+//strip any trailing / from the mountpoint name
+//			s = strrchr (dir, G_DIR_SEPARATOR);
+//			if (s != NULL && s != dir && *(s+1) == '\0')
+//				*s = '\0';
 
-# else
-#  define E2_TEST_MOUNTED_DEVICE \
-		if (   strcmp (dev, "none") \
-			&& strcmp (dev, "/dev") \
-			&& strcmp (type, "sysfs") \
-			&& strcmp (type, "proc") \
-			&& strcmp (dir, G_DIR_SEPARATOR_S)) /*no need to support umounting '/' */ \
+//finesse unmounted device name before inclusion in the list of mountable devices
+# define E2_PROCESS_UNMOUNTED_DEVICE
+			//strip any trailing / from the mountpoint name
+//			s = strrchr (dir, G_DIR_SEPARATOR);
+//			if (s != NULL && s != dir && *(s+1) == '\0')
+//				*s = '\0';
 
-# endif
-/*	these should be handled by dev="none"
-			&& strcmp (type, "devpts")
-			&& strcmp (type, "proc")
-			&& strcmp (type, "usbdevfs")
-			&& strcmp (type, "usbfs")
-			&& strcmp (type, "sysfs")
-			&& strstr (dir, "/dev/") */
+/**
+@brief test whether a device should be included in the list of unmountable ones
+@param dev
+@param type
+@param dir
+
+@return TRUE if so
+*/
+static gboolean _e2_fs_mount_is_unmountable (const gchar *dev, const gchar *type, const gchar *dir)
+{
+#if defined(__linux__) || defined(__E2BSD__)
+	const gchar *devices[] = { "none" };
+	const gchar *types[] = { "proc", "dev", "sysfs", "tmpfs", "nfsd", "fuse", "fusectl", "fuse.bindfs"};
+	const gchar *dirs[] = { G_DIR_SEPARATOR_S, "/sys", "/proc" };
+/*	these should be covered by device "none"
+	type "devpts"
+	type "proc"
+	type "usbdevfs"
+	type "usbfs"
+	type "sysfs"
+	dir "/dev/"
+*/
+#else
+	//CHECKME
+	const gchar *devices = { "none" };
+	const gchar *types = { "proc", "sysfs", "tmpfs", "nfsd" };
+	const gchar *dirs = { G_DIR_SEPARATOR_S };
+#endif
+	gint i, j;
+	j = sizeof (devices) / sizeof (gchar*);
+	for (i = 0; i < j; i++)
+	{
+		if (strncmp (dev, devices[i], strlen(devices[i])) == 0)
+		{
+			printd (DEBUG, "device %s not unmountable", dev);
+			return FALSE;
+		}
+	}
+	j = sizeof (types) / sizeof (gchar*);
+	for (i = 0; i < j; i++)
+	{
+		if (strncmp (type, types[i], strlen(types[i])) == 0)
+		{
+			printd (DEBUG, "type %s not unmountable", type);
+			return FALSE;
+		}
+	}
+	j = sizeof (dirs) / sizeof (gchar*);
+	for (i = 0; i < j; i++)
+	{
+		if (i == 0)
+		{
+			if (strcmp (dir, dirs[0]) == 0)
+			{
+				printd (DEBUG, "dir %s not unmountable", dir);
+				return FALSE;
+			}
+		}
+		else if (strncmp (dir, dirs[i], strlen(dirs[i])) == 0)
+		{
+			printd (DEBUG, "dir %s not unmountable", dir);
+			return FALSE;
+		}
+	}
+/* there may be things blocking an unmount eg monitoring with dnotify */
 /* permission-checking disabled pending doing it properly, if not forever
 			gchar *opt = mounted->mnt_opts;
 			if (!permcheck || _e2_complete_mount_permission (dir, opt)) */
@@ -140,55 +180,85 @@ Solaris  statvfs() with sys/statvfs.h */
 		mounted[i].f_flags	//mount flags
 		mounted[i].f_owner	//owner
 */
-/* there may be things blocking an unmount eg monitoring with dnotify */
+	printd (DEBUG, "dev %s, type %s, dir %s IS UNmountable", dev, type, dir);
+	return TRUE;
+}
 
-//finesse mounted device name before inclusion in the list of
-//unmountable devices
-# define E2_PROCESS_MOUNTED_DEVICE
-//strip any trailing / from the mountpoint name
-//			s = strrchr (dir, G_DIR_SEPARATOR);
-//			if (s != NULL && s != dir && *(s+1) == '\0')
-//				*s = '\0';
+/**
+@brief test whether a device should be included in the list of mountable ones
+@param dev
+@param type
+@param dir
 
-//test which unmounted devices should be included in the list
-//of mountable ones
+@return TRUE if so
+*/
+static gboolean _e2_fs_mount_is_mountable (const gchar *dev, const gchar *type, const gchar *dir)
+{
 //CHECKME does this need to be os-specific ? if so, how ?
-# define E2_TEST_UNMOUNTED_DEVICE \
-		if (   strcmp (dev, "none") \
-			&& strcmp (type, "swap") \
-			&& strcmp (type, "proc") \
-			&& strcmp (type, "sysfs") \
-			&& strcmp (type, "ignore") \
-			&& strcmp (dir, G_DIR_SEPARATOR_S)	/*no need to support mounting '/' */ \
-			&& type[0] != '\0' )
+	const gchar *devices[] = { "none" };
+	const gchar *types[] = { "swap", "proc", "sysfs", "tmpfs", "nfsd", "ignore", "", "fuse", "fusectl", "fuse.bindfs" };
+	const gchar *dirs[] = { G_DIR_SEPARATOR_S, "/boot" };
+	gint i, j;
+	j = sizeof (devices) / sizeof (gchar*);
+	for (i = 0; i < j; i++)
+	{
+		if (strncmp (dev, devices[i], strlen(devices[i])) == 0)
+		{
+			printd (DEBUG, "device %s not mountable", dev);
+			return FALSE;
+		}
+	}
+	j = sizeof (types) / sizeof (gchar*);
+	for (i = 0; i < j; i++)
+	{
+		if (*types[i] != '\0')
+		{
+			if (strncmp (type, types[i], strlen(types[i])) == 0)
+			{
+				printd (DEBUG, "type %s not mountable", type);
+				return FALSE;
+			}
+		}
+		else if (*dev == '\0') return FALSE;
+	}
+	j = sizeof (dirs) / sizeof (gchar*);
+	for (i = 0; i < j; i++)
+	{
+		if (i == 0)
+		{
+			if (strcmp (dir, dirs[0]) == 0)
+			{
+				printd (DEBUG, "dir %s not unmountable", dir);
+				return FALSE;
+			}
+		}
+		else if (strncmp (dir, dirs[i], strlen(dirs[i])) == 0)
+		{
+			printd (DEBUG, "dir %s not mountable", dir);
+			return FALSE;
+		}
+	}
 /*
-//			&& g_file_test (dir, G_FILE_TEST_IS_DIR) don't bother checking for this (slow)
-    these should be handled by dev="none"
-			&& strcmp (type, "devpts")
-			&& strcmp (type, "proc")
-			&& strcmp (type, "sysfs")
-			&& strcmp (type, "usbdevfs")
-			&& strcmp (type, "usbfs")
-			&& strstr (dir, "/dev/") == NULL
+	g_file_test (dir, G_FILE_TEST_IS_DIR) don't bother checking for this (slow)
+    these should be handled by dev "none"
+	type "devpts"
+	type "proc"
+	type "sysfs"
+	type "usbdevfs"
+	type "usbfs"
+	dir "/dev/"
 CHECKME sometimes, test may need to include
 	strncmp (dev, "/dev", 4)
 	realpath (dev, realdev) != NULL
 	gchar realdev[PATH_MAX];
 */
 /* permission-checking disabled pending doing it properly, if not forever
-			gchar *opt = fs.vfs_mntopts;
-			if (!permcheck || _e2_complete_mount_permission (dir, opt)) */
-
-//finesse unmounted device name before inclusion in the list of
-//mountable devices
-# define E2_PROCESS_UNMOUNTED_DEVICE
-			//strip any trailing / from the mountpoint name
-//			s = strrchr (dir, G_DIR_SEPARATOR);
-//			if (s != NULL && s != dir && *(s+1) == '\0')
-//				*s = '\0';
-
-//guint uid;
-//time_t fstab_mtime;
+	gchar *opt = fs.vfs_mntopts;
+	if (!permcheck || _e2_complete_mount_permission (dir, opt))
+*/
+	printd (DEBUG, "dev %s, type %s, dir %s IS mountable", dev, type, dir);
+	return TRUE;
+}
 
 #ifdef E2_HAL
 /**
@@ -796,11 +866,10 @@ static gboolean _e2_fs_mount_permission (gchar* device, gchar *options)
 } */
 
 /**
-@brief create list of mounted partitions (other than "/") in the native filesystem
+@brief create list of mounted partitions (with some exception) in the native filesystem
 
 The list is created by the appropriate os-specifc protocol.
-Root dir is exlcluded because we don't need to ever unmount that.
-@return list of mounted partitions (UTF-8 strings), or NULL
+@return list of partitions (UTF-8 strings), or NULL
 */
 GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 {
@@ -821,9 +890,9 @@ GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 		//check the mounted device meets our needs
 		//need to take a copy if string is altered
 		gchar *dev = mounted->mnt_fsname;
-		gchar *dir = mounted->mnt_dir;
 		gchar *type = mounted->mnt_type;
-		E2_TEST_MOUNTED_DEVICE
+		gchar *dir = mounted->mnt_dir;
+		if (_e2_fs_mount_is_unmountable (dev, type, dir))
 		{
 			E2_PROCESS_MOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &mounts_list);
@@ -841,9 +910,9 @@ GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 		//check the mounted device meets our needs
 		//need to take a copy if string is altered
 		gchar *dev = mounted[i].f_mntfromname;
-		gchar *dir = mounted[i].f_mntonname;
 		gchar *type = mounted[i].f_fstypename;
-		E2_TEST_MOUNTED_DEVICE
+		gchar *dir = mounted[i].f_mntonname;
+		if (_e2_fs_mount_is_unmountable (dev, type, dir))
 		{
 			E2_PROCESS_MOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &mounts_list);
@@ -862,9 +931,9 @@ GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 		//check the mounted device meets our needs
 		//need to take a copy if string is altered
 		gchar *dev = mounted.mnt_special;
+		gchar *type = " "; //TODO mounted.mnt_fstype;
 		gchar *dir = mounted.mnt_mountp;
-//		gchar *type = mounted.mnt_fstype;
-		E2_TEST_MOUNTED_DEVICE
+		if (_e2_fs_mount_is_unmountable (dev, type, dir))
 		{
 			E2_PROCESS_MOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &mounts_list);
@@ -885,9 +954,9 @@ GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 		//check the mounted device meets our needs
 		//need to take a copy if string is altered
 		gchar *dev = mounted->mnt_fsname;
+		gchar *type = " "; //TODO mounted->mnt_type;
 		gchar *dir = mounted->mnt_dir;
-//		gchar *type = mounted->mnt_type;
-		E2_TEST_MOUNTED_DEVICE
+		if (_e2_fs_mount_is_unmountable (dev, type, dir))
 		{
 			E2_PROCESS_MOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &mounts_list);
@@ -904,9 +973,9 @@ GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 	while (getmntent (f, &mounted) ==0)
 	{
 		gchar *dev = mounted.f_mntfromname;
+		gchar *type = " "; //TODO mounted.f_fstypename;
 		gchar *dir = mounted.f_mntonname;
-//		gchar *type = mounted.f_fstypename;
-		E2_TEST_MOUNTED_DEVICE
+		if (_e2_fs_mount_is_unmountable (dev, type, dir))
 		{
 			E2_PROCESS_MOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &mounts_list);
@@ -927,9 +996,9 @@ GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 		//check the mounted device meets our needs
 		//need to take a copy if string is altered
 		gchar *dev = mounted->mnt_fsname;
+		gchar *type = " "; //TODO mounted->mnt_type;
 		gchar *dir = mounted->mnt_dir;
-//		gchar *type = mounted->mnt_type;
-		E2_TEST_MOUNTED_DEVICE
+		if (_e2_fs_mount_is_unmountable (dev, type, dir))
 		{
 			E2_PROCESS_MOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &mounts_list);
@@ -960,16 +1029,17 @@ GList *e2_fs_mount_get_mountable_list (void)	//gboolean permcheck)
 	while ((fs = getfsent ()) != NULL)
 	{
 		gchar *dev = fs->fs_spec;
-		gchar *dir = fs->fs_file;
 		gchar *type = fs->fs_vfstype;
+		gchar *dir = fs->fs_file;
 /*		copy strings if they need to be modified
 		g_strlcpy (dev, fs->fs_spec, sizeof(dev));
 		g_strlcpy (dir, fs->fs_file, sizeof(dir));
 		g_strlcpy (type, fs->fs_vfstype, sizeof(type));
 		_e2_complete_mount_fix_fstab_name (dev);
 		_e2_complete_mount_fix_fstab_name (dir);
-		_e2_complete_mount_fix_fstab_name (type); */
-		E2_TEST_UNMOUNTED_DEVICE
+		_e2_complete_mount_fix_fstab_name (type);
+ */
+		if (_e2_fs_mount_is_mountable (dev, type, dir))
 		{
 			E2_PROCESS_UNMOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &fstab_list);
@@ -987,9 +1057,9 @@ GList *e2_fs_mount_get_mountable_list (void)	//gboolean permcheck)
 	while (getvfsent (f, &fs) == 0)
 	{
 		gchar *dev = fs.vfs_special;
+		gchar *type = " "; //TODO fs.vfs_fstype;
 		gchar *dir = fs.vfs_mountp;
-//		gchar *type = fs.vfs_fstype;
-		E2_TEST_UNMOUNTED_DEVICE
+		if (_e2_fs_mount_is_mountable (dev, type, dir))
 		{
 			E2_PROCESS_UNMOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &fstab_list);
@@ -1010,8 +1080,9 @@ GList *e2_fs_mount_get_mountable_list (void)	//gboolean permcheck)
 	while ((mountable = getmntent (f)) != NULL)
 	{
 		gchar *dev = mountable->mnt_fsname;
+		gchar *type = " "; //TODO
 		gchar *type = mountable->mnt_type;
-		E2_TEST_UNMOUNTED_DEVICE
+		if (_e2_fs_mount_is_mountable (dev, type, dir))
 		{
 			E2_PROCESS_UNMOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &fstab_list);
@@ -1028,8 +1099,9 @@ GList *e2_fs_mount_get_mountable_list (void)	//gboolean permcheck)
 	while (getmntent (f, &mountable) == 0)
 	{
 		gchar *dev = mountable.mnt_special;
+		gchar *type = " "; //TODO
 		gchar *dir = mountable.mnt_mountp;
-		E2_TEST_UNMOUNTED_DEVICE
+		if (_e2_fs_mount_is_mountable (dev, type, dir))
 		{
 			E2_PROCESS_UNMOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &fstab_list);
@@ -1046,10 +1118,10 @@ GList *e2_fs_mount_get_mountable_list (void)	//gboolean permcheck)
 	while ((mountable = getmntent (f)) != NULL)	//data is not thread- or process-safe
 	{
 		gchar *dev = mountable->mnt_fsname;
+		gchar *type = " "; //TODO mountable->mnt_type;
 		gchar *dir = mountable->mnt_dir;
-//		gchar *type = mountable->mnt_type;
 //		gchar *opts = mountable->mnt_opts;
-		E2_TEST_UNMOUNTED_DEVICE
+		if (_e2_fs_mount_is_mountable (dev, type, dir))
 		{
 			E2_PROCESS_UNMOUNTED_DEVICE
 			_e2_fs_mount_add_to_list (dir, &fstab_list);
@@ -1168,138 +1240,6 @@ gboolean e2_fs_mount_is_removable (const gchar *utfpath)
 #endif
 	return retval;
 }
-/**
-@brief check whether the filesystem associated with @a view uses case-sensitive paths/names
-
-MS-DOS/Windows FAT* are case-insensitive, but so-called "long" names are
-case-preserving.
-NTFS can be fully case sensitive or just case preserving.
-Classic Mac OS was case-insensitive but case-preserving. HFS+ on Mac OS X is
-probably the same. (its tag is "hfsplus") Mac HFSX is fully case senstive.
-Samba servers usually exhibit case-insensitive behaviour, regardless of the
-actual fs properties.
-For our purposes here, probably best to treat treat -preserving as -sensitive.
-
-@param view pointer to view data struct
-
-@return TRUE if paths/names are case-sensitive (usually the case)
-*/
-gboolean e2_fs_mount_is_cased (ViewInfo *view)
-{
-	if (view->case_sensitive_names == E2FSCASE_UNKNOWN)
-	{
-#ifdef E2_VFSTMP
-		if (view->case_sensitive_names == E2FSCASE_VIRTUAL)
-		{
-			figure it out
-		}
-		else
-		{
-#endif
-			gchar *local = D_FILENAME_TO_LOCALE (view->dir); //always dup, to avoid dirchange race
-//=========== OS-specific stuff =============
-#if defined(__linux__)
-#include <sys/vfs.h>
-//#include <linux/magic.h> only in glibc-devel
-			struct statfs buf;
-retry:
-			if (statfs (local, &buf))
-			{
-				if (errno == EINTR)
-					goto retry;
-				view->case_sensitive_names = E2FSCASE_NEVER;
-			}
-			else
-			{
-				switch (buf.f_type)
-				{
-					case MSDOS_SUPER_MAGIC:
-					case SMB_SUPER_MAGIC:
-						view->case_sensitive_names = E2FSCASE_ANY;
-						break;
-					default:
-						view->case_sensitive_names = E2FSCASE_SENSITIVE;
-						break;
-				}
-			}
-#if defined (E2_HAL) || defined (E2_DEVKIT)
-			if (e2_fs_mount_is_removable (view->dir)) //removable device
-#endif
-				view->case_sensitive_names |= E2FSCASE_NOCACHE;
-#elif defined(__E2BSD__)
-#include <sys/param.h>
-#include <sys/mount.h>
-			struct statfs buf;
-
-			if (statfs (local, &buf))
-			{
-				view->case_sensitive_names = E2FSCASE_NEVER;
-			}
-			else
-			{
-				gchar *type = buf.f_fstypename;
-				if (strcmp (type, "vfat") == 0)
-					view->case_sensitive_names = E2FSCASE_ANY;
-				else
-					view->case_sensitive_names = E2FSCASE_SENSITIVE;
-			}
-#if defined (E2_HAL) || defined (E2_DEVKIT)
-			if (e2_fs_mount_is_removable (view->dir)) //on removable device
-#endif
-				view->case_sensitive_names |= E2FSCASE_NOCACHE;
-#elif defined(__solaris__) || defined(sco) || defined(hpux) || defined(__svr4__)
-#include <sys/types.h>
-#include <sys/statvfs.h>
-			struct statvfs buf;
-
-			if (statvfs (local, &buf))
-			{
-				view->case_sensitive_names = E2FSCASE_NEVER;
-			}
-			else
-			{
-				gchar *type = buf.f_basetype;
-				if (strcmp (type, "vfat") == 0)
-					view->case_sensitive_names = E2FSCASE_ANY;
-				else
-					view->case_sensitive_names = E2FSCASE_SENSITIVE;
-			}
-#if defined (E2_HAL) || defined (E2_DEVKIT)
-			if (e2_fs_mount_is_removable (view->dir)) //on removable device
-#endif
-				view->case_sensitive_names |= E2FSCASE_NOCACHE;
-#else
-#include <sys/param.h>
-#include <sys/mount.h>
-			struct statfs buf;
-
-			if (statfs (local, &buf))
-			{
-				view->case_sensitive_names = E2FSCASE_NEVER;
-			}
-			else
-			{
-				gchar *type = buf.f_fstypename;
-				if (strcmp (type, "vfat") == 0)
-					view->case_sensitive_names = E2FSCASE_ANY;
-				else
-					view->case_sensitive_names = E2FSCASE_SENSITIVE;
-			}
-#if defined (E2_HAL) || defined (E2_DEVKIT)
-			if (e2_fs_mount_is_removable (view->dir)) //on removable place
-#endif
-				view->case_sensitive_names |= E2FSCASE_NOCACHE;
-#endif	//which OS
-//=========================
-			g_free (local);
-#ifdef E2_VFSTMP
-		}
-#endif
-	}
-	return (view->case_sensitive_names == E2FSCASE_SENSITIVE ||
-			view->case_sensitive_names == E2FSCASE_SENSITIVENOW);
-}
-
 /**
 @brief register mountpoint-related actions
 @return
@@ -1307,7 +1247,7 @@ retry:
 void e2_fs_mount_actions_register (void)
 {
 	E2_Action action =
-	{g_strconcat(_A(61),".",_A(27),NULL),e2_menu_create_mounts_menu,TRUE,E2_ACTION_TYPE_ITEM,E2_ACTION_EXCLUDE_MENU,NULL,NULL};
+	{g_strconcat(_A(64),".",_A(28),NULL),e2_menu_create_mounts_menu,TRUE,E2_ACTION_TYPE_ITEM,E2_ACTION_EXCLUDE_MENU,NULL,NULL};
 	e2_action_register (&action);
 }
 #endif //def E2_FS_MOUNTABLE
