@@ -1,6 +1,6 @@
-/* $Id: e2_hal.c 2309 2011-01-17 00:12:22Z tpgww $
+/* $Id: e2_hal.c 2742 2013-09-11 00:37:42Z tpgww $
 
-Copyright (C) 2008-2011 tooar <tooar@emelfm2.net>
+Copyright (C) 2008-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -97,7 +97,7 @@ static void _e2_hal_destroy_data (HalStorageData *data)
 }
 /* *
 @brief display hal/dbus error message
-Assumes BGL is closed
+Assumes BGL is open/off upon arrival
 @param message specific message
 @error dbus error data, or NULL
 
@@ -106,6 +106,7 @@ Assumes BGL is closed
 /*
 static void _e2_hal_advise (const gchar *message, const DBusError *error)
 {
+	CLOSEBGL
 	if (error != NULL && dbus_error_is_set (error))
 	{
 		gchar *msg = g_strdup_printf ("%s (%s)", message, error->message);
@@ -113,8 +114,16 @@ static void _e2_hal_advise (const gchar *message, const DBusError *error)
 	}
 	else
 		e2_output_print_error ((gchar *)message, FALSE);
+	OPENBGL
 }
 */
+/**
+@brief check for currently-known removable devices
+Assumes BGL open/off
+@param ctx pointer to HAL data struct
+
+@return TRUE if any device found
+*/
 static gboolean _e2_hal_detect_current_removables (LibHalContext *ctx)
 {
 	gint i, count;
@@ -191,9 +200,9 @@ static gboolean _e2_hal_mount_volume (LibHalVolume *vol, const gchar *realdevice
 				printd (WARN, "HAL: cannot mount %s in accord with fstab", realdevice);
 				str = g_strdup_printf (_("Cannot auto-mount device '%s' at '%s'"),
 					realdevice, mountpoint);
-				gdk_threads_enter ();
+				CLOSEBGL
 				e2_output_print_error (str, TRUE);
-				gdk_threads_leave ();
+				OPENBGL
 				g_free (mountpoint);
 				return FALSE;
 			}
@@ -348,9 +357,9 @@ static gboolean _e2_hal_mount_volume (LibHalVolume *vol, const gchar *realdevice
 	guint count = 0;
 #ifdef E2_VFS
 	VPATH ddata = { str, NULL };	//local mounting only
-	while (e2_fs_access3 (&ddata, F_OK, NULL) == 0)
+	while (e2_fs_access3 (&ddata, F_OK E2_ERR_NONE()) == 0)
 #else
-	while (e2_fs_access3 (str, F_OK) == 0)
+	while (e2_fs_access3 (str, F_OK E2_ERR_NONE()) == 0)
 #endif
 	{
 		g_free (str);
@@ -383,9 +392,7 @@ static gboolean _e2_hal_mount_volume (LibHalVolume *vol, const gchar *realdevice
 							ctx, udi, "volume.mount.valid_options", &error);
 	if (dbus_error_is_set (&error))
 	{
-//		gdk_threads_enter ();
 //		_e2_hal_advise (_("HAL: cannot get mount options"), &error);
-//		gdk_threads_leave ();
 #ifdef DEBUG_MESSAGES
 		if (!g_str_has_suffix (error.name, "NoSuchProperty")) //e.g. for audio cd
 			printd (WARN, "HAL: cannot get mount options - %s", error.message);
@@ -438,9 +445,7 @@ static gboolean _e2_hal_mount_volume (LibHalVolume *vol, const gchar *realdevice
 		if (instruction == NULL)
 		{
 badexit:
-//			gdk_threads_enter ();
 //			_e2_hal_advise (_("HAL: cannot initiate mount instruction"), &error);
-//			gdk_threads_leave ();
 			printd (WARN, "HAL: cannot initiate mount instruction - %s", error.message);
 			dbus_error_free (&error);
 			g_free (mountpoint);
@@ -530,13 +535,13 @@ badexit:
 		}
 		if (dbus_error_is_set (&error))
 		{
+//			_e2_hal_advise (_("HAL: cannot mount"), &error);
 			printd (WARN, "HAL: cannot mount %s - %s", realdevice, error.message);
 			str = g_strdup_printf (_("Cannot auto-mount device '%s' at '%s'"),
 					realdevice, mountpoint);
-			gdk_threads_enter ();
-//			_e2_hal_advise (_("HAL: cannot mount"), &error);
+			CLOSEBGL
 			e2_output_print_error (str, TRUE);
-			gdk_threads_leave ();
+			OPENBGL
 
 			dbus_error_free (&error);
 			retval = FALSE;
@@ -594,9 +599,7 @@ static gboolean _e2_hal_unmount_volume (LibHalVolume *vol, const gchar *realdevi
 							ctx, udi, "volume.unmount.valid_options", &error);
 	if (dbus_error_is_set (&error))
 	{
-//		gdk_threads_enter ();
 //		_e2_hal_advise (_("HAL: cannot get unmount options"), &error);
-//		gdk_threads_leave ();
 		printd (WARN, "HAL: cannot get unmount options - %s", error.message);
 		dbus_error_free (&error);
 		if (unmountopts != NULL)
@@ -628,9 +631,7 @@ static gboolean _e2_hal_unmount_volume (LibHalVolume *vol, const gchar *realdevi
 	if (instruction == NULL)
 	{
 badexit:
-//		gdk_threads_enter ();
 //		_e2_hal_advise (_("HAL: cannot initiate unmount instruction"), &error);
-//		gdk_threads_leave ();
 		printd (WARN, "HAL: cannot initiate unmount instruction - %s", error.message);
 		dbus_error_free (&error);
 //		g_ptr_array_free (extra_options, TRUE);
@@ -736,9 +737,7 @@ static gboolean _e2_hal_check_mount (HalStorageData *data)
 						{
 							if (libhal_volume_disc_is_blank (vol))
 							{
-//								gdk_threads_enter ();
 //								_e2_hal_advise (_("HAL: cannot mount blank medium"), NULL);
-//								gdk_threads_leave ();
 								printd (WARN, "HAL: cannot mount blank medium");
 							}
 							else
@@ -940,14 +939,14 @@ static void _e2_hal_remove_device_cb (LibHalContext *ctx, const gchar *udi)
 					{
 						found = TRUE;
 						e2_fs_get_valid_path (&elsewhere, TRUE E2_ERR_NONE());
-						gdk_threads_enter ();	//in case of downstream error message
 						gchar *msg = g_strdup_printf (_("Cannot access %s, going to %s instead"),
 							curr_view->dir, elsewhere);
 #ifdef E2_VFSTMP
 						//FIXME cater for any space-change message
 #endif
+						CLOSEBGL
 						e2_output_print_error (msg, TRUE);
-						gdk_threads_leave ();
+						OPENBGL
 						e2_pane_change_dir (curr_pane, elsewhere);
 					}
 #ifdef E2_VFSTMP
@@ -963,8 +962,9 @@ static void _e2_hal_remove_device_cb (LibHalContext *ctx, const gchar *udi)
 #ifdef E2_VFSTMP
 						//FIXME cater for any space-change message
 #endif
+						CLOSEBGL
 						e2_output_print_error (msg, TRUE);
-						gdk_threads_leave ();
+						OPENBGL
 						e2_pane_change_dir (other_pane, elsewhere);
 					}
 					g_free (elsewhere);
@@ -1030,14 +1030,18 @@ static void _e2_hal_property_change_cb (LibHalContext *ctx, const gchar *udi,
 		if (loaded)
 		{
 			printd (DEBUG, "HAL: removable media present now");
+//			CLOSEBGL
 			//do stuff
 			//mount if device mountpoint is not mounted (os should handle that)
+//			OPENBGL
 		}
 		else
 		{
 			printd (DEBUG, "HAL: removable media gone");
+//			CLOSEBGL
 			//eject if not just done to trigger this callback
 			//unmount if device mountpoint is mounted (os should handle that)
+//			OPENBGL
 		}
 	}
 }
@@ -1081,7 +1085,7 @@ gboolean e2_hal_device_is_ejectable (const gchar *devpath)
 }
 /**
 @brief initialise HAL connection
-
+Called with BGL open/off
 @return
 */
 void e2_hal_init (void)
