@@ -1,22 +1,22 @@
-/* $Id: stroke.c 1314 2008-10-06 21:11:20Z tpgww $
-	libstroke2 - a pointer-device gesture-management library for *NIX
-	Derived from libstroke 0.5.1 @ www.etla.net/libstroke
-	Copyright (C) 2008 tooar <tooar@gmx.net>
-	Portions copyright (C) 1996-1999  Mark F. Willey, ETLA Technical
+/* $Id: stroke.c 2064 2010-03-12 13:15:36Z tpgww $
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU Lesser General Public License as published by
-	the Free Software Foundation; either version 3, or (at your option)
-	any later version.
+libstroke2 - a pointer-device gesture-management library for *NIX
+Derived from libstroke 0.5.1 @ www.etla.net/libstroke
+Copyright (C) 2008-2010 tooar <tooar@emelfm2.net>
+Portions copyright (C) 1996-1999  Mark F. Willey, ETLA Technical
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-	See the GNU Lesser General Public License for more details.
+This library is free software; you can redistribute it and/or modify it
+under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation; either version 3, or (at your option)
+any later version.
 
-	You should have received a copy of the GNU Lesser General Public License
-	along with this file. If not, contact the Free Software Foundation,
-	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+This library is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+See the GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this file. If not, see http://www.gnu.org/licenses.
 */
 
 /* This code is independent of glib and gtk, though the documentation is mostly
