@@ -1,6 +1,6 @@
-/* $Id: e2_button.c 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_button.c 2790 2013-10-09 13:00:04Z tpgww $
 
-Copyright (C) 2004-2010 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -19,30 +19,31 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 
 #include "emelfm2.h"
 #include "e2_button.h"
+#include "e2_icons.h"
 
 E2_Button E2_BUTTON_YES =
-	{ NULL,GTK_STOCK_YES, NULL,	//label is always context-dependent
+	{ NULL,STOCK_NAME_YES, NULL,	//label is always context-dependent
 		E2_BTN_DEFAULT,E2_BTN_DEFAULT,GTK_RESPONSE_YES};
 E2_Button E2_BUTTON_NO =
-	{ NULL,GTK_STOCK_NO,NULL,0,0,GTK_RESPONSE_NO};	//label is always context-dependent
+	{ NULL,STOCK_NAME_NO,NULL,0,0,GTK_RESPONSE_NO};	//label is always context-dependent
 E2_Button E2_BUTTON_CANCEL =
-	{ N_("_Cancel"),GTK_STOCK_CANCEL,NULL,0,0,E2_RESPONSE_NOTOALL};
+	{ N_("_Cancel"),STOCK_NAME_CANCEL,NULL,0,0,E2_RESPONSE_NOTOALL};
 E2_Button E2_BUTTON_APPLY =
-	{ N_("Commi_t"),GTK_STOCK_OK,NULL,0,0,E2_RESPONSE_APPLY}; //often co-used with _Cancel, _All
+	{ N_("Commi_t"),STOCK_NAME_OK,NULL,0,0,E2_RESPONSE_APPLY}; //often co-used with _Cancel, _All
 E2_Button E2_BUTTON_APPLYTOALL =
 	{ N_("_All"),"apply_all"E2ICOND,N_("Change all unprocessed items"),
 		E2_BTN_TIPPED,E2_BTN_TIPPED,E2_RESPONSE_APPLYTOALL};
 E2_Button E2_BUTTON_DISCARD =
-	{ N_("_Discard"),GTK_STOCK_DISCARD,N_("Abandon any changes"),
+	{ N_("_Discard"),STOCK_NAME_DISCARD,N_("Abandon any changes"),
 		E2_BTN_TIPPED,E2_BTN_TIPPED,E2_RESPONSE_DISCARD};
 E2_Button E2_BUTTON_REFRESH =
-	{ N_("_Refresh"),GTK_STOCK_REFRESH,NULL,0,0,E2_RESPONSE_REFRESH};
+	{ N_("_Refresh"),STOCK_NAME_REFRESH,NULL,0,0,E2_RESPONSE_REFRESH};
 E2_Button E2_BUTTON_CLOSE =
-	{ N_("_Close"),GTK_STOCK_CLOSE,NULL,0,0,GTK_RESPONSE_CLOSE};
+	{ N_("_Close"),STOCK_NAME_CLOSE,NULL,0,0,GTK_RESPONSE_CLOSE};
 E2_Button E2_BUTTON_CREATE =
-	{ N_("C_reate"),GTK_STOCK_YES,NULL,0,0,E2_RESPONSE_CREATE};  //co-use with close/cancel
+	{ N_("C_reate"),STOCK_NAME_YES,NULL,0,0,E2_RESPONSE_CREATE};  //co-use with close/cancel
 E2_Button E2_BUTTON_REMOVE =
-	{ N_("_Remove"),GTK_STOCK_REMOVE,NULL,0,0,E2_RESPONSE_REMOVE}; //check no co-use with refresh
+	{ N_("_Remove"),STOCK_NAME_REMOVE,NULL,0,0,E2_RESPONSE_REMOVE}; //check no co-use with refresh
 E2_Button E2_BUTTON_MORE =
 	{ N_("_Apply"),"proceed"E2ICOND,NULL,0,0,E2_RESPONSE_MORE};
 
@@ -126,7 +127,7 @@ void e2_button_derive (E2_Button *button, E2_Button *base, E2_ButtonText type)
 		else
 			button->showflags &= ~E2_BTN_TIPPED;
 		if (type == BTN_NO_CANCEL)
-			button->stock = GTK_STOCK_CANCEL;
+			button->name = STOCK_NAME_CANCEL;
 	}
 }
 /**
@@ -264,7 +265,7 @@ This is used for toolbar buttons, and (maybe indirectly) for dialog buttons
 @param data data specified for the callback
 @param flags flags indicating button properties
 
-@return the (shown) button widget
+@return the (unshown) button widget
 */
 GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSize size,
 	const gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data,
@@ -293,10 +294,17 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 			(flags & E2_BUTTON_FREE_DATA) ? (GClosureNotify) g_free : NULL, 0);
 
 	GtkWidget *bbox;
+#ifdef USE_GTK3_0
 	if (flags & E2_BUTTON_ICON_ABOVE_LABEL)
-		bbox = gtk_vbox_new (FALSE, 2);	//constant gap, not E2_PADDING_XSMALL
+		bbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);	//constant gap, not E2_PADDING_XSMALL
+	else
+		bbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+#else
+	if (flags & E2_BUTTON_ICON_ABOVE_LABEL)
+		bbox = gtk_vbox_new (FALSE, 2);
 	else
 		bbox = gtk_hbox_new (FALSE, 2);
+#endif
 
 	gboolean need_align = FALSE;
 
@@ -304,7 +312,7 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 	{
 		GtkWidget *img = e2_widget_get_icon (stock, size);
 		if (img == NULL && (flags & E2_BUTTON_SHOW_MISSING_ICON))
-			img = e2_widget_get_icon (GTK_STOCK_MISSING_IMAGE, size);
+			img = e2_widget_get_icon (STOCK_NAME_MISSING_IMAGE, size);
 		if (img != NULL)
 		{
 			gtk_box_pack_start (GTK_BOX (bbox),
@@ -315,11 +323,10 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 
 	if ((label != NULL) && (*label != '\0'))
 	{
-		GtkWidget *lab = gtk_label_new ("");
-		gtk_label_set_markup_with_mnemonic (GTK_LABEL (lab), label);
-		gtk_box_pack_start (GTK_BOX (bbox), lab, TRUE, TRUE, 0);
-		if (stock != NULL)
-			gtk_widget_add_mnemonic_label (button, lab); //needed to make it work dunno why
+		GtkWidget *wlab = gtk_label_new ("");
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (wlab), label);
+		gtk_label_set_mnemonic_widget ((GtkLabel*)wlab, button); //NEEDED? label not a child of button
+		gtk_box_pack_start (GTK_BOX (bbox), wlab, TRUE, TRUE, 0);
 		need_align = TRUE;
 	}
 
@@ -342,7 +349,7 @@ Downstream code requires BGL closed
 @param callback callback function for button's "clicked" signal, or NULL
 @param data data to send to the callback
 
-@return the (shown) button widget
+@return the (unshown) new button widget
 */
 GtkWidget *e2_button_get (const gchar *label, const gchar *stock, const gchar *tip,
 	void (*callback)(/*GtkButton*,gpointer*/), gpointer data)
@@ -383,7 +390,7 @@ GtkWidget *e2_button_add (GtkWidget *box, gboolean exp, guint pad, gchar *label,
 @param callback callback function for "clicked" signal, or NULL
 @param data data specified for the callback
 
-@return the button widget
+@return the (unshown) new button widget
 */
 GtkWidget *e2_button_add_end (GtkWidget *box, gboolean exp, guint pad, gchar *label,
 	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data)
@@ -486,10 +493,10 @@ GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
 @param label button label which may contain mnemonic
 @param callback callback function for "clicked" signal, or NULL
 @param data data specified for the callback
-@param left
-@param right
-@param top
-@param bottom
+@param left index of table column left of attachment-position
+@param right index of table column right of attachment-position
+@param top index of table row above attachment-position
+@param bottom index of table row below attachment-position
 
 @return the button widget
 */
@@ -505,7 +512,12 @@ GtkWidget *e2_button_add_to_table (
 	if (callback != NULL)
 		g_signal_connect (G_OBJECT (button), "clicked",
 			G_CALLBACK (callback), data);
-	gtk_table_attach_defaults (GTK_TABLE (table), button, left, right, top, bottom);
+#ifdef USE_GTK3_2
+	gtk_grid_attach (GTK_GRID (table), button, left, top, right-left, bottom-top);
+#else
+	gtk_table_attach_defaults (GTK_TABLE (table), button,
+								left, right, top, bottom);
+#endif
 	return button;
 }
 /**
@@ -516,10 +528,10 @@ GtkWidget *e2_button_add_to_table (
 @param state the intitial state of the button
 @param callback callback function for "toggled" signal, or NULL
 @param data data specified for the callback
-@param left
-@param right
-@param top
-@param bottom
+@param left index of table column left of attachment-position
+@param right index of table column right of attachment-position
+@param top index of table row above attachment-position
+@param bottom index of table row below attachment-position
 
 @return the button widget
 */
@@ -539,8 +551,12 @@ GtkWidget *e2_button_add_toggle_to_table (
 	if (callback != NULL)
 		g_signal_connect (G_OBJECT (check_button), "toggled",
 			G_CALLBACK (callback), data);
+#ifdef USE_GTK3_2
+	gtk_grid_attach (GTK_GRID (table), check_button, left, top, right-left, bottom-top);
+#else
 	gtk_table_attach_defaults (GTK_TABLE (table), check_button,
 		                        left, right, top, bottom);
+#endif
 	return check_button;
 }
 /**
@@ -552,10 +568,10 @@ GtkWidget *e2_button_add_toggle_to_table (
 @param state the intitial state of the button
 @param callback callback function for "toggled" signal, or NULL
 @param data data specified for the callback
-@param left
-@param right
-@param top
-@param bottom
+@param left index of table column left of attachment-position
+@param right index of table column right of attachment-position
+@param top index of table row above attachment-position
+@param bottom index of table row below attachment-position
 
 @return the button widget
 */
@@ -574,7 +590,11 @@ GtkWidget *e2_button_add_radio_to_table (
 	if (callback != NULL)
 		g_signal_connect (G_OBJECT (radio_button), "toggled",
 			G_CALLBACK (callback), data);
+#ifdef USE_GTK3_2
+	gtk_grid_attach (GTK_GRID (table), radio_button, left, top, right-left, bottom-top);
+#else
 	gtk_table_attach_defaults (GTK_TABLE (table), radio_button,
 		                        left, right, top, bottom);
+#endif
 	return radio_button;
 }
