@@ -1,6 +1,6 @@
-/* $Id: e2_button.h 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_button.h 2748 2013-09-20 09:41:43Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -51,7 +51,7 @@ typedef enum
 typedef struct _E2_Button
 {
 	const gchar *label;
-	const gchar *stock;
+	const gchar *name; //name of icon file (without .png extension)
 	gchar *tip;	//custom tip or NULL, sometimes non-constant
 	E2_ButtonType showflags;
 	E2_ButtonType default_flags;
@@ -107,27 +107,27 @@ GtkWidget *e2_button_get (const gchar *label, const gchar *stock, const gchar *t
 	void (*callback)(/*GtkButton*,gpointer*/), gpointer data);
 GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSize size,
 	const gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data,
-	E2_ButtonFlags flags);
+	E2_ButtonFlags flags) G_GNUC_MALLOC;
 GtkWidget *e2_button_add (GtkWidget *box, gboolean exp, guint pad, gchar *label,
-	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data);
+	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data) G_GNUC_MALLOC;
 GtkWidget *e2_button_add_end (GtkWidget *box, gboolean exp, guint pad,gchar *label,
-	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data);
+	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data) G_GNUC_MALLOC;
 GtkWidget *e2_button_get_toggle (gboolean check, gboolean state, gchar *label,
-	gchar *tip, void (*callback)(GtkToggleButton*,gpointer), gpointer data);
+	gchar *tip, void (*callback)(GtkToggleButton*,gpointer), gpointer data) G_GNUC_MALLOC;
 GtkWidget *e2_button_add_toggle (GtkWidget *box,  gboolean check, gboolean state,
 	gchar *label, gchar *tip, gboolean exp, guint pad,
-	void (*callback)(/*GtkToggleButton*,gpointer*/), gpointer data);
+	void (*callback)(/*GtkToggleButton*,gpointer*/), gpointer data) G_GNUC_MALLOC;
 GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
 	gboolean state, gboolean exp, guint pad,
-	void (*callback)(GtkToggleButton*,gpointer), gpointer data);
+	void (*callback)(GtkToggleButton*,gpointer), gpointer data) G_GNUC_MALLOC;
 GtkWidget *e2_button_add_to_table (GtkWidget *table, gchar *label,
 	void (*callback)(GtkButton*,gpointer), gpointer data,
-	gint left, gint right, gint top, gint bottom);
+	gint left, gint right, gint top, gint bottom) G_GNUC_MALLOC;
 GtkWidget *e2_button_add_toggle_to_table(GtkWidget *table, gchar *label,
 	gboolean state, void (*callback)(/*GtkToggleButton*,gpointer*/), gpointer data,
-	gint left, gint right, gint top, gint bottom);
+	gint left, gint right, gint top, gint bottom) G_GNUC_MALLOC;
 GtkWidget *e2_button_add_radio_to_table (GtkWidget *table, gchar *label,
 	GSList *group, gboolean state, void (*callback)(/*GtkToggleButton*,gpointer*/),
-	gpointer data, gint left, gint right, gint top, gint bottom);
+	gpointer data, gint left, gint right, gint top, gint bottom) G_GNUC_MALLOC;
 
 #endif //ndef __E2_BUTTON_H__
