@@ -1,6 +1,6 @@
-/* $Id: e2_combobox.c 2464 2012-03-12 12:07:01Z tpgww $
+/* $Id: e2_combobox.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it under the
@@ -75,18 +75,24 @@ This applies to _all_ created comboboxes whether or not they have a text-entry
 static gboolean _e2_combobox_key_press_cb (GtkWidget *combo, GdkEventKey *event,
 	gpointer data)
 {
+	printd (DEBUG, "_e2_combobox_key_press_cb (combo:,wvent,data:)");
 	if (!(event->keyval == GDK_Up || event->keyval == GDK_Down))
 		return FALSE;
 
+	NEEDCLOSEBGL
 	if (event->keyval == GDK_Down &&
 		(event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK)
 	{
 		gtk_combo_box_popup (GTK_COMBO_BOX (combo));
+		NEEDOPENBGL
 		return TRUE;
 	}
 
 	if (data == NULL)
+	{
+		NEEDOPENBGL
 		return FALSE;	//no cycling allowed
+	}
 
 	gint pos = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
 	if (event->keyval == GDK_Down)
@@ -103,6 +109,7 @@ static gboolean _e2_combobox_key_press_cb (GtkWidget *combo, GdkEventKey *event,
 		else if (pos == 0 || pos == -1)
 			pos = _e2_combobox_last_index ((GtkComboBox*)combo);
 	}
+
 	e2_combobox_set_active (combo, pos);
 #ifdef USE_GTK2_24
 	if (gtk_combo_box_get_has_entry ((GtkComboBox*)combo))
@@ -111,50 +118,55 @@ static gboolean _e2_combobox_key_press_cb (GtkWidget *combo, GdkEventKey *event,
 #endif
 	{
 		GtkWidget *entry =
-# ifdef USE_GTK2_14
+#ifdef USE_GTK2_14
 			gtk_bin_get_child (GTK_BIN (combo));
-# else
+#else
 			GTK_BIN (combo)->child;
-# endif
+#endif
 		if (pos == -1)
 			gtk_entry_set_text (GTK_ENTRY (entry), "");
 		else
 			gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 	}
-
+	NEEDOPENBGL
 	return TRUE;
 }
 /**
-@brief "key-press-event" signal callback to handle GtkComboBoxEntry text shortening or history clearing
-A <Alt>Delete keypress in the combobox's entry triggers deletion of current
+@brief "key-press-event" signal callback
+These are default mechanisms, relevant only if not 'trumped' by a specific
+key-binding e.g. in main-window dirlines and commandline.
+<Alt>Delete keypress in the combobox's entry triggers deletion of current
 string and any matching item(s) in @a history.
-A <Shift><Alt>Delete keypress in the combobox entry triggers deletion of
+<Shift><Alt>Delete keypress in the combobox entry triggers deletion of
 current string and the whole history list @a history (hopefully that's not
 static if also used by other widgets)
-Otherwise, <Shift>Delete clears text from cursor to end
+<Shift>Delete clears text from cursor to end
 @param entry the entry widget for the combo box
 @param event pointer to event data struct
-@param history pointer to history list for the combo box
+@param history pointer to history list for the combo box, or NULL
 
-@return TRUE if the keypress was <Alt>Delete
+@return TRUE if the keypress was processed
 */
 static gboolean _e2_combobox_key_press_cb2 (GtkWidget *entry, GdkEventKey *event,
 	GList **history)
 {
-	//prevent sporadic bad latency in UI updates
-	gtk_widget_queue_draw (entry);
-
+	gboolean retval = FALSE;
 	if (event->keyval == GDK_Delete)
 	{
-		gint start;
+		OPENBGL //TODO do this if NEEDCLOSEBGL is empty
+		//prevent sporadic bad latency in UI updates
+		gtk_widget_queue_draw (entry);
+		CLOSEBGL
+
+		NEEDCLOSEBGLX //do this if NEEDCLOSEBGL is empty
 		guint modifiers = gtk_accelerator_get_default_mod_mask ();
 		if ((event->state & modifiers) == GDK_SHIFT_MASK)
-		{	//there is no action for clearing "the rest" of an entry
-			start = gtk_editable_get_position (GTK_EDITABLE (entry));
+		{
+			gint start = gtk_editable_get_position (GTK_EDITABLE (entry));
 			gtk_editable_delete_text (GTK_EDITABLE (entry), start, -1);
 			if (g_object_get_data (G_OBJECT (entry), "e2-dir-line") != NULL)
 				e2_command_line_highlight (entry, FALSE);
-			return TRUE;
+			retval = TRUE;
 		}
 		else if ((event->state & modifiers) == GDK_MOD1_MASK)
 		{
@@ -169,12 +181,10 @@ static gboolean _e2_combobox_key_press_cb2 (GtkWidget *entry, GdkEventKey *event
 #endif
 					this, TRUE);
 			}
-			return TRUE;
+			retval = TRUE;
 		}
 		else if ((event->state & modifiers) == (GDK_SHIFT_MASK | GDK_MOD1_MASK))
-		{	//hardcode history clearance for most comboboxes
-			//a keybinding probably applies to dirlines and commandline but
-			//in that case they don't use this function
+		{
 			GtkComboBox *combo;
 			GtkTreeModel *model;
 
@@ -191,10 +201,11 @@ static gboolean _e2_combobox_key_press_cb2 (GtkWidget *entry, GdkEventKey *event
 			if (history != NULL && *history != NULL)
 				//too bad for any other widget using the history, if that's static
 				e2_list_free_with_data (history);
-			return TRUE;
+			retval = TRUE;
 		}
+		NEEDOPENBGL
 	}
-	return FALSE;
+	return retval;
 }
 /**
 @brief "scroll-event" signal callback for the entry belonging to a GtkComboBoxEntry
@@ -213,6 +224,7 @@ static gboolean _e2_combobox_scroll_cb (GtkWidget *entry, GdkEventScroll *event,
 //	printd (DEBUG, "_e2_combobox_scroll_cb");
 	if (event->direction == GDK_SCROLL_UP)
 	{
+		NEEDCLOSEBGL
 		pos = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
 		if (pos > 0)
 		{
@@ -222,10 +234,12 @@ static gboolean _e2_combobox_scroll_cb (GtkWidget *entry, GdkEventScroll *event,
 			else
 				e2_combobox_set_active (combo, pos-1);
 		}
+		NEEDOPENBGL
 		return TRUE;
 	}
 	else if (event->direction == GDK_SCROLL_DOWN)
 	{
+		NEEDCLOSEBGL
 		pos = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
 		guint lastpos = _e2_combobox_last_index (GTK_COMBO_BOX (combo));
 		if (pos == -1 || pos < lastpos)
@@ -236,6 +250,7 @@ static gboolean _e2_combobox_scroll_cb (GtkWidget *entry, GdkEventScroll *event,
 			else
 				e2_combobox_set_active (combo, pos+1);
 		}
+		NEEDOPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -253,12 +268,13 @@ When the combobox entry is simply edited, nothing is done.
 */
 static void _e2_combobox_changed_cb (GtkWidget *combo, gpointer data)
 {
-//	printd (DEBUG, "_e2_combobox_changed_cb: combo:_ data: %s", (data == NULL) ? "FALSE":"TRUE");
+	printd (DEBUG, "_e2_combobox_changed_cb: combo:_ data: %s", (data == NULL) ? "FALSE":"TRUE");
 	if (g_object_get_data (G_OBJECT (combo), "e2_changed_blocked") != NULL)
 	{
 //		printd (DEBUG, "_e2_combobox_changed_cb BLOCKED");
 		return;
 	}
+	NEEDCLOSEBGL
 	//active item will be -1 whenever an item is edited
 	if (
 #ifdef USE_GTK2_18
@@ -283,80 +299,69 @@ static void _e2_combobox_changed_cb (GtkWidget *combo, gpointer data)
 		else
 		{
 			printd (DEBUG, "issue activate signal for combobox entry");
+			NEEDOPENBGL
 			g_signal_emit_by_name (G_OBJECT (entry), "activate");
+			return;
 		}
 	}
+	NEEDOPENBGL
+}
+
+/**
+@brief "activate" signal callback for the GtkEntry of a GtkComboBoxEntry combobox
+
+This cb is not connected if the parent combobox is created with flag
+E2_COMBOBOX_NO_AUTO_HISTORY.
+
+@param entry entry of a GtkComboBoxEntry
+@param data pointerised gboolean specified when callback was connected,
+  non-NULL/TRUE if repeated history-entries are allowed
+
+@return
+*/
+void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
+{
+	printd (DEBUG, "e2_combobox_activated_cb (entry: data:%x", data);
+	NEEDCLOSEBGL
+	GtkWidget *combo =
+#ifdef USE_GTK2_14
+		gtk_widget_get_parent (entry);
+#else
+		entry->parent;
+#endif
+	const gchar *text = gtk_entry_get_text (GTK_ENTRY (entry));
+	if (text != NULL && *text != '\0')
+	{
+		gchar *newtext;
+		gint limit;
+		if (g_object_get_data (G_OBJECT (entry), "e2-dir-line") != NULL)
+		{
+			newtext = g_strdup (text);
+			newtext = e2_utils_path_clean (newtext);
+			gtk_entry_set_text ((GtkEntry*)entry, newtext);
+			limit = -2;
+		}
+		else if (g_object_get_data (G_OBJECT (entry), "e2-command-line") != NULL)
+		{
+			newtext = (gchar*)text;
+			limit = -1;
+		}
+		else
+		{
+			newtext = (gchar*)text;
+			limit = 0;
+		}
+		e2_combobox_prepend_history (combo, newtext, limit, GPOINTER_TO_INT (data));
+		if (newtext != (gchar*)text)
+			g_free (newtext);
+	}
+	NEEDOPENBGL
 }
 
   /******************/
  /***** public *****/
 /******************/
 
-/**
-@brief "activate" signal callback for created GtkComboBoxEntry comboboxes
-
-This function is a callback for the "activate" signal on the GtkEntry that
-belongs to a combobox with a text entry, unless that combobox is created with
-flag E2_COMBOBOX_NO_AUTO_HISTORY.
-The current contents of @a entry, if not empty, are prepended to the box's
-history list.
-
-@param entry entry of a GtkComboBoxEntry
-@param data pointerised gboolean, TRUE if double history entries are allowed
-
-@return
-*/
-void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
-{
-	printd (DEBUG, "e2_combobox_activated_cb");
-	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
-	//a directory path may properly have trailing whitespace, so no strip
-	g_strchug (text);
-	if (*text != '\0')
-	{
-#ifdef USE_GTK2_24
-		GtkComboBoxText *combo = GTK_COMBO_BOX_TEXT (
-#else
-		GtkComboBox *combo = GTK_COMBO_BOX (
-#endif
-#ifdef USE_GTK2_14
-			gtk_widget_get_parent (entry)
-#else
-			entry->parent
-#endif
-		);
-
-		if (data == NULL)	// !GPOINTER_TO_INT (data))
-		{
-			GtkTreeIter iter;
-#ifdef USE_GTK2_24
-			GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
-#else
-			GtkTreeModel *model = gtk_combo_box_get_model (combo);
-#endif
-			if (gtk_tree_model_get_iter_first (model, &iter))
-			{
-				//actually, there should only ever be 1 match at most
-				while (e2_tree_find_iter_from_str_simple (model, 0, text, &iter, FALSE))
-				{
-					GtkTreeIter test;
-					test = iter;
-					gboolean more = gtk_tree_model_iter_next (model, &test);
-					gtk_list_store_remove (GTK_LIST_STORE (model), &iter); //after this, iter has data for next item, if any
-					if (!more)
-						break;
-				}
-			}
-		}
-#ifdef USE_GTK2_24
-		gtk_combo_box_text_prepend_text (combo, text);
-#else
-		gtk_combo_box_prepend_text (combo, text);
-#endif
-		e2_combobox_set_active (GTK_WIDGET (combo), 0);
-	}
-	g_free (text);
-}
 /**
 @brief determine whether the history model of @a combo has any content
 
@@ -537,6 +542,91 @@ void e2_combobox_clear_value (GtkWidget *combo, const gchar *value, gboolean wit
 }
 */
 /**
+@brief prepend @a newtext to history of @a combo
+
+Does nothing if @a newtext is NULL or empty. Trailing whitespace is omitted from the entry.
+After processing, the history will be shortened if needed.
+
+@param combo a GtkComboBox, possibly one of those in a toolbar command-line
+@param newtext the content to be added to the history
+@param limit integer: -2 for dirline, -1 for commandline, 0 for no length-limit, N for limit N > 0
+@param multi TRUE if repeated history-entries are allowed
+
+@return
+*/
+void e2_combobox_prepend_history (GtkWidget *combo, const gchar *newtext,
+	gint limit, gboolean multi)
+{
+	if (newtext != NULL)
+	{
+		gchar *text = g_strdup (newtext);
+		//a directory path may properly have trailing whitespace, so no strip
+		g_strchug (text);
+		if (*text != '\0')
+		{
+			gboolean add = TRUE;
+			GtkTreeIter iter;
+			GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+			if (gtk_tree_model_get_iter_first (model, &iter))
+			{
+				if (multi)
+				{
+					//just prevent adjacent matching items
+					gchar *first;
+					gtk_tree_model_get (model, &iter, 0, &first, -1);
+					if (first != NULL)
+					{
+						add = (strcmp (first, text) != 0);
+						g_free (first);
+					}
+				}
+				else
+				{
+					//remove all existing matching history-item(s)
+					//actually, there should only ever be 1 match at most
+					while (e2_tree_find_iter_from_str_simple (model, 0, text, &iter, FALSE))
+					{
+						GtkTreeIter test;
+						test = iter;
+						gboolean more = gtk_tree_model_iter_next (model, &test);
+						gtk_list_store_remove (GTK_LIST_STORE (model), &iter); //after this, iter has data for next item, if any
+						if (!more)
+							break;
+					}
+				}
+			}
+			if (add)
+			{
+#ifdef USE_GTK2_24
+				gtk_combo_box_text_prepend_text (GTK_COMBO_BOX_TEXT (combo), text);
+#else
+				gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), text);
+#endif
+			}
+			e2_combobox_set_active (GTK_WIDGET (combo), 0);
+			
+			gint max;
+			if (limit == -2)
+				max = e2_option_int_get ("dir-line-history-max");
+			else if (limit == -1) 
+				max = e2_option_int_get ("command-line-history-max");
+			else
+				max = limit;
+			if (max > 1) //ensure at least 1 entry
+			{
+				if (gtk_tree_model_iter_nth_child (model, &iter, NULL, max))
+				{
+					while (gtk_list_store_remove (GTK_LIST_STORE(model), &iter))
+					{
+//debugging				now--; 
+					}
+				}
+			}
+		}
+		g_free (text);
+	}
+}
+/**
 @brief append a GList to a GtkComboBox's history
 
 This function appends a GList of strings (@a list) to the history of a GtkComboBoxEntry
@@ -609,12 +699,14 @@ model, so it should not be static if there's a prospect of multiple widgets
 clearing the history
 If E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, calls gtk_widget_set_name()
 which MAY? need BGL closed
+The returned widget is not shown.
+
 @param activate_cb "activate" callback for the combobox entry, or NULL
 @param activate_data data to provide to the activate callback
 @param history history-list pointer, or NULL
 @param flags flags to influence the usage of the GtkComboBox
 
-@return the GtkComboBox that was created and packed
+@return the created GtkComboBox
 */
 GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 	gpointer activate_data, GList **history, E2_ComboBoxFlags flags)
@@ -628,7 +720,8 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 
 	if (!(flags & E2_COMBOBOX_MENU_STYLE))
 	{
-#ifdef USE_GTK3_0
+		//early gtk3 cannot cope with combobox styling ?
+#ifdef USE_GTK3_6
 		//use distinct style-object for each combo, cuz all are destroyed during any config window-recreation
 		GtkCssProvider *liststyle = gtk_css_provider_new ();
 		const gchar *cssdata = "GtkComboBox { -GtkComboBox-appears-as-list:1; }";
@@ -637,7 +730,7 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 		{
 			GtkStyleContext *sc = gtk_widget_get_style_context (combo);
 			gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER(liststyle),
-				GTK_STYLE_PROVIDER_PRIORITY_USER);
+				GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 		}
 		else
 		{
@@ -645,7 +738,7 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 			g_error_free (err);
 		}
 		g_object_unref (G_OBJECT (liststyle)); //back to refcount=1, or 0
-#else
+#elif !defined(USE_GTK3_0)
 		static gboolean style_parsed = FALSE;
 		if (!style_parsed)
 		{
@@ -657,7 +750,6 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 		gtk_widget_set_name (combo, "list-style");
 #endif
 	}
-
 	if (history != NULL)
 		e2_combobox_append_history (combo, *history);
 
@@ -666,18 +758,20 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 #ifdef USE_GTK2_14
 		GtkWidget *child = gtk_bin_get_child (GTK_BIN (combo));
 #endif
+		 //<Delete> processing, no key translation needed, NULL history ok
+		g_signal_connect (G_OBJECT
+#ifdef USE_GTK2_14
+			(child),
+#else
+			(GTK_BIN (combo)->child),
+#endif
+			"key-press-event",
+			G_CALLBACK (_e2_combobox_key_press_cb2), history);
+
 		if (history != NULL)
 		{
 			g_object_set_data (G_OBJECT(combo), "e2-combo-history", history);
-			//enable prompt UI updates and deletions
-			g_signal_connect (G_OBJECT
-#ifdef USE_GTK2_14
-				(child),
-#else
-				(GTK_BIN (combo)->child),
-#endif
-				"key-press-event",
-				G_CALLBACK (_e2_combobox_key_press_cb2), history); //for <Delete>, no key translation needed
+			//enable prompt UI updates
 			g_signal_connect (G_OBJECT
 #ifdef USE_GTK2_14
 				(child),
@@ -686,6 +780,7 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 #endif
 				"scroll-event",
 				G_CALLBACK (_e2_combobox_scroll_cb), combo);
+
 			if (!(flags & E2_COMBOBOX_NO_AUTO_HISTORY))
 				g_signal_connect (G_OBJECT
 #ifdef USE_GTK2_14
@@ -725,7 +820,8 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 Creates a GtkComboBox using e2_combobox_get(), and then packs it into @a box
 and connects @a func to the "activate" signal.
 If E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, downstream calls
-gtk_widget_set_name() which MAY? need BGL closed
+gtk_widget_set_name() which MAY? need BGL closed.
+None of the nested widgets is shown.
 
 @param box parent box to add the GtkComboBox to
 @param expand packing property
@@ -742,13 +838,19 @@ GtkWidget *e2_combobox_add (GtkWidget *box, gboolean expand, guint padding,
 	GList **history, E2_ComboBoxFlags flags)
 {
 	GtkWidget *combo = e2_combobox_get (activate_cb, activate_data, history, flags);
-
-	//add padding to the left & right side of the combobox
+	//add padding to the left & right sides of the combobox
 	GtkWidget *pad_box;
+#ifdef USE_GTK3_0
+	if (gtk_orientable_get_orientation(GTK_ORIENTABLE (box)) == GTK_ORIENTATION_HORIZONTAL)
+		pad_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, padding);
+	else
+		pad_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, padding);
+#else
 	if (GTK_IS_HBOX (box))
 		pad_box = gtk_vbox_new (FALSE, padding);
 	else
 		pad_box = gtk_hbox_new (FALSE, padding);
+#endif
 
 	//now add the combo to the padding container
 	gtk_box_pack_start (GTK_BOX (pad_box), combo, TRUE, TRUE, padding);
