@@ -1,6 +1,6 @@
-/* $Id: e2_combobox.h 2126 2010-08-12 23:33:55Z tpgww $
+/* $Id: e2_combobox.h 2658 2013-08-10 06:35:52Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -48,10 +48,8 @@ typedef enum
 									//will (if necessary) be cycled upon pressing GDK_UP or GDK_DOWN
 } E2_ComboBoxFlags;
 
-//callbacks
 void e2_combobox_activated_cb (GtkWidget *entry, gpointer data);
 
-//public
 void e2_combobox_block_changed (GtkWidget *combo);
 //void e2_combobox_unblock_changed (GtkWidget *combo);
 void e2_combobox_set_active (GtkWidget *combo, gint num);
@@ -59,6 +57,8 @@ gchar *e2_combobox_get_active_text (GtkWidget *combo);
 void e2_combobox_clear_value (GtkWidget *combo, const gchar *value,
 	gboolean with_entry);
 //void e2_combobox_save_history (GtkWidget *combo, GList **list);
+void e2_combobox_prepend_history (GtkWidget *combo, const gchar *newtext,
+	gint limit, gboolean multi);
 void e2_combobox_append_history (GtkWidget *combo, GList *list);
 void e2_combobox_append_history_counted (GtkWidget *combo, guint num, gchar **array);
 void e2_combobox_append_history_strv (GtkWidget *combo, gchar **strv);
