@@ -1,6 +1,6 @@
-/* $Id: e2_hook.h 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_hook.h 2743 2013-09-19 22:29:00Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -22,9 +22,9 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 
 #include "e2_utils.h"
 
-void e2_hook_unattach_cb (E2_Duo *duo);
+void e2_hook_unattach (E2_Duo *duo);
 
-GHook *e2_hook_register (GHookList *hook_list, gboolean (*func)(gpointer,gpointer), gpointer data);
+GHook *e2_hook_register (GHookList *hook_list, gboolean (*func)(gpointer,gpointer), gpointer data) G_GNUC_MALLOC;
 gboolean e2_hook_unregister (GHookList *hook_list, gboolean (*func)(gpointer,gpointer),
 	gpointer data, gboolean use_data);
 #define e2_hook_unregister_simple(hook_list, func) \
