@@ -1,6 +1,6 @@
-/* $Id: e2_list.h 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_list.h 2743 2013-09-19 22:29:00Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -23,10 +23,10 @@ along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 #include "emelfm2.h"
 
 gint e2_list_strcmp (const gchar *a, const gchar *b);
-void e2_list_update_history (const gchar *command, GList **history,
-	guint *cur, guint max, gboolean doubl);
-gchar **e2_list_to_strv (GList *list);
-GList *e2_list_copy_with_data (GList *list);
+void e2_list_update_history (GList **history, const gchar *latest, guint *cur,
+	guint max, gboolean doubl);
+gchar **e2_list_to_strv (GList *list) G_GNUC_MALLOC;
+GList *e2_list_copy_with_data (GList *list) G_GNUC_MALLOC;
 void e2_list_free_with_data (GList **list);
 //void e2_list_free_data_only (GList **list);
 //GList *e2_list_find_data_string (GList *list, gchar *search_text);
