@@ -1,6 +1,6 @@
-/* $Id: e2_textiter.h 1663 2009-05-09 09:59:37Z tpgww $
+/* $Id: e2_textiter.h 2318 2011-01-27 03:50:22Z tpgww $
 
- Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+ Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,14 +14,15 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #ifndef __E2_TEXT_ITER_H__
 #define __E2_TEXT_ITER_H__
 
-#include <gtk/gtktextiter.h>
+#ifndef USE_GTK3_0
+# include <gtk/gtktextiter.h>
+#endif
 
 //this is a superset of GtkTextSearchFlags
 typedef enum
