@@ -1,6 +1,6 @@
-/* $Id: e2_tree.c 2064 2010-03-12 13:15:36Z tpgww $
+/* $Id: e2_tree.c 2815 2013-10-13 07:00:55Z tpgww $
 
-Copyright (C) 2004-2010 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -63,7 +63,7 @@ gboolean e2_tree_find_iter_from_str (GtkTreeModel *model, gint column,
 @a iter is set to matching iter if any, or is unchanged if no match is found
 @param model treemodel to be interrogated
 @param column index of the column in @a model to scan
-@param search the string to be searched for
+@param search the string to be searched for, may be NULL
 @param iter pointer to initial treeiter for model interrogation
 @param with_children TRUE if matched iter must have child(ren)
 
@@ -79,32 +79,35 @@ gboolean e2_tree_find_iter_from_str_simple (GtkTreeModel *model, gint column,
 		GtkTreeIter iter2;
 		gchar *value;
 		gtk_tree_model_get (model, iter, column, &value, -1);
-		if (value == NULL)
-			continue;
-		if (!strcmp (value, search))
+		if (value != NULL)
 		{
-			g_free (value);
-			if (with_children)
+			if (search && strcmp (value, search) == 0)
 			{
-				if (gtk_tree_model_iter_children (model, &iter2, iter))
+				if (!with_children ||
+					gtk_tree_model_iter_children (model, &iter2, iter))
+				{
+					g_free (value);
 					return TRUE;
-				else
-					continue;
+				}
 			}
-			else
-				return TRUE;
+			g_free (value);
 		}
+		else if (search == NULL)
+		{
+			if (!with_children ||
+				gtk_tree_model_iter_children (model, &iter2, iter))
+					return TRUE;
+		}
+
 		if (gtk_tree_model_iter_children (model, &iter2, iter))
 		{
 			if (e2_tree_find_iter_from_str_simple (model, column, search,
 					&iter2, with_children))
 			{
 				*iter = iter2;
-				g_free (value);
 				return TRUE;
 			}
 		}
-		g_free (value);
 	} while (gtk_tree_model_iter_next (model, iter));
 	*iter = backup;
 	return FALSE;
@@ -390,7 +393,9 @@ gboolean e2_tree_ref_to_iter (GtkTreeStore *store, GtkTreeRowReference *ref,
 */
 void e2_tree_expand_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
+	NEEDCLOSEBGL
 	gtk_tree_view_expand_all (treeview);
+	NEEDOPENBGL
 }
 /**
 @brief tree collapse-all callback
@@ -402,7 +407,9 @@ void e2_tree_expand_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 */
 void e2_tree_collapse_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
+	NEEDCLOSEBGL
 	gtk_tree_view_collapse_all (treeview);
+	NEEDOPENBGL
 }
 /**
 @brief find previous iter in a tree model
@@ -522,10 +529,11 @@ void e2_tree_delete (GtkTreeView *treeview)
 		gtk_tree_path_free (newpath);
 	}
 }
+
 /**
 @brief add treemodel row, and all its descendants, to list
 
-A recursive depth-first copy process puts the path and corresponding
+A recursive depth-first copy process puts the treepath and corresponding
 column values for @a iter into an array, then appends that array to
 the list @a rowscopy
 
@@ -539,27 +547,20 @@ the list @a rowscopy
 static void _e2_tree_copy_branch (GtkTreeModel *model, GtkTreeIter *iter,
 	gint columncount, GList **rowscopy)
 {
-	GValueArray *row_array = g_value_array_new (columncount + 1);
-
+	//probably, this doesn't need to be a dynamic array
+	GArray *row_array = g_array_sized_new (FALSE, TRUE, sizeof(GValue), columncount + 1);
+	//glib doesn't actually clear stuff, so we do so
+	memset (row_array->data, 0, sizeof(GValue)*(columncount + 1));
 	//add path of copied node to the start of the buffer entry, to support proper pasting
-	GValue value = {0, };
-//	GValue *valptr = g_value_init (&value, G_TYPE_STRING);
-	GValue *valptr = g_value_init (&value, G_TYPE_POINTER);
+	GValue *valptr = (GValue*)row_array->data;
+	g_value_init (valptr, G_TYPE_POINTER);
 	GtkTreePath *path = gtk_tree_model_get_path (model, iter);
-//	gchar *pathstring = gtk_tree_path_to_string (path);
-//	g_value_set_static_string (valptr, pathstring);  //no pathstring copy created, don't free it here
 	g_value_set_pointer (valptr, path);
-	g_value_array_append (row_array, valptr);
-	g_value_unset (valptr);
-//	gtk_tree_path_free (path);
-	//add store column values to the buffer entry
+	//add store-column values to the array
 	gint i;
 	for (i = 0; i < columncount ; i++)
-	{
-		gtk_tree_model_get_value (model, iter, i, valptr);
-		g_value_array_append (row_array, valptr);
-		g_value_unset (valptr);
-	}
+		gtk_tree_model_get_value (model, iter, i, ++valptr);
+	row_array->len = columncount + 1;
 	//add this row to the buffer list
 	*rowscopy = g_list_append (*rowscopy, row_array);
 	//recurse to handle any child(ren)
@@ -635,8 +636,8 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 	{
 		//get source-view path for the 1st buffered item, the
 		//root of the copied branch (or one of several roots)
-		GValueArray *value_array = rowscopied->data;
-		GValue *value = g_value_array_get_nth (value_array, 0);
+		GArray *value_array = rowscopied->data;
+		GValue *value = &g_array_index (value_array, GValue, 0);
 //		gchar *pathstring = g_value_dup_string (value);
 //		GtkTreePath *path = gtk_tree_path_new_from_string (pathstring);
 		GtkTreePath *rootpath = (GtkTreePath *) g_value_get_pointer (value);
@@ -657,7 +658,7 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 		GtkTreeIter *parent = &local2;
 
 		//find a selected item where it's valid to paste
-		for (tmp = selpaths; tmp!=NULL; tmp=tmp->next)
+		for (tmp = selpaths; tmp != NULL; tmp = tmp->next)
 		{
 			if (gtk_tree_model_get_iter (model, iter, (GtkTreePath *) tmp->data))
 			{
@@ -695,7 +696,7 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 			gint i;
 			for (i = 0; i < columns; i++)
 			{
-				value = g_value_array_get_nth (value_array, i+1);
+				value = &g_array_index (value_array, GValue, i+1);
 				gtk_tree_store_set_value (GTK_TREE_STORE (model), iter, i, value);
 			}
 
@@ -720,11 +721,11 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 			GtkTreePath *src_path;
 			gchar *ppstring;
 			GList *tmp2 = rowscopied;
-			while ((tmp2=tmp2->next) != NULL)
+			while ((tmp2 = tmp2->next) != NULL)
 			{
 				value_array = tmp2->data;
 				//get source-view path and find where it should sit in the new tree
-				value = g_value_array_get_nth (value_array, 0);
+				value = &g_array_index (value_array, GValue, 0);
 //				pathstring = g_value_dup_string (value);
 				src_path = (GtkTreePath *) g_value_get_pointer (value);
 				path = gtk_tree_path_copy (src_path);
@@ -774,7 +775,7 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 				//populate row
 				for (i = 0; i < columns; i++)
 				{
-					value = g_value_array_get_nth (value_array, i+1);
+					value = &g_array_index (value_array, GValue, i+1);
 					gtk_tree_store_set_value (GTK_TREE_STORE (model), iter, i, value);
 				}
 //				g_free (pathstring);
@@ -783,8 +784,7 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 				ref = gtk_tree_row_reference_new (model, path);
 				gtk_tree_path_free (path);
 				ppstring = gtk_tree_path_to_string (src_path);
-//				g_hash_table_insert (ref_hash, g_value_dup_string (value),
-//					ref);
+//				g_hash_table_insert (ref_hash, g_value_dup_string (value), ref);
 				g_hash_table_insert (ref_hash, ppstring, ref);
 			}
 			//no change to (single row) selection, as all new rows are after
