@@ -1,6 +1,6 @@
-/* $Id: e2_utf8.c 2487 2012-04-06 10:19:53Z tpgww $
+/* $Id: e2_utf8.c 2746 2013-09-19 22:59:03Z tpgww $
 
-Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 Portion _maybe_ copyright (C) 2004-2005 Tarot Osuji
 
 This file is part of emelFM2.
@@ -217,10 +217,11 @@ gboolean e2_utf8_caseless_match (const gchar *s1, const gchar *s2,
 	return ret;
 }
 /**
-@brief duplicate utf8 string, truncated after @a num characters if the string is longer than that
+@brief duplicate UTF-8 string, truncated after @a num characters if the string is longer than that
+TODO error message assumes BGL closed
 @param str the string to be duplicated
 @param num maximum no. of characters in @a str to be processed
-@return the duplicated string
+@return the duplicated string, or NULL upon memory error
 */
 gchar *e2_utf8_ndup (const gchar *str, glong num)
 {
@@ -229,14 +230,18 @@ gchar *e2_utf8_ndup (const gchar *str, glong num)
 		num = size;
 	gchar *end = g_utf8_offset_to_pointer (str, num);
 	glong byte_size = end - str + 1;
-	gchar *utf8 = g_malloc (byte_size);
-	return g_utf8_strncpy (utf8, str, num);
+	gchar *utf8 = g_try_malloc (byte_size);
+	CHECKALLOCATEDWARN (utf8, return NULL;); //TODO depends on closed BGL
+	if (utf8 != NULL)
+		return g_utf8_strncpy (utf8, str, num);
+	return NULL;
 }
 /**
 @brief escape all instances of character @a c in utf-8 string @a str
+TODO memory-allocation error message assumes BGL closed
 @param str the string to be processed
 @param c the character to be escaped if found in @a str
-@return newly-allocated string with any occurrence of @a c escaped with a '\'
+@return newly-allocated string with any occurrence of @a c escaped with a '\', or NULL upon memory error
 */
 gchar *e2_utf8_escape (const gchar *str, gunichar c)
 {
@@ -244,32 +249,40 @@ gchar *e2_utf8_escape (const gchar *str, gunichar c)
 	glong i;
 	gint count = 0;
 	const gchar *tmp = str;
+
 	for (i = 0; i < clen; i++)
 	{
 		if (g_utf8_get_char (tmp) == c)
 			count++;
 		tmp = g_utf8_next_char (tmp);
 	}
+
 	if (count > 0)
 	{
 		gint len = strlen (str);
 		gint size = (len + count + 1) * sizeof (gchar);
-		gchar *ret = g_malloc (size);
-		count = 0;
-		for (i = 0; i < clen; i++)
+		gchar *ret = g_try_malloc (size);
+		CHECKALLOCATEDWARN (ret, return NULL;); //TODO assumes BGL closed
+		if (ret != NULL)
 		{
-			gchar *cur = g_utf8_offset_to_pointer (str, i);
-			if (g_utf8_get_char (cur) == c)
-				g_utf8_strncpy (g_utf8_offset_to_pointer (ret, count++), "\\", 1);
-			g_utf8_strncpy (g_utf8_offset_to_pointer (ret, count++), cur, 1);
+			count = 0;
+			for (i = 0; i < clen; i++)
+			{
+				gchar *cur = g_utf8_offset_to_pointer (str, i);
+				if (g_utf8_get_char (cur) == c)
+					g_utf8_strncpy (g_utf8_offset_to_pointer (ret, count++), "\\", 1);
+				g_utf8_strncpy (g_utf8_offset_to_pointer (ret, count++), cur, 1);
+			}
+			ret[size - 1] = '\0';
 		}
-		ret[size - 1] = '\0';
 		return ret;
-	} else
+	}
+	else
 		return g_strdup (str);
 }
 /**
 @brief unescape all instances of character @a c in utf-8 string @a str
+TODO allocation-error message assumes BGL closed
 @param str the string to be processed
 @param c the character to be unescaped if found in @a str
 @return newly-allocated string without any occurrence of '\'in front of @a c, or NULL after error
@@ -278,8 +291,8 @@ gchar *e2_utf8_unescape (const gchar *str, gunichar c)
 {
 	glong count = 0;
 	gint size = (strlen (str) + 1) * sizeof (gchar);
-	gchar *ret = g_try_malloc (size);	//FIXME nandle error
-	CHECKALLOCATEDWARN (ret, return NULL);
+	gchar *ret = g_try_malloc (size);
+	CHECKALLOCATEDWARN (ret, return NULL;); //TODO assumes BGL closed
 	*ret = '\0';	//in case str is empty
 	const gchar *tmp = str;
 	while (*tmp != '\0')
@@ -950,7 +963,6 @@ static const gchar *detect_charset_BOM (const guchar *text)
 const gchar *e2_utf8_detect_charset (const guchar *text, gboolean *isutf8)
 {
 	const gchar *charset;
-	const guchar *start;
 	guchar c;
 
 	charset = detect_charset_BOM (text);
@@ -961,7 +973,6 @@ const gchar *e2_utf8_detect_charset (const guchar *text, gboolean *isutf8)
 		return charset;
 	}
 
-	start = text;
 	*isutf8 = FALSE;
 	while ((c = *text++) != '\0')
 	{
