@@ -1,6 +1,6 @@
-/* $Id: e2_utf8.h 1663 2009-05-09 09:59:37Z tpgww $
+/* $Id: e2_utf8.h 2743 2013-09-19 22:29:00Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -125,14 +124,14 @@ void e2_utf8_set_name_conversion (gboolean convert);
 //#define PCHR(p) p=g_utf8_prev_char(p)
 
 //inline
-gchar *e2_utf8_filename_from_locale (const gchar *d);
+gchar *e2_utf8_filename_from_locale (const gchar *d) G_GNUC_MALLOC;
 //inline
-gchar *e2_utf8_filename_to_locale (const gchar *d);
+gchar *e2_utf8_filename_to_locale (const gchar *d) G_GNUC_MALLOC;
 //inline
-gchar *e2_utf8_to_locale (const gchar *d);
-gchar *e2_utf8_to_locale_fallback (const gchar *d);
-gchar *e2_utf8_from_locale (const gchar *d);
-gchar *e2_utf8_from_locale_fallback (const gchar *d);
+gchar *e2_utf8_to_locale (const gchar *d) G_GNUC_MALLOC;
+gchar *e2_utf8_to_locale_fallback (const gchar *d) G_GNUC_MALLOC;
+gchar *e2_utf8_from_locale (const gchar *d) G_GNUC_MALLOC;
+gchar *e2_utf8_from_locale_fallback (const gchar *d) G_GNUC_MALLOC;
 //macro for inlining speedier conversions
 #define e2_utf8_from_locale_fast(d) \
  (g_utf8_validate (d, -1, NULL) ) ? g_strdup (d) : \
@@ -143,9 +142,9 @@ const gchar *e2_utf8_strcasestr (const gchar *haystack, const gchar *needle);
 const gchar *e2_utf8_strrcasestr (const gchar *haystack, const gchar *needle);
 gboolean e2_utf8_caseless_match (const gchar *s1, const gchar *s2, gssize n1,
 	gssize n2);
-gchar *e2_utf8_ndup (const gchar *str, glong num);
-gchar *e2_utf8_escape (const gchar *str, gunichar c);
-gchar *e2_utf8_unescape (const gchar *str, gunichar c);
+gchar *e2_utf8_ndup (const gchar *str, glong num) G_GNUC_MALLOC;
+gchar *e2_utf8_escape (const gchar *str, gunichar c) G_GNUC_MALLOC;
+gchar *e2_utf8_unescape (const gchar *str, gunichar c) G_GNUC_MALLOC;
 
 const gchar *e2_utf8_detect_charset (const guchar *text, gboolean *isutf8);
 const gchar *e2_utf8_guess_charset (const guchar *text);
