@@ -1,6 +1,6 @@
-/* $Id: e2_utils.h 2469 2012-03-16 00:19:24Z tpgww $
+/* $Id: e2_utils.h 2910 2013-11-13 00:58:59Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2013 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -92,17 +92,17 @@ typedef struct
 } MagicIface;
 #endif
 
-//E2_Trio *e2_utils_trio_new (void);
-E2_Sextet *e2_utils_sextet_new (void);
-E2_Nontet *e2_utils_nontet_new (void);
+//E2_Trio *e2_utils_trio_new (void) G_GNUC_MALLOC;
+E2_Sextet *e2_utils_sextet_new (void) G_GNUC_MALLOC;
+E2_Nontet *e2_utils_nontet_new (void) G_GNUC_MALLOC;
 //void e2_utils_trio_destroy (E2_Trio *t);
 void e2_utils_sextet_destroy (E2_Sextet *s);
 void e2_utils_nontet_destroy (E2_Nontet *n);
 void e2_utils_show_memory_message (void);
 void e2_utils_memory_error (void);
 void e2_utils_show_help (gchar *title);
-gchar *e2_utils_color2str (GdkColor *color);
-gchar *e2_utils_str_replace (const gchar *str, const gchar *old, const gchar *new);
+gchar *e2_utils_color2str (GdkColor *color) G_GNUC_MALLOC;
+gchar *e2_utils_str_replace (const gchar *str, const gchar *old, const gchar *new) G_GNUC_MALLOC;
 gint e2_utils_LF_line_ends (gchar *text);
 gchar *e2_utils_revert_line_ends (gchar *text, guint linecount, gint separator);
 #ifdef E2_MAGIC
@@ -111,55 +111,49 @@ gboolean e2_utils_fill_magic_iface (MagicIface *iface);
 gchar *e2_utils_get_mimetype (VPATH *localpath);
 void e2_utils_get_charset (const gchar **encoding);
 void e2_utils_rowstr_split (gchar *line, gint columns, gchar *parts[]);
-gchar *e2_utils_str_stretch (gchar *string);
-gchar *e2_utils_str_shorten (gchar *string, gint limit, E2_DotMode position);
-gchar *e2_utils_str_to_lower (gchar *string);
+gchar *e2_utils_str_stretch (gchar *string) G_GNUC_MALLOC;
+gchar *e2_utils_str_shorten (gchar *string, gint limit, E2_DotMode position) G_GNUC_MALLOC;
+gchar *e2_utils_str_to_lower (gchar *string) G_GNUC_MALLOC;
 gint e2_utils_get_byte_position (const gchar *utf8_string, gint charoffset);
-gchar *e2_utils_dircat (ViewInfo *view, const gchar *string, gboolean localised);
-gchar *e2_utils_strcat (const gchar *string1, const gchar *string2);
+gchar *e2_utils_dircat (ViewInfo *view, const gchar *string, gboolean localised) G_GNUC_MALLOC;
+gchar *e2_utils_strcat (const gchar *string1, const gchar *string2) G_GNUC_MALLOC;
 gchar **e2_utils_str_breakup (const gchar *string, const gchar *delimiter,
-	gint max_tokens);
+	gint max_tokens) G_GNUC_MALLOC;
 gchar *e2_utils_find_whitespace (gchar *string);
 gchar *e2_utils_pass_whitespace (gchar *string);
-gchar *e2_utils_quote_string (const gchar *string);
-gchar *e2_utils_unquote_string (const gchar *utf8string);
+gchar *e2_utils_quote_string (const gchar *string) G_GNUC_MALLOC;
+gchar *e2_utils_unquote_string (const gchar *utf8string) G_GNUC_MALLOC;
 gchar *e2_utils_bare_strchr (gchar *string, gchar c);
-gchar *e2_utils_get_first_part (gchar *string, gboolean quoted);
-gchar *e2_utils_get_tempname (const gchar *orig);
+gchar *e2_utils_get_first_part (gchar *string, gboolean quoted) G_GNUC_MALLOC;
+gchar *e2_utils_get_tempname (const gchar *orig) G_GNUC_MALLOC;
 gboolean e2_utils_get_parent_path (gchar *path, gboolean ignore_trailer);
 gchar *e2_utils_path_clean (gchar *path)
 #ifdef G_GNUC_WARN_UNUSED_RESULT
  G_GNUC_WARN_UNUSED_RESULT
 #endif
 ;
-gchar *e2_utils_translate_relative_path (gchar *base_dir, gchar *new_path);
-gchar *e2_utils_create_relative_path (VPATH *src, VPATH *dest);
+gchar *e2_utils_translate_relative_path (const gchar *base_dir, const gchar *new_path) G_GNUC_MALLOC;
+gchar *e2_utils_create_relative_path (VPATH *src, VPATH *dest) G_GNUC_MALLOC;
 const gchar *e2_utils_skip_relative_path (VPATH *localpath);
-gchar *e2_utils_get_temp_path (const gchar *id);
-gchar *e2_utils_get_home_path (const gchar *utfpath);
-gchar *e2_utils_get_trash_path (gchar *localpath, gboolean filesplace);
-GList *e2_utils_get_trash_all (void);
-gchar *e2_utils_get_icons_path (gboolean withtrailer);
-#ifdef E2_IMAGECACHE
-void e2_utils_init_icon_sizes (void);
-gint e2_utils_get_icon_size (GtkIconSize size);
-GtkIconSize e2_utils_get_best_icon_size (gint psize);
-#endif
-gboolean e2_utils_check_stock_icon (const gchar *name);
+gchar *e2_utils_get_temp_path (const gchar *id) G_GNUC_MALLOC;
+gchar *e2_utils_get_home_path (const gchar *utfpath) G_GNUC_MALLOC;
+gchar *e2_utils_get_trash_path (gchar *localpath, gboolean filesplace) G_GNUC_MALLOC;
+GList *e2_utils_get_trash_all (void) G_GNUC_MALLOC;
+
 const gchar *e2_utils_get_output_font (void);
 void e2_utils_update_gtk_settings (void);
-gchar *e2_utils_expand_macros (const gchar *text, const gchar *for_each);
-gchar *e2_utils_replace_name_macros (const gchar *text, const gchar *utfpath);
+gchar *e2_utils_expand_macros (const gchar *text, const gchar *for_each) G_GNUC_MALLOC;
+gchar *e2_utils_replace_name_macros (const gchar *text, const gchar *utfpath) G_GNUC_MALLOC;
 gchar *e2_utils_replace_multiname (const gchar *text, gchar *path, GPtrArray *array,
-	gboolean single);
+	gboolean single) G_GNUC_MALLOC;
 gboolean e2_utils_get_variable (gchar **variable);
-gchar *e2_utils_replace_vars (gchar *raw, gboolean rawpath);
+gchar *e2_utils_replace_vars (gchar *raw, gboolean rawpath) G_GNUC_MALLOC;
 gchar *e2_utils_replace_wildcards (gchar *raw);
 void e2_utils_get_abs_pos (GtkWidget *widget, gint *x, gint *y);
 #ifdef USE_GTK3_0
 void e2_utils_save_state (GtkWidget *widget);
 GdkModifierType e2_utils_get_savedstate (GtkWidget *widget);
-GList *e2_utils_get_pointer_devices_for_widget (GtkWidget *widget);
+gboolean e2_utils_get_pointer_position (GtkWidget *widget, gint *x, gint *y);
 #else
 GdkModifierType e2_utils_get_modifiers (void);
 #endif
