@@ -1,6 +1,6 @@
-/* $Id: e2_widget.c 2445 2012-03-04 07:47:21Z tpgww $
+/* $Id: e2_widget.c 3001 2014-01-17 23:05:07Z tpgww $
 
-Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2014 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -29,6 +29,7 @@ This file contains various miscelleanous GtkWidget utility functions.
 #include <string.h>
 #include "e2_widget.h"
 #include "e2_dialog.h"
+#include "e2_icons.h"
 
 #ifdef USE_GTK2_12TIPS
 typedef struct
@@ -65,7 +66,9 @@ static gboolean _e2_widget_show_tip_cb (GtkWidget  *widget,
 	gchar *text = (gchar *) g_object_get_data (G_OBJECT (widget), "_e2-tiptext");
 	if (text != NULL && tooltip != NULL)
 	{
+		NEEDCLOSEBGL
 		gtk_tooltip_set_text (tooltip, text);
+		NEEDOPENBGL
 		return TRUE;
 	}
 	return FALSE;
@@ -73,14 +76,14 @@ static gboolean _e2_widget_show_tip_cb (GtkWidget  *widget,
 /**
 @brief idle-callback to add or remove a tip with BGL safely closed
 @param widget the widget whose tip is to be altered
-@return FALSE always, to cancel the idle
+@return FALSE always, to cancel the source
 */
 static gboolean _e2_widget_defer_tip (GtkWidget *widget)
 {
 	gpointer tip = g_object_get_data (G_OBJECT (widget), "_e2-tiptext");
-	gdk_threads_enter ();
+	CLOSEBGL
 	gtk_widget_set_has_tooltip (widget, (tip != NULL));
-	gdk_threads_leave ();
+	OPENBGL
 	if (tip != NULL)
 		g_signal_connect (G_OBJECT (widget), "query-tooltip",
 			G_CALLBACK(_e2_widget_show_tip_cb), NULL);
@@ -146,11 +149,11 @@ void e2_widget_set_tip (GtkWidget *widget, const gchar *tiptext1, const gchar *t
 	{
 #ifdef NATIVE_BGL
 		printd (WARN, "Closing BGL with default mutex will sometimes be recursively bad");
-		gdk_threads_enter ();
+		CLOSEBGL
 #else
 # ifdef DEBUG_MESSAGES
 		//FIXME find a way to make errorcheck mutex block, to avoid having to abort
-		gint _lockres = pthread_mutex_trylock (&gdklock);
+		gint _lockres = pthread_mutex_trylock (&display_mutex);
 		if (_lockres != 0)
 		{
 			printd (DEBUG, "%s BGL trylock result %d", __PRETTY_FUNCTION__, _lockres);
@@ -162,7 +165,7 @@ void e2_widget_set_tip (GtkWidget *widget, const gchar *tiptext1, const gchar *t
 		}
 # else
 		//recursive mutex is more tolerant
-		gdk_threads_enter ();
+		CLOSEBGL
 # endif
 #endif
 	}
@@ -172,7 +175,7 @@ void e2_widget_set_tip (GtkWidget *widget, const gchar *tiptext1, const gchar *t
 	gtk_tooltips_set_tip (app.tooltips, widget, tiptext1, tiptext2);
 
 	if (close)
-		gdk_threads_leave ();
+		OPENBGL
 }
 #endif //def USE_GTK2_12TIPS
 
@@ -228,14 +231,15 @@ GtkWidget *e2_widget_get_icon (const gchar *icon, GtkIconSize size)
 	if (icon == NULL || *icon == '\0')
 		return NULL;
 
-#ifdef E2_IMAGECACHE
-	//get cached image, after adding it to the cache if not there already
-	E2_Image *cache_img = e2_cache_image_get (icon, size);
-	return (gtk_image_new_from_pixbuf (cache_img->pixbuf));
+#ifdef E2_ICONCACHE
+	GdkPixbuf *pxb = e2_icons_get_puxbuf (icon, size, TRUE);
+	if (pxb != NULL)
+		return (gtk_image_new_from_pixbuf (pxb));
+	return NULL;
 #else
 	GtkWidget *image;
-	if (e2_utils_check_stock_icon (icon))
-		image = gtk_image_new_from_stock (icon, size);
+	if (e2_icons_check_stock (icon))
+		image = gtk_image_new_from_icon_name (icon, size);
 	else
 	{
 		GdkPixbuf *pixbuf;
@@ -244,7 +248,7 @@ GtkWidget *e2_widget_get_icon (const gchar *icon, GtkIconSize size)
 			fullname = icon;
 		else
 		{
-			gchar *localpath = e2_utils_get_icons_path (TRUE);
+			gchar *localpath = e2_icons_get_custom_path (TRUE);
 			fullname = e2_utils_strcat (localpath, icon);
 			g_free (localpath);
 		}
@@ -255,7 +259,7 @@ GtkWidget *e2_widget_get_icon (const gchar *icon, GtkIconSize size)
 			gint width, height;
 			if (!gtk_icon_size_lookup (size, &width, &height))
 			{
-				width = 16; height = 16;
+				width = 16; height = 16; //unlikely default to minimum size
 			}
 			pixbuf2 = gdk_pixbuf_scale_simple (pixbuf, width, height,
 				GDK_INTERP_BILINEAR);
@@ -273,21 +277,20 @@ GtkWidget *e2_widget_get_icon (const gchar *icon, GtkIconSize size)
 #endif
 }
 /**
-@brief create a vertically-centred GtkLabel and add it to @a table
+@brief create a vertically-centred GtkLabel and add it to @a table at specified position
 
 @param table the table to attach the new label into
 @param text the text for the new label, may have markup
 @param xalign the label's horizontal alignment
-@param left column number to attach the left side of the label to
-@param right column number to attach the right side of the label to
-@param top row number to attach the top of the label to
-@param bottom row number to attach the bottom of the label to
+@param left index of column to left of attach-position
+@param right index of column to right of attach-position
+@param top index of row above attach-position
+@param bottom index of row below attach-position
 
 @return  the label widget
 */
-GtkWidget *e2_widget_add_mid_label_to_table (GtkWidget *table,
-	gchar *text, gfloat xalign,
-	gint left, gint right, gint top, gint bottom)
+GtkWidget *e2_widget_add_mid_label_to_table (GtkWidget *table, gchar *text,
+	gfloat xalign, gint left, gint right, gint top, gint bottom)
 {
 	GtkWidget *label;
 	label = gtk_label_new (NULL);
@@ -295,7 +298,11 @@ GtkWidget *e2_widget_add_mid_label_to_table (GtkWidget *table,
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), xalign, 0.5);
 //	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
+#ifdef USE_GTK3_2
+	gtk_grid_attach (GTK_GRID (table), label, left, top, right-left, bottom-top);
+#else
 	gtk_table_attach_defaults (GTK_TABLE (table), label, left, right, top, bottom);
+#endif
 	return label;
 }
 /**
@@ -351,9 +358,12 @@ GtkWidget *e2_widget_add_label (GtkWidget *box, const gchar *text,
 static gboolean _e2_widget_entry_keypress_cb (GtkWidget *entry, GdkEventKey *event,
 	gpointer user_data)
 {
+	OPENBGL //TODO do this if NEEDCLOSEBGL is empty
 	//prevent sporadic bad latency in UI updates
 	gtk_widget_queue_draw (entry);
+	CLOSEBGL
 
+	NEEDCLOSEBGLX //do this if NEEDCLOSEBGL is empty
 	if (event->keyval == GDK_Delete)
 	{
 		guint modifiers = gtk_accelerator_get_default_mod_mask ();
@@ -361,9 +371,11 @@ static gboolean _e2_widget_entry_keypress_cb (GtkWidget *entry, GdkEventKey *eve
 		{
 			gint start = gtk_editable_get_position (GTK_EDITABLE (entry));
 			gtk_editable_delete_text (GTK_EDITABLE (entry), start, -1);
+			NEEDOPENBGL
 			return TRUE;
 		}
 	}
+	NEEDOPENBGL
 	return FALSE;
 }
 /**
@@ -402,10 +414,10 @@ GtkWidget *e2_widget_add_entry (GtkWidget *box, gchar *init_text,
 @param table the table to attach the new entry into
 @param init_text the initial text for the new entry, or NULL
 @param select_text TRUE to select the initial text in the entry
-@param left column number to attach the left side of the entry to
-@param right column number to attach the right side of the entry to
-@param top row number to attach the top of the entry to
-@param bottom row number to attach the bottom of the entry to
+@param left index of column to left of attach-position
+@param right index of column to right of attach-position
+@param top index of row above attach-position
+@param bottom index of row below attach-position
 
 @return the entry widget
 */
@@ -424,8 +436,11 @@ GtkWidget *e2_widget_add_entry_to_table (GtkWidget *table,
 	}
 	g_signal_connect (G_OBJECT (entry), "key-press-event",
 		G_CALLBACK (_e2_widget_entry_keypress_cb), NULL);
+#ifdef USE_GTK3_2
+	gtk_grid_attach (GTK_GRID (table), entry, left, top, right-left, bottom-top);
+#else
 	gtk_table_attach_defaults (GTK_TABLE (table), entry, left, right, top, bottom);
-
+#endif
 	return entry;
 }
 */
@@ -440,10 +455,17 @@ GtkWidget *e2_widget_add_entry_to_table (GtkWidget *table,
 */
 GtkWidget *e2_widget_get_box (gboolean vertical, gboolean homogen, gint spacing)
 {
-//gtk3-type box creation is helpful only if box orientation needs to change during a session
+#ifdef USE_GTK3_0
+	GtkWidget *box = (vertical) ?
+		gtk_box_new (GTK_ORIENTATION_VERTICAL, spacing):
+		gtk_box_new (GTK_ORIENTATION_HORIZONTAL, spacing);
+	if (homogen)
+		gtk_box_set_homogeneous ((GtkBox*)box, TRUE);
+#else
 	GtkWidget *box = (vertical) ?
 		gtk_vbox_new (homogen, spacing):
 		gtk_hbox_new (homogen, spacing);
+#endif
 	return box;
 }
 /**
@@ -497,10 +519,14 @@ GtkWidget *e2_widget_get_sw (GtkPolicyType h_policy, GtkPolicyType v_policy,
 	//set bars positions even for policy none, that may change later
 	gtk_scrolled_window_set_placement (GTK_SCROLLED_WINDOW (scrolled_window),
 		e2_option_sel_get ("scrollbar-position"));
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+	gtk_scrolled_window_set_policy ((GtkScrolledWindow*) scrolled_window,
 		h_policy, v_policy);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+	gtk_scrolled_window_set_shadow_type ((GtkScrolledWindow*) scrolled_window,
 		shadow);
+//#ifdef USE_GTK3_4
+//	gtk_scrolled_window_set_kinetic_scrolling (scrolled_window, FALSE);
+//#endif
+
  	return scrolled_window;
 }
 /**
@@ -542,7 +568,11 @@ set to GTK_SHADOW_NONE.
 */
 void e2_widget_sw_add_with_viewport (GtkWidget *sw, GtkWidget *child)
 {
+#ifdef USE_GTK3_8
+	gtk_container_add (GTK_CONTAINER (sw), child);
+#else
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), child);
+#endif
 	gtk_viewport_set_shadow_type (
 #ifdef USE_GTK2_14
 		GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (sw))),
@@ -563,12 +593,20 @@ void e2_widget_sw_add_with_viewport (GtkWidget *sw, GtkWidget *child)
 GtkWidget *e2_widget_add_separator (GtkWidget *box, gboolean exp, guint pad)
 {
 	GtkWidget *sep;
+#ifdef USE_GTK3_2
+	if (box != NULL &&
+		gtk_orientable_get_orientation (GTK_ORIENTABLE (box)) == GTK_ORIENTATION_HORIZONTAL)
+		sep = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
+	else
+		sep = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+#else
 	if (GTK_IS_HBOX (box))
 		sep = gtk_vseparator_new ();
 	else if (box == NULL || GTK_IS_VBOX (box))
 		sep = gtk_hseparator_new ();
 	else
 		return NULL;
+#endif
 	if (box != NULL)
 		gtk_box_pack_start (GTK_BOX (box), sep, exp, TRUE, pad);
 	return sep;
@@ -589,7 +627,18 @@ GtkWidget *e2_widget_add_table (GtkWidget *box, gint rows, gint cols,
 	gboolean homogen, gboolean exp, guint pad)
 {
 	GtkWidget *table;
+#ifdef USE_GTK3_2
+	gint i;
+	table = gtk_grid_new ();
+	if (homogen)
+		gtk_grid_set_column_homogeneous ((GtkGrid *)table, TRUE);
+	for (i = 0; i < cols; i++)
+		gtk_grid_insert_column ((GtkGrid *)table, i);
+	for (i = 0; i < rows; i++)
+		gtk_grid_insert_row ((GtkGrid *)table, i);
+#else
 	table = gtk_table_new (rows, cols, homogen);
+#endif
 	if (box != NULL)
 		gtk_box_pack_start (GTK_BOX (box), table, exp, TRUE, pad);
 	return table;
@@ -603,9 +652,19 @@ GtkWidget *e2_widget_add_framed_table (GtkWidget *box, gchar *title, gint rows,
 	frame = gtk_frame_new (title);
 	gtk_box_pack_start (GTK_BOX (box), frame, exp, TRUE, pad);
 
+#ifdef USE_GTK3_2
+	gint i;
+	table = gtk_grid_new ();
+	for (i = 0; i < cols; i++)
+		gtk_grid_insert_column ((GtkGrid *)table, i);
+	for (i = 0; i < rows; i++)
+		gtk_grid_insert_row ((GtkGrid *)table, i);
+	gtk_grid_set_row_spacing ((GtkGrid *)table, E2_PADDING);
+#else
 	table = gtk_table_new (rows, cols, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), E2_PADDING_XSMALL);
 	gtk_table_set_row_spacings (GTK_TABLE (table), E2_PADDING);
+#endif
+	gtk_container_set_border_width (GTK_CONTAINER (table), E2_PADDING_XSMALL);
 	gtk_container_add (GTK_CONTAINER (frame), table);
 	return table;
 }
@@ -741,7 +800,13 @@ void e2_widget_get_font_pixels (GtkWidget *widget, gint *width, gint *height)
 	PangoContext *context = gtk_widget_get_pango_context (widget);
 #ifdef USE_GTK3_0
 	GtkStyleContext *sc = gtk_widget_get_style_context (widget);
+# ifdef USE_GTK3_8
+	const PangoFontDescription *desc;
+	gtk_style_context_get (sc, GTK_STATE_FLAG_NORMAL,
+		GTK_STYLE_PROPERTY_FONT, &desc, NULL);
+# else
 	const PangoFontDescription *desc = gtk_style_context_get_font (sc, GTK_STATE_NORMAL);
+# endif
 	PangoFontMetrics *metrics = pango_context_get_metrics (context, desc,
 		pango_context_get_language (context));
 #else
@@ -817,13 +882,17 @@ static void _e2_widget_check_changed_cb (GtkToggleButton *button, E2_OptionSet *
 		"e2-controller-widget");
 	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
 		"e2-controller-blocked")))
+	{
+//		NEEDCLOSEBGL
 		e2_option_bool_set_direct (boolset,
 #ifdef USE_GTK2_14
 			gtk_toggle_button_get_active (button)
 #else
 			button->active
 #endif
-	);
+		);
+//		NEEDOPENBGL
+	}
 }
 /**
 @brief set @a button state if it's not blocked and not already at the desired state
